@@ -34,10 +34,11 @@ convert_toml_crates_to_key_value() {
         grep --extended-regexp --invert-match "^\[|path|^$|^#" |
             cut --delimiter ',' --field 1 |
                 sed "s/version//g" |
-                    tr -d '"{=' |
-                        tr -d "'" |
-                            tr -s ' ' |
-                                sort
+                    sed "s/=/ = /g" |
+                        tr -d '"{=}' |
+                            tr -d "'" |
+                                tr -s ' ' |
+                                    sort
 }
 
 get_crate_latest_version() {
@@ -88,8 +89,8 @@ update_crate_version_in_toml() {
     toml_file="$4"
 
     # update crate version in toml
-    sed -i -- "s/${crate} = { version = \"${actual_version}\"/${crate} = { version = \"${last_version}\"/g" "${toml_file}"
-    sed -i -- "s/${crate} = \"${actual_version}\"/${crate} = \"${last_version}\"/g" "${toml_file}"
+    sed -i -- "s/^${crate}.*=.*{.*version.*=.*\"${actual_version}\"/${crate} = { version = \"${last_version}\"/g" "${toml_file}"
+    sed -i -- "s/^${crate}.*=.*\"${actual_version}\"/${crate} = \"${last_version}\"/g" "${toml_file}"
     if grep --extended-regexp --silent "${crate}.*=.*${last_version}" "${toml_file}"; then
         echo "${color_blue}updated to ${last_version}${color_reset}"
     else
@@ -98,24 +99,12 @@ update_crate_version_in_toml() {
     fi
 }
 
-which_is_the_newest_version() {
-    # init vars
-    first_version="$1"
-    second_version="$2"
-
-    # which is the newest version
-    echo -e "${first_version}\n${second_version}" |
-        sort --version-sort |
-        tail -1
-}
-
 main() {
     # init vars
     crates_api_root_url="https://crates.io/api/v1/crates"
     arg="${1:-}"
     check_args "${arg}"
     updated_count=0
-    blacklisted="indexmap"
 
     # update toml
     for package in packages/*; do
@@ -130,18 +119,11 @@ main() {
                 return 1
             fi
             echo -n "- ${crate} ${actual_version} "
-            newest_version=$(which_is_the_newest_version "${last_version}" "${actual_version}")
-            if [ "${newest_version}" == "${actual_version}" ]; then
+            if [ "${last_version}" == "${actual_version}" ]; then
                 echo "${color_green}newest${color_reset}"
             else
-                # Blacklist some crates
-                if [[ ${crate} =~ ${blacklisted} ]]; then
-                    echo "${color_yellow} blacklisted latest ${last_version}${color_reset}"
-                    continue
-                fi
-
                 if [ "${arg}" = "--check" ]; then
-                    echo "${color_red}old, please update to latest ${last_version}${color_reset}"
+                    echo "${color_red}please update to max stable version ${last_version}${color_reset}"
                     updated_count=$((updated_count + 1))
                 else
                     update_crate_version_in_toml "${crate}" "${actual_version}" "${last_version}" "${toml_file}"
