@@ -35,22 +35,32 @@ def test(script_file: str):
     print(cmd)
     result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
-    test_exit_code(os.path.splitext(script_file)[0] + ".exit", result)
-    test_stdout(os.path.splitext(script_file)[0] + ".out", result)
-    test_stdout_pattern(os.path.splitext(script_file)[0] + ".out.pattern", result)
-    test_stderr(os.path.splitext(script_file)[0] + ".err", result)
-    test_stderr_pattern(os.path.splitext(script_file)[0] + ".err.pattern", result)
+    basename = os.path.splitext(script_file)[0]
+
+    _continue = test_exit_code(f"{basename}.exit", result)
+    if not _continue:
+        print(f"{cmd} - skipped")
+        return
+    test_stdout(f"{basename}.out", result)
+    test_stdout_pattern(f"{basename}.out.pattern", result)
+    test_stderr(f"{basename}.err", result)
+    test_stderr_pattern(f"{basename}.err.pattern", result)
 
 
-def test_exit_code(f, result) -> int:
-    """test exit code"""
+def test_exit_code(f: str, result: subprocess.CompletedProcess) -> bool:
+    """Test actual exit code `result` against an expected exit code in file `f`"""
     if os.path.exists(f):
         expected = int(open(f, encoding="utf-8").read().strip())
     else:
         expected = 0
-    if result.returncode != expected:
+    actual = result.returncode
+    # Exit code 255 is the signal to skip test.
+    if actual == 255:
+        return False
+
+    if actual != expected:
         print(">>> error in return code")
-        print(f"expected: {expected}  actual:{result.returncode}")
+        print(f"expected: {expected}  actual:{actual}")
         stderr = decode_string(result.stderr).strip()
         if stderr != "":
             print(stderr)
@@ -60,7 +70,7 @@ def test_exit_code(f, result) -> int:
 
         sys.exit(1)
 
-    return expected
+    return True
 
 
 def test_stdout(f, result):
@@ -86,12 +96,12 @@ def test_stdout_pattern(f, result):
 
     # curl debug logs are too dependent on the context, so we filter
     # them and not take them into account for testing differences.
-    expected = remove_curl_debug_lines(expected)
+    expected = ignore_lines(expected)
     expected_lines = expected.split("\n")
     expected_pattern_lines = [parse_pattern(line) for line in expected_lines]
 
     actual = decode_string(result.stdout)
-    actual = remove_curl_debug_lines(actual)
+    actual = ignore_lines(actual)
     actual_lines = re.split(r"\r?\n", actual)
 
     if len(actual_lines) != len(expected_pattern_lines):
@@ -127,8 +137,8 @@ def test_stderr(f, result):
     if not os.path.exists(f):
         return
 
-    expected = open(f, encoding="utf-8").read()
-    actual = decode_string(result.stderr)
+    expected = ignore_lines(open(f, encoding="utf-8").read())
+    actual = ignore_lines(decode_string(result.stderr))
     if actual != expected:
         print(">>> error in stderr")
         print(f"actual  : <{actual}>\nexpected: <{expected}>")
@@ -145,12 +155,12 @@ def test_stderr_pattern(f, result):
 
     # curl debug logs are too dependent on the context, so we filter
     # them and not take them into account for testing differences.
-    expected = remove_curl_debug_lines(expected)
+    expected = ignore_lines(expected)
     expected_lines = expected.split("\n")
     expected_pattern_lines = [parse_pattern(line) for line in expected_lines]
 
     actual = decode_string(result.stderr)
-    actual = remove_curl_debug_lines(actual)
+    actual = ignore_lines(actual)
     actual_lines = re.split(r"\r?\n", actual)
 
     if len(actual_lines) != len(expected_pattern_lines):
@@ -190,10 +200,16 @@ def parse_pattern(s: str) -> str:
     return s
 
 
-def remove_curl_debug_lines(text: str) -> str:
+def ignore_lines(text: str) -> str:
     """Removes curl debug logs from text and returns the new text."""
-    lines = text.split("\n")
-    lines = [line for line in lines if not line.startswith("**")]
+    lines = []
+    for line in text.split("\n"):
+        if line.startswith("**"):  # curl debug info
+            continue
+        if "libcurl.so.4: no version information available" in line:
+            continue
+        lines.append(line)
+
     return "\n".join(lines)
 
 
