@@ -15,20 +15,21 @@
  * limitations under the License.
  *
  */
-use super::variables::{parse as parse_variable, parse_value};
-use super::OptionsError;
-use crate::cli::options::ErrorFormat;
-use crate::cli::OutputType;
-use atty::Stream;
+use std::collections::HashMap;
+use std::fs::File;
+use std::io::{BufRead, BufReader, IsTerminal};
+use std::path::{Path, PathBuf};
+use std::time::Duration;
+use std::{env, io};
+
 use clap::ArgMatches;
 use hurl::runner::Value;
 use hurl_core::ast::Retry;
-use std::collections::HashMap;
-use std::env;
-use std::fs::File;
-use std::io::{BufRead, BufReader};
-use std::path::{Path, PathBuf};
-use std::time::Duration;
+
+use super::variables::{parse as parse_variable, parse_value};
+use super::OptionsError;
+use crate::cli::options::{ErrorFormat, HttpVersion, IpResolve};
+use crate::cli::OutputType;
 
 pub fn cacert_file(arg_matches: &ArgMatches) -> Result<Option<String>, OptionsError> {
     match get_string(arg_matches, "cacert_file") {
@@ -47,6 +48,10 @@ pub fn cacert_file(arg_matches: &ArgMatches) -> Result<Option<String>, OptionsEr
     }
 }
 
+pub fn aws_sigv4(arg_matches: &ArgMatches) -> Option<String> {
+    get::<String>(arg_matches, "aws_sigv4")
+}
+
 pub fn client_cert_file(arg_matches: &ArgMatches) -> Result<Option<String>, OptionsError> {
     match get::<String>(arg_matches, "client_cert_file") {
         None => Ok(None),
@@ -88,7 +93,7 @@ pub fn color(arg_matches: &ArgMatches) -> bool {
             return false;
         }
     }
-    atty::is(Stream::Stdout)
+    io::stdout().is_terminal()
 }
 
 pub fn compressed(arg_matches: &ArgMatches) -> bool {
@@ -104,6 +109,14 @@ pub fn connects_to(arg_matches: &ArgMatches) -> Vec<String> {
     get_strings(arg_matches, "connect_to").unwrap_or_default()
 }
 
+pub fn continue_on_error(arg_matches: &ArgMatches) -> bool {
+    if has_flag(arg_matches, "fail_at_end") {
+        eprintln!("The option fail-at-end is deprecated. Use continue-on-error instead");
+        return true;
+    }
+    has_flag(arg_matches, "continue_on_error")
+}
+
 pub fn cookie_input_file(arg_matches: &ArgMatches) -> Option<String> {
     get::<String>(arg_matches, "cookies_input_file")
 }
@@ -112,6 +125,11 @@ pub fn cookie_output_file(arg_matches: &ArgMatches) -> Option<String> {
     get::<String>(arg_matches, "cookies_output_file")
 }
 
+pub fn delay(arg_matches: &ArgMatches) -> Duration {
+    let millis = get::<u64>(arg_matches, "delay").unwrap();
+    Duration::from_millis(millis)
+}
+
 pub fn error_format(arg_matches: &ArgMatches) -> ErrorFormat {
     let error_format = get::<String>(arg_matches, "error_format");
     match error_format.as_deref() {
@@ -121,16 +139,15 @@ pub fn error_format(arg_matches: &ArgMatches) -> ErrorFormat {
     }
 }
 
-pub fn fail_fast(arg_matches: &ArgMatches) -> bool {
-    !has_flag(arg_matches, "fail_at_end")
-}
-
 pub fn file_root(arg_matches: &ArgMatches) -> Option<String> {
     get::<String>(arg_matches, "file_root")
 }
 
-pub fn follow_location(arg_matches: &ArgMatches) -> bool {
-    has_flag(arg_matches, "follow_location")
+pub fn follow_location(arg_matches: &ArgMatches) -> (bool, bool) {
+    let follow_location = has_flag(arg_matches, "follow_location")
+        || has_flag(arg_matches, "follow_location_trusted");
+    let follow_location_trusted = has_flag(arg_matches, "follow_location_trusted");
+    (follow_location, follow_location_trusted)
 }
 
 pub fn html_dir(arg_matches: &ArgMatches) -> Result<Option<PathBuf>, OptionsError> {
@@ -139,7 +156,7 @@ pub fn html_dir(arg_matches: &ArgMatches) -> Result<Option<PathBuf>, OptionsErro
         if !path.exists() {
             match std::fs::create_dir(path) {
                 Err(_) => Err(OptionsError::Error(format!(
-                    "Html dir {} can not be created",
+                    "HTML dir {} can not be created",
                     path.display()
                 ))),
                 Ok(_) => Ok(Some(path.to_path_buf())),
@@ -157,6 +174,20 @@ pub fn html_dir(arg_matches: &ArgMatches) -> Result<Option<PathBuf>, OptionsErro
     }
 }
 
+pub fn http_version(arg_matches: &ArgMatches) -> Option<HttpVersion> {
+    if has_flag(arg_matches, "http3") {
+        Some(HttpVersion::V3)
+    } else if has_flag(arg_matches, "http2") {
+        Some(HttpVersion::V2)
+    } else if has_flag(arg_matches, "http11") {
+        Some(HttpVersion::V11)
+    } else if has_flag(arg_matches, "http10") {
+        Some(HttpVersion::V10)
+    } else {
+        None
+    }
+}
+
 pub fn ignore_asserts(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "ignore_asserts")
 }
@@ -184,7 +215,7 @@ pub fn input_files(arg_matches: &ArgMatches) -> Result<Vec<String>, OptionsError
     for filename in glob_files(arg_matches)? {
         files.push(filename);
     }
-    if files.is_empty() && !atty::is(Stream::Stdin) {
+    if files.is_empty() && !io::stdin().is_terminal() {
         files.push("-".to_string());
     }
     Ok(files)
@@ -198,8 +229,18 @@ pub fn interactive(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "interactive")
 }
 
+pub fn ip_resolve(arg_matches: &ArgMatches) -> Option<IpResolve> {
+    if has_flag(arg_matches, "ipv6") {
+        Some(IpResolve::IpV6)
+    } else if has_flag(arg_matches, "ipv4") {
+        Some(IpResolve::IpV4)
+    } else {
+        None
+    }
+}
+
 pub fn junit_file(arg_matches: &ArgMatches) -> Option<String> {
-    get::<String>(arg_matches, "junit")
+    get::<String>(arg_matches, "report_junit")
 }
 
 pub fn max_redirect(arg_matches: &ArgMatches) -> Option<usize> {
@@ -237,7 +278,7 @@ pub fn progress_bar(arg_matches: &ArgMatches) -> bool {
         && !verbose
         && !interactive(arg_matches)
         && !is_ci()
-        && atty::is(Stream::Stderr)
+        && io::stderr().is_terminal()
 }
 
 pub fn proxy(arg_matches: &ArgMatches) -> Option<String> {
@@ -250,8 +291,8 @@ pub fn resolves(arg_matches: &ArgMatches) -> Vec<String> {
 
 pub fn retry(arg_matches: &ArgMatches) -> Retry {
     match get::<i32>(arg_matches, "retry").unwrap() {
-        r if r == -1 => Retry::Infinite,
-        r if r == 0 => Retry::None,
+        -1 => Retry::Infinite,
+        0 => Retry::None,
         r => Retry::Finite(r as usize),
     }
 }
@@ -265,6 +306,10 @@ pub fn ssl_no_revoke(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "ssl_no_revoke")
 }
 
+pub fn tap_file(arg_matches: &ArgMatches) -> Option<String> {
+    get::<String>(arg_matches, "report_tap")
+}
+
 pub fn test(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "test")
 }
@@ -278,6 +323,10 @@ pub fn to_entry(arg_matches: &ArgMatches) -> Option<usize> {
     get::<u32>(arg_matches, "to_entry").map(|x| x as usize)
 }
 
+pub fn unix_socket(arg_matches: &ArgMatches) -> Option<String> {
+    get::<String>(arg_matches, "unix_socket")
+}
+
 pub fn user(arg_matches: &ArgMatches) -> Option<String> {
     get::<String>(arg_matches, "user")
 }
