@@ -129,7 +129,7 @@ pub fn html_unescape(text: &str) -> String {
                         return "\u{FFFD}".to_string();
                     }
                     if INVALID_CODEPOINTS.contains(&num) {
-                        return "".to_string();
+                        return String::new();
                     }
                     char::from_u32(num).unwrap().to_string()
                 } else {
@@ -470,7 +470,7 @@ mod tests {
             check(e, e)
         }
 
-        // Check that non-existent named entities are returned unchanged
+        // Check that nonexistent named entities are returned unchanged
         check("&svadilfari;", "&svadilfari;");
 
         // The following examples are in the html5 specs
