@@ -16,6 +16,8 @@
 *
 */
 
+use crate::http::RequestedHttpVersion;
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum HttpError {
     CouldNotParseResponse,
@@ -31,27 +33,25 @@ pub enum HttpError {
     Libcurl {
         code: i32,
         description: String,
-        url: String,
     },
-    StatuslineIsMissing {
-        url: String,
+    LibcurlUnknownOption {
+        option: String,
+        minimum_version: String,
     },
+    StatuslineIsMissing,
     TooManyRedirect,
     UnsupportedContentEncoding {
         description: String,
     },
+    UnsupportedHttpVersion(RequestedHttpVersion),
     InvalidUrl(String),
+    InvalidUrlPrefix(String),
 }
 
 impl From<curl::Error> for HttpError {
     fn from(err: curl::Error) -> Self {
         let code = err.code() as i32;
         let description = err.description().to_string();
-        let url = "".to_string();
-        HttpError::Libcurl {
-            code,
-            description,
-            url,
-        }
+        HttpError::Libcurl { code, description }
     }
 }
