@@ -17,15 +17,15 @@
  */
 pub use self::call::Call;
 pub use self::certificate::Certificate;
-pub use self::client::Client;
+pub(crate) use self::client::Client;
 pub use self::cookie::{CookieAttribute, ResponseCookie};
-pub use self::core::{Cookie, Param, RequestCookie};
-pub use self::error::HttpError;
+pub(crate) use self::core::{Cookie, Param, RequestCookie};
+pub(crate) use self::error::HttpError;
 pub use self::header::Header;
-pub use self::options::{ClientOptions, Verbosity};
-pub use self::request::Request;
-pub use self::request_spec::{Body, FileParam, Method, MultipartParam, RequestSpec};
-pub use self::response::{Response, Version};
+pub(crate) use self::options::{ClientOptions, Verbosity};
+pub use self::request::{IpResolve, Request, RequestedHttpVersion};
+pub(crate) use self::request_spec::{Body, FileParam, Method, MultipartParam, RequestSpec};
+pub use self::response::{HttpVersion, Response};
 #[cfg(test)]
 pub use self::tests::*;
 pub use self::timings::Timings;
