@@ -15,11 +15,16 @@
  * limitations under the License.
  *
  */
-use hurl_core::ast::Retry;
 use std::time::Duration;
 
+use hurl_core::ast::Retry;
+
+use crate::http::request::RequestedHttpVersion;
+use crate::http::IpResolve;
+
 #[derive(Debug, Clone)]
 pub struct ClientOptions {
+    pub aws_sigv4: Option<String>,
     pub cacert_file: Option<String>,
     pub client_cert_file: Option<String>,
     pub client_key_file: Option<String>,
@@ -28,7 +33,10 @@ pub struct ClientOptions {
     pub connects_to: Vec<String>,
     pub cookie_input_file: Option<String>,
     pub follow_location: bool,
+    pub follow_location_trusted: bool,
+    pub http_version: RequestedHttpVersion,
     pub insecure: bool,
+    pub ip_resolve: IpResolve,
     pub max_redirect: Option<usize>,
     pub no_proxy: Option<String>,
     pub path_as_is: bool,
@@ -37,11 +45,13 @@ pub struct ClientOptions {
     pub retry: Retry,
     pub ssl_no_revoke: bool,
     pub timeout: Duration,
+    pub unix_socket: Option<String>,
     pub user: Option<String>,
     pub user_agent: Option<String>,
     pub verbosity: Option<Verbosity>,
 }
 
+// FIXME/ we could implement copy here
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum Verbosity {
     Verbose,
@@ -51,6 +61,7 @@ pub enum Verbosity {
 impl Default for ClientOptions {
     fn default() -> Self {
         ClientOptions {
+            aws_sigv4: None,
             cacert_file: None,
             client_cert_file: None,
             client_key_file: None,
@@ -59,7 +70,10 @@ impl Default for ClientOptions {
             connects_to: vec![],
             cookie_input_file: None,
             follow_location: false,
+            follow_location_trusted: false,
+            http_version: RequestedHttpVersion::default(),
             insecure: false,
+            ip_resolve: IpResolve::default(),
             max_redirect: Some(50),
             no_proxy: None,
             path_as_is: false,
@@ -68,6 +82,7 @@ impl Default for ClientOptions {
             retry: Retry::None,
             ssl_no_revoke: false,
             timeout: Duration::from_secs(300),
+            unix_socket: None,
             user: None,
             user_agent: None,
             verbosity: None,
@@ -80,6 +95,10 @@ impl ClientOptions {
     pub fn curl_args(&self) -> Vec<String> {
         let mut arguments = vec![];
 
+        if let Some(ref aws_sigv4) = self.aws_sigv4 {
+            arguments.push("--aws-sigv4".to_string());
+            arguments.push(aws_sigv4.clone());
+        }
         if let Some(ref cacert_file) = self.cacert_file {
             arguments.push("--cacert".to_string());
             arguments.push(cacert_file.clone());
@@ -107,10 +126,24 @@ impl ClientOptions {
             arguments.push("--cookie".to_string());
             arguments.push(cookie_file.clone());
         }
+        match self.http_version {
+            RequestedHttpVersion::Default => {}
+            RequestedHttpVersion::Http10 => arguments.push("--http1.0".to_string()),
+            RequestedHttpVersion::Http11 => arguments.push("--http1.1".to_string()),
+            RequestedHttpVersion::Http2 => arguments.push("--http2".to_string()),
+            RequestedHttpVersion::Http3 => arguments.push("--http3".to_string()),
+        }
         if self.insecure {
             arguments.push("--insecure".to_string());
         }
-        if self.follow_location {
+        match self.ip_resolve {
+            IpResolve::Default => {}
+            IpResolve::IpV4 => arguments.push("--ipv4".to_string()),
+            IpResolve::IpV6 => arguments.push("--ipv6".to_string()),
+        }
+        if self.follow_location_trusted {
+            arguments.push("--location-trusted".to_string());
+        } else if self.follow_location {
             arguments.push("--location".to_string());
         }
         if self.max_redirect != ClientOptions::default().max_redirect {
@@ -136,6 +169,10 @@ impl ClientOptions {
             arguments.push("--timeout".to_string());
             arguments.push(self.timeout.as_secs().to_string());
         }
+        if let Some(ref unix_socket) = self.unix_socket {
+            arguments.push("--unix-socket".to_string());
+            arguments.push(format!("'{unix_socket}'"))
+        }
         if let Some(ref user) = self.user {
             arguments.push("--user".to_string());
             arguments.push(format!("'{user}'"));
@@ -158,18 +195,23 @@ mod tests {
 
         assert_eq!(
             ClientOptions {
+                aws_sigv4: None,
                 cacert_file: None,
                 client_cert_file: None,
                 client_key_file: None,
+                compressed: true,
+                connect_timeout: Duration::from_secs(20),
                 connects_to: vec!["example.com:443:host-47.example.com:443".to_string()],
-                follow_location: true,
-                max_redirect: Some(10),
                 cookie_input_file: Some("cookie_file".to_string()),
+                follow_location: true,
+                follow_location_trusted: false,
+                http_version: RequestedHttpVersion::Http10,
+                insecure: true,
+                ip_resolve: IpResolve::IpV6,
+                max_redirect: Some(10),
                 path_as_is: true,
                 proxy: Some("localhost:3128".to_string()),
                 no_proxy: None,
-                verbosity: None,
-                insecure: true,
                 resolves: vec![
                     "foo.com:80:192.168.0.1".to_string(),
                     "bar.com:443:127.0.0.1".to_string()
@@ -177,38 +219,43 @@ mod tests {
                 retry: Retry::None,
                 ssl_no_revoke: false,
                 timeout: Duration::from_secs(10),
-                connect_timeout: Duration::from_secs(20),
+                unix_socket: Some("/var/run/example.sock".to_string()),
                 user: Some("user:password".to_string()),
                 user_agent: Some("my-useragent".to_string()),
-                compressed: true,
+                verbosity: None,
             }
             .curl_args(),
             [
-                "--compressed".to_string(),
-                "--connect-timeout".to_string(),
-                "20".to_string(),
-                "--connect-to".to_string(),
-                "example.com:443:host-47.example.com:443".to_string(),
-                "--cookie".to_string(),
-                "cookie_file".to_string(),
-                "--insecure".to_string(),
-                "--location".to_string(),
-                "--max-redirs".to_string(),
-                "10".to_string(),
-                "--path-as-is".to_string(),
-                "--proxy".to_string(),
-                "'localhost:3128'".to_string(),
-                "--resolve".to_string(),
-                "foo.com:80:192.168.0.1".to_string(),
-                "--resolve".to_string(),
-                "bar.com:443:127.0.0.1".to_string(),
-                "--timeout".to_string(),
-                "10".to_string(),
-                "--user".to_string(),
-                "'user:password'".to_string(),
-                "--user-agent".to_string(),
-                "'my-useragent'".to_string(),
+                "--compressed",
+                "--connect-timeout",
+                "20",
+                "--connect-to",
+                "example.com:443:host-47.example.com:443",
+                "--cookie",
+                "cookie_file",
+                "--http1.0",
+                "--insecure",
+                "--ipv6",
+                "--location",
+                "--max-redirs",
+                "10",
+                "--path-as-is",
+                "--proxy",
+                "'localhost:3128'",
+                "--resolve",
+                "foo.com:80:192.168.0.1",
+                "--resolve",
+                "bar.com:443:127.0.0.1",
+                "--timeout",
+                "10",
+                "--unix-socket",
+                "'/var/run/example.sock'",
+                "--user",
+                "'user:password'",
+                "--user-agent",
+                "'my-useragent'",
             ]
+            .map(|a| a.to_string())
         );
     }
 }
