@@ -15,6 +15,8 @@
  * limitations under the License.
  *
  */
+use std::fmt;
+
 use url::Url;
 
 use crate::http::core::*;
@@ -28,6 +30,37 @@ pub struct Request {
     pub body: Vec<u8>,
 }
 
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Default)]
+pub enum RequestedHttpVersion {
+    #[default]
+    Default, // The effective HTTP version will be chosen by libcurl
+    Http10,
+    Http11,
+    Http2,
+    Http3,
+}
+
+impl fmt::Display for RequestedHttpVersion {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        let value = match self {
+            RequestedHttpVersion::Default => "HTTP (default)",
+            RequestedHttpVersion::Http10 => "HTTP/1.0",
+            RequestedHttpVersion::Http11 => "HTTP/1.1",
+            RequestedHttpVersion::Http2 => "HTTP/2",
+            RequestedHttpVersion::Http3 => "HTTP/3",
+        };
+        write!(f, "{value}")
+    }
+}
+
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Default)]
+pub enum IpResolve {
+    #[default]
+    Default, // Default, can use addresses of all IP versions that your system allows.
+    IpV4,
+    IpV6,
+}
+
 impl Request {
     /// Extracts query string params from the url of the request.
     pub fn query_string_params(&self) -> Vec<Param> {
@@ -57,7 +90,7 @@ impl Request {
     /// Returns optional Content-type header value.
     pub fn content_type(&self) -> Option<String> {
         header::get_values(&self.headers, "Content-Type")
-            .get(0)
+            .first()
             .cloned()
     }
 
@@ -68,17 +101,19 @@ impl Request {
             Ok(url) => url,
             Err(_) => return Err(HttpError::InvalidUrl(self.url.clone())),
         };
-        let base_url = format!(
-            "{}://{}{}",
-            url.scheme(),
-            url.host().unwrap(),
-            if let Some(port) = url.port() {
-                format!(":{port}")
-            } else {
-                "".to_string()
-            }
-        );
-        Ok(base_url)
+        let scheme = url.scheme();
+        if scheme != "http" && scheme != "https" {
+            return Err(HttpError::InvalidUrlPrefix(self.url.clone()));
+        }
+        let host = match url.host() {
+            Some(host) => host,
+            None => return Err(HttpError::InvalidUrl(self.url.clone())),
+        };
+        let port = match url.port() {
+            Some(port) => format!(":{port}"),
+            None => String::new(),
+        };
+        Ok(format!("{scheme}://{host}{port}"))
     }
 }
 
@@ -94,7 +129,7 @@ fn parse_cookie(s: &str) -> RequestCookie {
         },
         None => RequestCookie {
             name: s.to_string(),
-            value: "".to_string(),
+            value: String::new(),
         },
     }
 }
@@ -147,7 +182,7 @@ mod tests {
                 },
                 Param {
                     name: "param2".to_string(),
-                    value: "".to_string(),
+                    value: String::new(),
                 },
                 Param {
                     name: "param3".to_string(),
@@ -212,7 +247,7 @@ mod tests {
         assert_eq!(
             Request {
                 url: "http://localhost".to_string(),
-                method: "".to_string(),
+                method: String::new(),
                 headers: vec![],
                 body: vec![],
             }
@@ -223,7 +258,7 @@ mod tests {
         assert_eq!(
             Request {
                 url: "http://localhost:8000/redirect-relative".to_string(),
-                method: "".to_string(),
+                method: String::new(),
                 headers: vec![],
                 body: vec![],
             }
@@ -234,7 +269,7 @@ mod tests {
         assert_eq!(
             Request {
                 url: "https://localhost:8000".to_string(),
-                method: "".to_string(),
+                method: String::new(),
                 headers: vec![],
                 body: vec![],
             }
