@@ -20,7 +20,7 @@ use crate::util::logger::Logger;
 
 impl Request {
     /// Log request body.
-    pub fn log_body(&self, logger: &Logger) {
+    pub fn log_body(&self, debug: bool, logger: &Logger) {
         logger.debug_important("Request body:");
 
         // We try to decode the HTTP body as text if the response has a text kind content type.
@@ -28,13 +28,13 @@ impl Request {
         // print the body first 64 bytes.
         if let Some(content_type) = self.content_type() {
             if !mimetype::is_kind_of_text(&content_type) {
-                debug::log_bytes(&self.body, 64, logger);
+                debug::log_bytes(&self.body, 64, debug, logger);
                 return;
             }
         }
         match self.text() {
-            Ok(text) => debug::log_text(&text, logger),
-            Err(_) => debug::log_bytes(&self.body, 64, logger),
+            Ok(text) => debug::log_text(&text, debug, logger),
+            Err(_) => debug::log_bytes(&self.body, 64, debug, logger),
         }
     }
 }
