@@ -15,7 +15,7 @@
  * limitations under the License.
  *
  */
-use core::fmt;
+use std::fmt;
 use std::time::Duration;
 
 use crate::http::certificate::Certificate;
@@ -24,7 +24,7 @@ use crate::http::{header, Header};
 /// Represents an HTTP response.
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Response {
-    pub version: Version,
+    pub version: HttpVersion,
     pub status: u32,
     pub headers: Vec<Header>,
     pub body: Vec<u8>,
@@ -37,35 +37,17 @@ pub struct Response {
 impl Default for Response {
     fn default() -> Self {
         Response {
-            version: Version::Http10,
+            version: HttpVersion::Http10,
             status: 200,
             headers: vec![],
             body: vec![],
             duration: Default::default(),
-            url: "".to_string(),
+            url: String::new(),
             certificate: None,
         }
     }
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub enum Version {
-    Http10,
-    Http11,
-    Http2,
-}
-
-impl fmt::Display for Version {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let value = match self {
-            Version::Http10 => "HTTP/1.0",
-            Version::Http11 => "HTTP/1.1",
-            Version::Http2 => "HTTP/2",
-        };
-        write!(f, "{value}")
-    }
-}
-
 impl Response {
     /// Returns all header values.
     pub fn get_header_values(&self, name: &str) -> Vec<String> {
@@ -75,11 +57,33 @@ impl Response {
     /// Returns optional Content-type header value.
     pub fn content_type(&self) -> Option<String> {
         header::get_values(&self.headers, "Content-Type")
-            .get(0)
+            .first()
             .cloned()
     }
 }
 
+/// Represents the HTTP version of a HTTP transaction.
+/// See <https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/Evolution_of_HTTP>
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+pub enum HttpVersion {
+    Http10,
+    Http11,
+    Http2,
+    Http3,
+}
+
+impl fmt::Display for HttpVersion {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        let value = match self {
+            HttpVersion::Http10 => "HTTP/1.0",
+            HttpVersion::Http11 => "HTTP/1.1",
+            HttpVersion::Http2 => "HTTP/2",
+            HttpVersion::Http3 => "HTTP/3",
+        };
+        write!(f, "{value}")
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
