@@ -264,7 +264,7 @@ pub mod tests {
 
     #[test]
     fn test_uncompress_brotli() {
-        let data = vec![
+        let data = [
             0x21, 0x2c, 0x00, 0x04, 0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x57, 0x6f, 0x72, 0x6c,
             0x64, 0x21, 0x03,
         ];
@@ -273,7 +273,7 @@ pub mod tests {
 
     #[test]
     fn test_uncompress_gzip() {
-        let data = vec![
+        let data = [
             0x1f, 0x8b, 0x08, 0x08, 0xa7, 0x52, 0x85, 0x5f, 0x00, 0x03, 0x64, 0x61, 0x74, 0x61,
             0x2e, 0x74, 0x78, 0x74, 0x00, 0xf3, 0x48, 0xcd, 0xc9, 0xc9, 0x57, 0x08, 0xcf, 0x2f,
             0xca, 0x49, 0x51, 0x04, 0x00, 0xa3, 0x1c, 0x29, 0x1c, 0x0c, 0x00, 0x00, 0x00,
@@ -283,7 +283,7 @@ pub mod tests {
 
     #[test]
     fn test_uncompress_zlib() {
-        let data = vec![
+        let data = [
             0x78, 0x9c, 0xf3, 0x48, 0xcd, 0xc9, 0xc9, 0x57, 0x08, 0xcf, 0x2f, 0xca, 0x49, 0x51,
             0x04, 0x00, 0x1c, 0x49, 0x04, 0x3e,
         ];
@@ -292,7 +292,7 @@ pub mod tests {
 
     #[test]
     fn test_uncompress_error() {
-        let data = vec![0x21];
+        let data = [0x21];
         assert_eq!(
             uncompress_brotli(&data[..]).err().unwrap(),
             HttpError::CouldNotUncompressResponse {
