@@ -18,16 +18,16 @@
 
 use base64::engine::general_purpose;
 use base64::Engine;
+use std::str::FromStr;
 
-use crate::runner::Value;
+use crate::runner::{Number, Value};
 
 impl Value {
     pub fn to_json(&self) -> serde_json::Value {
         match self {
             Value::Bool(v) => serde_json::Value::Bool(*v),
             Value::Date(v) => serde_json::Value::String(v.to_string()),
-            Value::Integer(v) => serde_json::Value::Number(serde_json::Number::from(*v)),
-            Value::Float(f) => serde_json::Value::Number(serde_json::Number::from_f64(*f).unwrap()),
+            Value::Number(v) => v.to_json(),
             Value::String(s) => serde_json::Value::String(s.clone()),
             Value::List(values) => {
                 let values = values.iter().map(|v| v.to_json()).collect();
@@ -60,3 +60,18 @@ impl Value {
         }
     }
 }
+
+impl Number {
+    pub fn to_json(&self) -> serde_json::Value {
+        match self {
+            Number::Integer(v) => serde_json::Value::Number(serde_json::Number::from(*v)),
+            Number::Float(f) => {
+                serde_json::Value::Number(serde_json::Number::from_f64(*f).unwrap())
+            }
+            Number::BigInteger(s) => {
+                let number = serde_json::Number::from_str(s).unwrap();
+                serde_json::Value::Number(number)
+            }
+        }
+    }
+}
