@@ -19,12 +19,12 @@ use super::error::*;
 use super::reader::Reader;
 use super::{ParseFunc, ParseResult};
 
-pub fn zero_or_more<'a, T>(f: ParseFunc<'a, T>, p: &mut Reader) -> ParseResult<'a, Vec<T>> {
-    let _start = p.state.clone();
+pub fn zero_or_more<T>(f: ParseFunc<T>, p: &mut Reader) -> ParseResult<Vec<T>> {
+    let _start = p.state;
 
     let mut v: Vec<T> = Vec::new();
     loop {
-        let initial_state = p.state.clone();
+        let initial_state = p.state;
         if p.is_eof() {
             return Ok(v);
         }
@@ -48,9 +48,9 @@ pub fn zero_or_more<'a, T>(f: ParseFunc<'a, T>, p: &mut Reader) -> ParseResult<'
 
 /// Tries to apply the list of parser functions `fs` until one of them succeeds.
 /// Typically this should be recoverable
-pub fn choice<'a, T>(fs: &[ParseFunc<'a, T>], reader: &mut Reader) -> ParseResult<'a, T> {
+pub fn choice<T>(fs: &[ParseFunc<T>], reader: &mut Reader) -> ParseResult<T> {
     for (pos, f) in fs.iter().enumerate() {
-        let start = reader.state.clone();
+        let start = reader.state;
         if pos == fs.len() - 1 {
             return f(reader);
         }
