@@ -19,14 +19,14 @@
 use error::Error;
 use reader::Reader;
 
-#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 pub struct Pos {
     pub line: usize,
     pub column: usize,
 }
 
-pub type ParseResult<'a, T> = std::result::Result<T, Error>;
-pub type ParseFunc<'a, T> = fn(&mut Reader) -> ParseResult<'a, T>;
+pub type ParseResult<T> = Result<T, Error>;
+pub type ParseFunc<T> = fn(&mut Reader) -> ParseResult<T>;
 
 pub use self::parse::parse;
 
