@@ -33,7 +33,7 @@ fn query(reader: &mut Reader) -> ParseResult<Query> {
     let selectors = zero_or_more(selector, reader)?;
     if !reader.is_eof() {
         return Err(Error {
-            pos: reader.state.pos.clone(),
+            pos: reader.state.pos,
             recoverable: false,
             inner: ParseError::Expecting {
                 value: "eof".to_string(),
@@ -61,6 +61,7 @@ fn selector(reader: &mut Reader) -> ParseResult<Selector> {
 }
 
 fn selector_array_index_or_array_indices(reader: &mut Reader) -> Result<Selector, Error> {
+    let initial_state = reader.state;
     try_left_bracket(reader)?;
     let mut indexes = vec![];
     let i = match natural(reader) {
@@ -75,7 +76,7 @@ fn selector_array_index_or_array_indices(reader: &mut Reader) -> Result<Selector
     };
     indexes.push(i);
     loop {
-        let state = reader.state.clone();
+        let state = reader.state;
         if try_literal(",", reader).is_ok() {
             let i = match natural(reader) {
                 Err(e) => {
@@ -93,7 +94,16 @@ fn selector_array_index_or_array_indices(reader: &mut Reader) -> Result<Selector
             break;
         }
     }
-    literal("]", reader)?;
+    // you will have a ':' for a slice
+    // TODO: combine array index, indices and slice in the same function
+    if let Err(e) = try_literal("]", reader) {
+        reader.state = initial_state;
+        return Err(Error {
+            pos: reader.state.pos,
+            recoverable: true,
+            inner: e.inner,
+        });
+    }
     let selector = if indexes.len() == 1 {
         Selector::ArrayIndex(*indexes.first().unwrap())
     } else {
@@ -106,15 +116,15 @@ fn selector_array_wildcard(reader: &mut Reader) -> Result<Selector, Error> {
     try_left_bracket(reader)?;
     try_literal("*", reader)?;
     literal("]", reader)?;
-    Ok(Selector::ArrayWildcard {})
+    Ok(Selector::ArrayWildcard)
 }
 
 fn selector_array_slice(reader: &mut Reader) -> Result<Selector, Error> {
     try_left_bracket(reader)?;
-    let state = reader.state.clone();
+    let state = reader.state;
     let start = match integer(reader) {
         Err(_) => {
-            reader.state = state.clone();
+            reader.state = state;
             None
         }
         Ok(v) => Some(v),
@@ -128,7 +138,7 @@ fn selector_array_slice(reader: &mut Reader) -> Result<Selector, Error> {
             },
         });
     };
-    let state = reader.state.clone();
+    let state = reader.state;
     let end = match integer(reader) {
         Err(_) => {
             reader.state = state;
@@ -152,7 +162,7 @@ fn selector_object_key_bracket(reader: &mut Reader) -> Result<Selector, Error> {
     try_left_bracket(reader)?;
     match string_value(reader) {
         Err(_) => Err(Error {
-            pos: reader.state.pos.clone(),
+            pos: reader.state.pos,
             recoverable: true,
             inner: ParseError::Expecting {
                 value: "value string".to_string(),
@@ -168,7 +178,7 @@ fn selector_object_key_bracket(reader: &mut Reader) -> Result<Selector, Error> {
 fn selector_object_key(reader: &mut Reader) -> Result<Selector, Error> {
     if !reader.try_literal(".") {
         return Err(Error {
-            pos: reader.state.pos.clone(),
+            pos: reader.state.pos,
             recoverable: true,
             inner: ParseError::Expecting {
                 value: "[ or .".to_string(),
@@ -179,7 +189,7 @@ fn selector_object_key(reader: &mut Reader) -> Result<Selector, Error> {
     let s = reader.read_while(|c| c.is_alphanumeric() || *c == '_' || *c == '-');
     if s.is_empty() {
         return Err(Error {
-            pos: reader.state.pos.clone(),
+            pos: reader.state.pos,
             recoverable: false,
             inner: ParseError::Expecting {
                 value: "empty value".to_string(),
@@ -191,12 +201,12 @@ fn selector_object_key(reader: &mut Reader) -> Result<Selector, Error> {
 
 fn selector_wildcard(reader: &mut Reader) -> Result<Selector, Error> {
     try_literal(".*", reader)?;
-    Ok(Selector::Wildcard {})
+    Ok(Selector::Wildcard)
 }
 
 fn selector_recursive_wildcard(reader: &mut Reader) -> Result<Selector, Error> {
     try_literal("..*", reader)?;
-    Ok(Selector::RecursiveWildcard {})
+    Ok(Selector::RecursiveWildcard)
 }
 
 fn selector_recursive_key(reader: &mut Reader) -> Result<Selector, Error> {
@@ -206,7 +216,7 @@ fn selector_recursive_key(reader: &mut Reader) -> Result<Selector, Error> {
 }
 
 fn try_left_bracket(reader: &mut Reader) -> Result<(), Error> {
-    let start = reader.state.clone();
+    let start = reader.state;
     if literal(".[", reader).is_err() {
         reader.state = start;
         try_literal("[", reader)?;
@@ -226,12 +236,12 @@ fn predicate(reader: &mut Reader) -> ParseResult<Predicate> {
     // @.key>=value   GreaterThanOrEqual(Key, Value)
     literal("@.", reader)?; // assume key value for the time being
     let key = key_path(reader)?;
-    let state = reader.state.clone();
+    let state = reader.state;
     let func = match predicate_func(reader) {
         Ok(f) => f,
         Err(_) => {
             reader.state = state;
-            PredicateFunc::KeyExist {}
+            PredicateFunc::KeyExist
         }
     };
     Ok(Predicate { key, func })
@@ -386,7 +396,7 @@ mod tests {
             selector(&mut reader).unwrap(),
             Selector::Filter(Predicate {
                 key: vec!["isbn".to_string()],
-                func: PredicateFunc::KeyExist {},
+                func: PredicateFunc::KeyExist,
             })
         );
         assert_eq!(reader.state.cursor, 11);
@@ -458,18 +468,28 @@ mod tests {
     #[test]
     pub fn test_selector_wildcard() {
         let mut reader = Reader::new("[*]");
-        assert_eq!(selector(&mut reader).unwrap(), Selector::ArrayWildcard {});
+        assert_eq!(selector(&mut reader).unwrap(), Selector::ArrayWildcard);
         assert_eq!(reader.state.cursor, 3);
 
         // you don't need to keep the exact string
         // this is not part of the AST
         let mut reader = Reader::new(".[*]");
-        assert_eq!(selector(&mut reader).unwrap(), Selector::ArrayWildcard {});
+        assert_eq!(selector(&mut reader).unwrap(), Selector::ArrayWildcard);
         assert_eq!(reader.state.cursor, 4);
     }
 
     #[test]
     pub fn test_selector_array_slice() {
+        let mut reader = Reader::new("[1:]");
+        assert_eq!(
+            selector(&mut reader).unwrap(),
+            Selector::ArraySlice(Slice {
+                start: Some(1),
+                end: None
+            })
+        );
+        assert_eq!(reader.state.cursor, 4);
+
         let mut reader = Reader::new("[-1:]");
         assert_eq!(
             selector(&mut reader).unwrap(),
@@ -539,7 +559,7 @@ mod tests {
             predicate(&mut Reader::new("@.isbn")).unwrap(),
             Predicate {
                 key: vec!["isbn".to_string()],
-                func: PredicateFunc::KeyExist {},
+                func: PredicateFunc::KeyExist,
             }
         );
 
