@@ -19,8 +19,8 @@ use super::super::ast::*;
 use super::error::{Error, ParseError};
 use super::{ParseResult, Reader};
 
-pub fn natural(reader: &mut Reader) -> ParseResult<'static, usize> {
-    let start = reader.state.clone();
+pub fn natural(reader: &mut Reader) -> ParseResult<usize> {
+    let start = reader.state;
 
     if reader.is_eof() {
         return Err(Error {
@@ -42,7 +42,7 @@ pub fn natural(reader: &mut Reader) -> ParseResult<'static, usize> {
         });
     }
 
-    let save = reader.state.clone();
+    let save = reader.state;
     let s = reader.read_while(|c| c.is_ascii_digit());
 
     // if the first digit is zero, you should not have any more digits
@@ -58,19 +58,19 @@ pub fn natural(reader: &mut Reader) -> ParseResult<'static, usize> {
     Ok(format!("{first_digit}{s}").parse().unwrap())
 }
 
-pub fn integer(reader: &mut Reader) -> ParseResult<'static, i64> {
+pub fn integer(reader: &mut Reader) -> ParseResult<i64> {
     let sign = if reader.try_literal("-") { -1 } else { 1 };
     let nat = natural(reader)?;
     Ok(sign * (nat as i64))
 }
 
-pub fn number(reader: &mut Reader) -> ParseResult<'static, Number> {
+pub fn number(reader: &mut Reader) -> ParseResult<Number> {
     let int = integer(reader)?;
 
     let decimal = if reader.try_literal(".") {
         if reader.is_eof() {
             return Err(Error {
-                pos: reader.clone().state.pos,
+                pos: reader.state.pos,
                 recoverable: false,
                 inner: ParseError::Expecting {
                     value: String::from("natural"),
@@ -81,7 +81,7 @@ pub fn number(reader: &mut Reader) -> ParseResult<'static, Number> {
         let s = reader.read_while(|c| c.is_ascii_digit());
         if s.is_empty() {
             return Err(Error {
-                pos: reader.clone().state.pos,
+                pos: reader.state.pos,
                 recoverable: false,
                 inner: ParseError::Expecting {
                     value: String::from("natural"),
@@ -98,12 +98,12 @@ pub fn number(reader: &mut Reader) -> ParseResult<'static, Number> {
 
 pub fn string_value(reader: &mut Reader) -> Result<String, Error> {
     try_literal("'", reader)?;
-    let mut s = "".to_string();
+    let mut s = String::new();
     loop {
         match reader.read() {
             None => {
                 return Err(Error {
-                    pos: reader.state.pos.clone(),
+                    pos: reader.state.pos,
                     recoverable: false,
                     inner: ParseError::Expecting {
                         value: String::from("'"),
@@ -119,7 +119,7 @@ pub fn string_value(reader: &mut Reader) -> Result<String, Error> {
                     }
                     _ => {
                         return Err(Error {
-                            pos: reader.state.pos.clone(),
+                            pos: reader.state.pos,
                             recoverable: false,
                             inner: ParseError::Expecting {
                                 value: String::from("'"),
@@ -148,7 +148,7 @@ pub fn key_name(reader: &mut Reader) -> Result<String, Error> {
                 c
             } else {
                 return Err(Error {
-                    pos: reader.state.pos.clone(),
+                    pos: reader.state.pos,
                     recoverable: false,
                     inner: ParseError::Expecting {
                         value: "key".to_string(),
@@ -158,7 +158,7 @@ pub fn key_name(reader: &mut Reader) -> Result<String, Error> {
         }
         None => {
             return Err(Error {
-                pos: reader.state.pos.clone(),
+                pos: reader.state.pos,
                 recoverable: false,
                 inner: ParseError::Expecting {
                     value: "key".to_string(),
@@ -183,12 +183,12 @@ pub fn key_path(reader: &mut Reader) -> Result<Vec<String>, Error> {
     Ok(path)
 }
 
-pub fn literal(s: &str, reader: &mut Reader) -> ParseResult<'static, ()> {
+pub fn literal(s: &str, reader: &mut Reader) -> ParseResult<()> {
     // does not return a value
     // non recoverable reader
     // => use combinator recover to make it recoverable
-    let start = reader.state.clone();
-    if reader.clone().is_eof() {
+    let start = reader.state;
+    if reader.is_eof() {
         return Err(Error {
             pos: start.pos,
             recoverable: false,
@@ -198,7 +198,7 @@ pub fn literal(s: &str, reader: &mut Reader) -> ParseResult<'static, ()> {
         });
     }
     for c in s.chars() {
-        let _state = reader.state.clone();
+        let _state = reader.state;
         match reader.read() {
             None => {
                 return Err(Error {
@@ -228,7 +228,7 @@ pub fn literal(s: &str, reader: &mut Reader) -> ParseResult<'static, ()> {
     Ok(())
 }
 
-pub fn try_literal(s: &str, p: &mut Reader) -> ParseResult<'static, ()> {
+pub fn try_literal(s: &str, p: &mut Reader) -> ParseResult<()> {
     match literal(s, p) {
         Ok(_) => Ok(()),
         Err(Error { pos, inner, .. }) => Err(Error {
