@@ -26,7 +26,7 @@ pub struct Reader {
     pub state: ReaderState,
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub struct ReaderState {
     pub cursor: usize,
     pub pos: Pos,
@@ -70,7 +70,7 @@ impl Reader {
     /// Returns `count` chars from the buffer advancing the internal state.
     /// This methods can returns less than `count` chars if there is not enough chars in the buffer.
     pub fn read_n(&mut self, count: usize) -> String {
-        let mut s = String::from("");
+        let mut s = String::new();
         for _ in 0..count {
             match self.read() {
                 None => {}
@@ -82,7 +82,7 @@ impl Reader {
 
     /// Returns chars from the buffer while `predicate` is true, advancing the internal state.
     pub fn read_while(&mut self, predicate: fn(&char) -> bool) -> String {
-        let mut s = String::from("");
+        let mut s = String::new();
         loop {
             match self.peek() {
                 None => return s,
