@@ -15,7 +15,6 @@
  * limitations under the License.
  *
  */
-#![cfg_attr(feature = "strict", deny(warnings))]
 
 //! This crate provides a function to run a Hurl formatted content.
 //! Hurl uses a plain text format to run and tests HTTP requests. The fully documented
@@ -41,11 +40,10 @@
 //!
 //! This crate works on Windows, macOS and Linux.
 mod html;
-mod http;
+pub mod http;
 mod json;
 mod jsonpath;
 pub mod output;
 pub mod report;
 pub mod runner;
 pub mod util;
-pub use http::libcurl_version_info;
