@@ -22,13 +22,14 @@ use std::path::Path;
 use std::time::Instant;
 use std::{env, process};
 
-use crate::cli::options::OptionsError;
 use colored::control;
-use hurl::report::{html, junit};
+use hurl::report::{html, junit, tap};
 use hurl::runner::HurlResult;
 use hurl::util::logger::{BaseLogger, Logger, LoggerOptionsBuilder, Verbosity};
 use hurl::{output, runner};
 
+use crate::cli::options::OptionsError;
+
 const EXIT_OK: i32 = 0;
 const EXIT_ERROR_COMMANDLINE: i32 = 1;
 const EXIT_ERROR_PARSING: i32 = 2;
@@ -57,6 +58,10 @@ fn main() {
                 print!("{message}");
                 process::exit(EXIT_OK);
             }
+            OptionsError::NoInput(message) => {
+                eprintln!("{message}");
+                process::exit(EXIT_ERROR_COMMANDLINE);
+            }
             OptionsError::Error(message) => {
                 eprintln!("error: {message}");
                 process::exit(EXIT_ERROR_COMMANDLINE);
@@ -68,11 +73,9 @@ fn main() {
     // We'll use a more advanced logger for rich error report when running Hurl files.
     let verbose = opts.verbose || opts.very_verbose || opts.interactive;
     let base_logger = BaseLogger::new(opts.color, verbose);
-
     let current_dir = env::current_dir();
     let current_dir = unwrap_or_exit(current_dir, EXIT_ERROR_UNDEFINED, &base_logger);
     let current_dir = current_dir.as_path();
-
     let start = Instant::now();
     let mut runs = vec![];
 
@@ -89,14 +92,13 @@ fn main() {
         let verbosity = Verbosity::from(opts.verbose, opts.very_verbose);
         let logger_options = LoggerOptionsBuilder::new()
             .color(opts.color)
-            .error_format(opts.error_format.clone().into())
+            .error_format(opts.error_format.into())
             .filename(filename)
             .progress_bar(opts.progress_bar)
             .test(opts.test)
             .verbosity(verbosity)
             .build();
         let logger = Logger::from(&logger_options);
-
         let total = opts.input_files.len();
         logger.test_running(current + 1, total);
 
@@ -106,6 +108,7 @@ fn main() {
             Ok(h) => h,
             Err(_) => process::exit(EXIT_ERROR_PARSING),
         };
+
         logger.test_completed(&hurl_result);
         let success = hurl_result.success;
 
@@ -125,7 +128,6 @@ fn main() {
             );
             unwrap_or_exit(result, EXIT_ERROR_RUNTIME, &base_logger);
         }
-
         if matches!(opts.output_type, cli::OutputType::Json) {
             let result = output::write_json(&hurl_result, &content, filename, &opts.output);
             unwrap_or_exit(result, EXIT_ERROR_RUNTIME, &base_logger);
@@ -145,6 +147,12 @@ fn main() {
         unwrap_or_exit(result, EXIT_ERROR_UNDEFINED, &base_logger);
     }
 
+    if let Some(filename) = opts.tap_file {
+        base_logger.debug(format!("Writing TAP report to {filename}").as_str());
+        let result = create_tap_report(&runs, &filename);
+        unwrap_or_exit(result, EXIT_ERROR_UNDEFINED, &base_logger);
+    }
+
     if let Some(dir) = opts.html_dir {
         base_logger.debug(format!("Writing HTML report to {}", dir.display()).as_str());
         let result = create_html_report(&runs, &dir);
@@ -219,6 +227,16 @@ fn create_junit_report(runs: &[HurlRun], filename: &str) -> Result<(), cli::CliE
     Ok(())
 }
 
+/// Create a TAP report for this run.
+fn create_tap_report(runs: &[HurlRun], filename: &str) -> Result<(), cli::CliError> {
+    let testcases: Vec<tap::Testcase> = runs
+        .iter()
+        .map(|r| tap::Testcase::from(&r.hurl_result, &r.filename))
+        .collect();
+    tap::write_report(filename, &testcases)?;
+    Ok(())
+}
+
 /// Create an HTML report for this run.
 fn create_html_report(runs: &[HurlRun], dir_path: &Path) -> Result<(), cli::CliError> {
     // We ensure that the containing folder exists.
@@ -310,14 +328,17 @@ fn get_summary(runs: &[HurlRun], duration: u128) -> String {
 
 #[cfg(test)]
 pub mod tests {
-    use super::*;
     use hurl::runner::EntryResult;
+    use hurl_core::ast::{Pos, SourceInfo};
+
+    use super::*;
 
     #[test]
     fn create_run_summary() {
         fn new_run(success: bool, entries_count: usize) -> HurlRun {
             let dummy_entry = EntryResult {
                 entry_index: 0,
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                 calls: vec![],
                 captures: vec![],
                 asserts: vec![],
@@ -326,13 +347,14 @@ pub mod tests {
                 compressed: false,
             };
             HurlRun {
-                content: "".to_string(),
-                filename: "".to_string(),
+                content: String::new(),
+                filename: String::new(),
                 hurl_result: HurlResult {
                     entries: vec![dummy_entry; entries_count],
                     time_in_ms: 0,
                     success,
                     cookies: vec![],
+                    timestamp: 1,
                 },
             }
         }
