@@ -15,10 +15,11 @@
  * limitations under the License.
  *
  */
-use chrono::{DateTime, Utc};
 use std::iter::zip;
 use std::time::Duration;
 
+use chrono::{DateTime, Utc};
+
 use crate::http::Call;
 use crate::report::html::timeline::nice::NiceScale;
 use crate::report::html::timeline::svg::Attribute::{
@@ -56,7 +57,7 @@ impl Testcase {
         let times = get_times_interval(calls);
         let times = match times {
             Some(t) => t,
-            None => return "".to_string(),
+            None => return String::new(),
         };
 
         let margin_top = 50.px();
@@ -70,7 +71,7 @@ impl Testcase {
         root.add_attr(Width(width.0.to_string()));
         root.add_attr(Height(height.0.to_string()));
 
-        // Add styles, filters, symbols fo success and failure icons:
+        // Add styles, filters, symbols for success and failure icons:
         let elt = svg::new_style(include_str!("../resources/waterfall.css"));
         root.add_child(elt);
         let elt = new_filters();
@@ -167,6 +168,7 @@ fn new_vert_lines(
 
     // Draw the vertical lines:
     let mut lines = svg::new_group();
+    lines.add_attr(Class("grid-ticks".to_string()));
     lines.add_attr(Stroke("#ccc".to_string()));
     values.iter().for_each(|(x, _)| {
         if *x <= 0.0 {
@@ -315,7 +317,7 @@ fn new_call_tooltip(
     let offset_y = CALL_HEIGHT * (call_ctx.call_index - 1) + pixels_y.start;
     let offset_y = offset_y + CALL_HEIGHT - CALL_INSET;
     let max_width = pixels_x.end - pixels_x.start;
-    // We bound the tooltip background to the overall bouding box.
+    // We bound the tooltip background to the overall bounding box.
     let offset_x = Pixel::max(offset_x, 6.px());
     let offset_x = Pixel::min(offset_x, max_width - width - 6.px());
 
@@ -326,6 +328,7 @@ fn new_call_tooltip(
     let mut y = offset_y;
 
     let mut elt = svg::new_rect(x.0, y.0, width.0, height.0, "white");
+    elt.add_attr(Class("call-back".to_string()));
     elt.add_attr(Filter("url(#shadow)".to_string()));
     elt.add_attr(Stroke("#ccc".to_string()));
     elt.add_attr(StrokeWidth(1.0));
@@ -335,6 +338,10 @@ fn new_call_tooltip(
     y += 14.px();
     let delta_y = 30.px();
 
+    let mut legend = svg::new_group();
+    legend.add_attr(Class("call-legend".to_string()));
+    legend.add_attr(Fill("#555".to_string()));
+
     // Icon + URL + method
     let mut elt = svg::new_use();
     let icon = match call_ctx.kind {
@@ -347,19 +354,17 @@ fn new_call_tooltip(
     elt.add_attr(Y(y.0));
     elt.add_attr(Width("20".to_string()));
     elt.add_attr(Height("20".to_string()));
-    group.add_child(elt);
+    legend.add_child(elt);
 
     let text = format!("{} {}", call.request.method, call.request.url);
     let text = trunc_str(&text, 54);
     let text = format!("{text}  {}", call.response.status);
     let mut elt = svg::new_text(x.0 + 30.0, y.0 + 16.0, &text);
-    let color = match call_ctx.kind {
-        CallContextKind::Success | CallContextKind::Retry => "#555",
-        CallContextKind::Failure => "red",
-    };
-    elt.add_attr(Fill(color.to_string()));
+    if call_ctx.kind == CallContextKind::Failure {
+        elt.add_attr(Fill("red".to_string()));
+    }
     elt.add_attr(FontWeight("bold".to_string()));
-    group.add_child(elt);
+    legend.add_child(elt);
 
     x += 12.px();
     y += 32.px();
@@ -368,7 +373,7 @@ fn new_call_tooltip(
     let duration = call.timings.name_lookup.as_micros();
     let duration = Microsecond(duration as f64);
     let elt = new_legend(x, y, "DNS lookup", Some("#1d9688"), duration);
-    group.add_child(elt);
+    legend.add_child(elt);
     y += delta_y;
 
     // TCP handshake
@@ -376,7 +381,7 @@ fn new_call_tooltip(
     if let Some(duration) = duration {
         let duration = Microsecond(duration.as_micros() as f64);
         let elt = new_legend(x, y, "TCP handshake", Some("#fa7f03"), duration);
-        group.add_child(elt);
+        legend.add_child(elt);
         y += delta_y;
     }
 
@@ -385,7 +390,7 @@ fn new_call_tooltip(
     if let Some(duration) = duration {
         let duration = Microsecond(duration.as_micros() as f64);
         let elt = new_legend(x, y, "SSL handshake", Some("#9933ff"), duration);
-        group.add_child(elt);
+        legend.add_child(elt);
         y += delta_y;
     }
 
@@ -397,7 +402,7 @@ fn new_call_tooltip(
     if let Some(duration) = duration {
         let duration = Microsecond(duration.as_micros() as f64);
         let elt = new_legend(x, y, "Wait", Some("#18c852"), duration);
-        group.add_child(elt);
+        legend.add_child(elt);
         y += delta_y;
     }
 
@@ -406,7 +411,7 @@ fn new_call_tooltip(
     if let Some(duration) = duration {
         let duration = Microsecond(duration.as_micros() as f64);
         let elt = new_legend(x, y, "Data transfer", Some("#36a9f4"), duration);
-        group.add_child(elt);
+        legend.add_child(elt);
         y += delta_y;
     }
 
@@ -415,7 +420,7 @@ fn new_call_tooltip(
     let duration = Microsecond(duration as f64);
     let mut elt = new_legend(x, y, "Total", None, duration);
     elt.add_attr(FontWeight("bold".to_string()));
-    group.add_child(elt);
+    legend.add_child(elt);
     y += delta_y;
 
     // Start and stop timestamps
@@ -426,18 +431,18 @@ fn new_call_tooltip(
     let value = Microsecond(start.as_micros() as f64);
     let value = value.to_human_string();
     let elt = new_value("Start:", &value, x, y);
-    group.add_child(elt);
+    legend.add_child(elt);
     y += delta_y;
     let value = Microsecond(end.as_micros() as f64);
     let value = value.to_human_string();
     let elt = new_value("Stop:", &value, x, y);
-    group.add_child(elt);
+    legend.add_child(elt);
 
     y += delta_y;
     let value = Byte(call.response.body.len() as f64);
     let value = value.to_human_string();
     let elt = new_value("Transferred:", &value, x, y);
-    group.add_child(elt);
+    legend.add_child(elt);
 
     // Run URL
     y += 56.px();
@@ -446,12 +451,12 @@ fn new_call_tooltip(
         call_ctx.run_filename, call_ctx.entry_index, call_ctx.call_entry_index
     );
     let elt = new_link(x, y, "(view run)", &href);
-    group.add_child(elt);
+    legend.add_child(elt);
 
     // Source URL
     let href = format!("{}#l{}", call_ctx.source_filename, call_ctx.line);
     let elt = new_link(x + 90.px(), y, "(view source)", &href);
-    group.add_child(elt);
+    legend.add_child(elt);
 
     // Timings explanation
     y += delta_y;
@@ -461,7 +466,8 @@ fn new_call_tooltip(
         "Explanation",
         "https://hurl.dev/docs/response.html#timings",
     );
-    group.add_child(elt);
+    legend.add_child(elt);
+    group.add_child(legend);
 
     group
 }
@@ -499,7 +505,7 @@ fn new_call_sel(
         color,
     );
     elt.add_attr(Opacity(0.05));
-    elt.add_attr(Class("call-cell".to_string()));
+    elt.add_attr(Class("call-sel".to_string()));
     elt
 }
 
@@ -521,13 +527,11 @@ fn new_legend(
         group.add_child(color_elt);
     }
 
-    let mut text_elt = svg::new_text((x + dx_label).0, (y + dy_label).0, text);
-    text_elt.add_attr(Fill("#555".to_string()));
+    let text_elt = svg::new_text((x + dx_label).0, (y + dy_label).0, text);
     group.add_child(text_elt);
 
     let duration = duration.to_human_string();
-    let mut duration_elt = svg::new_text((x + dx_duration).0, (y + dy_label).0, &duration);
-    duration_elt.add_attr(Fill("#333".to_string()));
+    let duration_elt = svg::new_text((x + dx_duration).0, (y + dy_label).0, &duration);
     group.add_child(duration_elt);
 
     group
@@ -535,13 +539,11 @@ fn new_legend(
 
 fn new_value(label: &str, value: &str, x: Pixel, y: Pixel) -> Element {
     let mut group = svg::new_group();
-    let mut elt = svg::new_text(x.0, y.0, label);
-    elt.add_attr(Fill("#555".to_string()));
+    let elt = svg::new_text(x.0, y.0, label);
     group.add_child(elt);
 
     let x = x + 100.px();
-    let mut elt = svg::new_text(x.0, y.0, value);
-    elt.add_attr(Fill("#333".to_string()));
+    let elt = svg::new_text(x.0, y.0, value);
     group.add_child(elt);
 
     group
@@ -572,7 +574,7 @@ fn new_filters() -> Element {
 }
 
 impl Microsecond {
-    /// Returns a human readable sting of a microsecond.
+    /// Returns a human readable string of a microsecond.
     fn to_human_string(self) -> String {
         match self.0 {
             d if d < 0.0 => "_".to_string(),
@@ -584,7 +586,7 @@ impl Microsecond {
 }
 
 impl Byte {
-    /// Returns a human readable sting of a byte.
+    /// Returns a human readable string of a byte.
     fn to_human_string(self) -> String {
         match self.0 {
             d if d < 0.0 => "_".to_string(),
@@ -597,16 +599,16 @@ impl Byte {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
-    use crate::report::html::timeline::unit::Microsecond;
-    use crate::report::html::timeline::unit::{Interval, Scale};
     use chrono::{Duration, TimeZone, Utc};
 
+    use super::*;
+    use crate::report::html::timeline::unit::{Interval, Microsecond, Scale};
+
     #[test]
     fn legend_svg() {
         let x = 20.px();
         let y = 30.px();
-        let text = "Hellow world";
+        let text = "Hello world";
         let color = "red";
         let duration = Microsecond(2000.0);
 
@@ -615,8 +617,8 @@ mod tests {
             elt.to_string(),
             "<g>\
                 <rect x=\"20\" y=\"30\" width=\"20\" height=\"20\" fill=\"red\" />\
-                <text x=\"56\" y=\"47\" fill=\"#555\">Hellow world</text>\
-                <text x=\"200\" y=\"47\" fill=\"#333\">2.0 ms</text>\
+                <text x=\"56\" y=\"47\">Hello world</text>\
+                <text x=\"200\" y=\"47\">2.0 ms</text>\
             </g>"
         );
     }
@@ -639,7 +641,7 @@ mod tests {
         assert_eq!(
             elt.to_string(),
             "<g>\
-                <g stroke=\"#ccc\">\
+                <g class=\"grid-ticks\" stroke=\"#ccc\">\
                     <line x1=\"100\" y1=\"0\" x2=\"100\" y2=\"100\" />\
                     <line x1=\"200\" y1=\"0\" x2=\"200\" y2=\"100\" />\
                     <line x1=\"300\" y1=\"0\" x2=\"300\" y2=\"100\" />\
@@ -649,7 +651,7 @@ mod tests {
                     <line x1=\"700\" y1=\"0\" x2=\"700\" y2=\"100\" />\
                     <line x1=\"800\" y1=\"0\" x2=\"800\" y2=\"100\" />\
                     <line x1=\"900\" y1=\"0\" x2=\"900\" y2=\"100\" />\
-                    </g>\
+                </g>\
                 <g font-size=\"15px\" font-family=\"sans-serif\" fill=\"#777\">\
                     <text x=\"5\" y=\"20\">0 ms</text>\
                     <text x=\"105\" y=\"20\">100 ms</text>\
