@@ -15,9 +15,7 @@
  * limitations under the License.
  *
  */
-use indexmap::map::IndexMap;
-use xmltree::{Element, XMLNode};
-
+use crate::report::junit::xml::Element;
 use crate::runner::HurlResult;
 use crate::util::logger;
 
@@ -58,44 +56,21 @@ impl Testcase {
 
     /// Serializes this testcase to XML.
     pub fn to_xml(&self) -> Element {
-        let name = "testcase".to_string();
-        let mut attributes = IndexMap::new();
-        attributes.insert("id".to_string(), self.id.clone());
-        attributes.insert("name".to_string(), self.name.clone());
         let time_in_seconds = format!("{:.3}", self.time_in_ms as f64 / 1000.0);
-        attributes.insert("time".to_string(), time_in_seconds);
 
-        let mut children = vec![];
-        for message in self.failures.clone() {
-            let element = Element {
-                prefix: None,
-                namespace: None,
-                namespaces: None,
-                name: "failure".to_string(),
-                attributes: IndexMap::new(),
-                children: vec![XMLNode::Text(message)],
-            };
-            children.push(XMLNode::Element(element));
+        let mut element = Element::new("testcase")
+            .attr("id", &self.id)
+            .attr("name", &self.name)
+            .attr("time", &time_in_seconds);
+
+        for failure in self.failures.iter() {
+            element = element.add_child(Element::new("failure").text(failure))
         }
-        for message in self.errors.clone() {
-            let element = Element {
-                prefix: None,
-                namespace: None,
-                namespaces: None,
-                name: "error".to_string(),
-                attributes: IndexMap::new(),
-                children: vec![XMLNode::Text(message)],
-            };
-            children.push(XMLNode::Element(element));
-        }
-        Element {
-            name,
-            prefix: None,
-            namespace: None,
-            namespaces: None,
-            attributes,
-            children,
+
+        for error in self.errors.iter() {
+            element = element.add_child(Element::new("error").text(error))
         }
+        element
     }
 
     pub fn get_error_count(&self) -> usize {
@@ -109,9 +84,10 @@ impl Testcase {
 
 #[cfg(test)]
 mod test {
-    use hurl_core::ast::SourceInfo;
+    use hurl_core::ast::{Pos, SourceInfo};
 
     use crate::report::junit::testcase::Testcase;
+    use crate::report::junit::xml::XmlDocument;
     use crate::runner::{EntryResult, Error, HurlResult, RunnerError};
 
     #[test]
@@ -121,18 +97,16 @@ mod test {
             time_in_ms: 230,
             success: true,
             cookies: vec![],
+            timestamp: 1,
         };
 
-        let mut buffer = Vec::new();
         let content = "";
         let filename = "test.hurl";
-        Testcase::from(&hurl_result, content, filename)
-            .to_xml()
-            .write(&mut buffer)
-            .unwrap();
+        let element = Testcase::from(&hurl_result, content, filename).to_xml();
+        let doc = XmlDocument::new(element);
         assert_eq!(
-            std::str::from_utf8(&buffer).unwrap(),
-            r#"<?xml version="1.0" encoding="UTF-8"?><testcase id="test.hurl" name="test.hurl" time="0.230" />"#
+            doc.to_string().unwrap(),
+            r#"<?xml version="1.0" encoding="utf-8"?><testcase id="test.hurl" name="test.hurl" time="0.230" />"#
         );
     }
 
@@ -145,31 +119,31 @@ HTTP/1.0 200
         let hurl_result = HurlResult {
             entries: vec![EntryResult {
                 entry_index: 1,
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 35)),
                 calls: vec![],
                 captures: vec![],
                 asserts: vec![],
-                errors: vec![Error {
-                    source_info: SourceInfo::new(2, 10, 2, 13),
-                    inner: RunnerError::AssertStatus {
+                errors: vec![Error::new(
+                    SourceInfo::new(Pos::new(2, 10), Pos::new(2, 13)),
+                    RunnerError::AssertStatus {
                         actual: "404".to_string(),
                     },
-                    assert: true,
-                }],
+                    true,
+                )],
                 time_in_ms: 0,
                 compressed: false,
             }],
             time_in_ms: 230,
             success: true,
             cookies: vec![],
+            timestamp: 1,
         };
-        let mut buffer = Vec::new();
-        Testcase::from(&hurl_result, content, filename)
-            .to_xml()
-            .write(&mut buffer)
-            .unwrap();
+
+        let element = Testcase::from(&hurl_result, content, filename).to_xml();
+        let doc = XmlDocument::new(element);
         assert_eq!(
-            std::str::from_utf8(&buffer).unwrap(),
-            r#"<?xml version="1.0" encoding="UTF-8"?><testcase id="test.hurl" name="test.hurl" time="0.230"><failure>Assert status code
+            doc.to_string().unwrap(),
+            r#"<?xml version="1.0" encoding="utf-8"?><testcase id="test.hurl" name="test.hurl" time="0.230"><failure>Assert status code
   --> test.hurl:2:10
    |
  2 | HTTP/1.0 200
@@ -185,32 +159,28 @@ HTTP/1.0 200
         let hurl_result = HurlResult {
             entries: vec![EntryResult {
                 entry_index: 1,
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 18)),
                 calls: vec![],
                 captures: vec![],
                 asserts: vec![],
-                errors: vec![Error {
-                    source_info: SourceInfo::new(1, 5, 1, 19),
-                    inner: RunnerError::HttpConnection {
-                        url: "http://unknown".to_string(),
-                        message: "(6) Could not resolve host: unknown".to_string(),
-                    },
-                    assert: false,
-                }],
+                errors: vec![Error::new(
+                    SourceInfo::new(Pos::new(1, 5), Pos::new(1, 19)),
+                    RunnerError::HttpConnection("(6) Could not resolve host: unknown".to_string()),
+                    false,
+                )],
                 time_in_ms: 0,
                 compressed: false,
             }],
             time_in_ms: 230,
             success: true,
             cookies: vec![],
+            timestamp: 1,
         };
-        let mut buffer = Vec::new();
-        Testcase::from(&hurl_result, content, filename)
-            .to_xml()
-            .write(&mut buffer)
-            .unwrap();
+        let element = Testcase::from(&hurl_result, content, filename).to_xml();
+        let doc = XmlDocument::new(element);
         assert_eq!(
-            std::str::from_utf8(&buffer).unwrap(),
-            r#"<?xml version="1.0" encoding="UTF-8"?><testcase id="test.hurl" name="test.hurl" time="0.230"><error>HTTP connection
+            doc.to_string().unwrap(),
+            r#"<?xml version="1.0" encoding="utf-8"?><testcase id="test.hurl" name="test.hurl" time="0.230"><error>HTTP connection
   --> test.hurl:1:5
    |
  1 | GET http://unknown
