@@ -20,11 +20,13 @@ use std::collections::HashMap;
 use hurl_core::ast::*;
 
 use crate::http;
-use crate::runner::core::{Error, RunnerError, *};
+use crate::runner::error::{Error, RunnerError};
 use crate::runner::filter::eval_filters;
 use crate::runner::predicate::eval_predicate;
 use crate::runner::query::eval_query;
-use crate::runner::value::Value;
+use crate::runner::result::AssertResult;
+use crate::runner::Value;
+use crate::util::path::ContextDir;
 
 impl AssertResult {
     /// Evaluates an assert and returns `None` if assert is succeeded or an `Error` if failed.
@@ -41,13 +43,10 @@ impl AssertResult {
                 {
                     None
                 } else {
-                    Some(Error {
-                        source_info: source_info.clone(),
-                        inner: RunnerError::AssertVersion {
-                            actual: actual.clone(),
-                        },
-                        assert: false,
-                    })
+                    let inner = RunnerError::AssertVersion {
+                        actual: actual.to_string(),
+                    };
+                    Some(Error::new(*source_info, inner, false))
                 }
             }
             AssertResult::Status {
@@ -58,13 +57,10 @@ impl AssertResult {
                 if actual == expected {
                     None
                 } else {
-                    Some(Error {
-                        source_info: source_info.clone(),
-                        inner: RunnerError::AssertStatus {
-                            actual: actual.to_string(),
-                        },
-                        assert: false,
-                    })
+                    let inner = RunnerError::AssertStatus {
+                        actual: actual.to_string(),
+                    };
+                    Some(Error::new(*source_info, inner, false))
                 }
             }
             AssertResult::Header {
@@ -77,11 +73,8 @@ impl AssertResult {
                     if s == expected {
                         None
                     } else {
-                        Some(Error {
-                            source_info: source_info.clone(),
-                            inner: RunnerError::AssertHeaderValueError { actual: s.clone() },
-                            assert: false,
-                        })
+                        let inner = RunnerError::AssertHeaderValueError { actual: s.clone() };
+                        Some(Error::new(*source_info, inner, false))
                     }
                 }
             },
@@ -99,16 +92,12 @@ impl AssertResult {
                         } else {
                             let actual = actual.to_string();
                             let expected = expected.to_string();
-                            Some(Error {
-                                source_info: source_info.clone(),
-                                inner: RunnerError::AssertBodyValueError { actual, expected },
-                                assert: false,
-                            })
+                            let inner = RunnerError::AssertBodyValueError { actual, expected };
+                            Some(Error::new(*source_info, inner, false))
                         }
                     }
                 },
             },
-
             AssertResult::Explicit { actual: Err(e), .. } => Some(e.clone()),
             AssertResult::Explicit {
                 predicate_result: Some(Err(e)),
@@ -132,6 +121,7 @@ pub fn eval_assert(
     assert: &Assert,
     variables: &HashMap<String, Value>,
     http_response: &http::Response,
+    context_dir: &ContextDir,
 ) -> AssertResult {
     let query_result = eval_query(&assert.query, variables, http_response);
 
@@ -145,9 +135,8 @@ pub fn eval_assert(
                     .first()
                     .expect("at least one filter")
                     .1
-                    .source_info
-                    .clone(),
-                inner: RunnerError::FilterMissingInput {},
+                    .source_info,
+                inner: RunnerError::FilterMissingInput,
                 assert: true,
             }),
             Some(value) => {
@@ -162,15 +151,20 @@ pub fn eval_assert(
         query_result
     };
 
-    let source_info = &assert.predicate.predicate_func.source_info;
+    let source_info = assert.predicate.predicate_func.source_info;
     let predicate_result = match &actual {
         Err(_) => None,
-        Ok(actual) => Some(eval_predicate(&assert.predicate, variables, actual)),
+        Ok(actual) => Some(eval_predicate(
+            &assert.predicate,
+            variables,
+            actual,
+            context_dir,
+        )),
     };
 
     AssertResult::Explicit {
         actual,
-        source_info: source_info.clone(),
+        source_info,
         predicate_result,
     }
 }
@@ -178,25 +172,28 @@ pub fn eval_assert(
 #[cfg(test)]
 pub mod tests {
     use hurl_core::ast::SourceInfo;
+    use std::path::Path;
 
     use super::super::query;
     use super::*;
     use crate::http::xml_three_users_http_response;
+    use crate::runner::Number;
 
-    // xpath //user countEquals 3
+    // `xpath "//user" count == 3`
     pub fn assert_count_user() -> Assert {
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::new(1, 1, 1, 1),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
         };
         let predicate = Predicate {
             not: false,
             space0: whitespace.clone(),
             predicate_func: PredicateFunc {
-                source_info: SourceInfo::new(1, 14, 1, 27),
-                value: PredicateFuncValue::CountEqual {
+                source_info: SourceInfo::new(Pos::new(1, 22), Pos::new(1, 24)),
+                value: PredicateFuncValue::Equal {
                     space0: whitespace.clone(),
-                    value: PredicateValue::Integer(3),
+                    value: PredicateValue::Number(hurl_core::ast::Number::Integer(3)),
+                    operator: true,
                 },
             },
         };
@@ -204,7 +201,13 @@ pub mod tests {
             line_terminators: vec![],
             space0: whitespace.clone(),
             query: query::tests::xpath_users(),
-            filters: vec![],
+            filters: vec![(
+                whitespace.clone(),
+                Filter {
+                    source_info: SourceInfo::new(Pos::new(1, 16), Pos::new(1, 21)),
+                    value: FilterValue::Count,
+                },
+            )],
             space1: whitespace.clone(),
             predicate,
             line_terminator0: LineTerminator {
@@ -221,15 +224,19 @@ pub mod tests {
     #[test]
     fn test_eval() {
         let variables = HashMap::new();
+        let current_dir = std::env::current_dir().unwrap();
+        let file_root = Path::new("file_root");
+        let context_dir = ContextDir::new(current_dir.as_path(), file_root);
         assert_eq!(
             eval_assert(
                 &assert_count_user(),
                 &variables,
                 &xml_three_users_http_response(),
+                &context_dir
             ),
             AssertResult::Explicit {
-                actual: Ok(Some(Value::Nodeset(3))),
-                source_info: SourceInfo::new(1, 14, 1, 27),
+                actual: Ok(Some(Value::Number(Number::Integer(3)))),
+                source_info: SourceInfo::new(Pos::new(1, 22), Pos::new(1, 24)),
                 predicate_result: Some(Ok(())),
             }
         );
