@@ -21,7 +21,7 @@ use std::path::PathBuf;
 use hurl_core::ast::*;
 
 use crate::http;
-use crate::runner::core::{Error, RunnerError};
+use crate::runner::error::{Error, RunnerError};
 use crate::runner::json::eval_json_value;
 use crate::runner::multiline::eval_multiline;
 use crate::runner::template::eval_template;
@@ -58,33 +58,32 @@ pub fn eval_bytes(
         Bytes::Base64(Base64 { value, .. }) => Ok(http::Body::Binary(value.clone())),
         Bytes::Hex(Hex { value, .. }) => Ok(http::Body::Binary(value.clone())),
         Bytes::File(File { filename, .. }) => {
-            let value = eval_file(filename, context_dir)?;
-            Ok(http::Body::File(value, filename.value.clone()))
+            let value = eval_file(filename, variables, context_dir)?;
+            let filename = eval_template(filename, variables)?;
+            Ok(http::Body::File(value, filename))
         }
     }
 }
 
-pub fn eval_file(filename: &Filename, context_dir: &ContextDir) -> Result<Vec<u8>, Error> {
+pub fn eval_file(
+    filename: &Template,
+    variables: &HashMap<String, Value>,
+    context_dir: &ContextDir,
+) -> Result<Vec<u8>, Error> {
+    let file = eval_template(filename, variables)?;
     // In order not to leak any private date, we check that the user provided file
     // is a child of the context directory.
-    let file = filename.value.clone();
     if !context_dir.is_access_allowed(&file) {
-        return Err(Error {
-            source_info: filename.source_info.clone(),
-            inner: RunnerError::UnauthorizedFileAccess {
-                path: PathBuf::from(file),
-            },
-            assert: false,
-        });
+        let inner = RunnerError::UnauthorizedFileAccess {
+            path: PathBuf::from(file),
+        };
+        return Err(Error::new(filename.source_info, inner, false));
     }
     let resolved_file = context_dir.get_path(&file);
+    let inner = RunnerError::FileReadAccess { file };
     match std::fs::read(resolved_file) {
         Ok(value) => Ok(value),
-        Err(_) => Err(Error {
-            source_info: filename.source_info.clone(),
-            inner: RunnerError::FileReadAccess { value: file },
-            assert: false,
-        }),
+        Err(_) => Err(Error::new(filename.source_info, inner, false)),
     }
 }
 
@@ -101,14 +100,18 @@ mod tests {
         // file, data.bin;
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
 
         let bytes = Bytes::File(File {
             space0: whitespace.clone(),
-            filename: Filename {
-                value: String::from("tests/data.bin"),
-                source_info: SourceInfo::new(1, 7, 1, 15),
+            filename: Template {
+                delimiter: None,
+                source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15)),
+                elements: vec![TemplateElement::String {
+                    value: "tests/data.bin".to_string(),
+                    encoded: "tests/data.bin".to_string(),
+                }],
             },
             space1: whitespace,
         });
@@ -128,14 +131,18 @@ mod tests {
         // file, data.bin;
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
 
         let bytes = Bytes::File(File {
             space0: whitespace.clone(),
-            filename: Filename {
-                value: String::from("data.bin"),
-                source_info: SourceInfo::new(1, 7, 1, 15),
+            filename: Template {
+                delimiter: None,
+                source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15)),
+                elements: vec![TemplateElement::String {
+                    value: "data.bin".to_string(),
+                    encoded: "data.bin".to_string(),
+                }],
             },
             space1: whitespace,
         });
@@ -149,9 +156,12 @@ mod tests {
         assert_eq!(
             error.inner,
             RunnerError::FileReadAccess {
-                value: "data.bin".to_string()
+                file: "data.bin".to_string()
             }
         );
-        assert_eq!(error.source_info, SourceInfo::new(1, 7, 1, 15));
+        assert_eq!(
+            error.source_info,
+            SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15))
+        );
     }
 }
