@@ -20,10 +20,12 @@ use std::collections::HashMap;
 use hurl_core::ast::*;
 
 use crate::http;
-use crate::runner::core::{CaptureResult, Error, RunnerError};
+use crate::runner::error::{Error, RunnerError};
 use crate::runner::filter::eval_filters;
 use crate::runner::query::eval_query;
-use crate::runner::value::Value;
+use crate::runner::result::CaptureResult;
+use crate::runner::template::eval_template;
+use crate::runner::Value;
 
 /// Evaluates a `capture` with `variables` map and `http_response`, returns a
 /// [`CaptureResult`] on success or an [`Error`] .
@@ -32,25 +34,25 @@ pub fn eval_capture(
     variables: &HashMap<String, Value>,
     http_response: &http::Response,
 ) -> Result<CaptureResult, Error> {
-    let name = &capture.name.value;
+    let name = eval_template(&capture.name, variables)?;
     let value = eval_query(&capture.query, variables, http_response)?;
     let value = match value {
         None => {
-            return Err(Error {
-                source_info: capture.query.source_info.clone(),
-                inner: RunnerError::NoQueryResult {},
-                assert: false,
-            });
+            return Err(Error::new(
+                capture.query.source_info,
+                RunnerError::NoQueryResult,
+                false,
+            ));
         }
         Some(value) => {
             let filters = capture.filters.iter().map(|(_, f)| f.clone()).collect();
             match eval_filters(&filters, &value, variables, false)? {
                 None => {
-                    return Err(Error {
-                        source_info: capture.query.source_info.clone(),
-                        inner: RunnerError::NoQueryResult {},
-                        assert: false,
-                    })
+                    return Err(Error::new(
+                        capture.query.source_info,
+                        RunnerError::NoQueryResult,
+                        false,
+                    ));
                 }
                 Some(v) => v,
             }
@@ -69,21 +71,24 @@ pub mod tests {
 
     use self::super::super::query;
     use super::*;
+    use crate::runner::Number;
 
     pub fn user_count_capture() -> Capture {
         // non scalar value
         let whitespace = Whitespace {
-            value: String::from(""),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            value: String::new(),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
         Capture {
             line_terminators: vec![],
             space0: whitespace.clone(),
-            name: EncodedString {
-                quotes: false,
-                value: "UserCount".to_string(),
-                encoded: "UserCount".to_string(),
-                source_info: SourceInfo::new(0, 0, 0, 0),
+            name: Template {
+                delimiter: None,
+                elements: vec![TemplateElement::String {
+                    value: "UserCount".to_string(),
+                    encoded: "UserCount".to_string(),
+                }],
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             space1: whitespace.clone(),
             space2: whitespace.clone(),
@@ -102,17 +107,19 @@ pub mod tests {
     pub fn duration_capture() -> Capture {
         // non scalar value
         let whitespace = Whitespace {
-            value: String::from(""),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            value: String::new(),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
         Capture {
             line_terminators: vec![],
             space0: whitespace.clone(),
-            name: EncodedString {
-                quotes: false,
-                value: "duration".to_string(),
-                encoded: "duration".to_string(),
-                source_info: SourceInfo::new(0, 0, 0, 0),
+            name: Template {
+                delimiter: None,
+                elements: vec![TemplateElement::String {
+                    value: "duration".to_string(),
+                    encoded: "duration".to_string(),
+                }],
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             space1: whitespace.clone(),
             space2: whitespace.clone(),
@@ -132,17 +139,19 @@ pub mod tests {
     fn test_invalid_xpath() {
         let variables = HashMap::new();
         let whitespace = Whitespace {
-            value: String::from(""),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            value: String::new(),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
         let capture = Capture {
             line_terminators: vec![],
             space0: whitespace.clone(),
-            name: EncodedString {
-                quotes: false,
-                value: "count".to_string(),
-                encoded: "count".to_string(),
-                source_info: SourceInfo::new(0, 0, 0, 0),
+            name: Template {
+                delimiter: None,
+                elements: vec![TemplateElement::String {
+                    value: "count".to_string(),
+                    encoded: "count".to_string(),
+                }],
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             filters: vec![],
             space1: whitespace.clone(),
@@ -167,24 +176,26 @@ pub mod tests {
     fn test_capture_unsupported() {
         // non scalar value
         let whitespace = Whitespace {
-            value: String::from(""),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            value: String::new(),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
         let _capture = Capture {
             line_terminators: vec![],
             space0: whitespace.clone(),
-            name: EncodedString {
-                quotes: false,
-                value: "???".to_string(),
-                encoded: "???".to_string(),
-                source_info: SourceInfo::new(0, 0, 0, 0),
+            name: Template {
+                delimiter: None,
+                elements: vec![TemplateElement::String {
+                    value: "???".to_string(),
+                    encoded: "???".to_string(),
+                }],
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             space1: whitespace.clone(),
             space2: whitespace.clone(),
 
             // xpath //user
             query: Query {
-                source_info: SourceInfo::new(1, 1, 1, 13),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
                 value: QueryValue::Xpath {
                     space0: whitespace.clone(),
                     expr: Template {
@@ -193,7 +204,7 @@ pub mod tests {
                             value: "//user".to_string(),
                             encoded: "//user".to_string(),
                         }],
-                        source_info: SourceInfo::new(1, 7, 1, 13),
+                        source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 13)),
                     },
                 },
             },
@@ -218,7 +229,7 @@ pub mod tests {
             .unwrap(),
             CaptureResult {
                 name: "UserCount".to_string(),
-                value: Value::from_f64(3.0),
+                value: Value::Number(Number::from(3.0)),
             }
         );
 
@@ -226,7 +237,7 @@ pub mod tests {
             eval_capture(&duration_capture(), &variables, &http::json_http_response()).unwrap(),
             CaptureResult {
                 name: "duration".to_string(),
-                value: Value::from_f64(1.5),
+                value: Value::Number(Number::from(1.5)),
             }
         );
     }
