@@ -22,11 +22,11 @@ use hurl_core::ast::*;
 
 use crate::http;
 use crate::http::ClientOptions;
-use crate::runner::core::{Error, RunnerError, *};
+use crate::runner::error::Error;
 use crate::runner::request::{cookie_storage_clear, cookie_storage_set, eval_request};
 use crate::runner::response::{eval_asserts, eval_captures, eval_version_status_asserts};
+use crate::runner::result::{AssertResult, EntryResult};
 use crate::runner::runner_options::RunnerOptions;
-use crate::runner::template::eval_template;
 use crate::runner::value::Value;
 use crate::util::logger::{Logger, Verbosity};
 
@@ -44,11 +44,13 @@ pub fn run(
     runner_options: &RunnerOptions,
     logger: &Logger,
 ) -> EntryResult {
-    let http_request = match eval_request(&entry.request, variables, &runner_options.context_dir) {
+    let context_dir = &runner_options.context_dir;
+    let http_request = match eval_request(&entry.request, variables, context_dir) {
         Ok(r) => r,
         Err(error) => {
             return EntryResult {
                 entry_index,
+                source_info: entry.source_info(),
                 calls: vec![],
                 captures: vec![],
                 asserts: vec![],
@@ -82,8 +84,13 @@ pub fn run(
     log_request_spec(&http_request, logger);
 
     logger.debug("Request can be run with the following curl command:");
-    let curl_command =
-        http_client.curl_command_line(&http_request, &runner_options.context_dir, &client_options);
+    let output = &runner_options.output;
+    let curl_command = http_client.curl_command_line(
+        &http_request,
+        context_dir,
+        output.as_deref(),
+        &client_options,
+    );
     logger.debug(curl_command.as_str());
     logger.debug("");
 
@@ -91,17 +98,13 @@ pub fn run(
     let calls = match http_client.execute_with_redirect(&http_request, &client_options, logger) {
         Ok(calls) => calls,
         Err(http_error) => {
-            let runner_error = RunnerError::from(http_error);
-            let error = Error {
-                source_info: SourceInfo {
-                    start: entry.request.url.source_info.start.clone(),
-                    end: entry.request.url.source_info.end.clone(),
-                },
-                inner: runner_error,
-                assert: false,
-            };
+            let start = entry.request.url.source_info.start;
+            let end = entry.request.url.source_info.end;
+            let error_source_info = SourceInfo::new(start, end);
+            let error = Error::new(error_source_info, http_error.into(), false);
             return EntryResult {
                 entry_index,
+                source_info: entry.source_info(),
                 calls: vec![],
                 captures: vec![],
                 asserts: vec![],
@@ -136,6 +139,7 @@ pub fn run(
                 logger.debug("");
                 return EntryResult {
                     entry_index,
+                    source_info: entry.source_info(),
                     calls,
                     captures: vec![],
                     asserts,
@@ -155,6 +159,7 @@ pub fn run(
             Err(e) => {
                 return EntryResult {
                     entry_index,
+                    source_info: entry.source_info(),
                     calls,
                     captures: vec![],
                     asserts: all_asserts,
@@ -177,12 +182,7 @@ pub fn run(
     // Compute asserts
     if !runner_options.ignore_asserts {
         if let Some(response_spec) = &entry.response {
-            let mut asserts = eval_asserts(
-                response_spec,
-                variables,
-                http_response,
-                &runner_options.context_dir,
-            );
+            let mut asserts = eval_asserts(response_spec, variables, http_response, context_dir);
             all_asserts.append(&mut asserts);
         }
     };
@@ -191,6 +191,7 @@ pub fn run(
 
     EntryResult {
         entry_index,
+        source_info: entry.source_info(),
         calls,
         captures,
         asserts: all_asserts,
@@ -208,11 +209,7 @@ fn asserts_to_errors(asserts: &[AssertResult]) -> Vec<Error> {
         .map(
             |Error {
                  source_info, inner, ..
-             }| Error {
-                source_info,
-                inner,
-                assert: true,
-            },
+             }| Error::new(source_info, inner, true),
         )
         .collect()
 }
@@ -220,30 +217,35 @@ fn asserts_to_errors(asserts: &[AssertResult]) -> Vec<Error> {
 impl ClientOptions {
     fn from(runner_options: &RunnerOptions, verbosity: Option<Verbosity>) -> Self {
         ClientOptions {
+            aws_sigv4: runner_options.aws_sigv4.clone(),
             cacert_file: runner_options.cacert_file.clone(),
             client_cert_file: runner_options.client_cert_file.clone(),
             client_key_file: runner_options.client_key_file.clone(),
+            compressed: runner_options.compressed,
+            connect_timeout: runner_options.connect_timeout,
             connects_to: runner_options.connects_to.clone(),
-            follow_location: runner_options.follow_location,
-            max_redirect: runner_options.max_redirect,
             cookie_input_file: runner_options.cookie_input_file.clone(),
+            follow_location: runner_options.follow_location,
+            follow_location_trusted: runner_options.follow_location_trusted,
+            http_version: runner_options.http_version,
+            ip_resolve: runner_options.ip_resolve,
+            max_redirect: runner_options.max_redirect,
             path_as_is: runner_options.path_as_is,
             proxy: runner_options.proxy.clone(),
             no_proxy: runner_options.no_proxy.clone(),
-            verbosity: match verbosity {
-                Some(Verbosity::Verbose) => Some(http::Verbosity::Verbose),
-                Some(Verbosity::VeryVerbose) => Some(http::Verbosity::VeryVerbose),
-                _ => None,
-            },
             insecure: runner_options.insecure,
             resolves: runner_options.resolves.clone(),
             retry: runner_options.retry,
             ssl_no_revoke: runner_options.ssl_no_revoke,
             timeout: runner_options.timeout,
-            connect_timeout: runner_options.connect_timeout,
+            unix_socket: runner_options.unix_socket.clone(),
             user: runner_options.user.clone(),
             user_agent: runner_options.user_agent.clone(),
-            compressed: runner_options.compressed,
+            verbosity: match verbosity {
+                Some(Verbosity::Verbose) => Some(http::Verbosity::Verbose),
+                Some(Verbosity::VeryVerbose) => Some(http::Verbosity::VeryVerbose),
+                _ => None,
+            },
         }
     }
 }
@@ -285,144 +287,3 @@ fn log_request_spec(request: &http::RequestSpec, logger: &Logger) {
     }
     logger.debug("");
 }
-
-/// Returns a new [`RunnerOptions`] based on the `entry` optional Options section
-/// and a default `runner_options`.
-/// The [`variables`] can also be updated if `variable` keys are present in the section.
-pub fn get_entry_options(
-    entry: &Entry,
-    runner_options: &RunnerOptions,
-    variables: &mut HashMap<String, Value>,
-    logger: &Logger,
-) -> Result<RunnerOptions, Error> {
-    let mut runner_options = runner_options.clone();
-    if !has_options(entry) {
-        return Ok(runner_options);
-    }
-
-    logger.debug("");
-    logger.debug_important("Entry options:");
-
-    for section in &entry.request.sections {
-        if let SectionValue::Options(options) = &section.value {
-            for option in options {
-                match option {
-                    EntryOption::CaCertificate(option) => {
-                        runner_options.cacert_file = Some(option.filename.value.clone());
-                        logger.debug(format!("cacert: {}", option.filename.value).as_str());
-                    }
-                    EntryOption::ClientCert(option) => {
-                        runner_options.client_cert_file = Some(option.filename.value.clone());
-                        logger.debug(format!("cert: {}", option.filename.value).as_str());
-                    }
-                    EntryOption::ClientKey(option) => {
-                        runner_options.client_key_file = Some(option.filename.value.clone());
-                        logger.debug(format!("key: {}", option.filename.value).as_str());
-                    }
-                    EntryOption::Compressed(option) => {
-                        runner_options.compressed = option.value;
-                        logger.debug(format!("compressed: {}", option.value).as_str());
-                    }
-                    EntryOption::FollowLocation(option) => {
-                        runner_options.follow_location = option.value;
-                        logger.debug(format!("location: {}", option.value).as_str());
-                    }
-                    EntryOption::Insecure(option) => {
-                        runner_options.insecure = option.value;
-                        logger.debug(format!("insecure: {}", option.value).as_str());
-                    }
-                    EntryOption::MaxRedirect(option) => {
-                        runner_options.max_redirect = Some(option.value);
-                        logger.debug(format!("max-redirs: {}", option.value).as_str());
-                    }
-                    EntryOption::PathAsIs(option) => {
-                        runner_options.path_as_is = option.value;
-                        logger.debug(format!("path-as-is: {}", option.value).as_str());
-                    }
-                    EntryOption::Proxy(option) => {
-                        runner_options.proxy = Some(option.value.clone());
-                        logger.debug(format!("proxy: {}", option.value).as_str());
-                    }
-                    EntryOption::Retry(option) => {
-                        runner_options.retry = option.value;
-                        logger.debug(format!("retry: {}", option.value).as_str());
-                    }
-                    EntryOption::RetryInterval(option) => {
-                        runner_options.retry_interval = Duration::from_millis(option.value);
-                        logger.debug(format!("retry-interval: {}", option.value).as_str());
-                    }
-                    EntryOption::Variable(VariableOption {
-                        value: VariableDefinition { name, value, .. },
-                        ..
-                    }) => {
-                        let value = eval_variable_value(value, variables)?;
-                        variables.insert(name.clone(), value);
-                    }
-                    EntryOption::Verbose(option) => {
-                        logger.debug(format!("verbose: {}", option.value).as_str());
-                    }
-
-                    EntryOption::VeryVerbose(option) => {
-                        logger.debug(format!("very-verbose: {}", option.value).as_str());
-                    }
-                }
-            }
-        }
-    }
-    Ok(runner_options)
-}
-
-fn eval_variable_value(
-    variable_value: &VariableValue,
-    variables: &mut HashMap<String, Value>,
-) -> Result<Value, Error> {
-    match variable_value {
-        VariableValue::Null {} => Ok(Value::Null),
-        VariableValue::Bool(v) => Ok(Value::Bool(*v)),
-        VariableValue::Integer(v) => Ok(Value::Integer(*v)),
-        VariableValue::Float(Float { value, .. }) => Ok(Value::Float(*value)),
-        VariableValue::String(template) => {
-            let s = eval_template(template, variables)?;
-            Ok(Value::String(s))
-        }
-    }
-}
-
-/// Returns [`true`] if this `entry` has an Option section, [`false`] otherwise.
-fn has_options(entry: &Entry) -> bool {
-    entry
-        .request
-        .sections
-        .iter()
-        .any(|s| matches!(s.value, SectionValue::Options(_)))
-}
-
-/// Returns the overridden `entry` verbosity, or the default `verbosity` file.
-pub fn get_entry_verbosity(entry: &Entry, verbosity: &Option<Verbosity>) -> Option<Verbosity> {
-    let mut verbosity = *verbosity;
-
-    for section in &entry.request.sections {
-        if let SectionValue::Options(options) = &section.value {
-            for option in options {
-                match option {
-                    EntryOption::Verbose(option) => {
-                        verbosity = if option.value {
-                            Some(Verbosity::Verbose)
-                        } else {
-                            None
-                        }
-                    }
-                    EntryOption::VeryVerbose(option) => {
-                        verbosity = if option.value {
-                            Some(Verbosity::VeryVerbose)
-                        } else {
-                            None
-                        }
-                    }
-                    _ => {}
-                }
-            }
-        }
-    }
-    verbosity
-}
