@@ -16,20 +16,21 @@
  *
  */
 use std::collections::HashMap;
+use std::path::PathBuf;
 use std::thread;
 use std::time::Instant;
 
+use chrono::Utc;
 use hurl_core::ast::VersionValue::VersionAnyLegacy;
-use hurl_core::ast::*;
+use hurl_core::ast::{Body, Bytes, Entry, HurlFile, MultilineString, Request, Response, Retry};
 use hurl_core::error::Error;
 use hurl_core::parser;
 
-use crate::http;
 use crate::http::Call;
-use crate::runner::core::*;
 use crate::runner::runner_options::RunnerOptions;
-use crate::runner::{entry, Value};
+use crate::runner::{entry, options, EntryResult, HurlResult, RunnerError, Value};
 use crate::util::logger::{ErrorFormat, Logger, LoggerOptions, LoggerOptionsBuilder};
+use crate::{http, runner};
 
 /// Runs a Hurl `content` and returns a [`HurlResult`] upon completion.
 ///
@@ -90,8 +91,7 @@ pub fn run(
     log_run_info(&hurl_file, runner_options, variables, &logger);
 
     // Now, we have a syntactically correct HurlFile instance, we can run it.
-    let cookie_input_file = runner_options.cookie_input_file.clone();
-    let mut http_client = http::Client::new(cookie_input_file);
+    let mut http_client = http::Client::new();
     let mut entries = vec![];
     let mut variables = variables.clone();
     let mut entry_index = 1;
@@ -102,17 +102,23 @@ pub fn run(
         hurl_file.entries.len()
     };
     let start = Instant::now();
+    let timestamp = Utc::now().timestamp();
 
+    // Main loop processing each entry.
+    // The `entry_index` is not always incremented of each loop tick: an entry can be retried upon
+    // errors for instance. Each entry is executed with options that are computed from the global
+    // runner options and the "overridden" request options.
     loop {
         if entry_index > n {
             break;
         }
         let entry = &hurl_file.entries[entry_index - 1];
 
-        // We compute the new logger for this entry, before entering into the `run`
-        // function because entry options can modify the logger and we want the preamble
-        // "Executing entry..." to be displayed based on the entry level verbosity.
-        let logger = get_entry_logger(entry, logger_options);
+        // We compute the new logger for this entry, before entering into the `run` function because
+        // entry options can modify the logger and we want the preamble "Executing entry..." to be
+        // displayed based on the entry level verbosity.
+        let logger =
+            get_entry_logger(entry, logger_options, &variables).map_err(|e| e.description())?;
         if let Some(pre_entry) = runner_options.pre_entry {
             let exit = pre_entry(entry.clone());
             if exit {
@@ -130,18 +136,40 @@ pub fn run(
         logger.test_progress(entry_index, n);
 
         // The real execution of the entry happens here, with the overridden entry options.
-        let options = entry::get_entry_options(entry, runner_options, &mut variables, &logger);
+        let options = options::get_entry_options(entry, runner_options, &mut variables, &logger);
         let entry_result = match &options {
-            Ok(options) => entry::run(
-                entry,
-                entry_index,
-                &mut http_client,
-                &mut variables,
-                options,
-                &logger,
-            ),
+            Ok(options) => {
+                if options.skip {
+                    logger
+                        .debug_important(format!("Entry {entry_index} has been skipped").as_str());
+                    logger.debug("");
+                    entry_index += 1;
+                    continue;
+                }
+
+                let delay = options.delay;
+                let delay_ms = delay.as_millis();
+                if delay_ms > 0 {
+                    logger.debug("");
+                    logger.debug_important(
+                        format!("Delay entry {entry_index} (x{retry_count} by {delay_ms} ms)")
+                            .as_str(),
+                    );
+                    thread::sleep(delay);
+                };
+
+                entry::run(
+                    entry,
+                    entry_index,
+                    &mut http_client,
+                    &mut variables,
+                    options,
+                    &logger,
+                )
+            }
             Err(error) => EntryResult {
                 entry_index,
+                source_info: entry.source_info(),
                 calls: vec![],
                 captures: vec![],
                 asserts: vec![],
@@ -163,8 +191,8 @@ pub fn run(
         } else {
             false
         };
-        // If `retry_max_reached` is true, we print now a warning, before displaying
-        // any assert error so any potential error is the last thing displayed to the user.
+        // If `retry_max_reached` is true, we print now a warning, before displaying any assert
+        // error so any potential error is the last thing displayed to the user.
         // If `retry_max_reached` is not true (for instance `retry`is true, or there is no error
         // we first log the error and a potential warning about retrying.
         logger.test_erase_line();
@@ -173,10 +201,36 @@ pub fn run(
             logger.debug("");
         }
 
+        // We logs eventual errors, only if we're not retrying the current entry...
         let retry = !matches!(retry_opts, Retry::None) && !retry_max_reached && has_error;
         if has_error {
             log_errors(&entry_result, content, retry, &logger);
         }
+
+        // When --output is overriden on a request level, we output the HTTP response only if the
+        // call has succeeded.
+        if let Ok(RunnerOptions {
+            output: Some(output),
+            ..
+        }) = options
+        {
+            if !has_error {
+                // TODO: make output write and access error as part of entry result errors.
+                // For the moment, we deal the --output request failure as a simple warning and not
+                // an error. If we want to treat it as an error, we've to add it to the current
+                // `entry_result` errors, and optionally deals with retry if we can't write to the
+                // specified path.
+                if !runner_options.context_dir.is_access_allowed(&output) {
+                    let inner = RunnerError::UnauthorizedFileAccess {
+                        path: PathBuf::from(output.clone()),
+                    };
+                    let error = runner::Error::new(entry.request.source_info, inner, false);
+                    logger.warning(&error.fixme());
+                } else if let Err(error) = entry_result.write_response(output) {
+                    logger.warning(&error.fixme());
+                }
+            }
+        }
         entries.push(entry_result);
 
         if retry {
@@ -186,9 +240,9 @@ pub fn run(
                 format!("Retry entry {entry_index} (x{retry_count} pause {delay} ms)").as_str(),
             );
             retry_count += 1;
-            // If we retry the entry, we do not want to display a 'blank' progress bar
-            // during the sleep delay. During the pause, we artificially show the previsoulsy
-            // erased progress line.
+            // If we retry the entry, we do not want to display a 'blank' progress bar during the
+            // sleep delay. During the pause, we artificially show the previously erased progress
+            // line.
             logger.test_progress(entry_index, n);
             thread::sleep(retry_interval);
             logger.test_erase_line();
@@ -201,7 +255,7 @@ pub fn run(
                 break;
             }
         }
-        if runner_options.fail_fast && has_error {
+        if !runner_options.continue_on_error && has_error {
             break;
         }
 
@@ -218,6 +272,7 @@ pub fn run(
         time_in_ms,
         success,
         cookies,
+        timestamp,
     })
 }
 
@@ -307,6 +362,57 @@ fn warn_deprecated(entry: &Entry, logger: &Logger) {
     }
 }
 
+// Returns the list of options that have non-default values.
+fn get_non_default_options(options: &RunnerOptions) -> Vec<(&'static str, String)> {
+    let default_options = RunnerOptions::default();
+
+    let mut non_default_options = vec![];
+
+    if options.continue_on_error != default_options.continue_on_error {
+        non_default_options.push(("continue_on_error", options.continue_on_error.to_string()));
+    }
+
+    if options.delay != default_options.delay {
+        // FIXME: the cast to u64 seems not necessary.
+        //  If we dont cast from u128 and try to format! or println!
+        //  we have a segfault on Alpine Docker images and Rust 1.68.0, whereas it was
+        //  ok with Rust >= 1.67.0.
+        non_default_options.push(("delay", format!("{}ms", options.delay.as_millis() as u64)));
+    }
+
+    if options.follow_location != default_options.follow_location {
+        non_default_options.push(("follow redirect", options.follow_location.to_string()));
+    }
+
+    if options.insecure != default_options.insecure {
+        non_default_options.push(("insecure", options.insecure.to_string()));
+    }
+
+    if options.max_redirect != default_options.max_redirect {
+        if let Some(n) = options.max_redirect {
+            non_default_options.push(("max redirect", n.to_string()));
+        }
+    }
+
+    if options.proxy != default_options.proxy {
+        if let Some(proxy) = &options.proxy {
+            non_default_options.push(("proxy", proxy.to_string()));
+        }
+    }
+
+    if options.retry != default_options.retry {
+        non_default_options.push(("retry", options.retry.to_string()));
+    }
+
+    if options.unix_socket != default_options.unix_socket {
+        if let Some(unix_socket) = &options.unix_socket {
+            non_default_options.push(("unix socket", unix_socket.to_string()))
+        }
+    }
+
+    non_default_options
+}
+
 /// Logs various debug information at the start of `hurl_file` run.
 fn log_run_info(
     hurl_file: &HurlFile,
@@ -314,17 +420,16 @@ fn log_run_info(
     variables: &HashMap<String, Value>,
     logger: &Logger,
 ) {
-    logger.debug_important("Options:");
-    logger.debug(format!("    fail fast: {}", runner_options.fail_fast).as_str());
-    logger.debug(format!("    follow redirect: {}", runner_options.follow_location).as_str());
-    logger.debug(format!("    insecure: {}", runner_options.insecure).as_str());
-    if let Some(n) = runner_options.max_redirect {
-        logger.debug(format!("    max redirect: {n}").as_str());
+    if logger.verbosity.is_some() {
+        let non_default_options = get_non_default_options(runner_options);
+        if !non_default_options.is_empty() {
+            logger.debug_important("Options:");
+            for (name, value) in non_default_options.iter() {
+                logger.debug(format!("    {name}: {value}").as_str());
+            }
+        }
     }
-    if let Some(proxy) = &runner_options.proxy {
-        logger.debug(format!("    proxy: {proxy}").as_str());
-    }
-    logger.debug(format!("    retry: {}", runner_options.retry).as_str());
+
     if !variables.is_empty() {
         logger.debug_important("Variables:");
         for (name, value) in variables.iter() {
@@ -365,8 +470,13 @@ fn log_errors(entry_result: &EntryResult, content: &str, retry: bool, logger: &L
 /// Creates a new logger for this entry.
 /// Verbosity can be overridden at entry level with an Options section so each
 /// entry has its own logger.
-fn get_entry_logger(entry: &Entry, logger_options: &LoggerOptions) -> Logger {
-    let entry_verbosity = entry::get_entry_verbosity(entry, &logger_options.verbosity);
+fn get_entry_logger(
+    entry: &Entry,
+    logger_options: &LoggerOptions,
+    variables: &HashMap<String, Value>,
+) -> Result<Logger, runner::Error> {
+    let entry_verbosity =
+        options::get_entry_verbosity(entry, &logger_options.verbosity, variables)?;
     let entry_logger_options = LoggerOptionsBuilder::new()
         .color(logger_options.color)
         .filename(&logger_options.filename)
@@ -375,5 +485,32 @@ fn get_entry_logger(entry: &Entry, logger_options: &LoggerOptions) -> Logger {
         .verbosity(entry_verbosity)
         .test(logger_options.test)
         .build();
-    Logger::from(&entry_logger_options)
+    Ok(Logger::from(&entry_logger_options))
+}
+
+#[cfg(test)]
+mod test {
+    use super::*;
+    use crate::runner::RunnerOptionsBuilder;
+
+    #[test]
+    fn get_non_default_options_returns_empty_when_default() {
+        let options = RunnerOptions::default();
+        assert!(get_non_default_options(&options).is_empty());
+    }
+
+    #[test]
+    fn get_non_default_options_returns_only_non_default_options() {
+        let options = RunnerOptionsBuilder::new()
+            .delay(std::time::Duration::from_millis(500))
+            .build();
+
+        let non_default_options = get_non_default_options(&options);
+        assert_eq!(non_default_options.len(), 1);
+
+        let first_non_default = non_default_options.get(0).unwrap();
+
+        assert_eq!(first_non_default.0, "delay");
+        assert_eq!(first_non_default.1, "500ms");
+    }
 }
