@@ -18,20 +18,17 @@
 
 //! A runner for Hurl files. If you want to execute an Hurl file, this is the right place.
 
-pub use self::core::{AssertResult, CaptureResult, EntryResult, Error, HurlResult, RunnerError};
+pub use self::error::{Error, RunnerError};
 pub use self::hurl_file::run;
+pub use self::number::Number;
+pub use self::output::Output;
+pub use self::result::{AssertResult, CaptureResult, EntryResult, HurlResult};
 pub use self::runner_options::{RunnerOptions, RunnerOptionsBuilder};
 pub use self::value::Value;
 
-// We only expose specific structs from the module `http`.
-// We don't want to expose the whole internal HTTP client, but needs those structs as
-// they're exposed through `EntryResult`.
-pub use crate::http::{Call, Certificate, Cookie, Header, Request, Response, Timings, Version};
-
 mod assert;
 mod body;
 mod capture;
-mod core;
 mod entry;
 mod error;
 mod expr;
@@ -40,12 +37,16 @@ mod hurl_file;
 mod json;
 mod multiline;
 mod multipart;
+mod number;
+mod options;
+mod output;
 mod predicate;
 mod predicate_value;
 mod query;
 mod regex;
 mod request;
 mod response;
+mod result;
 mod runner_options;
 mod template;
 mod value;
