@@ -21,12 +21,10 @@ use hurl_core::ast::*;
 use regex::Regex;
 use sha2::Digest;
 
-use crate::runner::core::{Error, RunnerError};
-use crate::runner::filter;
+use crate::http;
+use crate::runner::error::{Error, RunnerError};
 use crate::runner::template::eval_template;
-use crate::runner::value::Value;
-
-use crate::{http, jsonpath};
+use crate::runner::{filter, Number, Value};
 
 pub type QueryResult = Result<Option<Value>, Error>;
 
@@ -37,28 +35,28 @@ pub fn eval_query(
     http_response: &http::Response,
 ) -> QueryResult {
     match query.value.clone() {
-        QueryValue::Status {} => eval_query_status(http_response),
-        QueryValue::Url {} => eval_query_url(http_response),
+        QueryValue::Status => eval_query_status(http_response),
+        QueryValue::Url => eval_query_url(http_response),
         QueryValue::Header { name, .. } => eval_query_header(http_response, &name, variables),
         QueryValue::Cookie {
             expr: CookiePath { name, attribute },
             ..
         } => eval_query_cookie(http_response, &name, &attribute, variables),
-        QueryValue::Body {} => eval_query_body(http_response, &query.source_info),
+        QueryValue::Body => eval_query_body(http_response, query.source_info),
         QueryValue::Xpath { expr, .. } => {
-            eval_query_xpath(http_response, &expr, variables, &query.source_info)
+            eval_query_xpath(http_response, &expr, variables, query.source_info)
         }
         QueryValue::Jsonpath { expr, .. } => {
-            eval_query_jsonpath(http_response, &expr, variables, &query.source_info)
+            eval_query_jsonpath(http_response, &expr, variables, query.source_info)
         }
         QueryValue::Regex { value, .. } => {
-            eval_query_regex(http_response, &value, variables, &query.source_info)
+            eval_query_regex(http_response, &value, variables, query.source_info)
         }
         QueryValue::Variable { name, .. } => eval_query_variable(&name, variables),
-        QueryValue::Duration {} => eval_query_duration(http_response),
-        QueryValue::Bytes {} => eval_query_bytes(http_response, &query.source_info),
-        QueryValue::Sha256 {} => eval_query_sha256(http_response, &query.source_info),
-        QueryValue::Md5 {} => eval_query_md5(http_response, &query.source_info),
+        QueryValue::Duration => eval_query_duration(http_response),
+        QueryValue::Bytes => eval_query_bytes(http_response, query.source_info),
+        QueryValue::Sha256 => eval_query_sha256(http_response, query.source_info),
+        QueryValue::Md5 => eval_query_md5(http_response, query.source_info),
         QueryValue::Certificate {
             attribute_name: field,
             ..
@@ -67,7 +65,9 @@ pub fn eval_query(
 }
 
 fn eval_query_status(response: &http::Response) -> QueryResult {
-    Ok(Some(Value::Integer(i64::from(response.status))))
+    Ok(Some(Value::Number(Number::Integer(i64::from(
+        response.status,
+    )))))
 }
 
 fn eval_query_url(response: &http::Response) -> QueryResult {
@@ -115,15 +115,11 @@ fn eval_query_cookie(
     }
 }
 
-fn eval_query_body(response: &http::Response, query_source_info: &SourceInfo) -> QueryResult {
+fn eval_query_body(response: &http::Response, query_source_info: SourceInfo) -> QueryResult {
     // Can return a string if encoding is known and utf8.
     match response.text() {
         Ok(s) => Ok(Some(Value::String(s))),
-        Err(inner) => Err(Error {
-            source_info: query_source_info.clone(),
-            inner: RunnerError::from(inner),
-            assert: false,
-        }),
+        Err(inner) => Err(Error::new(query_source_info, inner.into(), false)),
     }
 }
 
@@ -131,17 +127,13 @@ fn eval_query_xpath(
     response: &http::Response,
     expr: &Template,
     variables: &HashMap<String, Value>,
-    source_info: &SourceInfo,
+    query_source_info: SourceInfo,
 ) -> QueryResult {
     match response.text() {
-        Err(inner) => Err(Error {
-            source_info: source_info.clone(),
-            inner: RunnerError::from(inner),
-            assert: false,
-        }),
         Ok(xml) => {
-            filter::eval_xpath_string(&xml, expr, variables, source_info, response.is_html())
+            filter::eval_xpath_string(&xml, expr, variables, query_source_info, response.is_html())
         }
+        Err(inner) => Err(Error::new(query_source_info, inner.into(), false)),
     }
 }
 
@@ -149,48 +141,11 @@ fn eval_query_jsonpath(
     response: &http::Response,
     expr: &Template,
     variables: &HashMap<String, Value>,
-    query_source_info: &SourceInfo,
+    query_source_info: SourceInfo,
 ) -> QueryResult {
-    let value = eval_template(expr, variables)?;
-    let expr_source_info = &expr.source_info;
-    let jsonpath_query = match jsonpath::parse(value.as_str()) {
-        Ok(q) => q,
-        Err(_) => {
-            return Err(Error {
-                source_info: expr_source_info.clone(),
-                inner: RunnerError::QueryInvalidJsonpathExpression { value },
-                assert: false,
-            });
-        }
-    };
-    let json = match response.text() {
-        Err(inner) => {
-            return Err(Error {
-                source_info: query_source_info.clone(),
-                inner: RunnerError::from(inner),
-                assert: false,
-            });
-        }
-        Ok(v) => v,
-    };
-    let value = match serde_json::from_str(json.as_str()) {
-        Err(_) => {
-            return Err(Error {
-                source_info: query_source_info.clone(),
-                inner: RunnerError::QueryInvalidJson,
-                assert: false,
-            });
-        }
-        Ok(v) => v,
-    };
-
-    let results = jsonpath_query.eval(&value);
-    match results {
-        None => Ok(None),
-        Some(jsonpath::JsonpathResult::SingleEntry(value)) => Ok(Some(Value::from_json(&value))),
-        Some(jsonpath::JsonpathResult::Collection(values)) => {
-            Ok(Some(Value::from_json(&serde_json::Value::Array(values))))
-        }
+    match response.text() {
+        Ok(json) => filter::eval_jsonpath_string(&json, expr, variables, query_source_info),
+        Err(inner) => Err(Error::new(query_source_info, inner.into(), false)),
     }
 }
 
@@ -198,31 +153,18 @@ fn eval_query_regex(
     response: &http::Response,
     regex: &RegexValue,
     variables: &HashMap<String, Value>,
-    query_source_info: &SourceInfo,
+    query_source_info: SourceInfo,
 ) -> QueryResult {
     let s = match response.text() {
-        Err(inner) => {
-            return Err(Error {
-                source_info: query_source_info.clone(),
-                inner: RunnerError::from(inner),
-                assert: false,
-            });
-        }
         Ok(v) => v,
+        Err(inner) => return Err(Error::new(query_source_info, inner.into(), false)),
     };
     let re = match regex {
         RegexValue::Template(t) => {
             let value = eval_template(t, variables)?;
             match Regex::new(value.as_str()) {
                 Ok(re) => re,
-                Err(_) => {
-                    let source_info = t.source_info.clone();
-                    return Err(Error {
-                        source_info,
-                        inner: RunnerError::InvalidRegex(),
-                        assert: false,
-                    });
-                }
+                Err(_) => return Err(Error::new(t.source_info, RunnerError::InvalidRegex, false)),
             }
         }
         RegexValue::Regex(re) => re.inner.clone(),
@@ -246,29 +188,23 @@ fn eval_query_variable(name: &Template, variables: &HashMap<String, Value>) -> Q
 }
 
 fn eval_query_duration(response: &http::Response) -> QueryResult {
-    Ok(Some(Value::Integer(response.duration.as_millis() as i64)))
+    Ok(Some(Value::Number(Number::Integer(
+        response.duration.as_millis() as i64,
+    ))))
 }
 
-fn eval_query_bytes(response: &http::Response, query_source_info: &SourceInfo) -> QueryResult {
+fn eval_query_bytes(response: &http::Response, query_source_info: SourceInfo) -> QueryResult {
     match response.uncompress_body() {
         Ok(s) => Ok(Some(Value::Bytes(s))),
-        Err(inner) => Err(Error {
-            source_info: query_source_info.clone(),
-            inner: RunnerError::from(inner),
-            assert: false,
-        }),
+        Err(inner) => Err(Error::new(query_source_info, inner.into(), false)),
     }
 }
 
-fn eval_query_sha256(response: &http::Response, query_source_info: &SourceInfo) -> QueryResult {
+fn eval_query_sha256(response: &http::Response, query_source_info: SourceInfo) -> QueryResult {
     let bytes = match response.uncompress_body() {
         Ok(s) => s,
         Err(inner) => {
-            return Err(Error {
-                source_info: query_source_info.clone(),
-                inner: RunnerError::from(inner),
-                assert: false,
-            })
+            return Err(Error::new(query_source_info, inner.into(), false));
         }
     };
     let mut hasher = sha2::Sha256::new();
@@ -278,15 +214,11 @@ fn eval_query_sha256(response: &http::Response, query_source_info: &SourceInfo)
     Ok(Some(bytes))
 }
 
-fn eval_query_md5(response: &http::Response, query_source_info: &SourceInfo) -> QueryResult {
+fn eval_query_md5(response: &http::Response, query_source_info: SourceInfo) -> QueryResult {
     let bytes = match response.uncompress_body() {
         Ok(s) => s,
         Err(inner) => {
-            return Err(Error {
-                source_info: query_source_info.clone(),
-                inner: RunnerError::from(inner),
-                assert: false,
-            })
+            return Err(Error::new(query_source_info, inner.into(), false));
         }
     };
     let bytes = md5::compute(bytes).to_vec();
@@ -326,7 +258,9 @@ fn eval_cookie_attribute_name(
                 Err(_) => todo!(),
             }
         }
-        CookieAttributeName::MaxAge(_) => cookie.max_age().map(Value::Integer),
+        CookieAttributeName::MaxAge(_) => {
+            cookie.max_age().map(|v| Value::Number(Number::Integer(v)))
+        }
         CookieAttributeName::Domain(_) => cookie.domain().map(Value::String),
         CookieAttributeName::Path(_) => cookie.path().map(Value::String),
         CookieAttributeName::Secure(_) => {
@@ -354,9 +288,11 @@ impl Value {
             serde_json::Value::Bool(bool) => Value::Bool(*bool),
             serde_json::Value::Number(n) => {
                 if n.is_f64() {
-                    Value::from_f64(n.as_f64().unwrap())
+                    Value::Number(Number::from(n.as_f64().unwrap()))
+                } else if n.is_i64() {
+                    Value::Number(Number::from(n.as_i64().unwrap()))
                 } else {
-                    Value::Integer(n.as_i64().unwrap())
+                    Value::Number(Number::BigInteger(n.to_string()))
                 }
             }
             serde_json::Value::String(s) => Value::String(s.to_string()),
@@ -378,7 +314,6 @@ impl Value {
 #[cfg(test)]
 pub mod tests {
     use hex_literal::hex;
-
     use hurl_core::ast::{Pos, SourceInfo};
 
     use super::*;
@@ -387,10 +322,10 @@ pub mod tests {
         // xpath ???
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
         Query {
-            source_info: SourceInfo::new(1, 1, 1, 13),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
             value: QueryValue::Xpath {
                 space0: whitespace,
                 expr: Template {
@@ -399,7 +334,7 @@ pub mod tests {
                         value: "???".to_string(),
                         encoded: "???".to_string(),
                     }],
-                    source_info: SourceInfo::new(1, 7, 1, 10),
+                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10)),
                 },
             },
         }
@@ -408,10 +343,10 @@ pub mod tests {
     pub fn xpath_count_user_query() -> Query {
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
         Query {
-            source_info: SourceInfo::new(1, 1, 1, 13),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
             value: QueryValue::Xpath {
                 space0: whitespace,
                 expr: Template {
@@ -420,7 +355,7 @@ pub mod tests {
                         value: "count(//user)".to_string(),
                         encoded: "count(//user)".to_string(),
                     }],
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
             },
         }
@@ -429,10 +364,10 @@ pub mod tests {
     pub fn xpath_users() -> Query {
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
         Query {
-            source_info: SourceInfo::new(1, 1, 1, 13),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
             value: QueryValue::Xpath {
                 space0: whitespace,
                 expr: Template {
@@ -441,7 +376,7 @@ pub mod tests {
                         value: "//user".to_string(),
                         encoded: "/user".to_string(),
                     }],
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
             },
         }
@@ -450,11 +385,11 @@ pub mod tests {
     pub fn jsonpath_success() -> Query {
         // jsonpath $.success
         Query {
-            source_info: SourceInfo::new(1, 1, 1, 19),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 19)),
             value: QueryValue::Jsonpath {
                 space0: Whitespace {
-                    value: String::from(""),
-                    source_info: SourceInfo::new(1, 9, 1, 10),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -462,7 +397,7 @@ pub mod tests {
                         encoded: String::from("$.success"),
                     }],
                     delimiter: Some('"'),
-                    source_info: SourceInfo::new(1, 10, 1, 19),
+                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 19)),
                 },
             },
         }
@@ -471,11 +406,11 @@ pub mod tests {
     pub fn jsonpath_errors() -> Query {
         // jsonpath $.errors
         Query {
-            source_info: SourceInfo::new(1, 1, 1, 19),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 19)),
             value: QueryValue::Jsonpath {
                 space0: Whitespace {
-                    value: String::from(""),
-                    source_info: SourceInfo::new(1, 9, 1, 10),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -483,7 +418,7 @@ pub mod tests {
                         encoded: String::from("$.errors"),
                     }],
                     delimiter: Some('"'),
-                    source_info: SourceInfo::new(1, 10, 1, 18),
+                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 18)),
                 },
             },
         }
@@ -492,11 +427,11 @@ pub mod tests {
     pub fn jsonpath_duration() -> Query {
         // jsonpath $.errors
         Query {
-            source_info: SourceInfo::new(1, 1, 1, 19),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 19)),
             value: QueryValue::Jsonpath {
                 space0: Whitespace {
-                    value: String::from(""),
-                    source_info: SourceInfo::new(1, 9, 1, 10),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -504,7 +439,7 @@ pub mod tests {
                         encoded: String::from("$.duration"),
                     }],
                     delimiter: Some('"'),
-                    source_info: SourceInfo::new(1, 10, 1, 18),
+                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 18)),
                 },
             },
         }
@@ -513,11 +448,11 @@ pub mod tests {
     pub fn regex_name() -> Query {
         // regex "Hello ([a-zA-Z]+)!"
         Query {
-            source_info: SourceInfo::new(1, 1, 1, 26),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 26)),
             value: QueryValue::Regex {
                 space0: Whitespace {
-                    value: String::from(""),
-                    source_info: SourceInfo::new(1, 6, 1, 7),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
                 value: RegexValue::Template(Template {
                     delimiter: Some('"'),
@@ -525,7 +460,7 @@ pub mod tests {
                         value: "Hello ([a-zA-Z]+)!".to_string(),
                         encoded: "Hello ([a-zA-Z]+)!".to_string(),
                     }],
-                    source_info: SourceInfo::new(1, 7, 1, 26),
+                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 26)),
                 }),
             },
         }
@@ -534,11 +469,11 @@ pub mod tests {
     pub fn regex_invalid() -> Query {
         // regex ???"
         Query {
-            source_info: SourceInfo::new(1, 1, 1, 26),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 26)),
             value: QueryValue::Regex {
                 space0: Whitespace {
-                    value: String::from(""),
-                    source_info: SourceInfo::new(1, 6, 1, 7),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
                 value: RegexValue::Template(Template {
                     delimiter: Some('"'),
@@ -546,27 +481,56 @@ pub mod tests {
                         value: "???".to_string(),
                         encoded: "???".to_string(),
                     }],
-                    source_info: SourceInfo::new(1, 7, 1, 10),
+                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10)),
                 }),
             },
         }
     }
 
+    #[test]
+    pub fn value_from_json() {
+        let json_number: serde_json::Value = serde_json::from_str("1000").unwrap();
+        assert_eq!(
+            Value::from_json(&json_number),
+            Value::Number(Number::Integer(1000))
+        );
+
+        let json_number: serde_json::Value = serde_json::from_str("1.0").unwrap();
+        assert_eq!(
+            Value::from_json(&json_number),
+            Value::Number(Number::Float(1.0))
+        );
+
+        let json_number: serde_json::Value =
+            serde_json::from_str("1000000000000000000000").unwrap();
+        assert_eq!(
+            Value::from_json(&json_number),
+            Value::Number(Number::BigInteger("1000000000000000000000".to_string()))
+        );
+
+        let json_number: serde_json::Value =
+            serde_json::from_str("1000000000000000000000.5").unwrap();
+        assert_eq!(
+            Value::from_json(&json_number),
+            Value::Number(Number::Float(1000000000000000000000.5f64))
+        )
+    }
+
     #[test]
     fn test_query_status() {
         let variables = HashMap::new();
         assert_eq!(
             eval_query(
                 &Query {
-                    source_info: SourceInfo::new(0, 0, 0, 0),
-                    value: QueryValue::Status {},
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                    value: QueryValue::Status,
                 },
                 &variables,
                 &http::hello_http_response(),
             )
             .unwrap()
             .unwrap(),
-            Value::Integer(200)
+            Value::Number(Number::Integer(200))
         );
     }
 
@@ -575,11 +539,11 @@ pub mod tests {
         let variables = HashMap::new();
         // header Custom
         let query_header = Query {
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Header {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 7, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 8)),
                 },
                 name: Template {
                     delimiter: Some('"'),
@@ -587,7 +551,7 @@ pub mod tests {
                         value: "Custom".to_string(),
                         encoded: "Custom".to_string(),
                     }],
-                    source_info: SourceInfo::new(2, 8, 2, 14),
+                    source_info: SourceInfo::new(Pos::new(2, 8), Pos::new(2, 14)),
                 },
             },
         };
@@ -605,11 +569,11 @@ pub mod tests {
         // header Content-Type
         let variables = HashMap::new();
         let query_header = Query {
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Header {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 7, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 8)),
                 },
                 name: Template {
                     delimiter: Some('"'),
@@ -617,7 +581,7 @@ pub mod tests {
                         value: "Content-Type".to_string(),
                         encoded: "Content-Type".to_string(),
                     }],
-                    source_info: SourceInfo::new(1, 8, 1, 16),
+                    source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 16)),
                 },
             },
         };
@@ -633,8 +597,8 @@ pub mod tests {
     fn test_query_cookie() {
         let variables = HashMap::new();
         let space = Whitespace {
-            value: String::from(""),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            value: String::new(),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
         let response = http::Response {
             headers: vec![
@@ -645,7 +609,7 @@ pub mod tests {
 
         // cookie "LSID"
         let query = Query {
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
@@ -655,7 +619,7 @@ pub mod tests {
                             value: "LSID".to_string(),
                             encoded: "LSID".to_string(),
                         }],
-                        source_info: SourceInfo::new(0, 0, 0, 0),
+                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                     },
                     attribute: None,
                 },
@@ -668,7 +632,7 @@ pub mod tests {
 
         // cookie "LSID[Path]"
         let query = Query {
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
@@ -678,7 +642,7 @@ pub mod tests {
                             value: "LSID".to_string(),
                             encoded: "LSID".to_string(),
                         }],
-                        source_info: SourceInfo::new(0, 0, 0, 0),
+                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                     },
                     attribute: Some(CookieAttribute {
                         space0: space.clone(),
@@ -695,7 +659,7 @@ pub mod tests {
 
         // cookie "LSID[Secure]"
         let query = Query {
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
@@ -705,7 +669,7 @@ pub mod tests {
                             value: "LSID".to_string(),
                             encoded: "LSID".to_string(),
                         }],
-                        source_info: SourceInfo::new(0, 0, 0, 0),
+                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                     },
                     attribute: Some(CookieAttribute {
                         space0: space.clone(),
@@ -722,7 +686,7 @@ pub mod tests {
 
         // cookie "LSID[Domain]"
         let query = Query {
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
@@ -732,7 +696,7 @@ pub mod tests {
                             value: "LSID".to_string(),
                             encoded: "LSID".to_string(),
                         }],
-                        source_info: SourceInfo::new(0, 0, 0, 0),
+                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                     },
                     attribute: Some(CookieAttribute {
                         space0: space.clone(),
@@ -833,8 +797,8 @@ pub mod tests {
         assert_eq!(
             eval_query(
                 &Query {
-                    source_info: SourceInfo::new(0, 0, 0, 0),
-                    value: QueryValue::Body {},
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                    value: QueryValue::Body,
                 },
                 &variables,
                 &http::hello_http_response(),
@@ -845,15 +809,18 @@ pub mod tests {
         );
         let error = eval_query(
             &Query {
-                source_info: SourceInfo::new(1, 1, 1, 2),
-                value: QueryValue::Body {},
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 2)),
+                value: QueryValue::Body,
             },
             &variables,
             &http::bytes_http_response(),
         )
         .err()
         .unwrap();
-        assert_eq!(error.source_info, SourceInfo::new(1, 1, 1, 2));
+        assert_eq!(
+            error.source_info,
+            SourceInfo::new(Pos::new(1, 1), Pos::new(1, 2))
+        );
         assert_eq!(
             error.inner,
             RunnerError::InvalidDecoding {
@@ -886,11 +853,11 @@ pub mod tests {
         let variables = HashMap::new();
         // xpath ^^^
         let query = Query {
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Xpath {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 6, 1, 7),
+                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
                 expr: Template {
                     delimiter: Some('"'),
@@ -898,7 +865,7 @@ pub mod tests {
                         value: "^^^".to_string(),
                         encoded: "^^^".to_string(),
                     }],
-                    source_info: SourceInfo::new(1, 7, 1, 10),
+                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10)),
                 },
             },
         };
@@ -931,7 +898,7 @@ pub mod tests {
             )
             .unwrap()
             .unwrap(),
-            Value::Float(2.0)
+            Value::Number(Number::Float(2.0))
         );
     }
 
@@ -940,10 +907,10 @@ pub mod tests {
         // $x("normalize-space(/html/head/meta/@charset)")
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         };
         Query {
-            source_info: SourceInfo::new(1, 1, 1, 13),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
             value: QueryValue::Xpath {
                 space0: whitespace,
                 expr: Template {
@@ -952,7 +919,7 @@ pub mod tests {
                         value: "normalize-space(/html/head/meta/@charset)".to_string(),
                         encoded: "normalize-space(/html/head/meta/@charset)".to_string(),
                     }],
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
             },
         }
@@ -979,11 +946,11 @@ pub mod tests {
 
         // jsonpath xxx
         let jsonpath_query = Query {
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Jsonpath {
                 space0: Whitespace {
-                    value: String::from(""),
-                    source_info: SourceInfo::new(1, 9, 1, 10),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -991,7 +958,7 @@ pub mod tests {
                         encoded: String::from("xxx"),
                     }],
                     delimiter: Some('"'),
-                    source_info: SourceInfo::new(1, 10, 1, 13),
+                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 13)),
                 },
             },
         };
@@ -1081,8 +1048,11 @@ pub mod tests {
         let error = eval_query(&regex_invalid(), &variables, &http::hello_http_response())
             .err()
             .unwrap();
-        assert_eq!(error.source_info, SourceInfo::new(1, 7, 1, 10));
-        assert_eq!(error.inner, RunnerError::InvalidRegex());
+        assert_eq!(
+            error.source_info,
+            SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10))
+        );
+        assert_eq!(error.inner, RunnerError::InvalidRegex);
     }
 
     #[test]
@@ -1091,8 +1061,8 @@ pub mod tests {
         assert_eq!(
             eval_query(
                 &Query {
-                    source_info: SourceInfo::new(0, 0, 0, 0),
-                    value: QueryValue::Bytes {},
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                    value: QueryValue::Bytes,
                 },
                 &variables,
                 &http::hello_http_response(),
@@ -1109,7 +1079,7 @@ pub mod tests {
         assert_eq!(
             eval_query(
                 &Query {
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                     value: QueryValue::Sha256 {},
                 },
                 &variables,
@@ -1141,10 +1111,10 @@ pub mod tests {
                 &http::Response {
                     certificate: Some(http::Certificate {
                         subject: "A=B, C=D".to_string(),
-                        issuer: "".to_string(),
+                        issuer: String::new(),
                         start_date: Default::default(),
                         expire_date: Default::default(),
-                        serial_number: "".to_string()
+                        serial_number: String::new()
                     }),
                     ..Default::default()
                 },
