@@ -23,7 +23,7 @@ use hurl_core::ast::*;
 
 use crate::http;
 use crate::runner::body::eval_body;
-use crate::runner::core::Error;
+use crate::runner::error::Error;
 use crate::runner::multipart::eval_multipart_param;
 use crate::runner::template::eval_template;
 use crate::runner::value::Value;
@@ -41,16 +41,17 @@ pub fn eval_request(
     // Headers
     let mut headers: Vec<http::Header> = vec![];
     for header in &request.headers {
-        let name = &header.key.value;
+        let name = eval_template(&header.key, variables)?;
         let value = eval_template(&header.value, variables)?;
-        let header = http::Header::new(name, &value);
+        let header = http::Header::new(&name, &value);
         headers.push(header);
     }
 
     // Basic auth
     if let Some(kv) = &request.basic_auth() {
+        let name = eval_template(&kv.key, variables)?;
         let value = eval_template(&kv.value, variables)?;
-        let user_password = format!("{}:{}", kv.key.value, value);
+        let user_password = format!("{}:{}", name, value);
         let user_password = user_password.as_bytes();
         let authorization = general_purpose::STANDARD.encode(user_password);
         let value = format!("Basic {authorization}");
@@ -61,7 +62,7 @@ pub fn eval_request(
     // Query string params
     let mut querystring: Vec<http::Param> = vec![];
     for param in &request.querystring_params() {
-        let name = param.key.value.clone();
+        let name = eval_template(&param.key, variables)?;
         let value = eval_template(&param.value, variables)?;
         let param = http::Param { name, value };
         querystring.push(param);
@@ -70,7 +71,7 @@ pub fn eval_request(
     // Form params
     let mut form: Vec<http::Param> = vec![];
     for param in &request.form_params() {
-        let name = param.key.value.clone();
+        let name = eval_template(&param.key, variables)?;
         let value = eval_template(&param.value, variables)?;
         let param = http::Param { name, value };
         form.push(param);
@@ -79,8 +80,8 @@ pub fn eval_request(
     // Cookies
     let mut cookies = vec![];
     for cookie in &request.cookies() {
+        let name = eval_template(&cookie.name, variables)?;
         let value = eval_template(&cookie.value, variables)?;
-        let name = cookie.name.value.clone();
         let cookie = http::RequestCookie { name, value };
         cookies.push(cookie);
     }
@@ -168,13 +169,13 @@ fn eval_method(method: &Method) -> http::Method {
 mod tests {
     use hurl_core::ast::SourceInfo;
 
-    use super::super::core::RunnerError;
+    use super::super::error::RunnerError;
     use super::*;
 
     fn whitespace() -> Whitespace {
         Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         }
     }
 
@@ -195,7 +196,7 @@ mod tests {
                         space0: whitespace(),
                         variable: Variable {
                             name: String::from("base_url"),
-                            source_info: SourceInfo::new(1, 7, 1, 15),
+                            source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15)),
                         },
                         space1: whitespace(),
                     }),
@@ -205,17 +206,17 @@ mod tests {
                     },
                 ],
                 delimiter: None,
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             line_terminator0: line_terminator,
             headers: vec![],
             sections: vec![],
             body: None,
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         }
     }
 
-    fn simple_key_value(key: EncodedString, value: Template) -> KeyValue {
+    fn simple_key_value(key: Template, value: Template) -> KeyValue {
         let line_terminator = LineTerminator {
             space0: whitespace(),
             comment: None,
@@ -249,7 +250,7 @@ mod tests {
                     encoded: String::from("http://localhost:8000/querystring-params"),
                 }],
                 delimiter: None,
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             line_terminator0: line_terminator.clone(),
             headers: vec![],
@@ -259,11 +260,13 @@ mod tests {
                 line_terminator0: line_terminator,
                 value: SectionValue::QueryParams(vec![
                     simple_key_value(
-                        EncodedString {
-                            quotes: false,
-                            value: "param1".to_string(),
-                            encoded: "param1".to_string(),
-                            source_info: SourceInfo::new(0, 0, 0, 0),
+                        Template {
+                            delimiter: None,
+                            elements: vec![TemplateElement::String {
+                                value: "param1".to_string(),
+                                encoded: "param1".to_string(),
+                            }],
+                            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                         },
                         Template {
                             delimiter: None,
@@ -271,19 +274,21 @@ mod tests {
                                 space0: whitespace(),
                                 variable: Variable {
                                     name: String::from("param1"),
-                                    source_info: SourceInfo::new(1, 7, 1, 15),
+                                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15)),
                                 },
                                 space1: whitespace(),
                             })],
-                            source_info: SourceInfo::new(0, 0, 0, 0),
+                            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                         },
                     ),
                     simple_key_value(
-                        EncodedString {
-                            quotes: false,
-                            value: "param2".to_string(),
-                            encoded: "param2".to_string(),
-                            source_info: SourceInfo::new(0, 0, 0, 0),
+                        Template {
+                            delimiter: None,
+                            elements: vec![TemplateElement::String {
+                                value: "param2".to_string(),
+                                encoded: "param2".to_string(),
+                            }],
+                            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                         },
                         Template {
                             delimiter: None,
@@ -291,14 +296,14 @@ mod tests {
                                 value: "a b".to_string(),
                                 encoded: "a b".to_string(),
                             }],
-                            source_info: SourceInfo::new(0, 0, 0, 0),
+                            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                         },
                     ),
                 ]),
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             }],
             body: None,
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         }
     }
 
@@ -308,7 +313,10 @@ mod tests {
         let error = eval_request(&hello_request(), &variables, &ContextDir::default())
             .err()
             .unwrap();
-        assert_eq!(error.source_info, SourceInfo::new(1, 7, 1, 15));
+        assert_eq!(
+            error.source_info,
+            SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15))
+        );
         assert_eq!(
             error.inner,
             RunnerError::TemplateVariableNotDefined {
@@ -354,7 +362,8 @@ mod tests {
             line_terminators: vec![LineTerminator {
                 space0: whitespace(),
                 comment: Some(Comment {
-                    value: "@cookie_storage_clear".to_string()
+                    value: "@cookie_storage_clear".to_string(),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 }),
                 newline: whitespace(),
             }],
@@ -367,13 +376,13 @@ mod tests {
                     encoded: String::from("http://localhost"),
                 },],
                 delimiter: None,
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             line_terminator0: line_terminator,
             headers: vec![],
             sections: vec![],
             body: None,
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         }));
     }
 
@@ -394,7 +403,8 @@ mod tests {
                     comment: Some(Comment {
                         value:
                             "@cookie_storage_set: localhost\tFALSE\t/\tFALSE\t0\tcookie1\tvalueA"
-                                .to_string()
+                                .to_string(),
+                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                     }),
                     newline: whitespace(),
                 }],
@@ -407,13 +417,13 @@ mod tests {
                         encoded: String::from("http://localhost"),
                     },],
                     delimiter: None,
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
                 line_terminator0: line_terminator,
                 headers: vec![],
                 sections: vec![],
                 body: None,
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             })
         );
     }
