@@ -23,11 +23,12 @@ use crate::http;
 use crate::runner::assert::eval_assert;
 use crate::runner::body::eval_body;
 use crate::runner::capture::eval_capture;
-use crate::runner::core::*;
+use crate::runner::error::{Error, RunnerError};
 use crate::runner::json::eval_json_value;
 use crate::runner::multiline::eval_multiline;
+use crate::runner::result::{AssertResult, CaptureResult};
 use crate::runner::template::eval_template;
-use crate::runner::value::Value;
+use crate::runner::Value;
 use crate::util::path::ContextDir;
 
 /// Returns a list of assert results on the response status code and HTTP version,
@@ -42,7 +43,7 @@ pub fn eval_version_status_asserts(
     asserts.push(AssertResult::Version {
         actual: http_response.version.to_string(),
         expected: version.value.to_string(),
-        source_info: version.source_info.clone(),
+        source_info: version.source_info,
     });
 
     let status = &response.status;
@@ -50,7 +51,7 @@ pub fn eval_version_status_asserts(
         asserts.push(AssertResult::Status {
             actual: http_response.status as u64,
             expected: v,
-            source_info: status.source_info.clone(),
+            source_info: status.source_info,
         });
     }
     asserts
@@ -71,55 +72,70 @@ pub fn eval_asserts(
     for header in response.headers.iter() {
         match eval_template(&header.value, variables) {
             Err(e) => {
-                asserts.push(AssertResult::Header {
+                let result = AssertResult::Header {
                     actual: Err(e),
-                    expected: String::from(""),
-                    source_info: header.key.clone().source_info,
-                });
+                    expected: String::new(),
+                    source_info: header.key.source_info,
+                };
+                asserts.push(result);
             }
             Ok(expected) => {
-                let header_name = &header.key.value;
-                let actuals = http_response.get_header_values(header_name);
-                if actuals.is_empty() {
-                    asserts.push(AssertResult::Header {
-                        actual: Err(Error {
-                            source_info: header.key.source_info.clone(),
-                            inner: RunnerError::QueryHeaderNotFound {},
-                            assert: false,
-                        }),
-                        expected,
-                        source_info: header.key.source_info.clone(),
-                    });
-                } else if actuals.len() == 1 {
-                    let actual = actuals.first().unwrap().to_string();
-                    asserts.push(AssertResult::Header {
-                        actual: Ok(actual),
-                        expected,
-                        source_info: header.value.clone().source_info,
-                    });
-                } else {
-                    // failure by default
-                    // expected value not found in the list
-                    // actual is therefore the full list
-                    let mut actual = format!(
-                        "[{}]",
-                        actuals
-                            .iter()
-                            .map(|v| format!("\"{v}\""))
-                            .collect::<Vec<String>>()
-                            .join(", ")
-                    );
-                    for value in actuals {
-                        if value == expected {
-                            actual = value;
-                            break;
+                match eval_template(&header.key, variables) {
+                    Ok(header_name) => {
+                        let actuals = http_response.get_header_values(&header_name);
+                        if actuals.is_empty() {
+                            let result = AssertResult::Header {
+                                actual: Err(Error::new(
+                                    header.key.source_info,
+                                    RunnerError::QueryHeaderNotFound,
+                                    false,
+                                )),
+                                expected,
+                                source_info: header.key.source_info,
+                            };
+                            asserts.push(result);
+                        } else if actuals.len() == 1 {
+                            let actual = actuals.first().unwrap().to_string();
+                            let result = AssertResult::Header {
+                                actual: Ok(actual),
+                                expected,
+                                source_info: header.value.source_info,
+                            };
+                            asserts.push(result);
+                        } else {
+                            // failure by default
+                            // expected value not found in the list
+                            // actual is therefore the full list
+                            let mut actual = format!(
+                                "[{}]",
+                                actuals
+                                    .iter()
+                                    .map(|v| format!("\"{v}\""))
+                                    .collect::<Vec<String>>()
+                                    .join(", ")
+                            );
+                            for value in actuals {
+                                if value == expected {
+                                    actual = value;
+                                    break;
+                                }
+                            }
+                            let result = AssertResult::Header {
+                                actual: Ok(actual),
+                                expected,
+                                source_info: header.value.source_info,
+                            };
+                            asserts.push(result);
                         }
                     }
-                    asserts.push(AssertResult::Header {
-                        actual: Ok(actual),
-                        expected,
-                        source_info: header.value.clone().source_info,
-                    });
+                    Err(e) => {
+                        let result = AssertResult::Header {
+                            actual: Err(e),
+                            expected,
+                            source_info: header.value.source_info,
+                        };
+                        asserts.push(result);
+                    }
                 }
             }
         }
@@ -131,7 +147,7 @@ pub fn eval_asserts(
     }
 
     for assert in response.asserts().iter() {
-        let assert_result = eval_assert(assert, variables, http_response);
+        let assert_result = eval_assert(assert, variables, http_response, context_dir);
         asserts.push(assert_result);
     }
     asserts
@@ -152,38 +168,36 @@ fn eval_implicit_body_asserts(
             };
             let actual = match http_response.text() {
                 Ok(s) => Ok(Value::String(s)),
-                Err(e) => Err(Error {
-                    source_info: SourceInfo {
-                        start: spec_body.space0.source_info.end.clone(),
-                        end: spec_body.space0.source_info.end.clone(),
-                    },
-                    inner: RunnerError::from(e),
-                    assert: true,
-                }),
+                Err(e) => {
+                    let source_info = SourceInfo {
+                        start: spec_body.space0.source_info.end,
+                        end: spec_body.space0.source_info.end,
+                    };
+                    Err(Error::new(source_info, e.into(), true))
+                }
             };
             AssertResult::Body {
                 actual,
                 expected,
-                source_info: spec_body.space0.source_info.clone(),
+                source_info: spec_body.space0.source_info,
             }
         }
         Bytes::Xml(value) => {
             let expected = Ok(Value::String(value.to_string()));
             let actual = match http_response.text() {
                 Ok(s) => Ok(Value::String(s)),
-                Err(e) => Err(Error {
-                    source_info: SourceInfo {
-                        start: spec_body.space0.source_info.end.clone(),
-                        end: spec_body.space0.source_info.end.clone(),
-                    },
-                    inner: RunnerError::from(e),
-                    assert: true,
-                }),
+                Err(e) => {
+                    let source_info = SourceInfo {
+                        start: spec_body.space0.source_info.end,
+                        end: spec_body.space0.source_info.end,
+                    };
+                    Err(Error::new(source_info, e.into(), true))
+                }
             };
             AssertResult::Body {
                 actual,
                 expected,
-                source_info: spec_body.space0.source_info.clone(),
+                source_info: spec_body.space0.source_info,
             }
         }
         Bytes::OnelineString(value) => {
@@ -193,19 +207,18 @@ fn eval_implicit_body_asserts(
             };
             let actual = match http_response.text() {
                 Ok(s) => Ok(Value::String(s)),
-                Err(e) => Err(Error {
-                    source_info: SourceInfo {
-                        start: spec_body.space0.source_info.end.clone(),
-                        end: spec_body.space0.source_info.end.clone(),
-                    },
-                    inner: RunnerError::from(e),
-                    assert: true,
-                }),
+                Err(e) => {
+                    let source_info = SourceInfo {
+                        start: spec_body.space0.source_info.end,
+                        end: spec_body.space0.source_info.end,
+                    };
+                    Err(Error::new(source_info, e.into(), true))
+                }
             };
             AssertResult::Body {
                 actual,
                 expected,
-                source_info: value.source_info.clone(),
+                source_info: value.source_info,
             }
         }
         Bytes::MultilineString(multi) => {
@@ -215,14 +228,13 @@ fn eval_implicit_body_asserts(
             };
             let actual = match http_response.text() {
                 Ok(s) => Ok(Value::String(s)),
-                Err(e) => Err(Error {
-                    source_info: SourceInfo {
-                        start: spec_body.space0.source_info.end.clone(),
-                        end: spec_body.space0.source_info.end.clone(),
-                    },
-                    inner: RunnerError::from(e),
-                    assert: true,
-                }),
+                Err(e) => {
+                    let source_info = SourceInfo {
+                        start: spec_body.space0.source_info.end,
+                        end: spec_body.space0.source_info.end,
+                    };
+                    Err(Error::new(source_info, e.into(), true))
+                }
             };
             AssertResult::Body {
                 actual,
@@ -239,21 +251,20 @@ fn eval_implicit_body_asserts(
             let expected = Ok(Value::Bytes(value.to_vec()));
             let actual = match http_response.uncompress_body() {
                 Ok(b) => Ok(Value::Bytes(b)),
-                Err(e) => Err(Error {
-                    source_info: SourceInfo {
-                        start: spec_body.space0.source_info.end.clone(),
-                        end: spec_body.space0.source_info.end.clone(),
-                    },
-                    inner: RunnerError::from(e),
-                    assert: true,
-                }),
+                Err(e) => {
+                    let source_info = SourceInfo {
+                        start: spec_body.space0.source_info.end,
+                        end: spec_body.space0.source_info.end,
+                    };
+                    Err(Error::new(source_info, e.into(), true))
+                }
             };
             AssertResult::Body {
                 actual,
                 expected,
                 source_info: SourceInfo {
-                    start: space0.source_info.end.clone(),
-                    end: space1.source_info.start.clone(),
+                    start: space0.source_info.end,
+                    end: space1.source_info.start,
                 },
             }
         }
@@ -266,21 +277,20 @@ fn eval_implicit_body_asserts(
             let expected = Ok(Value::Bytes(value.to_vec()));
             let actual = match http_response.uncompress_body() {
                 Ok(b) => Ok(Value::Bytes(b)),
-                Err(e) => Err(Error {
-                    source_info: SourceInfo {
-                        start: spec_body.space0.source_info.end.clone(),
-                        end: spec_body.space0.source_info.end.clone(),
-                    },
-                    inner: RunnerError::from(e),
-                    assert: true,
-                }),
+                Err(e) => {
+                    let source_info = SourceInfo {
+                        start: spec_body.space0.source_info.end,
+                        end: spec_body.space0.source_info.end,
+                    };
+                    Err(Error::new(source_info, e.into(), true))
+                }
             };
             AssertResult::Body {
                 actual,
                 expected,
                 source_info: SourceInfo {
-                    start: space0.source_info.end.clone(),
-                    end: space1.source_info.start.clone(),
+                    start: space0.source_info.end,
+                    end: space1.source_info.start,
                 },
             }
         }
@@ -291,19 +301,18 @@ fn eval_implicit_body_asserts(
             };
             let actual = match http_response.uncompress_body() {
                 Ok(b) => Ok(Value::Bytes(b)),
-                Err(e) => Err(Error {
-                    source_info: SourceInfo {
-                        start: spec_body.space0.source_info.end.clone(),
-                        end: spec_body.space0.source_info.end.clone(),
-                    },
-                    inner: RunnerError::from(e),
-                    assert: true,
-                }),
+                Err(e) => {
+                    let source_info = SourceInfo {
+                        start: spec_body.space0.source_info.end,
+                        end: spec_body.space0.source_info.end,
+                    };
+                    Err(Error::new(source_info, e.into(), true))
+                }
             };
             AssertResult::Body {
                 actual,
                 expected,
-                source_info: spec_body.space0.source_info.clone(),
+                source_info: spec_body.space0.source_info,
             }
         }
     }
@@ -330,11 +339,12 @@ pub fn eval_captures(
 mod tests {
     use self::super::super::{assert, capture};
     use super::*;
+    use crate::runner::Number;
 
     pub fn user_response() -> Response {
         let whitespace = Whitespace {
             value: String::from(" "),
-            source_info: SourceInfo::new(1, 1, 1, 1),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
         };
         let line_terminator = LineTerminator {
             space0: whitespace.clone(),
@@ -346,12 +356,12 @@ mod tests {
             line_terminators: vec![],
             version: Version {
                 value: VersionValue::Version1,
-                source_info: SourceInfo::new(2, 1, 2, 9),
+                source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 9)),
             },
             space0: whitespace.clone(),
             status: Status {
                 value: StatusValue::Specific(200),
-                source_info: SourceInfo::new(2, 10, 2, 13),
+                source_info: SourceInfo::new(Pos::new(2, 10), Pos::new(2, 13)),
             },
             space1: whitespace.clone(),
             line_terminator0: line_terminator.clone(),
@@ -362,18 +372,18 @@ mod tests {
                     space0: whitespace.clone(),
                     line_terminator0: line_terminator.clone(),
                     value: SectionValue::Asserts(vec![assert::tests::assert_count_user()]),
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
                 Section {
                     line_terminators: vec![],
                     space0: whitespace,
                     line_terminator0: line_terminator,
                     value: SectionValue::Captures(vec![capture::tests::user_count_capture()]),
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
             ],
             body: None,
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         }
     }
 
@@ -389,17 +399,17 @@ mod tests {
                 &context_dir,
             ),
             vec![AssertResult::Explicit {
-                actual: Ok(Some(Value::Nodeset(2))),
-                source_info: SourceInfo::new(1, 14, 1, 27),
-                predicate_result: Some(Err(Error {
-                    source_info: SourceInfo::new(1, 0, 1, 0),
-                    inner: RunnerError::AssertFailure {
-                        actual: "2".to_string(),
-                        expected: "3".to_string(),
+                actual: Ok(Some(Value::Number(Number::Integer(2)))),
+                source_info: SourceInfo::new(Pos::new(1, 22), Pos::new(1, 24)),
+                predicate_result: Some(Err(Error::new(
+                    SourceInfo::new(Pos::new(1, 0), Pos::new(1, 0)),
+                    RunnerError::AssertFailure {
+                        actual: "int <2>".to_string(),
+                        expected: "int <3>".to_string(),
                         type_mismatch: false,
                     },
-                    assert: true,
-                })),
+                    true
+                ))),
             },]
         );
     }
@@ -412,12 +422,12 @@ mod tests {
                 AssertResult::Version {
                     actual: String::from("HTTP/1.0"),
                     expected: String::from("HTTP/1.0"),
-                    source_info: SourceInfo::new(2, 1, 2, 9),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 9)),
                 },
                 AssertResult::Status {
                     actual: 200,
                     expected: 200,
-                    source_info: SourceInfo::new(2, 10, 2, 13),
+                    source_info: SourceInfo::new(Pos::new(2, 10), Pos::new(2, 13)),
                 },
             ]
         );
@@ -435,7 +445,7 @@ mod tests {
             .unwrap(),
             vec![CaptureResult {
                 name: "UserCount".to_string(),
-                value: Value::Float(2.0),
+                value: Value::Number(Number::Float(2.0)),
             }]
         );
     }
