@@ -19,9 +19,11 @@ use std::time::Duration;
 
 use hurl_core::ast::{Entry, Retry};
 
+use crate::http::{IpResolve, RequestedHttpVersion};
 use crate::util::path::ContextDir;
 
 pub struct RunnerOptionsBuilder {
+    aws_sigv4: Option<String>,
     cacert_file: Option<String>,
     client_cert_file: Option<String>,
     client_key_file: Option<String>,
@@ -29,13 +31,18 @@ pub struct RunnerOptionsBuilder {
     connect_timeout: Duration,
     connects_to: Vec<String>,
     context_dir: ContextDir,
+    continue_on_error: bool,
     cookie_input_file: Option<String>,
-    fail_fast: bool,
+    delay: Duration,
     follow_location: bool,
+    follow_location_trusted: bool,
+    http_version: RequestedHttpVersion,
     ignore_asserts: bool,
     insecure: bool,
+    ip_resolve: IpResolve,
     max_redirect: Option<usize>,
     no_proxy: Option<String>,
+    output: Option<String>,
     path_as_is: bool,
     post_entry: Option<fn() -> bool>,
     pre_entry: Option<fn(Entry) -> bool>,
@@ -43,9 +50,11 @@ pub struct RunnerOptionsBuilder {
     resolves: Vec<String>,
     retry: Retry,
     retry_interval: Duration,
+    skip: bool,
     ssl_no_revoke: bool,
     timeout: Duration,
     to_entry: Option<usize>,
+    unix_socket: Option<String>,
     user: Option<String>,
     user_agent: Option<String>,
 }
@@ -53,6 +62,7 @@ pub struct RunnerOptionsBuilder {
 impl Default for RunnerOptionsBuilder {
     fn default() -> Self {
         RunnerOptionsBuilder {
+            aws_sigv4: None,
             cacert_file: None,
             client_cert_file: None,
             client_key_file: None,
@@ -60,13 +70,18 @@ impl Default for RunnerOptionsBuilder {
             connect_timeout: Duration::from_secs(300),
             connects_to: vec![],
             context_dir: ContextDir::default(),
+            continue_on_error: false,
             cookie_input_file: None,
-            fail_fast: true,
+            delay: Duration::from_millis(0),
             follow_location: false,
+            follow_location_trusted: false,
+            http_version: RequestedHttpVersion::default(),
             ignore_asserts: false,
             insecure: false,
+            ip_resolve: IpResolve::default(),
             max_redirect: Some(50),
             no_proxy: None,
+            output: None,
             path_as_is: false,
             post_entry: None,
             pre_entry: None,
@@ -74,9 +89,11 @@ impl Default for RunnerOptionsBuilder {
             resolves: vec![],
             retry: Retry::None,
             retry_interval: Duration::from_millis(1000),
+            skip: false,
             ssl_no_revoke: false,
             timeout: Duration::from_secs(300),
             to_entry: None,
+            unix_socket: None,
             user: None,
             user_agent: None,
         }
@@ -89,6 +106,12 @@ impl RunnerOptionsBuilder {
         RunnerOptionsBuilder::default()
     }
 
+    /// Specifies the AWS SigV4 option
+    pub fn aws_sigv4(&mut self, aws_sigv4: Option<String>) -> &mut Self {
+        self.aws_sigv4 = aws_sigv4;
+        self
+    }
+
     /// Specifies the certificate file for peer verification.
     /// The file may contain multiple CA certificates and must be in PEM format.
     pub fn cacert_file(&mut self, cacert_file: Option<String>) -> &mut Self {
@@ -132,6 +155,14 @@ impl RunnerOptionsBuilder {
         self
     }
 
+    /// Sets delay (timeout) before the request.
+    ///
+    /// Default is 0 ms.
+    pub fn delay(&mut self, delay: Duration) -> &mut Self {
+        self.delay = delay;
+        self
+    }
+
     /// Sets root file system to import files in Hurl.
     ///
     /// This is used for both files in multipart form data and request body.
@@ -140,6 +171,15 @@ impl RunnerOptionsBuilder {
         self
     }
 
+    /// Sets stopping or continuing executing requests to the end of the Hurl file even when an error occurs.
+    ///
+    /// By default, Hurl exits after an error in the HTTP response. Note that this option does
+    /// not affect the behavior with multiple input Hurl files.
+    pub fn continue_on_error(&mut self, continue_on_error: bool) -> &mut Self {
+        self.continue_on_error = continue_on_error;
+        self
+    }
+
     /// Reads cookies from this file (using the Netscape cookie file format).
     pub fn cookie_input_file(&mut self, cookie_input_file: Option<String>) -> &mut Self {
         self.cookie_input_file = cookie_input_file;
@@ -151,7 +191,7 @@ impl RunnerOptionsBuilder {
     /// By default, Hurl exits after an assert error in the HTTP response. Note that this option does
     /// not affect the behavior with multiple input Hurl files.
     pub fn fail_fast(&mut self, fail_fast: bool) -> &mut Self {
-        self.fail_fast = fail_fast;
+        self.continue_on_error = !fail_fast;
         self
     }
 
@@ -163,6 +203,20 @@ impl RunnerOptionsBuilder {
         self
     }
 
+    /// Sets follow redirect with trust.
+    ///
+    /// To limit the amount of redirects to follow use [`self.max_redirect()`]
+    pub fn follow_location_trusted(&mut self, follow_location_trusted: bool) -> &mut Self {
+        self.follow_location_trusted = follow_location_trusted;
+        self
+    }
+
+    /// Set requested HTTP version (can be different of the effective HTTP version).
+    pub fn http_version(&mut self, version: RequestedHttpVersion) -> &mut Self {
+        self.http_version = version;
+        self
+    }
+
     /// Ignores all asserts defined in the Hurl file.
     pub fn ignore_asserts(&mut self, ignore_asserts: bool) -> &mut Self {
         self.ignore_asserts = ignore_asserts;
@@ -175,6 +229,12 @@ impl RunnerOptionsBuilder {
         self
     }
 
+    /// Set IP version.
+    pub fn ip_resolve(&mut self, ip_resolve: IpResolve) -> &mut Self {
+        self.ip_resolve = ip_resolve;
+        self
+    }
+
     /// Set maximum number of redirection-followings allowed
     ///
     /// By default, the limit is set to 50 redirections
@@ -195,6 +255,12 @@ impl RunnerOptionsBuilder {
         self
     }
 
+    /// Specifies the file to output the HTTP response instead of stdout.
+    pub fn output(&mut self, output: Option<String>) -> &mut Self {
+        self.output = output;
+        self
+    }
+
     /// Sets function to be executed after each entry execution.
     ///
     /// If the function returns true, the run is stopped.
@@ -231,7 +297,7 @@ impl RunnerOptionsBuilder {
         self
     }
 
-    /// Sets suration between each retry.
+    /// Sets duration between each retry.
     ///
     /// Default is 1000 ms.
     pub fn retry_interval(&mut self, retry_interval: Duration) -> &mut Self {
@@ -258,6 +324,12 @@ impl RunnerOptionsBuilder {
         self
     }
 
+    /// Sets the specified unix domain socket to connect through, instead of using the network.
+    pub fn unix_socket(&mut self, unix_socket: Option<String>) -> &mut Self {
+        self.unix_socket = unix_socket;
+        self
+    }
+
     /// Adds basic Authentication header to each request.
     pub fn user(&mut self, user: Option<String>) -> &mut Self {
         self.user = user;
@@ -273,20 +345,26 @@ impl RunnerOptionsBuilder {
     /// Create an instance of [`RunnerOptions`].
     pub fn build(&self) -> RunnerOptions {
         RunnerOptions {
+            aws_sigv4: self.aws_sigv4.clone(),
             cacert_file: self.cacert_file.clone(),
             client_cert_file: self.client_cert_file.clone(),
             client_key_file: self.client_key_file.clone(),
             compressed: self.compressed,
             connect_timeout: self.connect_timeout,
             connects_to: self.connects_to.clone(),
+            delay: self.delay,
             context_dir: self.context_dir.clone(),
+            continue_on_error: self.continue_on_error,
             cookie_input_file: self.cookie_input_file.clone(),
-            fail_fast: self.fail_fast,
             follow_location: self.follow_location,
+            follow_location_trusted: self.follow_location_trusted,
+            http_version: self.http_version,
             ignore_asserts: self.ignore_asserts,
             insecure: self.insecure,
+            ip_resolve: self.ip_resolve,
             max_redirect: self.max_redirect,
             no_proxy: self.no_proxy.clone(),
+            output: self.output.clone(),
             path_as_is: self.path_as_is,
             post_entry: self.post_entry,
             pre_entry: self.pre_entry,
@@ -294,9 +372,11 @@ impl RunnerOptionsBuilder {
             resolves: self.resolves.clone(),
             retry: self.retry,
             retry_interval: self.retry_interval,
+            skip: self.skip,
             ssl_no_revoke: self.ssl_no_revoke,
             timeout: self.timeout,
             to_entry: self.to_entry,
+            unix_socket: self.unix_socket.clone(),
             user: self.user.clone(),
             user_agent: self.user_agent.clone(),
         }
@@ -305,20 +385,26 @@ impl RunnerOptionsBuilder {
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct RunnerOptions {
+    pub(crate) aws_sigv4: Option<String>,
     pub(crate) cacert_file: Option<String>,
     pub(crate) client_cert_file: Option<String>,
     pub(crate) client_key_file: Option<String>,
     pub(crate) compressed: bool,
     pub(crate) connect_timeout: Duration,
     pub(crate) connects_to: Vec<String>,
+    pub(crate) delay: Duration,
     pub(crate) context_dir: ContextDir,
+    pub(crate) continue_on_error: bool,
     pub(crate) cookie_input_file: Option<String>,
-    pub(crate) fail_fast: bool,
     pub(crate) follow_location: bool,
+    pub(crate) follow_location_trusted: bool,
+    pub(crate) http_version: RequestedHttpVersion,
     pub(crate) ignore_asserts: bool,
+    pub(crate) ip_resolve: IpResolve,
     pub(crate) insecure: bool,
     pub(crate) max_redirect: Option<usize>,
     pub(crate) no_proxy: Option<String>,
+    pub(crate) output: Option<String>,
     pub(crate) path_as_is: bool,
     pub(crate) post_entry: Option<fn() -> bool>,
     pub(crate) pre_entry: Option<fn(Entry) -> bool>,
@@ -326,9 +412,11 @@ pub struct RunnerOptions {
     pub(crate) resolves: Vec<String>,
     pub(crate) retry: Retry,
     pub(crate) retry_interval: Duration,
+    pub(crate) skip: bool,
     pub(crate) ssl_no_revoke: bool,
     pub(crate) timeout: Duration,
     pub(crate) to_entry: Option<usize>,
+    pub(crate) unix_socket: Option<String>,
     pub(crate) user: Option<String>,
     pub(crate) user_agent: Option<String>,
 }
