@@ -19,8 +19,8 @@ use std::collections::HashMap;
 
 use hurl_core::ast::*;
 
-use crate::runner::core::{Error, RunnerError};
-use crate::runner::value::Value;
+use crate::runner::error::{Error, RunnerError};
+use crate::runner::Value;
 
 /// Renders to string a `template` given a map of variables.
 pub fn eval_template(
@@ -28,7 +28,7 @@ pub fn eval_template(
     variables: &HashMap<String, Value>,
 ) -> Result<String, Error> {
     let Template { elements, .. } = template;
-    let mut value = String::from("");
+    let mut value = String::new();
     for elem in elements {
         match eval_template_element(elem, variables) {
             Ok(v) => value.push_str(v.as_str()),
@@ -44,33 +44,34 @@ fn eval_template_element(
 ) -> Result<String, Error> {
     match template_element {
         TemplateElement::String { value, .. } => Ok(value.clone()),
-        TemplateElement::Expression(expr) => eval_expression(expr, variables),
+        TemplateElement::Expression(expr) => render_expression(expr, variables),
     }
 }
 
-pub fn eval_expression(expr: &Expr, variables: &HashMap<String, Value>) -> Result<String, Error> {
-    let source_info = &expr.variable.source_info;
+pub fn render_expression(expr: &Expr, variables: &HashMap<String, Value>) -> Result<String, Error> {
+    let source_info = expr.variable.source_info;
+    let name = &expr.variable.name;
+    let value = eval_expression(expr, variables)?;
+    if value.is_renderable() {
+        Ok(value.to_string())
+    } else {
+        let inner = RunnerError::UnrenderableVariable {
+            name: name.to_string(),
+            value: value.to_string(),
+        };
+        Err(Error::new(source_info, inner, false))
+    }
+}
+
+pub fn eval_expression(expr: &Expr, variables: &HashMap<String, Value>) -> Result<Value, Error> {
+    let source_info = expr.variable.source_info;
     let name = &expr.variable.name;
     match variables.get(name.as_str()) {
-        Some(value) => {
-            if value.is_renderable() {
-                Ok(value.clone().to_string())
-            } else {
-                Err(Error {
-                    source_info: source_info.clone(),
-                    inner: RunnerError::UnrenderableVariable {
-                        name: name.to_string(),
-                        value: value.to_string(),
-                    },
-                    assert: false,
-                })
-            }
+        Some(value) => Ok(value.clone()),
+        _ => {
+            let inner = RunnerError::TemplateVariableNotDefined { name: name.clone() };
+            Err(Error::new(source_info, inner, false))
         }
-        _ => Err(Error {
-            source_info: source_info.clone(),
-            inner: RunnerError::TemplateVariableNotDefined { name: name.clone() },
-            assert: false,
-        }),
     }
 }
 
@@ -78,7 +79,7 @@ impl Value {
     pub fn is_renderable(&self) -> bool {
         matches!(
             self,
-            Value::Integer(_) | Value::Bool(_) | Value::Float(_) | Value::String(_) | Value::Null
+            Value::Number(_) | Value::Bool(_) | Value::String(_) | Value::Null
         )
     }
 }
@@ -88,21 +89,22 @@ mod tests {
     use hurl_core::ast::SourceInfo;
 
     use super::*;
+    use crate::runner::Number;
 
     fn template_element_expression() -> TemplateElement {
         // {{name}}
         TemplateElement::Expression(Expr {
             space0: Whitespace {
-                value: "".to_string(),
-                source_info: SourceInfo::new(1, 3, 1, 3),
+                value: String::new(),
+                source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 3)),
             },
             variable: Variable {
                 name: "name".to_string(),
-                source_info: SourceInfo::new(1, 3, 1, 7),
+                source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 7)),
             },
             space1: Whitespace {
-                value: "".to_string(),
-                source_info: SourceInfo::new(1, 7, 1, 7),
+                value: String::new(),
+                source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 7)),
             },
         })
     }
@@ -135,12 +137,18 @@ mod tests {
         let mut variables = HashMap::new();
         variables.insert(
             "name".to_string(),
-            Value::List(vec![Value::Integer(1), Value::Integer(2)]),
+            Value::List(vec![
+                Value::Number(Number::Integer(1)),
+                Value::Number(Number::Integer(2)),
+            ]),
         );
         let error = eval_template_element(&template_element_expression(), &variables)
             .err()
             .unwrap();
-        assert_eq!(error.source_info, SourceInfo::new(1, 3, 1, 7));
+        assert_eq!(
+            error.source_info,
+            SourceInfo::new(Pos::new(1, 3), Pos::new(1, 7))
+        );
         assert_eq!(
             error.inner,
             RunnerError::UnrenderableVariable {
@@ -149,4 +157,29 @@ mod tests {
             }
         );
     }
+
+    #[test]
+    fn test_render_expression() {
+        let mut variables = HashMap::new();
+        variables.insert("status".to_string(), Value::Bool(true));
+        let expr = Expr {
+            space0: Whitespace {
+                value: "".to_string(),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+            },
+            variable: Variable {
+                name: "status".to_string(),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+            },
+            space1: Whitespace {
+                value: "".to_string(),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+            },
+        };
+        assert_eq!(
+            eval_expression(&expr, &variables).unwrap(),
+            Value::Bool(true)
+        );
+        assert_eq!(render_expression(&expr, &variables).unwrap(), "true");
+    }
 }
