@@ -24,7 +24,7 @@ use libxml::bindings::{htmlReadMemory, xmlReadMemory};
 use libxml::parser::{ParseFormat, Parser, XmlParseError};
 use libxml::tree::Document;
 
-use crate::runner::value::Value;
+use crate::runner::{Number, Value};
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum XpathError {
@@ -40,12 +40,12 @@ pub fn eval_xml(xml: &str, expr: &str) -> Result<Value, XpathError> {
     match parse_html_string_patched(xml, &parser) {
         Ok(doc) => {
             if doc.get_root_element().is_none() {
-                Err(XpathError::InvalidXml {})
+                Err(XpathError::InvalidXml)
             } else {
                 eval(&doc, expr, true)
             }
         }
-        Err(_) => Err(XpathError::InvalidXml {}),
+        Err(_) => Err(XpathError::InvalidXml),
     }
 }
 
@@ -57,12 +57,12 @@ pub fn eval_html(html: &str, expr: &str) -> Result<Value, XpathError> {
             // You can have a doc structure even if the input xml is not valid
             // check that the root element exists
             if doc.get_root_element().is_none() {
-                Err(XpathError::InvalidHtml {})
+                Err(XpathError::InvalidHtml)
             } else {
                 eval(&doc, expr, false)
             }
         }
-        Err(_) => Err(XpathError::InvalidHtml {}),
+        Err(_) => Err(XpathError::InvalidHtml),
     }
 }
 
@@ -157,12 +157,12 @@ fn eval(doc: &Document, expr: &str, support_ns: bool) -> Result<Value, XpathErro
 
     let result = match context.evaluate(expr) {
         Ok(object) => object,
-        Err(_) => return Err(XpathError::Eval {}),
+        Err(_) => return Err(XpathError::Eval),
     };
 
     match unsafe { *result.ptr }.type_ {
         libxml::bindings::xmlXPathObjectType_XPATH_NUMBER => {
-            Ok(Value::from_f64(unsafe { *result.ptr }.floatval))
+            Ok(Value::Number(Number::from(unsafe { *result.ptr }.floatval)))
         }
         libxml::bindings::xmlXPathObjectType_XPATH_BOOLEAN => {
             Ok(Value::Bool(unsafe { *result.ptr }.boolval != 0))
@@ -180,7 +180,7 @@ fn eval(doc: &Document, expr: &str, support_ns: bool) -> Result<Value, XpathErro
         libxml::bindings::xmlXPathObjectType_XPATH_NODESET => {
             Ok(Value::Nodeset(result.get_number_of_nodes()))
         }
-        _ => Err(XpathError::Unsupported {}),
+        _ => Err(XpathError::Unsupported),
     }
 }
 
@@ -253,22 +253,31 @@ mod tests {
 </food>
 "#;
         let xpath = "count(//food/*)";
-        assert_eq!(eval_xml(xml, xpath).unwrap(), Value::from_f64(3.0));
+        assert_eq!(
+            eval_xml(xml, xpath).unwrap(),
+            Value::Number(Number::from(3.0))
+        );
 
         let xpath = "//food/*";
         assert_eq!(eval_xml(xml, xpath).unwrap(), Value::Nodeset(3));
 
         let xpath = "count(//*[@type='fruit'])";
-        assert_eq!(eval_xml(xml, xpath).unwrap(), Value::from_f64(2.0));
+        assert_eq!(
+            eval_xml(xml, xpath).unwrap(),
+            Value::Number(Number::from(2.0))
+        );
 
         let xpath = "number(//food/banana/@price)";
-        assert_eq!(eval_xml(xml, xpath).unwrap(), Value::from_f64(1.1));
+        assert_eq!(
+            eval_xml(xml, xpath).unwrap(),
+            Value::Number(Number::from(1.1))
+        );
     }
 
     #[test]
     fn test_error_eval() {
-        assert_eq!(eval_xml("<a/>", "^^^").err().unwrap(), XpathError::Eval {});
-        assert_eq!(eval_xml("<a/>", "//").err().unwrap(), XpathError::Eval {});
+        assert_eq!(eval_xml("<a/>", "^^^").err().unwrap(), XpathError::Eval);
+        assert_eq!(eval_xml("<a/>", "//").err().unwrap(), XpathError::Eval);
         // assert_eq!(1,2);
     }
 
