@@ -192,7 +192,7 @@ impl Default for LoggerOptionsBuilder {
         LoggerOptionsBuilder {
             color: false,
             error_format: ErrorFormat::Short,
-            filename: "".to_string(),
+            filename: String::new(),
             progress_bar: false,
             test: false,
             verbosity: None,
@@ -518,7 +518,7 @@ fn progress_string(entry_index: usize, count: usize) -> String {
     let completed = if col > 0 {
         "=".repeat(col)
     } else {
-        "".to_string()
+        String::new()
     };
     let void = " ".repeat(WIDTH - col - 1);
     format!("[{completed}>{void}] {entry_index}/{count}")
@@ -679,7 +679,7 @@ fn get_lines(text: &str) -> Vec<&str> {
 
 #[cfg(test)]
 pub mod tests {
-    use hurl_core::ast::SourceInfo;
+    use hurl_core::ast::{Pos, SourceInfo};
 
     use super::*;
     use crate::runner;
@@ -698,13 +698,14 @@ pub mod tests {
 HTTP/1.0 200
 "#;
         let filename = "test.hurl";
-        let error = runner::Error {
-            source_info: SourceInfo::new(2, 10, 2, 13),
-            inner: runner::RunnerError::AssertStatus {
-                actual: "404".to_string(),
-            },
-            assert: true,
+        let inner = runner::RunnerError::AssertStatus {
+            actual: "404".to_string(),
         };
+        let error = runner::Error::new(
+            SourceInfo::new(Pos::new(2, 10), Pos::new(2, 13)),
+            inner,
+            true,
+        );
         assert_eq!(
             error_string(filename, content, &error, false),
             r#"Assert status code
@@ -721,20 +722,20 @@ HTTP/1.0 200
         let content = r#"GET http://example.com
 HTTP/1.0 200
 [Asserts]
-xpath "strong(//head/title)" equals "Hello"
+xpath "strong(//head/title)" == "Hello"
 "#;
         let filename = "test.hurl";
-        let error = runner::Error {
-            source_info: SourceInfo::new(4, 7, 4, 29),
-            inner: runner::RunnerError::QueryInvalidXpathEval {},
-            assert: true,
-        };
+        let error = runner::Error::new(
+            SourceInfo::new(Pos::new(4, 7), Pos::new(4, 29)),
+            runner::RunnerError::QueryInvalidXpathEval,
+            true,
+        );
         assert_eq!(
             error_string(filename, content, &error, false),
             r#"Invalid XPath expression
   --> test.hurl:4:7
    |
- 4 | xpath "strong(//head/title)" equals "Hello"
+ 4 | xpath "strong(//head/title)" == "Hello"
    |       ^^^^^^^^^^^^^^^^^^^^^^ the XPath expression is not valid
    |"#
         )
@@ -749,7 +750,7 @@ jsonpath "$.count" >= 5
 "#;
         let filename = "test.hurl";
         let error = runner::Error {
-            source_info: SourceInfo::new(4, 0, 4, 0),
+            source_info: SourceInfo::new(Pos::new(4, 0), Pos::new(4, 0)),
             inner: runner::RunnerError::AssertFailure {
                 actual: "int <2>".to_string(),
                 expected: "greater than int <5>".to_string(),
@@ -777,14 +778,12 @@ HTTP/1.0 200
 ```
 "#;
         let filename = "test.hurl";
-        let error = runner::Error {
-            source_info: SourceInfo::new(3, 4, 4, 1),
-            inner: runner::RunnerError::AssertBodyValueError {
-                actual: "<p>Hello</p>\n\n".to_string(),
-                expected: "<p>Hello</p>\n".to_string(),
-            },
-            assert: true,
+        let inner = runner::RunnerError::AssertBodyValueError {
+            actual: "<p>Hello</p>\n\n".to_string(),
+            expected: "<p>Hello</p>\n".to_string(),
         };
+        let error =
+            runner::Error::new(SourceInfo::new(Pos::new(3, 4), Pos::new(4, 1)), inner, true);
         assert_eq!(
             error_string(filename, content, &error, false),
             r#"Assert body value
