@@ -32,6 +32,13 @@ pub struct Entry {
     pub response: Option<Response>,
 }
 
+impl Entry {
+    /// Returns the source information for this entry.
+    pub fn source_info(&self) -> SourceInfo {
+        self.request.space0.source_info
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Request {
     pub line_terminators: Vec<LineTerminator>,
@@ -84,7 +91,7 @@ impl Request {
     pub fn basic_auth(&self) -> Option<KeyValue> {
         for section in &self.sections {
             if let SectionValue::BasicAuth(kv) = &section.value {
-                return Some(kv.clone());
+                return kv.clone();
             }
         }
         None
@@ -150,6 +157,7 @@ pub enum VersionValue {
     Version1,
     Version11,
     Version2,
+    Version3,
     VersionAny,
     VersionAnyLegacy,
 }
@@ -208,7 +216,7 @@ impl Section {
 #[allow(clippy::large_enum_variant)]
 pub enum SectionValue {
     QueryParams(Vec<KeyValue>),
-    BasicAuth(KeyValue),
+    BasicAuth(Option<KeyValue>),
     FormParams(Vec<KeyValue>),
     MultipartFormData(Vec<MultipartParam>),
     Cookies(Vec<Cookie>),
@@ -221,7 +229,7 @@ pub enum SectionValue {
 pub struct Cookie {
     pub line_terminators: Vec<LineTerminator>,
     pub space0: Whitespace,
-    pub name: EncodedString,
+    pub name: Template,
     pub space1: Whitespace,
     pub space2: Whitespace,
     pub value: Template,
@@ -232,7 +240,7 @@ pub struct Cookie {
 pub struct KeyValue {
     pub line_terminators: Vec<LineTerminator>,
     pub space0: Whitespace,
-    pub key: EncodedString,
+    pub key: Template,
     pub space1: Whitespace,
     pub space2: Whitespace,
     pub value: Template,
@@ -249,7 +257,7 @@ pub enum MultipartParam {
 pub struct FileParam {
     pub line_terminators: Vec<LineTerminator>,
     pub space0: Whitespace,
-    pub key: EncodedString,
+    pub key: Template,
     pub space1: Whitespace,
     pub space2: Whitespace,
     pub value: FileValue,
@@ -259,7 +267,7 @@ pub struct FileParam {
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct FileValue {
     pub space0: Whitespace,
-    pub filename: Filename,
+    pub filename: Template,
     pub space1: Whitespace,
     pub space2: Whitespace,
     pub content_type: Option<String>,
@@ -269,7 +277,7 @@ pub struct FileValue {
 pub struct Capture {
     pub line_terminators: Vec<LineTerminator>,
     pub space0: Whitespace,
-    pub name: EncodedString,
+    pub name: Template,
     pub space1: Whitespace,
     pub space2: Whitespace,
     pub query: Query,
@@ -297,8 +305,8 @@ pub struct Query {
 #[derive(Clone, Debug, PartialEq, Eq)]
 #[allow(clippy::large_enum_variant)]
 pub enum QueryValue {
-    Status {},
-    Url {},
+    Status,
+    Url,
     Header {
         space0: Whitespace,
         name: Template,
@@ -307,7 +315,7 @@ pub enum QueryValue {
         space0: Whitespace,
         expr: CookiePath,
     },
-    Body {},
+    Body,
     Xpath {
         space0: Whitespace,
         expr: Template,
@@ -324,10 +332,10 @@ pub enum QueryValue {
         space0: Whitespace,
         name: Template,
     },
-    Duration {},
-    Bytes {},
-    Sha256 {},
-    Md5 {},
+    Duration,
+    Bytes,
+    Sha256,
+    Md5,
     Certificate {
         space0: Whitespace,
         attribute_name: CertificateAttributeName,
@@ -411,16 +419,16 @@ pub struct PredicateFunc {
 #[derive(Clone, Debug, PartialEq, Eq)]
 #[allow(clippy::large_enum_variant)]
 pub enum PredicateValue {
-    String(Template),
-    MultilineString(MultilineString),
-    Integer(i64),
-    Float(Float),
-    Bool(bool),
-    Null {},
-    Hex(Hex),
     Base64(Base64),
+    Bool(bool),
     Expression(Expr),
+    File(File),
+    Hex(Hex),
+    MultilineString(MultilineString),
+    Null,
+    Number(Number),
     Regex(Regex),
+    String(Template),
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -456,10 +464,6 @@ pub enum PredicateFuncValue {
         value: PredicateValue,
         operator: bool,
     },
-    CountEqual {
-        space0: Whitespace,
-        value: PredicateValue,
-    },
     StartWith {
         space0: Whitespace,
         value: PredicateValue,
@@ -480,13 +484,14 @@ pub enum PredicateFuncValue {
         space0: Whitespace,
         value: PredicateValue,
     },
-    IsInteger {},
-    IsFloat {},
-    IsBoolean {},
-    IsString {},
-    IsCollection {},
-    Exist {},
-    IsEmpty {},
+    IsInteger,
+    IsFloat,
+    IsBoolean,
+    IsString,
+    IsCollection,
+    IsDate,
+    Exist,
+    IsEmpty,
 }
 
 //
@@ -556,7 +561,7 @@ pub struct Base64 {
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct File {
     pub space0: Whitespace,
-    pub filename: Filename,
+    pub filename: Template,
     pub space1: Whitespace,
 }
 
@@ -577,6 +582,7 @@ pub enum TemplateElement {
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Comment {
     pub value: String,
+    pub source_info: SourceInfo,
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -594,15 +600,10 @@ pub struct Whitespace {
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
-pub struct Filename {
-    pub value: String,
-    pub source_info: SourceInfo,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct Number {
-    pub int: i64,
-    pub decimal: u64,
+pub enum Number {
+    Float(Float),
+    Integer(i64),
+    BigInteger(String),
 }
 
 // keep Number terminology for both Integer and Decimal Numbers
@@ -661,35 +662,27 @@ impl PartialEq for Regex {
 }
 impl Eq for Regex {}
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub struct Pos {
     pub line: usize,
     pub column: usize,
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+impl Pos {
+    pub fn new(line: usize, column: usize) -> Pos {
+        Pos { line, column }
+    }
+}
+
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub struct SourceInfo {
     pub start: Pos,
     pub end: Pos,
 }
 
 impl SourceInfo {
-    pub fn new(
-        start_line: usize,
-        start_col: usize,
-        end_line: usize,
-        end_column: usize,
-    ) -> SourceInfo {
-        SourceInfo {
-            start: Pos {
-                line: start_line,
-                column: start_col,
-            },
-            end: Pos {
-                line: end_line,
-                column: end_column,
-            },
-        }
+    pub fn new(start: Pos, end: Pos) -> SourceInfo {
+        SourceInfo { start, end }
     }
 }
 
@@ -707,161 +700,133 @@ pub struct Variable {
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
-pub enum EntryOption {
-    CaCertificate(CaCertificateOption),
-    ClientCert(ClientCertOption),
-    ClientKey(ClientKeyOption),
-    Compressed(CompressedOption),
-    Insecure(InsecureOption),
-    FollowLocation(FollowLocationOption),
-    MaxRedirect(MaxRedirectOption),
-    PathAsIs(PathAsIsOption),
-    Proxy(ProxyOption),
+pub struct EntryOption {
+    pub line_terminators: Vec<LineTerminator>,
+    pub space0: Whitespace,
+    pub space1: Whitespace,
+    pub space2: Whitespace,
+    pub kind: OptionKind,
+    pub line_terminator0: LineTerminator,
+}
+
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub enum OptionKind {
+    AwsSigV4(Template),
+    CaCertificate(Template),
+    ClientCert(Template),
+    ClientKey(Template),
+    Compressed(BooleanOption),
+    ConnectTo(Template),
+    Delay(NaturalOption),
+    Http10(BooleanOption),
+    Http11(BooleanOption),
+    Http2(BooleanOption),
+    Http3(BooleanOption),
+    Insecure(BooleanOption),
+    IpV4(BooleanOption),
+    IpV6(BooleanOption),
+    FollowLocation(BooleanOption),
+    FollowLocationTrusted(BooleanOption),
+    MaxRedirect(NaturalOption),
+    Output(Template),
+    PathAsIs(BooleanOption),
+    Proxy(Template),
+    Resolve(Template),
     Retry(RetryOption),
-    RetryInterval(RetryIntervalOption),
-    Variable(VariableOption),
-    Verbose(VerboseOption),
-    VeryVerbose(VeryVerboseOption),
+    RetryInterval(NaturalOption),
+    Skip(BooleanOption),
+    UnixSocket(Template),
+    Variable(VariableDefinition),
+    Verbose(BooleanOption),
+    VeryVerbose(BooleanOption),
+}
+
+impl OptionKind {
+    pub fn name(&self) -> &'static str {
+        match self {
+            OptionKind::AwsSigV4(_) => "aws-sigv4",
+            OptionKind::CaCertificate(_) => "cacert",
+            OptionKind::ClientCert(_) => "cert",
+            OptionKind::ClientKey(_) => "key",
+            OptionKind::Compressed(_) => "compressed",
+            OptionKind::ConnectTo(_) => "connect-to",
+            OptionKind::Delay(_) => "delay",
+            OptionKind::FollowLocation(_) => "location",
+            OptionKind::FollowLocationTrusted(_) => "location-trusted",
+            OptionKind::Http10(_) => "http1.0",
+            OptionKind::Http11(_) => "http1.1",
+            OptionKind::Http2(_) => "http2",
+            OptionKind::Http3(_) => "http3",
+            OptionKind::Insecure(_) => "insecure",
+            OptionKind::IpV4(_) => "ipv4",
+            OptionKind::IpV6(_) => "ipv6",
+            OptionKind::MaxRedirect(_) => "max-redirs",
+            OptionKind::Output(_) => "output",
+            OptionKind::PathAsIs(_) => "path-as-is",
+            OptionKind::Proxy(_) => "proxy",
+            OptionKind::Resolve(_) => "resolve",
+            OptionKind::Retry(_) => "retry",
+            OptionKind::RetryInterval(_) => "retry-interval",
+            OptionKind::Skip(_) => "skip",
+            OptionKind::UnixSocket(_) => "unix-socket",
+            OptionKind::Variable(_) => "variable",
+            OptionKind::Verbose(_) => "verbose",
+            OptionKind::VeryVerbose(_) => "very-verbose",
+        }
+    }
+
+    pub fn value_as_str(&self) -> String {
+        match self {
+            OptionKind::AwsSigV4(value) => value.to_string(),
+            OptionKind::CaCertificate(filename) => filename.to_string(),
+            OptionKind::ClientCert(filename) => filename.to_string(),
+            OptionKind::ClientKey(filename) => filename.to_string(),
+            OptionKind::Compressed(value) => value.to_string(),
+            OptionKind::ConnectTo(value) => value.to_string(),
+            OptionKind::Delay(value) => value.to_string(),
+            OptionKind::FollowLocation(value) => value.to_string(),
+            OptionKind::FollowLocationTrusted(value) => value.to_string(),
+            OptionKind::Http10(value) => value.to_string(),
+            OptionKind::Http11(value) => value.to_string(),
+            OptionKind::Http2(value) => value.to_string(),
+            OptionKind::Http3(value) => value.to_string(),
+            OptionKind::Insecure(value) => value.to_string(),
+            OptionKind::IpV4(value) => value.to_string(),
+            OptionKind::IpV6(value) => value.to_string(),
+            OptionKind::MaxRedirect(value) => value.to_string(),
+            OptionKind::Output(filename) => filename.to_string(),
+            OptionKind::PathAsIs(value) => value.to_string(),
+            OptionKind::Proxy(value) => value.to_string(),
+            OptionKind::Resolve(value) => value.to_string(),
+            OptionKind::Retry(value) => value.to_string(),
+            OptionKind::RetryInterval(value) => value.to_string(),
+            OptionKind::Skip(value) => value.to_string(),
+            OptionKind::UnixSocket(value) => value.to_string(),
+            OptionKind::Variable(VariableDefinition { name, value, .. }) => {
+                format!("{name}={value}")
+            }
+            OptionKind::Verbose(value) => value.to_string(),
+            OptionKind::VeryVerbose(value) => value.to_string(),
+        }
+    }
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
-pub struct CompressedOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: bool,
-    pub line_terminator0: LineTerminator,
+pub enum BooleanOption {
+    Literal(bool),
+    Expression(Expr),
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
-pub struct InsecureOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: bool,
-    pub line_terminator0: LineTerminator,
+pub enum NaturalOption {
+    Literal(u64),
+    Expression(Expr),
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
-pub struct CaCertificateOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub filename: Filename,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct ClientCertOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub filename: Filename,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct ClientKeyOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub filename: Filename,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct PathAsIsOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: bool,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct ProxyOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: String,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct RetryIntervalOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: u64,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct RetryOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: Retry,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct VerboseOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: bool,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct VeryVerboseOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: bool,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct FollowLocationOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: bool,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct MaxRedirectOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: usize,
-    pub line_terminator0: LineTerminator,
-}
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct VariableOption {
-    pub line_terminators: Vec<LineTerminator>,
-    pub space0: Whitespace,
-    pub space1: Whitespace,
-    pub space2: Whitespace,
-    pub value: VariableDefinition,
-    pub line_terminator0: LineTerminator,
+pub enum RetryOption {
+    Literal(Retry),
+    Expression(Expr),
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -874,10 +839,9 @@ pub struct VariableDefinition {
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum VariableValue {
-    Null {},
+    Null,
     Bool(bool),
-    Integer(i64),
-    Float(Float),
+    Number(Number),
     String(Template),
 }
 
@@ -902,6 +866,10 @@ pub enum FilterValue {
     },
     HtmlEscape,
     HtmlUnescape,
+    JsonPath {
+        space0: Whitespace,
+        expr: Template,
+    },
     Nth {
         space0: Whitespace,
         n: u64,
