@@ -40,14 +40,14 @@ pub enum Value {
         space0: String,
         elements: Vec<ObjectElement>,
     },
-    Null {},
+    Null,
 }
 
 impl Value {
     pub fn _type(&self) -> String {
         match self {
             Value::Number(_) => "number".to_string(),
-            Value::Null {} => "null".to_string(),
+            Value::Null => "null".to_string(),
             Value::Boolean(_) => "boolean".to_string(),
             Value::List { .. } => "list".to_string(),
             Value::Object { .. } => "object".to_string(),
@@ -101,7 +101,7 @@ impl fmt::Display for Value {
                     .collect::<Vec<String>>();
                 format!("{{{}{}}}", space0, elements.join(","))
             }
-            Value::Null { .. } => "null".to_string(),
+            Value::Null => "null".to_string(),
         };
         write!(f, "{s}")
     }
@@ -109,7 +109,7 @@ impl fmt::Display for Value {
 
 impl fmt::Display for ListElement {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let mut s = "".to_string();
+        let mut s = String::new();
         s.push_str(self.space0.as_str());
         s.push_str(self.value.to_string().as_str());
         s.push_str(self.space1.as_str());
@@ -119,7 +119,7 @@ impl fmt::Display for ListElement {
 
 impl fmt::Display for ObjectElement {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let mut s = "".to_string();
+        let mut s = String::new();
         s.push_str(self.space0.as_str());
         s.push('"');
         s.push_str(self.name.to_string().as_str());
@@ -160,14 +160,14 @@ impl Value {
                     .collect::<Vec<String>>();
                 format!("{{{}{}}}", space0, elements.join(","))
             }
-            Value::Null { .. } => "null".to_string(),
+            Value::Null => "null".to_string(),
         }
     }
 }
 
 impl ListElement {
     fn encoded(&self) -> String {
-        let mut s = "".to_string();
+        let mut s = String::new();
         s.push_str(self.space0.as_str());
         s.push_str(self.value.encoded().as_str());
         s.push_str(self.space1.as_str());
@@ -177,7 +177,7 @@ impl ListElement {
 
 impl ObjectElement {
     fn encoded(&self) -> String {
-        let mut s = "".to_string();
+        let mut s = String::new();
         s.push_str(self.space0.as_str());
         s.push_str(self.name.encoded().as_str());
         s.push_str(self.space1.as_str());
@@ -191,7 +191,7 @@ impl ObjectElement {
 
 impl Template {
     fn encoded(&self) -> String {
-        let mut s = "".to_string();
+        let mut s = String::new();
         if let Some(d) = self.delimiter {
             s.push(d)
         }
@@ -216,7 +216,7 @@ impl TemplateElement {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::ast::{SourceInfo, TemplateElement, Variable, Whitespace};
+    use crate::ast::{Pos, SourceInfo, TemplateElement, Variable, Whitespace};
 
     #[test]
     fn test_to_string() {
@@ -224,16 +224,16 @@ mod tests {
             "{{x}}".to_string(),
             Value::Expression(Expr {
                 space0: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
                 variable: Variable {
                     name: "x".to_string(),
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
                 space1: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
             })
             .to_string()
@@ -247,7 +247,7 @@ mod tests {
                     value: "hello".to_string(),
                     encoded: "hello".to_string(),
                 }],
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             })
             .to_string()
         );
@@ -255,7 +255,7 @@ mod tests {
         assert_eq!(
             "[]".to_string(),
             Value::List {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![],
             }
             .to_string()
@@ -263,22 +263,22 @@ mod tests {
         assert_eq!(
             "[1, 2, 3]".to_string(),
             Value::List {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![
                     ListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: Value::Number("1".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                     ListElement {
                         space0: " ".to_string(),
                         value: Value::Number("2".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                     ListElement {
                         space0: " ".to_string(),
                         value: Value::Number("3".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     }
                 ],
             }
@@ -287,7 +287,7 @@ mod tests {
         assert_eq!(
             "{}".to_string(),
             Value::Object {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![],
             }
             .to_string()
@@ -295,7 +295,7 @@ mod tests {
         assert_eq!(
             "{ \"id\": 123 }".to_string(),
             Value::Object {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![ObjectElement {
                     space0: " ".to_string(),
                     name: Template {
@@ -304,9 +304,9 @@ mod tests {
                             value: "id".to_string(),
                             encoded: "id".to_string(),
                         }],
-                        source_info: SourceInfo::new(1, 1, 1, 1),
+                        source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                     },
-                    space1: "".to_string(),
+                    space1: String::new(),
                     space2: " ".to_string(),
                     value: Value::Number("123".to_string()),
                     space3: " ".to_string(),
@@ -314,7 +314,7 @@ mod tests {
             }
             .to_string()
         );
-        assert_eq!("null".to_string(), Value::Null {}.to_string());
+        assert_eq!("null".to_string(), Value::Null.to_string());
     }
 
     #[test]
@@ -322,16 +322,16 @@ mod tests {
         assert_eq!(
             TemplateElement::Expression(Expr {
                 space0: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 1, 1, 1),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                 },
                 variable: Variable {
                     name: "name".to_string(),
-                    source_info: SourceInfo::new(1, 1, 1, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                 },
                 space1: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 1, 1, 1),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                 },
             })
             .encoded(),
@@ -342,19 +342,19 @@ mod tests {
                 delimiter: None,
                 elements: vec![TemplateElement::Expression(Expr {
                     space0: Whitespace {
-                        value: "".to_string(),
-                        source_info: SourceInfo::new(1, 1, 1, 1),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                     },
                     variable: Variable {
                         name: "name".to_string(),
-                        source_info: SourceInfo::new(1, 1, 1, 1),
+                        source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                     },
                     space1: Whitespace {
-                        value: "".to_string(),
-                        source_info: SourceInfo::new(1, 1, 1, 1),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                     },
                 })],
-                source_info: SourceInfo::new(1, 1, 1, 1),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             }
             .encoded(),
             "{{name}}".to_string()
