@@ -15,9 +15,10 @@
  * limitations under the License.
  *
  */
-use crate::ast::*;
 use std::fmt::Display;
 
+use crate::ast::*;
+
 /// Returns an HTML string of the Hurl file `hurl_file`.
 ///
 /// If `standalone` is true, a complete HTML body with inline styling is returned.
@@ -174,7 +175,11 @@ impl HtmlFormatter {
         match section_value {
             SectionValue::Asserts(items) => items.iter().for_each(|item| self.fmt_assert(item)),
             SectionValue::QueryParams(items) => items.iter().for_each(|item| self.fmt_kv(item)),
-            SectionValue::BasicAuth(item) => self.fmt_kv(item),
+            SectionValue::BasicAuth(item) => {
+                if let Some(kv) = item {
+                    self.fmt_kv(kv)
+                }
+            }
             SectionValue::FormParams(items) => items.iter().for_each(|item| self.fmt_kv(item)),
             SectionValue::MultipartFormData(items) => {
                 items.iter().for_each(|item| self.fmt_multipart_param(item))
@@ -191,7 +196,7 @@ impl HtmlFormatter {
         self.fmt_lts(&kv.line_terminators);
         self.fmt_span_open("line");
         self.fmt_space(&kv.space0);
-        self.fmt_string(&kv.key.encoded);
+        self.fmt_template(&kv.key);
         self.fmt_space(&kv.space1);
         self.buffer.push(':');
         self.fmt_space(&kv.space2);
@@ -200,179 +205,61 @@ impl HtmlFormatter {
         self.fmt_lt(&kv.line_terminator0);
     }
 
-    fn fmt_entry_option(&mut self, entry_option: &EntryOption) {
-        match entry_option {
-            EntryOption::CaCertificate(option) => self.fmt_ca_certificate_option(option),
-            EntryOption::ClientCert(option) => self.fmt_client_cert_option(option),
-            EntryOption::ClientKey(option) => self.fmt_client_key_option(option),
-            EntryOption::Compressed(option) => self.fmt_compressed_option(option),
-            EntryOption::Insecure(option) => self.fmt_insecure_option(option),
-            EntryOption::FollowLocation(option) => self.fmt_follow_location_option(option),
-            EntryOption::MaxRedirect(option) => self.fmt_max_redirect_option(option),
-            EntryOption::PathAsIs(option) => self.fmt_path_as_is_option(option),
-            EntryOption::Proxy(option) => self.fmt_proxy_option(option),
-            EntryOption::Retry(option) => self.fmt_retry_option(option),
-            EntryOption::RetryInterval(option) => self.fmt_retry_interval_option(option),
-            EntryOption::Variable(option) => self.fmt_variable_option(option),
-            EntryOption::Verbose(option) => self.fmt_verbose_option(option),
-            EntryOption::VeryVerbose(option) => self.fmt_very_verbose_option(option),
+    fn fmt_entry_option(&mut self, option: &EntryOption) {
+        self.fmt_lts(&option.line_terminators);
+        self.fmt_span_open("line");
+        self.fmt_space(&option.space0);
+        self.fmt_string(option.kind.name());
+        self.fmt_space(&option.space1);
+        self.buffer.push(':');
+        self.fmt_space(&option.space2);
+        match &option.kind {
+            OptionKind::AwsSigV4(value) => self.fmt_template(value),
+            OptionKind::CaCertificate(filename) => self.fmt_filename(filename),
+            OptionKind::ClientCert(filename) => self.fmt_filename(filename),
+            OptionKind::ClientKey(filename) => self.fmt_filename(filename),
+            OptionKind::Compressed(value) => self.fmt_bool_option(value),
+            OptionKind::ConnectTo(value) => self.fmt_template(value),
+            OptionKind::Delay(value) => self.fmt_natural_option(value),
+            OptionKind::FollowLocation(value) => self.fmt_bool_option(value),
+            OptionKind::FollowLocationTrusted(value) => self.fmt_bool_option(value),
+            OptionKind::Http10(value) => self.fmt_bool_option(value),
+            OptionKind::Http11(value) => self.fmt_bool_option(value),
+            OptionKind::Http2(value) => self.fmt_bool_option(value),
+            OptionKind::Http3(value) => self.fmt_bool_option(value),
+            OptionKind::Insecure(value) => self.fmt_bool_option(value),
+            OptionKind::IpV4(value) => self.fmt_bool_option(value),
+            OptionKind::IpV6(value) => self.fmt_bool_option(value),
+            OptionKind::MaxRedirect(value) => self.fmt_natural_option(value),
+            OptionKind::Output(filename) => self.fmt_filename(filename),
+            OptionKind::PathAsIs(value) => self.fmt_bool_option(value),
+            OptionKind::Proxy(value) => self.fmt_template(value),
+            OptionKind::Resolve(value) => self.fmt_template(value),
+            OptionKind::Retry(value) => self.fmt_retry_option(value),
+            OptionKind::RetryInterval(value) => self.fmt_natural_option(value),
+            OptionKind::Skip(value) => self.fmt_bool_option(value),
+            OptionKind::UnixSocket(value) => self.fmt_template(value),
+            OptionKind::Variable(value) => self.fmt_variable_definition(value),
+            OptionKind::Verbose(value) => self.fmt_bool_option(value),
+            OptionKind::VeryVerbose(value) => self.fmt_bool_option(value),
+        };
+        self.fmt_span_close();
+        self.fmt_lt(&option.line_terminator0);
+    }
+
+    fn fmt_retry_option(&mut self, retry_option: &RetryOption) {
+        match retry_option {
+            RetryOption::Literal(retry) => self.fmt_retry(retry),
+            RetryOption::Expression(expr) => self.fmt_expr(expr),
         };
     }
 
-    fn fmt_compressed_option(&mut self, option: &CompressedOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("compressed");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_bool(option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_insecure_option(&mut self, option: &InsecureOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("insecure");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_bool(option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_ca_certificate_option(&mut self, option: &CaCertificateOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("cacert");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_filename(&option.filename);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_client_cert_option(&mut self, option: &ClientCertOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("cert");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_filename(&option.filename);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_client_key_option(&mut self, option: &ClientKeyOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("key");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_filename(&option.filename);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_follow_location_option(&mut self, option: &FollowLocationOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("location");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_bool(option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_max_redirect_option(&mut self, option: &MaxRedirectOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("max-redirs");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_number(option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_path_as_is_option(&mut self, option: &PathAsIsOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("path-as-is");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_bool(option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_proxy_option(&mut self, option: &ProxyOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("proxy");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_string(&option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_retry_option(&mut self, option: &RetryOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("retry");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_retry(option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_retry_interval_option(&mut self, option: &RetryIntervalOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("retry-interval");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_number(option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_variable_option(&mut self, option: &VariableOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("variable");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_variable_definition(&option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
+    fn fmt_retry(&mut self, retry: &Retry) {
+        match retry {
+            Retry::Finite(n) => self.fmt_number(n),
+            Retry::Infinite => self.fmt_number(-1),
+            Retry::None => self.fmt_number(0),
+        };
     }
 
     fn fmt_variable_definition(&mut self, option: &VariableDefinition) {
@@ -384,40 +271,13 @@ impl HtmlFormatter {
 
     fn fmt_variable_value(&mut self, option: &VariableValue) {
         match option {
-            VariableValue::Null { .. } => self.fmt_span("null", "null"),
+            VariableValue::Null => self.fmt_span("null", "null"),
             VariableValue::Bool(v) => self.fmt_bool(*v),
-            VariableValue::Integer(v) => self.fmt_number(v),
-            VariableValue::Float(v) => self.fmt_number(&v.encoded),
+            VariableValue::Number(v) => self.fmt_number(v),
             VariableValue::String(t) => self.fmt_template(t),
         }
     }
 
-    fn fmt_verbose_option(&mut self, option: &VerboseOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("verbose");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_bool(option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
-    fn fmt_very_verbose_option(&mut self, option: &VeryVerboseOption) {
-        self.fmt_lts(&option.line_terminators);
-        self.fmt_span_open("line");
-        self.fmt_space(&option.space0);
-        self.fmt_string("very-verbose");
-        self.fmt_space(&option.space1);
-        self.buffer.push(':');
-        self.fmt_space(&option.space2);
-        self.fmt_bool(option.value);
-        self.fmt_span_close();
-        self.fmt_lt(&option.line_terminator0);
-    }
-
     fn fmt_multipart_param(&mut self, param: &MultipartParam) {
         match param {
             MultipartParam::Param(param) => self.fmt_kv(param),
@@ -429,7 +289,7 @@ impl HtmlFormatter {
         self.fmt_lts(&param.line_terminators);
         self.fmt_span_open("line");
         self.fmt_space(&param.space0);
-        self.fmt_string(&param.key.encoded);
+        self.fmt_template(&param.key);
         self.fmt_space(&param.space1);
         self.buffer.push(':');
         self.fmt_space(&param.space2);
@@ -450,9 +310,9 @@ impl HtmlFormatter {
         }
     }
 
-    fn fmt_filename(&mut self, filename: &Filename) {
+    fn fmt_filename(&mut self, filename: &Template) {
         self.fmt_span_open("filename");
-        let s = filename.value.replace(' ', "\\ ");
+        let s = filename.to_string().replace(' ', "\\ ");
         self.buffer.push_str(s.as_str());
         self.fmt_span_close();
     }
@@ -461,7 +321,7 @@ impl HtmlFormatter {
         self.fmt_lts(&cookie.line_terminators);
         self.fmt_span_open("line");
         self.fmt_space(&cookie.space0);
-        self.fmt_span("name", &cookie.name.value);
+        self.fmt_template(&cookie.name);
         self.fmt_space(&cookie.space1);
         self.buffer.push(':');
         self.fmt_space(&cookie.space2);
@@ -474,7 +334,7 @@ impl HtmlFormatter {
         self.fmt_lts(&capture.line_terminators);
         self.fmt_span_open("line");
         self.fmt_space(&capture.space0);
-        self.fmt_span("name", &capture.name.value);
+        self.fmt_template(&capture.name);
         self.fmt_space(&capture.space1);
         self.buffer.push(':');
         self.fmt_space(&capture.space2);
@@ -493,8 +353,8 @@ impl HtmlFormatter {
 
     fn fmt_query_value(&mut self, query_value: &QueryValue) {
         match query_value {
-            QueryValue::Status {} => self.fmt_span("query-type", "status"),
-            QueryValue::Url {} => self.fmt_span("query-type", "url"),
+            QueryValue::Status => self.fmt_span("query-type", "status"),
+            QueryValue::Url => self.fmt_span("query-type", "url"),
             QueryValue::Header { space0, name } => {
                 self.fmt_span("query-type", "header");
                 self.fmt_space(space0);
@@ -505,7 +365,7 @@ impl HtmlFormatter {
                 self.fmt_space(space0);
                 self.fmt_cookie_path(expr);
             }
-            QueryValue::Body {} => self.fmt_span("query-type", "body"),
+            QueryValue::Body => self.fmt_span("query-type", "body"),
             QueryValue::Xpath { space0, expr } => {
                 self.fmt_span("query-type", "xpath");
                 self.fmt_space(space0);
@@ -526,10 +386,10 @@ impl HtmlFormatter {
                 self.fmt_space(space0);
                 self.fmt_template(name);
             }
-            QueryValue::Duration {} => self.fmt_span("query-type", "duration"),
-            QueryValue::Bytes {} => self.fmt_span("query-type", "bytes"),
-            QueryValue::Sha256 {} => self.fmt_span("query-type", "sha256"),
-            QueryValue::Md5 {} => self.fmt_span("query-type", "md5"),
+            QueryValue::Duration => self.fmt_span("query-type", "duration"),
+            QueryValue::Bytes => self.fmt_span("query-type", "bytes"),
+            QueryValue::Sha256 => self.fmt_span("query-type", "sha256"),
+            QueryValue::Md5 => self.fmt_span("query-type", "md5"),
             QueryValue::Certificate {
                 space0,
                 attribute_name: field,
@@ -616,10 +476,6 @@ impl HtmlFormatter {
         self.fmt_span_close();
 
         match value {
-            PredicateFuncValue::CountEqual { space0, value, .. } => {
-                self.fmt_space(space0);
-                self.fmt_predicate_value(value);
-            }
             PredicateFuncValue::Equal { space0, value, .. } => {
                 self.fmt_space(space0);
                 self.fmt_predicate_value(value);
@@ -664,13 +520,14 @@ impl HtmlFormatter {
                 self.fmt_space(space0);
                 self.fmt_predicate_value(value);
             }
-            PredicateFuncValue::IsInteger {} => {}
-            PredicateFuncValue::IsFloat {} => {}
-            PredicateFuncValue::IsBoolean {} => {}
-            PredicateFuncValue::IsString {} => {}
-            PredicateFuncValue::IsCollection {} => {}
-            PredicateFuncValue::Exist {} => {}
-            PredicateFuncValue::IsEmpty {} => {}
+            PredicateFuncValue::IsInteger => {}
+            PredicateFuncValue::IsFloat => {}
+            PredicateFuncValue::IsBoolean => {}
+            PredicateFuncValue::IsString => {}
+            PredicateFuncValue::IsCollection => {}
+            PredicateFuncValue::IsDate => {}
+            PredicateFuncValue::Exist => {}
+            PredicateFuncValue::IsEmpty => {}
         }
     }
 
@@ -678,13 +535,13 @@ impl HtmlFormatter {
         match predicate_value {
             PredicateValue::String(value) => self.fmt_template(value),
             PredicateValue::MultilineString(value) => self.fmt_multiline_string(value, false),
-            PredicateValue::Integer(value) => self.fmt_number(value),
-            PredicateValue::Float(value) => self.fmt_number(&value.encoded),
+            PredicateValue::Number(value) => self.fmt_number(value),
             PredicateValue::Bool(value) => self.fmt_bool(*value),
+            PredicateValue::File(value) => self.fmt_file(value),
             PredicateValue::Hex(value) => self.fmt_hex(value),
             PredicateValue::Base64(value) => self.fmt_base64(value),
             PredicateValue::Expression(value) => self.fmt_expr(value),
-            PredicateValue::Null {} => self.fmt_span("null", "null"),
+            PredicateValue::Null => self.fmt_span("null", "null"),
             PredicateValue::Regex(value) => self.fmt_regex(value),
         };
     }
@@ -819,6 +676,20 @@ impl HtmlFormatter {
         self.fmt_span("boolean", &value.to_string());
     }
 
+    fn fmt_bool_option(&mut self, value: &BooleanOption) {
+        match value {
+            BooleanOption::Literal(value) => self.fmt_span("boolean", &value.to_string()),
+            BooleanOption::Expression(value) => self.fmt_expr(value),
+        }
+    }
+
+    fn fmt_natural_option(&mut self, value: &NaturalOption) {
+        match value {
+            NaturalOption::Literal(value) => self.fmt_span("number", &value.to_string()),
+            NaturalOption::Expression(value) => self.fmt_expr(value),
+        }
+    }
+
     fn fmt_number<T: Sized + Display>(&mut self, value: T) {
         self.fmt_span("number", &value.to_string());
     }
@@ -914,6 +785,11 @@ impl HtmlFormatter {
             }
             FilterValue::HtmlEscape => self.fmt_span("filter-type", "htmlEscape"),
             FilterValue::HtmlUnescape => self.fmt_span("filter-type", "htmlUnescape"),
+            FilterValue::JsonPath { space0, expr } => {
+                self.fmt_span("filter-type", "jsonpath");
+                self.fmt_space(space0);
+                self.fmt_template(expr);
+            }
             FilterValue::Nth { space0, n: value } => {
                 self.fmt_span("filter-type", "nth");
                 self.fmt_space(space0);
@@ -967,14 +843,6 @@ impl HtmlFormatter {
             self.fmt_lt(line_terminator);
         }
     }
-
-    fn fmt_retry(&mut self, retry: Retry) {
-        match retry {
-            Retry::Finite(n) => self.fmt_span("number", &n.to_string()),
-            Retry::Infinite => self.fmt_span("number", "-1"),
-            _ => {}
-        };
-    }
 }
 
 fn escape_xml(s: &str) -> String {
@@ -985,7 +853,7 @@ fn escape_xml(s: &str) -> String {
 
 impl Template {
     fn to_encoded_string(&self) -> String {
-        let mut s = "".to_string();
+        let mut s = String::new();
         if let Some(d) = self.delimiter {
             s.push(d);
         }
@@ -1033,10 +901,10 @@ mod tests {
         let multiline_string = MultilineString::OneLineText(Template {
             delimiter: None,
             elements: vec![TemplateElement::String {
-                value: "".to_string(),
+                value: String::new(),
                 encoded: "unused".to_string(),
             }],
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         });
         let mut fmt = HtmlFormatter::new();
         fmt.fmt_multiline_string(&multiline_string, false);
@@ -1057,7 +925,7 @@ mod tests {
                 value: "hello".to_string(),
                 encoded: "unused".to_string(),
             }],
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         });
         let mut fmt = HtmlFormatter::new();
         fmt.fmt_multiline_string(&multiline_string, false);
@@ -1076,7 +944,7 @@ mod tests {
         // ```
         let multiline_string = MultilineString::Text(Text {
             space: Whitespace {
-                value: "".to_string(),
+                value: String::new(),
                 source_info: SourceInfo {
                     start: Pos { line: 1, column: 4 },
                     end: Pos { line: 1, column: 4 },
@@ -1095,7 +963,7 @@ mod tests {
                     value: "line1\nline2\n".to_string(),
                     encoded: "unused".to_string(),
                 }],
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
         });
         let mut fmt = HtmlFormatter::new();
@@ -1151,7 +1019,7 @@ mod tests {
     fn test_json() {
         let mut fmt = HtmlFormatter::new();
         let value = JsonValue::Object {
-            space0: "".to_string(),
+            space0: String::new(),
             elements: vec![JsonObjectElement {
                 space0: "\n   ".to_string(),
                 name: Template {
@@ -1160,9 +1028,9 @@ mod tests {
                         value: "id".to_string(),
                         encoded: "id".to_string(),
                     }],
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
-                space1: "".to_string(),
+                space1: String::new(),
                 space2: " ".to_string(),
                 value: JsonValue::Number("1".to_string()),
                 space3: "\n".to_string(),
@@ -1188,7 +1056,7 @@ mod tests {
                 value: "\n".to_string(),
                 encoded: "\\n".to_string(),
             }],
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         });
         fmt.fmt_json_value(&value);
         assert_eq!(
