@@ -24,7 +24,7 @@ use crate::parser::reader::Reader;
 use crate::parser::string::backtick_template;
 use crate::parser::{xml, ParseResult};
 
-pub fn bytes(reader: &mut Reader) -> ParseResult<'static, Bytes> {
+pub fn bytes(reader: &mut Reader) -> ParseResult<Bytes> {
     choice(
         &[
             multiline_string_bytes,
@@ -39,37 +39,37 @@ pub fn bytes(reader: &mut Reader) -> ParseResult<'static, Bytes> {
     )
 }
 
-fn xml_bytes(reader: &mut Reader) -> ParseResult<'static, Bytes> {
+fn xml_bytes(reader: &mut Reader) -> ParseResult<Bytes> {
     match xml::parse(reader) {
         Err(e) => Err(e),
         Ok(value) => Ok(Bytes::Xml(value)),
     }
 }
 
-fn json_bytes(reader: &mut Reader) -> ParseResult<'static, Bytes> {
+fn json_bytes(reader: &mut Reader) -> ParseResult<Bytes> {
     match parse_json(reader) {
         Err(e) => Err(e),
         Ok(value) => Ok(Bytes::Json(value)),
     }
 }
 
-fn file_bytes(reader: &mut Reader) -> ParseResult<'static, Bytes> {
+fn file_bytes(reader: &mut Reader) -> ParseResult<Bytes> {
     file(reader).map(Bytes::File)
 }
 
-fn base64_bytes(reader: &mut Reader) -> ParseResult<'static, Bytes> {
+fn base64_bytes(reader: &mut Reader) -> ParseResult<Bytes> {
     base64(reader).map(Bytes::Base64)
 }
 
-fn hex_bytes(reader: &mut Reader) -> ParseResult<'static, Bytes> {
+fn hex_bytes(reader: &mut Reader) -> ParseResult<Bytes> {
     hex(reader).map(Bytes::Hex)
 }
 
-pub fn multiline_string_bytes(reader: &mut Reader) -> ParseResult<'static, Bytes> {
+pub fn multiline_string_bytes(reader: &mut Reader) -> ParseResult<Bytes> {
     multiline_string(reader).map(Bytes::MultilineString)
 }
 
-fn string_bytes(reader: &mut Reader) -> ParseResult<'static, Bytes> {
+fn string_bytes(reader: &mut Reader) -> ParseResult<Bytes> {
     backtick_template(reader).map(Bytes::OnelineString)
 }
 
@@ -84,22 +84,22 @@ mod tests {
         assert_eq!(
             bytes(&mut reader).unwrap(),
             Bytes::Json(JsonValue::List {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![
                     JsonListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: JsonValue::Number("1".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                     JsonListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: JsonValue::Number("2".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                     JsonListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: JsonValue::Number("3".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                 ],
             })
@@ -129,7 +129,7 @@ mod tests {
             Bytes::Json(JsonValue::String(Template {
                 delimiter: Some('"'),
                 elements: vec![],
-                source_info: SourceInfo::new(1, 2, 1, 2),
+                source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 2)),
             }))
         );
         assert_eq!(reader.state.cursor, 2);
@@ -153,7 +153,7 @@ mod tests {
             error.inner,
             ParseError::Expecting {
                 value: "\"".to_string()
-            }
+            },
         );
     }
 
@@ -198,7 +198,7 @@ mod tests {
                     value: "foo".to_string(),
                     encoded: "foo".to_string()
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 6)
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6))
             })
         );
         assert_eq!(reader.state.cursor, 5);
