@@ -23,8 +23,8 @@ use crate::parser::reader::Reader;
 use crate::parser::string::*;
 use crate::parser::ParseResult;
 
-pub fn cookiepath(reader: &mut Reader) -> ParseResult<'static, CookiePath> {
-    let start = reader.state.pos.clone();
+pub fn cookiepath(reader: &mut Reader) -> ParseResult<CookiePath> {
+    let start = reader.state.pos;
     let s = reader.read_while(|c| *c != '[');
     let mut template_reader = Reader::new(s.as_str());
     template_reader.state.pos = start;
@@ -33,7 +33,7 @@ pub fn cookiepath(reader: &mut Reader) -> ParseResult<'static, CookiePath> {
     Ok(CookiePath { name, attribute })
 }
 
-fn cookiepath_attribute(reader: &mut Reader) -> ParseResult<'static, CookieAttribute> {
+fn cookiepath_attribute(reader: &mut Reader) -> ParseResult<CookieAttribute> {
     try_literal("[", reader)?;
     let space0 = zero_or_more_spaces(reader)?;
     let name = cookiepath_attribute_name(reader)?;
@@ -46,8 +46,8 @@ fn cookiepath_attribute(reader: &mut Reader) -> ParseResult<'static, CookieAttri
     })
 }
 
-fn cookiepath_attribute_name(reader: &mut Reader) -> ParseResult<'static, CookieAttributeName> {
-    let start = reader.state.pos.clone();
+fn cookiepath_attribute_name(reader: &mut Reader) -> ParseResult<CookieAttributeName> {
+    let start = reader.state.pos;
     let s = reader.read_while(|c| c.is_alphabetic() || *c == '-');
     match s.to_lowercase().as_str() {
         "value" => Ok(CookieAttributeName::Value(s)),
@@ -58,11 +58,7 @@ fn cookiepath_attribute_name(reader: &mut Reader) -> ParseResult<'static, Cookie
         "secure" => Ok(CookieAttributeName::Secure(s)),
         "httponly" => Ok(CookieAttributeName::HttpOnly(s)),
         "samesite" => Ok(CookieAttributeName::SameSite(s)),
-        _ => Err(Error {
-            pos: start,
-            recoverable: false,
-            inner: ParseError::InvalidCookieAttribute {},
-        }),
+        _ => Err(Error::new(start, false, ParseError::InvalidCookieAttribute)),
     }
 }
 
@@ -83,7 +79,7 @@ mod tests {
                         value: "cookie1".to_string(),
                         encoded: "cookie1".to_string(),
                     }],
-                    source_info: SourceInfo::new(1, 1, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 8)),
                 },
                 attribute: None,
             }
@@ -103,17 +99,17 @@ mod tests {
                         value: "cookie1".to_string(),
                         encoded: "cookie1".to_string(),
                     }],
-                    source_info: SourceInfo::new(1, 1, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 8)),
                 },
                 attribute: Some(CookieAttribute {
                     space0: Whitespace {
-                        value: String::from(""),
-                        source_info: SourceInfo::new(1, 9, 1, 9),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 9)),
                     },
                     name: CookieAttributeName::Domain("Domain".to_string()),
                     space1: Whitespace {
-                        value: String::from(""),
-                        source_info: SourceInfo::new(1, 15, 1, 15),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 15)),
                     },
                 }),
             }
@@ -131,29 +127,29 @@ mod tests {
                     delimiter: None,
                     elements: vec![TemplateElement::Expression(Expr {
                         space0: Whitespace {
-                            value: String::from(""),
-                            source_info: SourceInfo::new(1, 3, 1, 3),
+                            value: String::new(),
+                            source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 3)),
                         },
                         variable: Variable {
                             name: "name".to_string(),
-                            source_info: SourceInfo::new(1, 3, 1, 7),
+                            source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 7)),
                         },
                         space1: Whitespace {
-                            value: String::from(""),
-                            source_info: SourceInfo::new(1, 7, 1, 7),
+                            value: String::new(),
+                            source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 7)),
                         },
                     })],
-                    source_info: SourceInfo::new(1, 1, 1, 9),
+                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 9)),
                 },
                 attribute: Some(CookieAttribute {
                     space0: Whitespace {
-                        value: String::from(""),
-                        source_info: SourceInfo::new(1, 10, 1, 10),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 10)),
                     },
                     name: CookieAttributeName::Domain("Domain".to_string()),
                     space1: Whitespace {
-                        value: String::from(""),
-                        source_info: SourceInfo::new(1, 16, 1, 16),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 16), Pos::new(1, 16)),
                     },
                 }),
             }
@@ -166,13 +162,13 @@ mod tests {
         let mut reader = Reader::new("cookie1[");
         let error = cookiepath(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 9 });
-        assert_eq!(error.inner, ParseError::InvalidCookieAttribute {});
+        assert_eq!(error.inner, ParseError::InvalidCookieAttribute);
         assert!(!error.recoverable);
 
         let mut reader = Reader::new("cookie1[{{field]");
         let error = cookiepath(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 9 });
-        assert_eq!(error.inner, ParseError::InvalidCookieAttribute {});
+        assert_eq!(error.inner, ParseError::InvalidCookieAttribute);
         assert!(!error.recoverable);
     }
 
@@ -195,6 +191,6 @@ mod tests {
         let mut reader = Reader::new("unknown");
         let error = cookiepath_attribute_name(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(error.inner, ParseError::InvalidCookieAttribute {});
+        assert_eq!(error.inner, ParseError::InvalidCookieAttribute);
     }
 }
