@@ -21,15 +21,12 @@ use crate::parser::primitives::*;
 use crate::parser::reader::Reader;
 use crate::parser::ParseResult;
 
-pub fn parse(reader: &mut Reader) -> ParseResult<'static, Expr> {
-    // let start = p.state.clone();
-
+pub fn parse(reader: &mut Reader) -> ParseResult<Expr> {
     try_literal("{{", reader)?;
     let space0 = zero_or_more_spaces(reader)?;
     let variable = variable_name(reader)?;
     let space1 = zero_or_more_spaces(reader)?;
 
-    //literal("}}", p)?;
     if try_literal("}}}", reader).is_err() {
         literal("}}", reader)?
     }
@@ -41,9 +38,7 @@ pub fn parse(reader: &mut Reader) -> ParseResult<'static, Expr> {
     })
 }
 
-pub fn parse2(reader: &mut Reader) -> ParseResult<'static, Expr> {
-    // let start = p.state.clone();
-
+pub fn parse2(reader: &mut Reader) -> ParseResult<Expr> {
     let space0 = zero_or_more_spaces(reader)?;
     let variable = variable_name(reader)?;
     let space1 = zero_or_more_spaces(reader)?;
@@ -55,24 +50,15 @@ pub fn parse2(reader: &mut Reader) -> ParseResult<'static, Expr> {
     })
 }
 
-fn variable_name(reader: &mut Reader) -> ParseResult<'static, Variable> {
-    let start = reader.state.clone();
+fn variable_name(reader: &mut Reader) -> ParseResult<Variable> {
+    let start = reader.state;
     let name = reader.read_while(|c| c.is_alphanumeric() || *c == '_' || *c == '-');
     if name.is_empty() {
-        return Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::TemplateVariable {},
-        });
+        return Err(Error::new(start.pos, false, ParseError::TemplateVariable));
     }
     Ok(Variable {
         name,
-        source_info: SourceInfo::new(
-            start.pos.line,
-            start.pos.column,
-            reader.state.pos.line,
-            reader.state.pos.column,
-        ),
+        source_info: SourceInfo::new(start.pos, reader.state.pos),
     })
 }
 
@@ -89,15 +75,15 @@ mod tests {
             Expr {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 3, 1, 4),
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
                 variable: Variable {
                     name: String::from("name"),
-                    source_info: SourceInfo::new(1, 4, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 8)),
                 },
                 space1: Whitespace {
-                    value: String::from(""),
-                    source_info: SourceInfo::new(1, 8, 1, 8),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 8)),
                 },
             }
         );
@@ -138,7 +124,7 @@ mod tests {
             variable_name(&mut reader).unwrap(),
             Variable {
                 name: String::from("name"),
-                source_info: SourceInfo::new(1, 1, 1, 5),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 5)),
             }
         );
 
@@ -147,7 +133,7 @@ mod tests {
             variable_name(&mut reader).unwrap(),
             Variable {
                 name: String::from("my-id"),
-                source_info: SourceInfo::new(1, 1, 1, 6),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6)),
             }
         );
     }
