@@ -17,12 +17,13 @@
  */
 use crate::ast::{JsonListElement, JsonObjectElement, JsonValue, Pos, SourceInfo, Template};
 use crate::parser::combinators::*;
+use crate::parser::error::*;
 use crate::parser::primitives::*;
 use crate::parser::reader::*;
 use crate::parser::template::*;
-use crate::parser::{error, expr, ParseResult};
+use crate::parser::{expr, ParseResult};
 
-pub fn parse(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
+pub fn parse(reader: &mut Reader) -> ParseResult<JsonValue> {
     choice(
         &[
             null_value,
@@ -37,26 +38,51 @@ pub fn parse(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
     )
 }
 
-pub fn null_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
-    try_literal("null", reader)?;
-    Ok(JsonValue::Null {})
+/// Helper for parse, but already knowing that we are inside a JSON body.
+fn parse_in_json(reader: &mut Reader) -> ParseResult<JsonValue> {
+    if let Some(c) = reader.peek() {
+        if c == ',' {
+            let inner = ParseError::Json(JsonErrorVariant::EmptyElement);
+            return Err(Error::new(reader.state.pos, false, inner));
+        }
+    }
+    match parse(reader) {
+        Ok(r) => Ok(r),
+        // The only error that is recoverable is caused by reaching object_value try_literal('{'),
+        // but this is not recoverable in this case, because we already know that we are in a JSON
+        // body. So, we change the error to CannotResolve for the object found.
+        Err(e) => match e {
+            Error {
+                recoverable: true, ..
+            } => {
+                let inner = ParseError::Json(JsonErrorVariant::ExpectingElement);
+                Err(Error::new(e.pos, false, inner))
+            }
+            _ => Err(Error::new(e.pos, false, e.inner)),
+        },
+    }
 }
 
-pub fn boolean_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
+pub fn null_value(reader: &mut Reader) -> ParseResult<JsonValue> {
+    try_literal("null", reader)?;
+    Ok(JsonValue::Null)
+}
+
+pub fn boolean_value(reader: &mut Reader) -> ParseResult<JsonValue> {
     let value = boolean(reader)?;
     Ok(JsonValue::Boolean(value))
 }
 
-fn string_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
+fn string_value(reader: &mut Reader) -> ParseResult<JsonValue> {
     let template = string_template(reader)?;
     Ok(JsonValue::String(template))
 }
 
-fn string_template(reader: &mut Reader) -> ParseResult<'static, Template> {
+fn string_template(reader: &mut Reader) -> ParseResult<Template> {
     try_literal("\"", reader)?;
     let delimiter = Some('"');
     let mut chars = vec![];
-    let start = reader.state.pos.clone();
+    let start = reader.state.pos;
     loop {
         if reader.peek() == Some('"') || reader.is_eof() {
             break;
@@ -64,13 +90,10 @@ fn string_template(reader: &mut Reader) -> ParseResult<'static, Template> {
         let char = any_char(reader)?;
         chars.push(char);
     }
-    let end = reader.state.pos.clone();
+    let end = reader.state.pos;
 
     let encoded_string = EncodedString {
-        source_info: SourceInfo {
-            start: start.clone(),
-            end: end.clone(),
-        },
+        source_info: SourceInfo { start, end },
         chars,
     };
     literal("\"", reader)?;
@@ -84,30 +107,26 @@ fn string_template(reader: &mut Reader) -> ParseResult<'static, Template> {
     Ok(template)
 }
 
-fn any_char(reader: &mut Reader) -> ParseResult<'static, (char, String, Pos)> {
-    let start = reader.state.clone();
+fn any_char(reader: &mut Reader) -> ParseResult<(char, String, Pos)> {
+    let start = reader.state;
     match escape_char(reader) {
         Ok(c) => Ok((c, reader.peek_back(start.cursor), start.pos)),
         Err(e) => {
             if e.recoverable {
-                reader.state = start.clone();
+                reader.state = start;
                 match reader.read() {
-                    None => Err(error::Error {
-                        pos: start.pos,
-                        recoverable: true,
-                        inner: error::ParseError::Expecting {
+                    None => {
+                        let inner = ParseError::Expecting {
                             value: "char".to_string(),
-                        },
-                    }),
+                        };
+                        Err(Error::new(start.pos, true, inner))
+                    }
                     Some(c) => {
-                        if vec!['\\', '\x08', '\n', '\x0c', '\r', '\t'].contains(&c) {
-                            Err(error::Error {
-                                pos: start.pos,
-                                recoverable: true,
-                                inner: error::ParseError::Expecting {
-                                    value: "char".to_string(),
-                                },
-                            })
+                        if ['\\', '\x08', '\n', '\x0c', '\r', '\t'].contains(&c) {
+                            let inner = ParseError::Expecting {
+                                value: "char".to_string(),
+                            };
+                            Err(Error::new(start.pos, true, inner))
                         } else {
                             Ok((c, reader.peek_back(start.cursor), start.pos))
                         }
@@ -120,9 +139,9 @@ fn any_char(reader: &mut Reader) -> ParseResult<'static, (char, String, Pos)> {
     }
 }
 
-fn escape_char(reader: &mut Reader) -> ParseResult<'static, char> {
+fn escape_char(reader: &mut Reader) -> ParseResult<char> {
     try_literal("\\", reader)?;
-    let start = reader.state.clone();
+    let start = reader.state;
     match reader.read() {
         Some('"') => Ok('"'),
         Some('\\') => Ok('\\'),
@@ -133,43 +152,66 @@ fn escape_char(reader: &mut Reader) -> ParseResult<'static, char> {
         Some('r') => Ok('\r'),
         Some('t') => Ok('\t'),
         Some('u') => unicode(reader),
-        _ => Err(error::Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: error::ParseError::EscapeChar {},
-        }),
+        _ => Err(Error::new(start.pos, false, ParseError::EscapeChar)),
     }
 }
 
-fn unicode(reader: &mut Reader) -> ParseResult<'static, char> {
-    let v = hex_value(reader)?;
-    let c = match std::char::from_u32(v) {
-        None => {
-            return Err(error::Error {
-                pos: reader.clone().state.pos,
-                recoverable: false,
-                inner: error::ParseError::Unicode {},
-            })
+fn unicode(reader: &mut Reader) -> ParseResult<char> {
+    let start = reader.state.pos;
+    let cp1 = hex_value(reader)?;
+    let cp = if is_surrogate(cp1) {
+        literal("\\u", reader)?;
+        let start = reader.state.pos;
+        let cp2 = hex_value(reader)?;
+        match cp_surrogate_pair(cp1, cp2) {
+            None => return Err(Error::new(start, false, ParseError::Unicode)),
+            Some(cp) => cp,
         }
+    } else {
+        cp1
+    };
+    let c = match char::from_u32(cp) {
+        None => return Err(Error::new(start, false, ParseError::Unicode)),
         Some(c) => c,
     };
     Ok(c)
 }
 
-fn hex_value(reader: &mut Reader) -> ParseResult<'static, u32> {
+// 0xd800-0xdc00 encodes the high 10 bits of a pair.
+// 0xdc00-0xe000 encodes the low 10 bits of a pair.
+// the value is those 20 bits plus 0x10000.
+const SURR1: u32 = 0xd800;
+const SURR2: u32 = 0xdc00;
+const SURR3: u32 = 0xe000;
+const SURR_SELF: u32 = 0x10000;
+
+/// Returns whether the specified Unicode code point can appear in a surrogate pair.
+fn is_surrogate(cp: u32) -> bool {
+    (SURR1..SURR3).contains(&cp)
+}
+
+fn cp_surrogate_pair(cp1: u32, cp2: u32) -> Option<u32> {
+    if (SURR1..SURR2).contains(&cp1) && (SURR2..SURR3).contains(&cp2) {
+        Some(((cp1 - SURR1) << 10) | ((cp2 - SURR2) + SURR_SELF))
+    } else {
+        None
+    }
+}
+
+fn hex_value(reader: &mut Reader) -> ParseResult<u32> {
     let digit1 = nonrecover(hex_digit, reader)?;
     let digit2 = nonrecover(hex_digit, reader)?;
     let digit3 = nonrecover(hex_digit, reader)?;
     let digit4 = nonrecover(hex_digit, reader)?;
-    let value = digit1 * (16 ^ 3) + digit2 * (16 ^ 2) + digit3 * 16 + digit4;
+    let value = digit1 * (16 * 16 * 16) + digit2 * (16 * 16) + digit3 * 16 + digit4;
     Ok(value)
 }
 
-pub fn number_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
-    let start = reader.state.clone();
+pub fn number_value(reader: &mut Reader) -> ParseResult<JsonValue> {
+    let start = reader.state;
 
     let sign = match try_literal("-", reader) {
-        Err(_) => "".to_string(),
+        Err(_) => String::new(),
         Ok(_) => "-".to_string(),
     };
 
@@ -177,13 +219,10 @@ pub fn number_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
         Err(_) => {
             let digits = reader.read_while(|c| c.is_ascii_digit());
             if digits.is_empty() {
-                return Err(error::Error {
-                    pos: start.pos,
-                    recoverable: true,
-                    inner: error::ParseError::Expecting {
-                        value: "number".to_string(),
-                    },
-                });
+                let inner = ParseError::Expecting {
+                    value: "number".to_string(),
+                };
+                return Err(Error::new(start.pos, true, inner));
             } else {
                 digits
             }
@@ -195,18 +234,15 @@ pub fn number_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
         Ok(_) => {
             let digits = reader.read_while(|c| c.is_ascii_digit());
             if digits.is_empty() {
-                return Err(error::Error {
-                    pos: reader.state.pos.clone(),
-                    recoverable: false,
-                    inner: error::ParseError::Expecting {
-                        value: "digits".to_string(),
-                    },
-                });
+                let inner = ParseError::Expecting {
+                    value: "digits".to_string(),
+                };
+                return Err(Error::new(reader.state.pos, false, inner));
             } else {
                 format!(".{digits}")
             }
         }
-        Err(_) => "".to_string(),
+        Err(_) => String::new(),
     };
 
     let exponent = if reader.peek() == Some('e') || reader.peek() == Some('E') {
@@ -215,13 +251,13 @@ pub fn number_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
             Ok(_) => "-".to_string(),
             Err(_) => match try_literal("+", reader) {
                 Ok(_) => "+".to_string(),
-                Err(_) => "".to_string(),
+                Err(_) => String::new(),
             },
         };
         let exponent_digits = reader.read_while(|c| c.is_ascii_digit());
         format!("e{exponent_sign}{exponent_digits}")
     } else {
-        "".to_string()
+        String::new()
     };
 
     Ok(JsonValue::Number(format!(
@@ -229,12 +265,12 @@ pub fn number_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
     )))
 }
 
-fn expression_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
+fn expression_value(reader: &mut Reader) -> ParseResult<JsonValue> {
     let exp = expr::parse(reader)?;
     Ok(JsonValue::Expression(exp))
 }
 
-fn list_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
+fn list_value(reader: &mut Reader) -> ParseResult<JsonValue> {
     try_literal("[", reader)?;
     let space0 = whitespace(reader);
     let mut elements = vec![];
@@ -248,10 +284,21 @@ fn list_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
             if reader.peek() == Some(']') {
                 break;
             }
+            // Reports "expecting ']'" in case the user forgot to add the last ']', e.g
+            // `[1, 2`
             if reader.peek() != Some(',') {
                 break;
             }
+            // The reader advances after literal(","), so this saves its position to report an
+            // error in case it happens.
+            let save = reader.state.pos;
             literal(",", reader)?;
+            // If there is one more comma, e.g. [1, 2,], it's better to report to the user because
+            // this occurrence is common.
+            if reader.peek_ignoring_whitespace() == Some(']') {
+                let inner = ParseError::Json(JsonErrorVariant::TrailingComma);
+                return Err(Error::new(save, false, inner));
+            }
             let element = list_element(reader)?;
             elements.push(element);
         }
@@ -261,19 +308,9 @@ fn list_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
     Ok(JsonValue::List { space0, elements })
 }
 
-fn list_element(reader: &mut Reader) -> ParseResult<'static, JsonListElement> {
-    let save = reader.state.pos.clone();
+fn list_element(reader: &mut Reader) -> ParseResult<JsonListElement> {
     let space0 = whitespace(reader);
-    let value = match parse(reader) {
-        Ok(r) => r,
-        Err(_) => {
-            return Err(error::Error {
-                pos: save,
-                recoverable: false,
-                inner: error::ParseError::Json {},
-            })
-        }
-    };
+    let value = parse_in_json(reader)?;
     let space1 = whitespace(reader);
     Ok(JsonListElement {
         space0,
@@ -282,7 +319,7 @@ fn list_element(reader: &mut Reader) -> ParseResult<'static, JsonListElement> {
     })
 }
 
-pub fn object_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
+pub fn object_value(reader: &mut Reader) -> ParseResult<JsonValue> {
     try_literal("{", reader)?;
     let space0 = whitespace(reader);
     let mut elements = vec![];
@@ -294,10 +331,21 @@ pub fn object_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
             if reader.peek() == Some('}') {
                 break;
             }
+            // Reports "expecting '}'" in case the user forgot to add the last '}', e.g
+            // `{"name": "abc"`
             if reader.peek() != Some(',') {
                 break;
             }
+            // The reader advances after literal(","), so this saves its position to report an
+            // error in case it happens.
+            let save = reader.state.pos;
             literal(",", reader)?;
+            // If there is one more comma, e.g. {"a": "b",}, it's better to report to the user
+            // because this occurrence is common.
+            if reader.peek_ignoring_whitespace() == Some('}') {
+                let inner = ParseError::Json(JsonErrorVariant::TrailingComma);
+                return Err(Error::new(save, false, inner));
+            }
             let element = object_element(reader)?;
             elements.push(element);
         }
@@ -310,38 +358,35 @@ pub fn object_value(reader: &mut Reader) -> ParseResult<'static, JsonValue> {
     Ok(JsonValue::Object { space0, elements })
 }
 
-fn key(reader: &mut Reader) -> ParseResult<'static, Template> {
-    let save = reader.state.clone();
+fn key(reader: &mut Reader) -> ParseResult<Template> {
+    let save = reader.state;
     let name = string_template(reader).map_err(|e| e.non_recoverable())?;
     if name.elements.is_empty() {
-        Err(error::Error {
-            pos: save.pos,
-            recoverable: false,
-            inner: error::ParseError::Json {},
-        })
+        let inner = ParseError::Json(JsonErrorVariant::EmptyElement);
+        Err(Error::new(save.pos, false, inner))
     } else {
         Ok(name)
     }
 }
-fn object_element(reader: &mut Reader) -> ParseResult<'static, JsonObjectElement> {
+
+fn object_element(reader: &mut Reader) -> ParseResult<JsonObjectElement> {
     let space0 = whitespace(reader);
     //literal("\"", reader)?;
     let name = key(reader)?;
     //literal("\"", reader)?;
     let space1 = whitespace(reader);
     literal(":", reader)?;
-    let save = reader.state.pos.clone();
+    let save = reader.state.pos;
     let space2 = whitespace(reader);
-    let value = match parse(reader) {
-        Ok(r) => r,
-        Err(_) => {
-            return Err(error::Error {
-                pos: save,
-                recoverable: false,
-                inner: error::ParseError::Json {},
-            })
-        }
-    };
+    // Checks if there is no element after ':'. In this case, a special error must be reported
+    // because this is a common occurance.
+    let next_char = reader.peek();
+    // Comparing to None because `next_char` can be EOF.
+    if next_char == Some('}') || next_char.is_none() {
+        let inner = ParseError::Json(JsonErrorVariant::EmptyElement);
+        return Err(Error::new(save, false, inner));
+    }
+    let value = parse_in_json(reader)?;
     let space3 = whitespace(reader);
     Ok(JsonObjectElement {
         space0,
@@ -367,20 +412,26 @@ mod tests {
         let mut reader = Reader::new("{ \"a\":\n}");
         let error = parse(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 7 });
-        assert_eq!(error.inner, error::ParseError::Json {});
+        assert_eq!(
+            error.inner,
+            ParseError::Json(JsonErrorVariant::EmptyElement)
+        );
         assert!(!error.recoverable);
 
         let mut reader = Reader::new("[0,1,]");
         let error = parse(&mut reader).err().unwrap();
-        assert_eq!(error.pos, Pos { line: 1, column: 6 });
-        assert_eq!(error.inner, error::ParseError::Json {});
+        assert_eq!(error.pos, Pos { line: 1, column: 5 });
+        assert_eq!(
+            error.inner,
+            ParseError::Json(JsonErrorVariant::TrailingComma),
+        );
         assert!(!error.recoverable);
     }
 
     #[test]
     fn test_null_value() {
         let mut reader = Reader::new("null");
-        assert_eq!(null_value(&mut reader).unwrap(), JsonValue::Null {});
+        assert_eq!(null_value(&mut reader).unwrap(), JsonValue::Null);
         assert_eq!(reader.state.cursor, 4);
 
         let mut reader = Reader::new("true");
@@ -388,7 +439,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "null".to_string()
             }
         );
@@ -409,7 +460,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "true|false".to_string()
             }
         );
@@ -427,16 +478,16 @@ mod tests {
                 },
                 TemplateElement::Expression(Expr {
                     space0: Whitespace {
-                        value: "".to_string(),
-                        source_info: SourceInfo::new(1, 15, 1, 15),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 15)),
                     },
                     variable: Variable {
                         name: "name".to_string(),
-                        source_info: SourceInfo::new(1, 15, 1, 19),
+                        source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 19)),
                     },
                     space1: Whitespace {
-                        value: "".to_string(),
-                        source_info: SourceInfo::new(1, 19, 1, 19),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 19), Pos::new(1, 19)),
                     },
                 }),
                 TemplateElement::String {
@@ -444,7 +495,7 @@ mod tests {
                     encoded: "!".to_string(),
                 },
             ],
-            source_info: SourceInfo::new(1, 2, 1, 22),
+            source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 22)),
         })
     }
 
@@ -456,7 +507,7 @@ mod tests {
             JsonValue::String(Template {
                 delimiter: Some('"'),
                 elements: vec![],
-                source_info: SourceInfo::new(1, 2, 1, 2),
+                source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 2)),
             })
         );
         assert_eq!(reader.state.cursor, 2);
@@ -474,7 +525,7 @@ mod tests {
                     value: "{}".to_string(),
                     encoded: "{}".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 2, 1, 4),
+                source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 4)),
             })
         );
         assert_eq!(reader.state.cursor, 4);
@@ -487,7 +538,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "\"".to_string()
             }
         );
@@ -498,7 +549,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 3 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "\"".to_string()
             }
         );
@@ -509,7 +560,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 5 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "}}".to_string()
             }
         );
@@ -582,7 +633,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "\\".to_string()
             }
         );
@@ -595,6 +646,31 @@ mod tests {
         let mut reader = Reader::new("000a");
         assert_eq!(unicode(&mut reader).unwrap(), '\n');
         assert_eq!(reader.state.cursor, 4);
+
+        let mut reader = Reader::new("c350");
+        assert_eq!(unicode(&mut reader).unwrap(), '썐');
+        assert_eq!(reader.state.cursor, 4);
+
+        let mut reader = Reader::new("d83c\\udf78");
+        assert_eq!(unicode(&mut reader).unwrap(), '🍸');
+        assert_eq!(reader.state.cursor, 10);
+
+        let mut reader = Reader::new("d800");
+        let error = unicode(&mut reader).unwrap_err();
+        assert_eq!(error.pos, Pos { line: 1, column: 5 });
+        assert_eq!(
+            error.inner,
+            ParseError::Expecting {
+                value: "\\u".to_string()
+            }
+        );
+        assert!(!error.recoverable);
+
+        let mut reader = Reader::new("d800\\ud800");
+        let error = unicode(&mut reader).unwrap_err();
+        assert_eq!(error.pos, Pos { line: 1, column: 7 });
+        assert_eq!(error.inner, ParseError::Unicode);
+        assert!(!error.recoverable);
     }
 
     #[test]
@@ -602,10 +678,13 @@ mod tests {
         let mut reader = Reader::new("0020x");
         assert_eq!(hex_value(&mut reader).unwrap(), 32);
 
+        let mut reader = Reader::new("d800");
+        assert_eq!(hex_value(&mut reader).unwrap(), 55296);
+
         let mut reader = Reader::new("x");
-        let error = hex_value(&mut reader).err().unwrap();
+        let error = hex_value(&mut reader).unwrap_err();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(error.inner, error::ParseError::HexDigit);
+        assert_eq!(error.inner, ParseError::HexDigit);
         assert!(!error.recoverable);
     }
 
@@ -668,7 +747,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "number".to_string()
             }
         );
@@ -679,7 +758,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 3 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "digits".to_string()
             }
         );
@@ -693,16 +772,16 @@ mod tests {
             expression_value(&mut reader).unwrap(),
             JsonValue::Expression(Expr {
                 space0: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 3, 1, 3)
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 3))
                 },
                 variable: Variable {
                     name: "n".to_string(),
-                    source_info: SourceInfo::new(1, 3, 1, 4)
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4))
                 },
                 space1: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 4, 1, 4)
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4))
                 }
             })
         );
@@ -715,7 +794,7 @@ mod tests {
         assert_eq!(
             list_value(&mut reader).unwrap(),
             JsonValue::List {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![]
             }
         );
@@ -731,19 +810,26 @@ mod tests {
         );
         assert_eq!(reader.state.cursor, 3);
 
-        let mut reader = Reader::new("[true]");
+        let mut reader = Reader::new("[true, false]");
         assert_eq!(
             list_value(&mut reader).unwrap(),
             JsonValue::List {
-                space0: "".to_string(),
-                elements: vec![JsonListElement {
-                    space0: "".to_string(),
-                    value: JsonValue::Boolean(true),
-                    space1: "".to_string(),
-                }],
+                space0: String::new(),
+                elements: vec![
+                    JsonListElement {
+                        space0: String::new(),
+                        value: JsonValue::Boolean(true),
+                        space1: String::new(),
+                    },
+                    JsonListElement {
+                        space0: String::from(" "),
+                        value: JsonValue::Boolean(false),
+                        space1: String::new(),
+                    }
+                ],
             }
         );
-        assert_eq!(reader.state.cursor, 6);
+        assert_eq!(reader.state.cursor, 13);
     }
 
     #[test]
@@ -753,7 +839,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "[".to_string()
             }
         );
@@ -761,8 +847,11 @@ mod tests {
 
         let mut reader = Reader::new("[1, 2,]");
         let error = list_value(&mut reader).err().unwrap();
-        assert_eq!(error.pos, Pos { line: 1, column: 7 });
-        assert_eq!(error.inner, error::ParseError::Json {});
+        assert_eq!(error.pos, Pos { line: 1, column: 6 });
+        assert_eq!(
+            error.inner,
+            ParseError::Json(JsonErrorVariant::TrailingComma),
+        );
         assert!(!error.recoverable);
     }
 
@@ -772,9 +861,9 @@ mod tests {
         assert_eq!(
             list_element(&mut reader).unwrap(),
             JsonListElement {
-                space0: "".to_string(),
+                space0: String::new(),
                 value: JsonValue::Boolean(true),
-                space1: "".to_string(),
+                space1: String::new(),
             }
         );
         assert_eq!(reader.state.cursor, 4);
@@ -786,7 +875,7 @@ mod tests {
         assert_eq!(
             object_value(&mut reader).unwrap(),
             JsonValue::Object {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![]
             }
         );
@@ -808,16 +897,16 @@ mod tests {
             JsonValue::Object {
                 space0: "\n  ".to_string(),
                 elements: vec![JsonObjectElement {
-                    space0: "".to_string(),
+                    space0: String::new(),
                     name: Template {
                         delimiter: Some('"'),
                         elements: vec![TemplateElement::String {
                             value: "a".to_string(),
                             encoded: "a".to_string()
                         }],
-                        source_info: SourceInfo::new(2, 4, 2, 5)
+                        source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(2, 5))
                     },
-                    space1: "".to_string(),
+                    space1: String::new(),
                     space2: " ".to_string(),
                     value: JsonValue::Boolean(true),
                     space3: "\n".to_string(),
@@ -831,7 +920,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "{".to_string()
             }
         );
@@ -843,7 +932,10 @@ mod tests {
         let mut reader = Reader::new("{ \"a\":\n}");
         let error = object_value(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 7 });
-        assert_eq!(error.inner, error::ParseError::Json {});
+        assert_eq!(
+            error.inner,
+            ParseError::Json(JsonErrorVariant::EmptyElement)
+        );
         assert!(!error.recoverable);
     }
 
@@ -853,19 +945,19 @@ mod tests {
         assert_eq!(
             object_element(&mut reader).unwrap(),
             JsonObjectElement {
-                space0: "".to_string(),
+                space0: String::new(),
                 name: Template {
                     delimiter: Some('"'),
                     elements: vec![TemplateElement::String {
                         value: "a".to_string(),
                         encoded: "a".to_string()
                     }],
-                    source_info: SourceInfo::new(1, 2, 1, 3)
+                    source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 3))
                 },
-                space1: "".to_string(),
+                space1: String::new(),
                 space2: " ".to_string(),
                 value: JsonValue::Boolean(true),
-                space3: "".to_string(),
+                space3: String::new(),
             }
         );
         assert_eq!(reader.state.cursor, 9);
@@ -878,7 +970,7 @@ mod tests {
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
             error.inner,
-            error::ParseError::Expecting {
+            ParseError::Expecting {
                 value: "\"".to_string()
             }
         );
@@ -887,14 +979,17 @@ mod tests {
         let mut reader = Reader::new("\"name\":\n");
         let error = object_element(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 8 });
-        assert_eq!(error.inner, error::ParseError::Json {});
+        assert_eq!(
+            error.inner,
+            ParseError::Json(JsonErrorVariant::EmptyElement),
+        );
         assert!(!error.recoverable);
     }
 
     #[test]
     fn test_whitespace() {
         let mut reader = Reader::new("");
-        assert_eq!(whitespace(&mut reader), "".to_string());
+        assert_eq!(whitespace(&mut reader), String::new());
         assert_eq!(reader.state.cursor, 0);
 
         let mut reader = Reader::new(" x");
