@@ -15,15 +15,15 @@
  * limitations under the License.
  *
  */
-pub type ParseResult<'a, T> = Result<T, Error>;
-pub type ParseFunc<'a, T> = fn(&mut Reader) -> ParseResult<'a, T>;
+pub type ParseResult<T> = Result<T, Error>;
+pub type ParseFunc<T> = fn(&mut Reader) -> ParseResult<T>;
 
-pub fn parse_hurl_file(s: &str) -> ParseResult<'static, HurlFile> {
+pub fn parse_hurl_file(s: &str) -> ParseResult<HurlFile> {
     let mut reader = Reader::new(s);
     parsers::hurl_file(&mut reader)
 }
 
-pub use self::error::{Error, ParseError};
+pub use self::error::{Error, JsonErrorVariant, ParseError};
 pub use self::json::{
     boolean_value as parse_json_boolean, null_value as parse_json_null,
     number_value as parse_json_number, parse as parse_json,
@@ -41,7 +41,10 @@ mod expr;
 mod filename;
 mod filter;
 mod json;
+mod key_string;
 mod multiline;
+mod number;
+mod option;
 mod parsers;
 mod predicate;
 mod predicate_value;
