@@ -22,9 +22,9 @@ use crate::parser::primitives::*;
 use crate::parser::reader::Reader;
 use crate::parser::{template, Error, ParseError, ParseResult};
 
-pub fn multiline_string(reader: &mut Reader) -> ParseResult<'static, MultilineString> {
+pub fn multiline_string(reader: &mut Reader) -> ParseResult<MultilineString> {
     try_literal("```", reader)?;
-    let save = reader.state.clone();
+    let save = reader.state;
 
     match choice(&[json_text, xml_text, graphql, plain_text], reader) {
         Ok(multi) => Ok(multi),
@@ -53,7 +53,7 @@ pub fn multiline_string(reader: &mut Reader) -> ParseResult<'static, MultilineSt
     }
 }
 
-fn text(lang: &str, reader: &mut Reader) -> ParseResult<'static, Text> {
+fn text(lang: &str, reader: &mut Reader) -> ParseResult<Text> {
     try_literal(lang, reader)?;
     let space = zero_or_more_spaces(reader)?;
     let newline = newline(reader)?;
@@ -65,30 +65,30 @@ fn text(lang: &str, reader: &mut Reader) -> ParseResult<'static, Text> {
     })
 }
 
-fn json_text(reader: &mut Reader) -> ParseResult<'static, MultilineString> {
+fn json_text(reader: &mut Reader) -> ParseResult<MultilineString> {
     let text = text("json", reader)?;
     Ok(MultilineString::Json(text))
 }
 
-fn xml_text(reader: &mut Reader) -> ParseResult<'static, MultilineString> {
+fn xml_text(reader: &mut Reader) -> ParseResult<MultilineString> {
     let text = text("xml", reader)?;
     Ok(MultilineString::Xml(text))
 }
 
-fn graphql(reader: &mut Reader) -> ParseResult<'static, MultilineString> {
+fn graphql(reader: &mut Reader) -> ParseResult<MultilineString> {
     try_literal("graphql", reader)?;
     let space = zero_or_more_spaces(reader)?;
     let newline = newline(reader)?;
 
     let mut chars = vec![];
 
-    let start = reader.state.pos.clone();
+    let start = reader.state.pos;
     while reader.peek_n(3) != "```" && !reader.is_eof() {
-        let pos = reader.state.pos.clone();
+        let pos = reader.state.pos;
         let c = reader.read().unwrap();
         chars.push((c, c.to_string(), pos));
         if c == '\n' {
-            let end = reader.state.pos.clone();
+            let end = reader.state.pos;
             let variables = optional(graphql_variables, reader)?;
             match variables {
                 None => continue,
@@ -96,10 +96,7 @@ fn graphql(reader: &mut Reader) -> ParseResult<'static, MultilineString> {
                     literal("```", reader)?;
 
                     let encoded_string = template::EncodedString {
-                        source_info: SourceInfo {
-                            start: start.clone(),
-                            end: end.clone(),
-                        },
+                        source_info: SourceInfo { start, end },
                         chars: chars.clone(),
                     };
 
@@ -120,14 +117,11 @@ fn graphql(reader: &mut Reader) -> ParseResult<'static, MultilineString> {
             }
         }
     }
-    let end = reader.state.pos.clone();
+    let end = reader.state.pos;
     literal("```", reader)?;
 
     let encoded_string = template::EncodedString {
-        source_info: SourceInfo {
-            start: start.clone(),
-            end: end.clone(),
-        },
+        source_info: SourceInfo { start, end },
         chars,
     };
 
@@ -146,69 +140,45 @@ fn graphql(reader: &mut Reader) -> ParseResult<'static, MultilineString> {
     }))
 }
 
-fn whitespace(reader: &mut Reader) -> ParseResult<'static, Whitespace> {
-    let start = reader.state.clone();
+fn whitespace(reader: &mut Reader) -> ParseResult<Whitespace> {
+    let start = reader.state;
     match reader.read() {
-        None => Err(Error {
-            pos: start.pos,
-            recoverable: true,
-            inner: ParseError::Space {},
-        }),
+        None => Err(Error::new(start.pos, true, ParseError::Space)),
         Some(c) => {
             if c == ' ' || c == '\t' || c == '\n' || c == '\r' {
                 Ok(Whitespace {
                     value: c.to_string(),
-                    source_info: SourceInfo::new(
-                        start.pos.line,
-                        start.pos.column,
-                        reader.state.pos.line,
-                        reader.state.pos.column,
-                    ),
+                    source_info: SourceInfo::new(start.pos, reader.state.pos),
                 })
             } else {
-                Err(Error {
-                    pos: start.pos,
-                    recoverable: true,
-                    inner: ParseError::Space {},
-                })
+                Err(Error::new(start.pos, true, ParseError::Space))
             }
         }
     }
 }
 
-fn zero_or_more_whitespaces<'a>(reader: &mut Reader) -> ParseResult<'a, Whitespace> {
-    let start = reader.state.clone();
+fn zero_or_more_whitespaces(reader: &mut Reader) -> ParseResult<Whitespace> {
+    let start = reader.state;
     match zero_or_more(whitespace, reader) {
         Ok(v) => {
             let s = v.iter().map(|x| x.value.clone()).collect();
             Ok(Whitespace {
                 value: s,
-                source_info: SourceInfo::new(
-                    start.pos.line,
-                    start.pos.column,
-                    reader.state.pos.line,
-                    reader.state.pos.column,
-                ),
+                source_info: SourceInfo::new(start.pos, reader.state.pos),
             })
         }
         Err(e) => Err(e),
     }
 }
 
-fn graphql_variables(reader: &mut Reader) -> ParseResult<'static, GraphQlVariables> {
+fn graphql_variables(reader: &mut Reader) -> ParseResult<GraphQlVariables> {
     try_literal("variables", reader)?;
     let space = zero_or_more_spaces(reader)?;
-    let start = reader.state.clone();
+    let start = reader.state;
     let object = object_value(reader);
     let value = match object {
         Ok(obj) => obj,
-        Err(_) => {
-            return Err(Error {
-                pos: start.pos,
-                recoverable: false,
-                inner: ParseError::GraphQlVariables,
-            });
-        }
+        Err(_) => return Err(Error::new(start.pos, false, ParseError::GraphQlVariables)),
     };
     let whitespace = zero_or_more_whitespaces(reader)?;
     Ok(GraphQlVariables {
@@ -218,7 +188,7 @@ fn graphql_variables(reader: &mut Reader) -> ParseResult<'static, GraphQlVariabl
     })
 }
 
-fn plain_text(reader: &mut Reader) -> ParseResult<'static, MultilineString> {
+fn plain_text(reader: &mut Reader) -> ParseResult<MultilineString> {
     let space = zero_or_more_spaces(reader)?;
     let newline = newline(reader)?;
     let value = multiline_string_value(reader)?;
@@ -229,23 +199,20 @@ fn plain_text(reader: &mut Reader) -> ParseResult<'static, MultilineString> {
     }))
 }
 
-fn multiline_string_value(reader: &mut Reader) -> ParseResult<'static, Template> {
+fn multiline_string_value(reader: &mut Reader) -> ParseResult<Template> {
     let mut chars = vec![];
 
-    let start = reader.state.pos.clone();
+    let start = reader.state.pos;
     while reader.peek_n(3) != "```" && !reader.is_eof() {
-        let pos = reader.state.pos.clone();
+        let pos = reader.state.pos;
         let c = reader.read().unwrap();
         chars.push((c, c.to_string(), pos));
     }
-    let end = reader.state.pos.clone();
+    let end = reader.state.pos;
     literal("```", reader)?;
 
     let encoded_string = template::EncodedString {
-        source_info: SourceInfo {
-            start: start.clone(),
-            end: end.clone(),
-        },
+        source_info: SourceInfo { start, end },
         chars,
     };
 
@@ -258,30 +225,23 @@ fn multiline_string_value(reader: &mut Reader) -> ParseResult<'static, Template>
     })
 }
 
-fn oneline_string_value(reader: &mut Reader) -> ParseResult<'static, Template> {
+fn oneline_string_value(reader: &mut Reader) -> ParseResult<Template> {
     let mut chars = vec![];
 
-    let start = reader.state.pos.clone();
+    let start = reader.state.pos;
     while reader.peek_n(3) != "```" && !reader.is_eof() {
-        let pos = reader.state.pos.clone();
+        let pos = reader.state.pos;
         let c = reader.read().unwrap();
         if c == '\n' {
-            return Err(Error {
-                pos: start,
-                recoverable: false,
-                inner: ParseError::Multiline,
-            });
+            return Err(Error::new(start, false, ParseError::Multiline));
         }
         chars.push((c, c.to_string(), pos));
     }
-    let end = reader.state.pos.clone();
+    let end = reader.state.pos;
     literal("```", reader)?;
 
     let encoded_string = template::EncodedString {
-        source_info: SourceInfo {
-            start: start.clone(),
-            end: end.clone(),
-        },
+        source_info: SourceInfo { start, end },
         chars,
     };
 
@@ -305,12 +265,12 @@ mod tests {
             multiline_string(&mut reader).unwrap(),
             MultilineString::Text(Text {
                 space: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 4, 1, 4),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 4, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
@@ -318,7 +278,7 @@ mod tests {
                         value: "line1\nline2\nline3\n".to_string(),
                         encoded: "line1\nline2\nline3\n".to_string(),
                     }],
-                    source_info: SourceInfo::new(2, 1, 5, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
                 },
             })
         );
@@ -329,11 +289,11 @@ mod tests {
             MultilineString::Text(Text {
                 space: Whitespace {
                     value: "         ".to_string(),
-                    source_info: SourceInfo::new(1, 4, 1, 13),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 13)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 13, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 13), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
@@ -341,7 +301,7 @@ mod tests {
                         value: "line1\nline2\nline3\n".to_string(),
                         encoded: "line1\nline2\nline3\n".to_string(),
                     }],
-                    source_info: SourceInfo::new(2, 1, 5, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
                 },
             })
         );
@@ -354,12 +314,12 @@ mod tests {
             multiline_string(&mut reader).unwrap(),
             MultilineString::Json(Text {
                 space: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 8, 1, 8),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 8)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 8, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
@@ -367,7 +327,7 @@ mod tests {
                         value: "line1\nline2\nline3\n".to_string(),
                         encoded: "line1\nline2\nline3\n".to_string(),
                     }],
-                    source_info: SourceInfo::new(2, 1, 5, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
                 },
             })
         );
@@ -380,12 +340,12 @@ mod tests {
             multiline_string(&mut reader).unwrap(),
             MultilineString::GraphQl(GraphQl {
                 space: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 11, 1, 11),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 11)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 11, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
@@ -393,7 +353,7 @@ mod tests {
                         value: "line1\nline2\nline3\n".to_string(),
                         encoded: "line1\nline2\nline3\n".to_string(),
                     }],
-                    source_info: SourceInfo::new(2, 1, 5, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
                 },
                 variables: None,
             })
@@ -405,11 +365,11 @@ mod tests {
             MultilineString::GraphQl(GraphQl {
                 space: Whitespace {
                     value: "      ".to_string(),
-                    source_info: SourceInfo::new(1, 11, 1, 17),
+                    source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 17)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 17, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 17), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
@@ -417,7 +377,7 @@ mod tests {
                         value: "line1\nline2\nline3\n".to_string(),
                         encoded: "line1\nline2\nline3\n".to_string(),
                     }],
-                    source_info: SourceInfo::new(2, 1, 5, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
                 },
                 variables: None,
             })
@@ -445,7 +405,7 @@ mod tests {
             MultilineString::OneLineText(Template {
                 delimiter: None,
                 elements: vec![],
-                source_info: SourceInfo::new(1, 4, 1, 4),
+                source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
             })
         );
 
@@ -454,17 +414,17 @@ mod tests {
             multiline_string(&mut reader).unwrap(),
             MultilineString::Text(Text {
                 space: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 4, 1, 4),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 4, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
                     elements: vec![],
-                    source_info: SourceInfo::new(2, 1, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1)),
                 },
             })
         );
@@ -473,17 +433,17 @@ mod tests {
             multiline_string(&mut reader).unwrap(),
             MultilineString::Text(Text {
                 space: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 4, 1, 4),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
                 },
                 newline: Whitespace {
                     value: "\r\n".to_string(),
-                    source_info: SourceInfo::new(1, 4, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
                     elements: vec![],
-                    source_info: SourceInfo::new(2, 1, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1)),
                 },
             })
         );
@@ -500,7 +460,7 @@ mod tests {
                     value: "Hello World!".to_string(),
                     encoded: "Hello World!".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 4, 1, 16),
+                source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 16)),
             })
         );
     }
@@ -516,7 +476,7 @@ mod tests {
                     value: "base64_inline".to_string(),
                     encoded: "base64_inline".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 4, 1, 17),
+                source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 17)),
             })
         );
     }
@@ -528,12 +488,12 @@ mod tests {
             multiline_string(&mut reader).unwrap(),
             MultilineString::Text(Text {
                 space: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 4, 1, 4),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 4, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
@@ -541,7 +501,7 @@ mod tests {
                         value: "line1\nline2\nline3\n".to_string(),
                         encoded: "line1\nline2\nline3\n".to_string(),
                     }],
-                    source_info: SourceInfo::new(2, 1, 5, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
                 },
             })
         );
@@ -554,12 +514,12 @@ mod tests {
             multiline_string(&mut reader).unwrap(),
             MultilineString::Text(Text {
                 space: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 4, 1, 4),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 4, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
@@ -567,7 +527,7 @@ mod tests {
                         value: "\n".to_string(),
                         encoded: "\n".to_string(),
                     }],
-                    source_info: SourceInfo::new(2, 1, 3, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(3, 1)),
                 },
             })
         );
@@ -578,12 +538,12 @@ mod tests {
             multiline_string(&mut reader).unwrap(),
             MultilineString::Text(Text {
                 space: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 4, 1, 4),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 4, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
@@ -591,7 +551,7 @@ mod tests {
                         value: "\r\n".to_string(),
                         encoded: "\r\n".to_string(),
                     }],
-                    source_info: SourceInfo::new(2, 1, 3, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(3, 1)),
                 },
             })
         );
@@ -636,7 +596,7 @@ mod tests {
             Template {
                 delimiter: None,
                 elements: vec![],
-                source_info: SourceInfo::new(1, 1, 1, 1),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             }
         );
         assert_eq!(reader.state.cursor, 3);
@@ -650,7 +610,7 @@ mod tests {
                     value: "hello".to_string(),
                     encoded: "hello".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 6),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6)),
             }
         );
         assert_eq!(reader.state.cursor, 8);
@@ -676,12 +636,12 @@ variables {
             multiline_string(&mut reader).unwrap(),
             MultilineString::GraphQl(GraphQl {
                 space: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 11, 1, 11),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 11)),
                 },
                 newline: Whitespace {
                     value: "\n".to_string(),
-                    source_info: SourceInfo::new(1, 11, 2, 1),
+                    source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(2, 1)),
                 },
                 value: Template {
                     delimiter: None,
@@ -690,17 +650,17 @@ variables {
                         encoded:
                             "query Human($name: String!) {\n  human(name: $name) {\n    name\n    height(unit: FOOT)\n}\n\n".to_string()
                     }],
-                    source_info: SourceInfo::new(2, 1, 8, 1),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(8, 1)),
                 },
                 variables: Some(GraphQlVariables {
                     space: Whitespace {
                         value: " ".to_string(),
-                        source_info: SourceInfo::new(8, 10, 8, 11),
+                        source_info: SourceInfo::new(Pos::new(8, 10), Pos::new(8, 11)),
                     },
                     value: JsonValue::Object {
                         space0: "\n  ".to_string(),
                         elements: vec![JsonObjectElement {
-                            space0: "".to_string(),
+                            space0: String::new(),
                             name: Template {
                                 delimiter: Some('"'),
                                 elements: vec![
@@ -709,9 +669,9 @@ variables {
                                         encoded: "name".to_string()
                                     }
                                 ],
-                                source_info: SourceInfo::new(9, 4, 9, 8)
+                                source_info: SourceInfo::new(Pos::new(9, 4), Pos::new(9, 8))
                             },
-                            space1: "".to_string(),
+                            space1: String::new(),
                             space2: " ".to_string(),
                             value: JsonValue::String(Template {
                                 delimiter: Some('"'),
@@ -721,14 +681,14 @@ variables {
                                         encoded: "Han Solo".to_string()
                                     }
                                 ],
-                                source_info: SourceInfo::new(9, 12, 9, 20)
+                                source_info: SourceInfo::new(Pos::new(9, 12), Pos::new(9, 20))
                             }),
                             space3: "\n".to_string()
                         }]
                     },
                     whitespace: Whitespace {
                         value: "\n".to_string(),
-                        source_info: SourceInfo::new(10, 2, 11, 1)
+                        source_info: SourceInfo::new(Pos::new(10, 2), Pos::new(11, 1))
                     }
                 })
             })
@@ -752,14 +712,7 @@ variables
         let error = multiline_string(&mut reader).err().unwrap();
         assert_eq!(
             error,
-            Error {
-                pos: Pos {
-                    line: 8,
-                    column: 10
-                },
-                recoverable: false,
-                inner: ParseError::GraphQlVariables
-            }
+            Error::new(Pos::new(8, 10), false, ParseError::GraphQlVariables)
         )
     }
 
@@ -784,14 +737,7 @@ variables [
         let error = multiline_string(&mut reader).err().unwrap();
         assert_eq!(
             error,
-            Error {
-                pos: Pos {
-                    line: 8,
-                    column: 11
-                },
-                recoverable: false,
-                inner: ParseError::GraphQlVariables
-            }
+            Error::new(Pos::new(8, 11), false, ParseError::GraphQlVariables)
         )
     }
 }
