@@ -20,13 +20,14 @@ use crate::ast::*;
 use crate::parser::bytes::*;
 use crate::parser::combinators::*;
 use crate::parser::error::*;
+use crate::parser::number::natural;
 use crate::parser::primitives::*;
 use crate::parser::reader::Reader;
 use crate::parser::sections::*;
 use crate::parser::url::url;
 use crate::parser::ParseResult;
 
-pub fn hurl_file(reader: &mut Reader) -> ParseResult<'static, HurlFile> {
+pub fn hurl_file(reader: &mut Reader) -> ParseResult<HurlFile> {
     let entries = zero_or_more(entry, reader)?;
     let line_terminators = optional_line_terminators(reader)?;
     eof(reader)?;
@@ -36,7 +37,7 @@ pub fn hurl_file(reader: &mut Reader) -> ParseResult<'static, HurlFile> {
     })
 }
 
-fn entry(reader: &mut Reader) -> ParseResult<'static, Entry> {
+fn entry(reader: &mut Reader) -> ParseResult<Entry> {
     let req = request(reader)?;
     let resp = optional(response, reader)?;
     Ok(Entry {
@@ -45,8 +46,8 @@ fn entry(reader: &mut Reader) -> ParseResult<'static, Entry> {
     })
 }
 
-fn request(reader: &mut Reader) -> ParseResult<'static, Request> {
-    let start = reader.state.clone();
+fn request(reader: &mut Reader) -> ParseResult<Request> {
+    let start = reader.state;
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
     let m = method(reader)?;
@@ -57,22 +58,17 @@ fn request(reader: &mut Reader) -> ParseResult<'static, Request> {
     let headers = zero_or_more(key_value, reader)?;
     let sections = request_sections(reader)?;
     let b = optional(body, reader)?;
-    let source_info = SourceInfo::new(
-        start.pos.line,
-        start.pos.column,
-        reader.state.pos.line,
-        reader.state.pos.column,
-    );
+    let source_info = SourceInfo::new(start.pos, reader.state.pos);
 
-    // check duplicated section
+    // Check duplicated section
     let mut section_names = vec![];
     for section in sections.clone() {
         if section_names.contains(&section.name().to_string()) {
-            return Err(Error {
-                pos: section.source_info.start,
-                recoverable: false,
-                inner: ParseError::DuplicateSection,
-            });
+            return Err(Error::new(
+                section.source_info.start,
+                false,
+                ParseError::DuplicateSection,
+            ));
         } else {
             section_names.push(section.name().to_string());
         }
@@ -92,8 +88,8 @@ fn request(reader: &mut Reader) -> ParseResult<'static, Request> {
     })
 }
 
-fn response(reader: &mut Reader) -> ParseResult<'static, Response> {
-    let start = reader.state.clone();
+fn response(reader: &mut Reader) -> ParseResult<Response> {
+    let start = reader.state;
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
     let _version = version(reader)?;
@@ -113,40 +109,29 @@ fn response(reader: &mut Reader) -> ParseResult<'static, Response> {
         headers,
         sections,
         body: b,
-        source_info: SourceInfo::new(
-            start.pos.line,
-            start.pos.column,
-            reader.state.pos.line,
-            reader.state.pos.column,
-        ),
+        source_info: SourceInfo::new(start.pos, reader.state.pos),
     })
 }
 
-fn method(reader: &mut Reader) -> ParseResult<'static, Method> {
+fn method(reader: &mut Reader) -> ParseResult<Method> {
     if reader.is_eof() {
-        return Err(Error {
-            pos: reader.state.pos.clone(),
-            recoverable: true,
-            inner: ParseError::Method {
-                name: "<EOF>".to_string(),
-            },
-        });
+        let inner = ParseError::Method {
+            name: "<EOF>".to_string(),
+        };
+        return Err(Error::new(reader.state.pos, true, inner));
     }
-    let start = reader.state.clone();
+    let start = reader.state;
     let name = reader.read_while(|c| c.is_ascii_alphabetic());
     if name.is_empty() || name.to_uppercase() != name {
-        Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::Method { name },
-        })
+        let inner = ParseError::Method { name };
+        Err(Error::new(start.pos, false, inner))
     } else {
         Ok(Method(name))
     }
 }
 
-fn version(reader: &mut Reader) -> ParseResult<'static, Version> {
-    let start = reader.state.clone();
+fn version(reader: &mut Reader) -> ParseResult<Version> {
+    let start = reader.state;
     try_literal("HTTP", reader)?;
 
     let next_c = reader.peek();
@@ -156,67 +141,44 @@ fn version(reader: &mut Reader) -> ParseResult<'static, Version> {
                 ("/1.0", VersionValue::Version1),
                 ("/1.1", VersionValue::Version11),
                 ("/2", VersionValue::Version2),
+                ("/3", VersionValue::Version3),
                 ("/*", VersionValue::VersionAnyLegacy),
             ];
             for (s, value) in available_version.iter() {
                 if try_literal(s, reader).is_ok() {
                     return Ok(Version {
                         value: value.clone(),
-                        source_info: SourceInfo::new(
-                            start.pos.line,
-                            start.pos.column,
-                            reader.state.pos.line,
-                            reader.state.pos.column,
-                        ),
+                        source_info: SourceInfo::new(start.pos, reader.state.pos),
                     });
                 }
             }
-            Err(Error {
-                pos: start.pos,
-                recoverable: false,
-                inner: ParseError::Version {},
-            })
+            Err(Error::new(start.pos, false, ParseError::Version))
         }
         Some(' ') => Ok(Version {
             value: VersionAny,
-            source_info: SourceInfo::new(
-                start.pos.line,
-                start.pos.column,
-                reader.state.pos.line,
-                reader.state.pos.column,
-            ),
-        }),
-        _ => Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::Version {},
+            source_info: SourceInfo::new(start.pos, reader.state.pos),
         }),
+        _ => Err(Error::new(start.pos, false, ParseError::Version)),
     }
 }
 
-fn status(reader: &mut Reader) -> ParseResult<'static, Status> {
-    let start = reader.state.pos.clone();
+fn status(reader: &mut Reader) -> ParseResult<Status> {
+    let start = reader.state.pos;
     let value = match try_literal("*", reader) {
         Ok(_) => StatusValue::Any,
         Err(_) => match natural(reader) {
             Ok(value) => StatusValue::Specific(value),
-            Err(_) => {
-                return Err(Error {
-                    pos: start,
-                    recoverable: false,
-                    inner: ParseError::Status {},
-                });
-            }
+            Err(_) => return Err(Error::new(start, false, ParseError::Status)),
         },
     };
-    let end = reader.state.pos.clone();
+    let end = reader.state.pos;
     Ok(Status {
         value,
         source_info: SourceInfo { start, end },
     })
 }
 
-fn body(reader: &mut Reader) -> ParseResult<'static, Body> {
+fn body(reader: &mut Reader) -> ParseResult<Body> {
     //  let start = reader.state.clone();
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
@@ -291,13 +253,13 @@ mod tests {
         let default_request = Request {
             line_terminators: vec![],
             space0: Whitespace {
-                value: "".to_string(),
-                source_info: SourceInfo::new(1, 1, 1, 1),
+                value: String::new(),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             },
             method: Method("GET".to_string()),
             space1: Whitespace {
                 value: " ".to_string(),
-                source_info: SourceInfo::new(1, 4, 1, 5),
+                source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 5)),
             },
             url: Template {
                 elements: vec![TemplateElement::String {
@@ -305,37 +267,38 @@ mod tests {
                     encoded: String::from("http://google.fr"),
                 }],
                 delimiter: None,
-                source_info: SourceInfo::new(1, 5, 1, 21),
+                source_info: SourceInfo::new(Pos::new(1, 5), Pos::new(1, 21)),
             },
             line_terminator0: LineTerminator {
                 space0: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 21, 1, 21),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 21), Pos::new(1, 21)),
                 },
                 comment: None,
                 newline: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 21, 1, 21),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 21), Pos::new(1, 21)),
                 },
             },
             headers: vec![],
             sections: vec![],
             body: None,
-            source_info: SourceInfo::new(1, 1, 1, 21),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 21)),
         };
-        assert_eq!(request(&mut reader), Ok(default_request));
+        assert_eq!(request(&mut reader).unwrap(), default_request);
+        assert_eq!(reader.state.cursor, 20);
 
         let mut reader = Reader::new("GET  http://google.fr # comment");
         let default_request = Request {
             line_terminators: vec![],
             space0: Whitespace {
-                value: "".to_string(),
-                source_info: SourceInfo::new(1, 1, 1, 1),
+                value: String::new(),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             },
             method: Method("GET".to_string()),
             space1: Whitespace {
                 value: "  ".to_string(),
-                source_info: SourceInfo::new(1, 4, 1, 6),
+                source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 6)),
             },
             url: Template {
                 elements: vec![TemplateElement::String {
@@ -343,31 +306,33 @@ mod tests {
                     encoded: String::from("http://google.fr"),
                 }],
                 delimiter: None,
-                source_info: SourceInfo::new(1, 6, 1, 22),
+                source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 22)),
             },
             line_terminator0: LineTerminator {
                 space0: Whitespace {
                     value: " ".to_string(),
-                    source_info: SourceInfo::new(1, 22, 1, 23),
+                    source_info: SourceInfo::new(Pos::new(1, 22), Pos::new(1, 23)),
                 },
                 comment: Some(Comment {
                     value: " comment".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 24), Pos::new(1, 32)),
                 }),
                 newline: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 32, 1, 32),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 32), Pos::new(1, 32)),
                 },
             },
             headers: vec![],
             sections: vec![],
             body: None,
-            source_info: SourceInfo::new(1, 1, 1, 32),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 32)),
         };
-        assert_eq!(request(&mut reader), Ok(default_request));
+        assert_eq!(request(&mut reader).unwrap(), default_request);
+        assert_eq!(reader.state.cursor, 31);
 
         let mut reader = Reader::new("GET http://google.fr\nGET http://google.fr");
-        let r = request(&mut reader);
-        assert_eq!(r.unwrap().method, Method("GET".to_string()));
+        let r = request(&mut reader).unwrap();
+        assert_eq!(r.method, Method("GET".to_string()));
         assert_eq!(reader.state.cursor, 21);
         let r = request(&mut reader).unwrap();
         assert_eq!(r.method, Method("GET".to_string()));
@@ -386,16 +351,16 @@ mod tests {
             Body {
                 line_terminators: vec![],
                 space0: Whitespace {
-                    value: String::from(""),
-                    source_info: SourceInfo::new(2, 1, 2, 1),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1)),
                 },
                 value: Bytes::MultilineString(MultilineString::Text(Text {
                     space: Whitespace {
-                        value: "".to_string(),
-                        source_info: SourceInfo::new(2, 4, 2, 4),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(2, 4)),
                     },
                     newline: Whitespace {
-                        source_info: SourceInfo::new(2, 4, 3, 1),
+                        source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(3, 1)),
                         value: "\n".to_string(),
                     },
                     value: Template {
@@ -404,18 +369,18 @@ mod tests {
                             encoded: String::from("Hello World!\n"),
                         }],
                         delimiter: None,
-                        source_info: SourceInfo::new(3, 1, 4, 1),
+                        source_info: SourceInfo::new(Pos::new(3, 1), Pos::new(4, 1)),
                     },
                 })),
                 line_terminator0: LineTerminator {
                     space0: Whitespace {
-                        value: "".to_string(),
-                        source_info: SourceInfo::new(4, 4, 4, 4),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(4, 4), Pos::new(4, 4)),
                     },
                     comment: None,
                     newline: Whitespace {
-                        value: "".to_string(),
-                        source_info: SourceInfo::new(4, 4, 4, 4),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(4, 4), Pos::new(4, 4)),
                     },
                 },
             }
@@ -430,22 +395,22 @@ mod tests {
         assert_eq!(
             r.body.unwrap().value,
             Bytes::Json(JsonValue::List {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![
                     JsonListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: JsonValue::Number("1".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                     JsonListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: JsonValue::Number("2".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                     JsonListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: JsonValue::Number("3".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                 ],
             })
@@ -462,7 +427,7 @@ mod tests {
                     value: "Hello".to_string(),
                     encoded: "Hello".to_string(),
                 }],
-                source_info: SourceInfo::new(2, 2, 2, 7),
+                source_info: SourceInfo::new(Pos::new(2, 2), Pos::new(2, 7)),
             }))
         );
 
@@ -546,22 +511,22 @@ mod tests {
         assert_eq!(
             b.value,
             Bytes::Json(JsonValue::List {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![
                     JsonListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: JsonValue::Number("1".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                     JsonListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: JsonValue::Number("2".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                     JsonListElement {
-                        space0: "".to_string(),
+                        space0: String::new(),
                         value: JsonValue::Number("3".to_string()),
-                        space1: "".to_string(),
+                        space1: String::new(),
                     },
                 ],
             })
@@ -574,7 +539,7 @@ mod tests {
         assert_eq!(
             b.value,
             Bytes::Json(JsonValue::Object {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![],
             })
         );
@@ -586,7 +551,7 @@ mod tests {
         assert_eq!(
             b.value,
             Bytes::Json(JsonValue::Object {
-                space0: "".to_string(),
+                space0: String::new(),
                 elements: vec![],
             })
         );
