@@ -23,7 +23,7 @@ use crate::parser::primitives::*;
 use crate::parser::reader::Reader;
 use crate::parser::ParseResult;
 
-pub fn predicate(reader: &mut Reader) -> ParseResult<'static, Predicate> {
+pub fn predicate(reader: &mut Reader) -> ParseResult<Predicate> {
     let (not, space0) = predicate_not(reader);
     let func = predicate_func(reader)?;
     Ok(Predicate {
@@ -35,12 +35,12 @@ pub fn predicate(reader: &mut Reader) -> ParseResult<'static, Predicate> {
 
 // can not fail
 fn predicate_not(reader: &mut Reader) -> (bool, Whitespace) {
-    let save = reader.state.clone();
+    let save = reader.state;
     let no_whitespace = Whitespace {
-        value: "".to_string(),
+        value: String::new(),
         source_info: SourceInfo {
-            start: save.pos.clone(),
-            end: save.pos.clone(),
+            start: save.pos,
+            end: save.pos,
         },
     };
     if try_literal("not", reader).is_ok() {
@@ -56,18 +56,18 @@ fn predicate_not(reader: &mut Reader) -> (bool, Whitespace) {
     }
 }
 
-fn predicate_func(reader: &mut Reader) -> ParseResult<'static, PredicateFunc> {
-    let start = reader.state.clone().pos;
+fn predicate_func(reader: &mut Reader) -> ParseResult<PredicateFunc> {
+    let start = reader.state.pos;
     let value = predicate_func_value(reader)?;
-    let end = reader.state.clone().pos;
+    let end = reader.state.pos;
     Ok(PredicateFunc {
         source_info: SourceInfo { start, end },
         value,
     })
 }
 
-fn predicate_func_value(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
-    let start = reader.state.clone();
+fn predicate_func_value(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
+    let start = reader.state;
     match choice(
         &[
             equal_predicate,
@@ -76,7 +76,6 @@ fn predicate_func_value(reader: &mut Reader) -> ParseResult<'static, PredicateFu
             greater_predicate,
             less_or_equal_predicate,
             less_predicate,
-            count_equal_predicate,
             start_with_predicate,
             end_with_predicate,
             contain_predicate,
@@ -87,6 +86,7 @@ fn predicate_func_value(reader: &mut Reader) -> ParseResult<'static, PredicateFu
             boolean_predicate,
             string_predicate,
             collection_predicate,
+            date_predicate,
             exist_predicate,
             is_empty_predicate,
         ],
@@ -94,18 +94,14 @@ fn predicate_func_value(reader: &mut Reader) -> ParseResult<'static, PredicateFu
     ) {
         Err(Error {
             recoverable: true, ..
-        }) => Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::Predicate,
-        }),
+        }) => Err(Error::new(start.pos, false, ParseError::Predicate)),
         x => x,
     }
 }
 
 impl PredicateValue {
     pub fn is_number(&self) -> bool {
-        matches!(self, PredicateValue::Integer(_) | PredicateValue::Float(_))
+        matches!(self, PredicateValue::Number(_))
     }
     pub fn is_string(&self) -> bool {
         matches!(self, PredicateValue::String(_))
@@ -120,7 +116,7 @@ impl PredicateValue {
     }
 }
 
-fn equal_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     let operator = try_literals("equals", "==", reader)? == "==";
     if !operator {
         eprintln!("'equals' predicate is now deprecated. Use '==' instead");
@@ -138,7 +134,7 @@ fn equal_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncVal
     })
 }
 
-fn not_equal_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn not_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     let operator = try_literals("notEquals", "!=", reader)? == "!=";
     if !operator {
         eprintln!("'notEquals' predicate is now deprecated. Use '!=' instead");
@@ -156,7 +152,7 @@ fn not_equal_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFun
     })
 }
 
-fn greater_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn greater_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     let operator = try_literals("greaterThan", ">", reader)? == ">";
     if !operator {
         eprintln!("'greaterThan' predicate is now deprecated. Use '>' instead");
@@ -166,7 +162,7 @@ fn greater_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncV
     } else {
         one_or_more_spaces(reader)?
     };
-    let start = reader.state.clone();
+    let start = reader.state;
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
         Ok(PredicateFuncValue::GreaterThan {
@@ -175,15 +171,11 @@ fn greater_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncV
             operator,
         })
     } else {
-        Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::PredicateValue {},
-        })
+        Err(Error::new(start.pos, false, ParseError::PredicateValue))
     }
 }
 
-fn greater_or_equal_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn greater_or_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     let operator = try_literals("greaterThanOrEquals", ">=", reader)? == ">=";
     if !operator {
         eprintln!("'greaterThanOrEquals' predicate is now deprecated. Use '>=' instead");
@@ -193,7 +185,7 @@ fn greater_or_equal_predicate(reader: &mut Reader) -> ParseResult<'static, Predi
     } else {
         one_or_more_spaces(reader)?
     };
-    let start = reader.state.clone();
+    let start = reader.state;
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
         Ok(PredicateFuncValue::GreaterThanOrEqual {
@@ -202,15 +194,11 @@ fn greater_or_equal_predicate(reader: &mut Reader) -> ParseResult<'static, Predi
             operator,
         })
     } else {
-        Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::PredicateValue {},
-        })
+        Err(Error::new(start.pos, false, ParseError::PredicateValue))
     }
 }
 
-fn less_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn less_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     let operator = try_literals("lessThan", "<", reader)? == "<";
     if !operator {
         eprintln!("'lessThan' predicate is now deprecated. Use '<' instead");
@@ -220,7 +208,7 @@ fn less_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValu
     } else {
         one_or_more_spaces(reader)?
     };
-    let start = reader.state.clone();
+    let start = reader.state;
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
         Ok(PredicateFuncValue::LessThan {
@@ -229,15 +217,11 @@ fn less_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValu
             operator,
         })
     } else {
-        Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::PredicateValue {},
-        })
+        Err(Error::new(start.pos, false, ParseError::PredicateValue))
     }
 }
 
-fn less_or_equal_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn less_or_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     let operator = try_literals("lessThanOrEquals", "<=", reader)? == "<=";
     if !operator {
         eprintln!("'lessThanOrEquals' predicate is now deprecated. Use '<=' instead");
@@ -247,7 +231,7 @@ fn less_or_equal_predicate(reader: &mut Reader) -> ParseResult<'static, Predicat
     } else {
         one_or_more_spaces(reader)?
     };
-    let start = reader.state.clone();
+    let start = reader.state;
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
         Ok(PredicateFuncValue::LessThanOrEqual {
@@ -256,129 +240,99 @@ fn less_or_equal_predicate(reader: &mut Reader) -> ParseResult<'static, Predicat
             operator,
         })
     } else {
-        Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::PredicateValue {},
-        })
+        Err(Error::new(start.pos, false, ParseError::PredicateValue))
     }
 }
 
-fn count_equal_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
-    try_literal("countEquals", reader)?;
-    let space0 = one_or_more_spaces(reader)?;
-    let save = reader.state.clone();
-    let value = predicate_value(reader)?;
-    if !matches!(value, PredicateValue::Integer(_)) {
-        return Err(Error {
-            pos: save.pos,
-            recoverable: false,
-            inner: ParseError::PredicateValue {},
-        });
-    }
-    Ok(PredicateFuncValue::CountEqual { space0, value })
-}
-
-fn start_with_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn start_with_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("startsWith", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let save = reader.state.clone();
+    let save = reader.state;
     let value = predicate_value(reader)?;
     if !value.is_string() && !value.is_bytearray() {
-        return Err(Error {
-            pos: save.pos,
-            recoverable: false,
-            inner: ParseError::PredicateValue {},
-        });
+        return Err(Error::new(save.pos, false, ParseError::PredicateValue));
     }
     Ok(PredicateFuncValue::StartWith { space0, value })
 }
 
-fn end_with_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn end_with_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("endsWith", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let save = reader.state.clone();
+    let save = reader.state;
     let value = predicate_value(reader)?;
     if !value.is_string() && !value.is_bytearray() {
-        return Err(Error {
-            pos: save.pos,
-            recoverable: false,
-            inner: ParseError::PredicateValue {},
-        });
+        return Err(Error::new(save.pos, false, ParseError::PredicateValue));
     }
     Ok(PredicateFuncValue::EndWith { space0, value })
 }
 
-fn contain_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn contain_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("contains", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let save = reader.state.clone();
+    let save = reader.state;
     let value = predicate_value(reader)?;
     if !value.is_string() && !value.is_bytearray() {
-        return Err(Error {
-            pos: save.pos,
-            recoverable: false,
-            inner: ParseError::PredicateValue {},
-        });
+        return Err(Error::new(save.pos, false, ParseError::PredicateValue));
     }
     Ok(PredicateFuncValue::Contain { space0, value })
 }
 
-fn include_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn include_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("includes", reader)?;
     let space0 = one_or_more_spaces(reader)?;
     let value = predicate_value(reader)?;
     Ok(PredicateFuncValue::Include { space0, value })
 }
 
-fn match_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn match_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("matches", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let save = reader.state.clone();
+    let save = reader.state;
     let value = predicate_value(reader)?;
     if !matches!(value, PredicateValue::String(_)) && !matches!(value, PredicateValue::Regex(_)) {
-        return Err(Error {
-            pos: save.pos,
-            recoverable: false,
-            inner: ParseError::PredicateValue {},
-        });
+        return Err(Error::new(save.pos, false, ParseError::PredicateValue));
     }
     Ok(PredicateFuncValue::Match { space0, value })
 }
 
-fn integer_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn integer_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("isInteger", reader)?;
-    Ok(PredicateFuncValue::IsInteger {})
+    Ok(PredicateFuncValue::IsInteger)
 }
 
-fn float_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn float_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("isFloat", reader)?;
-    Ok(PredicateFuncValue::IsFloat {})
+    Ok(PredicateFuncValue::IsFloat)
 }
 
-fn boolean_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn boolean_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("isBoolean", reader)?;
-    Ok(PredicateFuncValue::IsBoolean {})
+    Ok(PredicateFuncValue::IsBoolean)
 }
 
-fn string_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn string_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("isString", reader)?;
-    Ok(PredicateFuncValue::IsString {})
+    Ok(PredicateFuncValue::IsString)
 }
 
-fn collection_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn collection_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("isCollection", reader)?;
-    Ok(PredicateFuncValue::IsCollection {})
+    Ok(PredicateFuncValue::IsCollection)
 }
 
-fn exist_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn date_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
+    try_literal("isDate", reader)?;
+    Ok(PredicateFuncValue::IsDate)
+}
+
+fn exist_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("exists", reader)?;
-    Ok(PredicateFuncValue::Exist {})
+    Ok(PredicateFuncValue::Exist)
 }
 
-fn is_empty_predicate(reader: &mut Reader) -> ParseResult<'static, PredicateFuncValue> {
+fn is_empty_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("isEmpty", reader)?;
-    Ok(PredicateFuncValue::IsEmpty {})
+    Ok(PredicateFuncValue::IsEmpty)
 }
 
 #[cfg(test)]
@@ -394,8 +348,8 @@ mod tests {
             (
                 false,
                 Whitespace {
-                    value: String::from(""),
-                    source_info: SourceInfo::new(1, 1, 1, 1),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                 }
             )
         );
@@ -408,7 +362,7 @@ mod tests {
                 true,
                 Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 4, 1, 5),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 5)),
                 }
             )
         );
@@ -417,113 +371,85 @@ mod tests {
 
     #[test]
     fn test_predicate() {
-        let mut reader = Reader::new("not equals true");
+        let mut reader = Reader::new("not == true");
         assert_eq!(
             predicate(&mut reader).unwrap(),
             Predicate {
                 not: true,
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 4, 1, 5),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 5)),
                 },
                 predicate_func: PredicateFunc {
-                    source_info: SourceInfo::new(1, 5, 1, 16),
+                    source_info: SourceInfo::new(Pos::new(1, 5), Pos::new(1, 12)),
                     value: PredicateFuncValue::Equal {
                         space0: Whitespace {
                             value: String::from(" "),
-                            source_info: SourceInfo::new(1, 11, 1, 12),
+                            source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 8)),
                         },
                         value: PredicateValue::Bool(true),
-                        operator: false,
+                        operator: true,
                     },
                 },
             }
         );
     }
 
-    #[test]
-    fn test_predicate_error() {
-        let mut reader = Reader::new("countEquals true");
-        let error = predicate(&mut reader).err().unwrap();
-        assert_eq!(
-            error.pos,
-            Pos {
-                line: 1,
-                column: 13,
-            }
-        );
-        assert!(!error.recoverable);
-        assert_eq!(error.inner, ParseError::PredicateValue {});
-    }
-
     #[test]
     fn test_predicate_func() {
-        let mut reader = Reader::new("tata equals 1");
+        let mut reader = Reader::new("tata == 1");
         let error = predicate_func(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert!(!error.recoverable);
-        assert_eq!(error.inner, ParseError::Predicate {});
+        assert_eq!(error.inner, ParseError::Predicate);
     }
 
     #[test]
     fn test_equal_predicate() {
-        let mut reader = Reader::new("equals  true");
+        let mut reader = Reader::new("==  true");
         assert_eq!(
             equal_predicate(&mut reader).unwrap(),
             PredicateFuncValue::Equal {
                 value: PredicateValue::Bool(true),
                 space0: Whitespace {
                     value: String::from("  "),
-                    source_info: SourceInfo::new(1, 7, 1, 9),
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 5)),
                 },
 
-                operator: false,
+                operator: true,
             }
         );
 
-        let mut reader = Reader::new("equals 1.1");
+        let mut reader = Reader::new("== 1.1");
         assert_eq!(
             equal_predicate(&mut reader).unwrap(),
             PredicateFuncValue::Equal {
-                value: PredicateValue::Float(Float {
+                value: PredicateValue::Number(Number::Float(Float {
                     value: 1.1,
                     encoded: "1.1".to_string(),
-                }),
+                })),
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 7, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
-                operator: false,
+                operator: true,
             }
         );
 
-        let mut reader = Reader::new("equals 2");
-        assert_eq!(
-            equal_predicate(&mut reader).unwrap(),
-            PredicateFuncValue::Equal {
-                value: PredicateValue::Integer(2),
-                space0: Whitespace {
-                    value: String::from(" "),
-                    source_info: SourceInfo::new(1, 7, 1, 8),
-                },
-                operator: false,
-            },
-        );
-
         let mut reader = Reader::new("== 2");
         assert_eq!(
             equal_predicate(&mut reader).unwrap(),
             PredicateFuncValue::Equal {
-                value: PredicateValue::Integer(2),
+                value: PredicateValue::Number(Number::Integer(2)),
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 3, 1, 4),
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
                 operator: true,
             },
         );
 
-        let mut reader = Reader::new("equals \"Bob\"");
+        let mut reader = Reader::new("== \"Bob\"");
         assert_eq!(
             equal_predicate(&mut reader).unwrap(),
             PredicateFuncValue::Equal {
@@ -533,73 +459,46 @@ mod tests {
                         value: "Bob".to_string(),
                         encoded: "Bob".to_string(),
                     }],
-                    source_info: SourceInfo::new(1, 8, 1, 13),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 9)),
                 }),
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 7, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
-                operator: false,
+                operator: true,
             }
         );
     }
 
     #[test]
     fn test_equal_expression_predicate() {
-        let mut reader = Reader::new("equals {{count}}");
+        let mut reader = Reader::new("== {{count}}");
         assert_eq!(
             equal_predicate(&mut reader).unwrap(),
             PredicateFuncValue::Equal {
                 value: PredicateValue::Expression(Expr {
                     space0: Whitespace {
-                        value: String::from(""),
-                        source_info: SourceInfo::new(1, 10, 1, 10),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 6)),
                     },
                     variable: Variable {
                         name: "count".to_string(),
-                        source_info: SourceInfo::new(1, 10, 1, 15),
+                        source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 11)),
                     },
                     space1: Whitespace {
-                        value: String::from(""),
-                        source_info: SourceInfo::new(1, 15, 1, 15),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 11)),
                     },
                 }),
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 7, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
-                operator: false,
+                operator: true,
             }
         );
     }
 
-    #[test]
-    fn test_count_equal_predicate() {
-        let mut reader = Reader::new("countEquals 2");
-        assert_eq!(
-            count_equal_predicate(&mut reader).unwrap(),
-            PredicateFuncValue::CountEqual {
-                value: PredicateValue::Integer(2),
-                space0: Whitespace {
-                    value: String::from(" "),
-                    source_info: SourceInfo::new(1, 12, 1, 13),
-                },
-            }
-        );
-
-        let mut reader = Reader::new("countEquals true");
-        let error = count_equal_predicate(&mut reader).err().unwrap();
-        assert_eq!(
-            error.pos,
-            Pos {
-                line: 1,
-                column: 13,
-            }
-        );
-        assert!(!error.recoverable);
-        assert_eq!(error.inner, ParseError::PredicateValue {});
-    }
-
     #[test]
     fn test_start_with_predicate() {
         let mut reader = Reader::new("startsWith 2");
@@ -612,6 +511,13 @@ mod tests {
             }
         );
         assert!(!error.recoverable);
-        assert_eq!(error.inner, ParseError::PredicateValue {});
+        assert_eq!(error.inner, ParseError::PredicateValue);
+    }
+
+    #[test]
+    fn test_date_predicate() {
+        let mut reader = Reader::new("isDate");
+        let result = date_predicate(&mut reader);
+        assert_eq!(result.unwrap(), PredicateFuncValue::IsDate);
     }
 }
