@@ -18,28 +18,29 @@
 use crate::ast::*;
 use crate::parser::combinators::*;
 use crate::parser::multiline::multiline_string;
+use crate::parser::number::number;
 use crate::parser::primitives::*;
 use crate::parser::reader::Reader;
 use crate::parser::string::*;
 use crate::parser::{expr, Error, ParseError, ParseResult};
 
-pub fn predicate_value(reader: &mut Reader) -> ParseResult<'static, PredicateValue> {
+pub fn predicate_value(reader: &mut Reader) -> ParseResult<PredicateValue> {
     choice(
         &[
             |p1| match null(p1) {
-                Ok(()) => Ok(PredicateValue::Null {}),
+                Ok(()) => Ok(PredicateValue::Null),
                 Err(e) => Err(e),
             },
             |p1| match boolean(p1) {
                 Ok(value) => Ok(PredicateValue::Bool(value)),
                 Err(e) => Err(e),
             },
-            |p1| match float(p1) {
-                Ok(value) => Ok(PredicateValue::Float(value)),
+            |p1| match number(p1) {
+                Ok(value) => Ok(PredicateValue::Number(value)),
                 Err(e) => Err(e),
             },
-            |p1| match integer(p1) {
-                Ok(value) => Ok(PredicateValue::Integer(value)),
+            |p1| match file(p1) {
+                Ok(value) => Ok(PredicateValue::File(value)),
                 Err(e) => Err(e),
             },
             |p1| match hex(p1) {
@@ -69,14 +70,13 @@ pub fn predicate_value(reader: &mut Reader) -> ParseResult<'static, PredicateVal
         ],
         reader,
     )
-    .map_err(|e| Error {
-        pos: e.pos,
-        recoverable: false,
-        inner: if e.recoverable {
+    .map_err(|e| {
+        let inner = if e.recoverable {
             ParseError::PredicateValue
         } else {
             e.inner
-        },
+        };
+        Error::new(e.pos, false, inner)
     })
 }
 
@@ -97,16 +97,16 @@ mod tests {
         let mut reader = Reader::new("1");
         assert_eq!(
             predicate_value(&mut reader).unwrap(),
-            PredicateValue::Integer(1)
+            PredicateValue::Number(Number::Integer(1))
         );
 
         let mut reader = Reader::new("1.1");
         assert_eq!(
             predicate_value(&mut reader).unwrap(),
-            PredicateValue::Float(Float {
+            PredicateValue::Number(Number::Float(Float {
                 value: 1.1,
                 encoded: "1.1".to_string(),
-            })
+            }))
         );
     }
 
@@ -115,7 +115,7 @@ mod tests {
         let mut reader = Reader::new("xx");
         let error = predicate_value(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(error.inner, ParseError::PredicateValue {});
+        assert_eq!(error.inner, ParseError::PredicateValue);
         assert!(!error.recoverable);
     }
 
