@@ -23,17 +23,17 @@ use crate::parser::reader::Reader;
 use crate::parser::string::*;
 use crate::parser::{Error, ParseError, ParseResult};
 
-pub fn query(reader: &mut Reader) -> ParseResult<'static, Query> {
-    let start = reader.state.pos.clone();
+pub fn query(reader: &mut Reader) -> ParseResult<Query> {
+    let start = reader.state.pos;
     let value = query_value(reader)?;
-    let end = reader.state.pos.clone();
+    let end = reader.state.pos;
     Ok(Query {
         source_info: SourceInfo { start, end },
         value,
     })
 }
 
-fn query_value(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn query_value(reader: &mut Reader) -> ParseResult<QueryValue> {
     choice(
         &[
             status_query,
@@ -55,28 +55,28 @@ fn query_value(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
     )
 }
 
-fn status_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn status_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("status", reader)?;
-    Ok(QueryValue::Status {})
+    Ok(QueryValue::Status)
 }
 
-fn url_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn url_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("url", reader)?;
-    Ok(QueryValue::Url {})
+    Ok(QueryValue::Url)
 }
 
-fn header_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn header_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("header", reader)?;
     let space0 = one_or_more_spaces(reader)?;
     let name = quoted_template(reader).map_err(|e| e.non_recoverable())?;
     Ok(QueryValue::Header { space0, name })
 }
 
-fn cookie_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn cookie_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("cookie", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let start = reader.state.pos.clone();
-    let s = quoted_string(reader)?;
+    let start = reader.state.pos;
+    let s = quoted_oneline_string(reader)?;
     // todo should work with an encodedString in order to support escape sequence
     // or decode escape sequence with the cookiepath parser
 
@@ -90,19 +90,19 @@ fn cookie_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
     Ok(QueryValue::Cookie { space0, expr })
 }
 
-fn body_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn body_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("body", reader)?;
-    Ok(QueryValue::Body {})
+    Ok(QueryValue::Body)
 }
 
-fn xpath_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn xpath_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("xpath", reader)?;
     let space0 = one_or_more_spaces(reader)?;
     let expr = quoted_template(reader).map_err(|e| e.non_recoverable())?;
     Ok(QueryValue::Xpath { space0, expr })
 }
 
-fn jsonpath_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn jsonpath_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("jsonpath", reader)?;
     let space0 = one_or_more_spaces(reader)?;
     //let expr = jsonpath_expr(reader)?;
@@ -123,14 +123,14 @@ fn jsonpath_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
     Ok(QueryValue::Jsonpath { space0, expr })
 }
 
-fn regex_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn regex_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("regex", reader)?;
     let space0 = one_or_more_spaces(reader)?;
     let value = regex_value(reader)?;
     Ok(QueryValue::Regex { space0, value })
 }
 
-pub fn regex_value(reader: &mut Reader) -> ParseResult<'static, RegexValue> {
+pub fn regex_value(reader: &mut Reader) -> ParseResult<RegexValue> {
     choice(
         &[
             |p1| match quoted_template(p1) {
@@ -144,43 +144,42 @@ pub fn regex_value(reader: &mut Reader) -> ParseResult<'static, RegexValue> {
         ],
         reader,
     )
-    .map_err(|e| Error {
-        pos: e.pos,
-        recoverable: false,
-        inner: ParseError::Expecting {
+    .map_err(|e| {
+        let inner = ParseError::Expecting {
             value: "\" or /".to_string(),
-        },
+        };
+        Error::new(e.pos, false, inner)
     })
 }
 
-fn variable_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn variable_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("variable", reader)?;
     let space0 = one_or_more_spaces(reader)?;
     let name = quoted_template(reader).map_err(|e| e.non_recoverable())?;
     Ok(QueryValue::Variable { space0, name })
 }
 
-fn duration_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn duration_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("duration", reader)?;
-    Ok(QueryValue::Duration {})
+    Ok(QueryValue::Duration)
 }
 
-fn bytes_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn bytes_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("bytes", reader)?;
-    Ok(QueryValue::Bytes {})
+    Ok(QueryValue::Bytes)
 }
 
-fn sha256_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn sha256_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("sha256", reader)?;
-    Ok(QueryValue::Sha256 {})
+    Ok(QueryValue::Sha256)
 }
 
-fn md5_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn md5_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("md5", reader)?;
-    Ok(QueryValue::Md5 {})
+    Ok(QueryValue::Md5)
 }
 
-fn certificate_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
+fn certificate_query(reader: &mut Reader) -> ParseResult<QueryValue> {
     try_literal("certificate", reader)?;
     let space0 = one_or_more_spaces(reader)?;
     let field = certificate_field(reader)?;
@@ -190,7 +189,7 @@ fn certificate_query(reader: &mut Reader) -> ParseResult<'static, QueryValue> {
     })
 }
 
-fn certificate_field(reader: &mut Reader) -> ParseResult<'static, CertificateAttributeName> {
+fn certificate_field(reader: &mut Reader) -> ParseResult<CertificateAttributeName> {
     literal("\"", reader)?;
     if try_literal(r#"Subject""#, reader).is_ok() {
         Ok(CertificateAttributeName::Subject)
@@ -204,14 +203,10 @@ fn certificate_field(reader: &mut Reader) -> ParseResult<'static, CertificateAtt
         Ok(CertificateAttributeName::SerialNumber)
     } else {
         let value =
-            "Field <Subject>, <Issuer>,<Start-Date>, <Expire-Date> or <Serial-Number>".to_string();
+            "Field <Subject>, <Issuer>, <Start-Date>, <Expire-Date> or <Serial-Number>".to_string();
         let inner = ParseError::Expecting { value };
-        let pos = reader.state.pos.clone();
-        Err(Error {
-            pos,
-            recoverable: false,
-            inner,
-        })
+        let pos = reader.state.pos;
+        Err(Error::new(pos, false, inner))
     }
 }
 
@@ -226,8 +221,8 @@ mod tests {
         assert_eq!(
             query(&mut reader).unwrap(),
             Query {
-                source_info: SourceInfo::new(1, 1, 1, 7),
-                value: QueryValue::Status {},
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 7)),
+                value: QueryValue::Status,
             }
         );
     }
@@ -238,8 +233,8 @@ mod tests {
         assert_eq!(
             query(&mut reader).unwrap(),
             Query {
-                source_info: SourceInfo::new(1, 1, 1, 7),
-                value: QueryValue::Status {},
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 7)),
+                value: QueryValue::Status,
             }
         );
     }
@@ -252,7 +247,7 @@ mod tests {
             QueryValue::Header {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 7, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 8)),
                 },
                 name: Template {
                     delimiter: Some('"'),
@@ -260,7 +255,7 @@ mod tests {
                         value: "Foo".to_string(),
                         encoded: "Foo".to_string(),
                     }],
-                    source_info: SourceInfo::new(1, 8, 1, 13),
+                    source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 13)),
                 },
             }
         );
@@ -274,7 +269,7 @@ mod tests {
             QueryValue::Cookie {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 7, 1, 8),
+                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 8)),
                 },
                 expr: CookiePath {
                     name: Template {
@@ -283,17 +278,17 @@ mod tests {
                             value: "Foo".to_string(),
                             encoded: "Foo".to_string(),
                         }],
-                        source_info: SourceInfo::new(1, 9, 1, 12),
+                        source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 12)),
                     },
                     attribute: Some(CookieAttribute {
                         space0: Whitespace {
-                            value: String::from(""),
-                            source_info: SourceInfo::new(1, 13, 1, 13),
+                            value: String::new(),
+                            source_info: SourceInfo::new(Pos::new(1, 13), Pos::new(1, 13)),
                         },
                         name: CookieAttributeName::Domain("Domain".to_string()),
                         space1: Whitespace {
-                            value: String::from(""),
-                            source_info: SourceInfo::new(1, 19, 1, 19),
+                            value: String::new(),
+                            source_info: SourceInfo::new(Pos::new(1, 19), Pos::new(1, 19)),
                         },
                     }),
                 },
@@ -313,7 +308,7 @@ mod tests {
             QueryValue::Xpath {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 6, 1, 7),
+                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
                 expr: Template {
                     delimiter: Some('"'),
@@ -321,14 +316,14 @@ mod tests {
                         value: String::from("normalize-space(//head/title)"),
                         encoded: String::from("normalize-space(//head/title)"),
                     }],
-                    source_info: SourceInfo::new(1, 7, 1, 38),
+                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 38)),
                 },
             },
         );
 
         let mut reader = Reader::new("xpath \"normalize-space(//div[contains(concat(' ',normalize-space(@class),' '),' monthly-price ')])\"");
         assert_eq!(xpath_query(&mut reader).unwrap(), QueryValue::Xpath {
-            space0: Whitespace { value: String::from(" "), source_info: SourceInfo::new(1, 6, 1, 7) },
+            space0: Whitespace { value: String::from(" "), source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)) },
             expr: Template {
                 delimiter: Some('"'),
                 elements: vec![
@@ -337,7 +332,7 @@ mod tests {
                         encoded: String::from("normalize-space(//div[contains(concat(' ',normalize-space(@class),' '),' monthly-price ')])"),
                     }
                 ],
-                source_info: SourceInfo::new(1, 7, 1, 100),
+                source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 100)),
             },
 
         });
@@ -351,7 +346,7 @@ mod tests {
             QueryValue::Jsonpath {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 9, 1, 10),
+                    source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -359,7 +354,7 @@ mod tests {
                         encoded: "$['statusCode']".to_string(),
                     }],
                     delimiter: Some('"'),
-                    source_info: SourceInfo::new(1, 10, 1, 27),
+                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 27)),
                 },
             },
         );
@@ -369,7 +364,7 @@ mod tests {
             QueryValue::Jsonpath {
                 space0: Whitespace {
                     value: String::from(" "),
-                    source_info: SourceInfo::new(1, 9, 1, 10),
+                    source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
                 expr: Template {
                     elements: vec![TemplateElement::String {
@@ -377,7 +372,7 @@ mod tests {
                         encoded: "$.success".to_string(),
                     }],
                     delimiter: Some('"'),
-                    source_info: SourceInfo::new(1, 10, 1, 21),
+                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 21)),
                 },
             },
         );
@@ -389,8 +384,8 @@ mod tests {
         assert_eq!(
             query(&mut reader).unwrap(),
             Query {
-                source_info: SourceInfo::new(1, 1, 1, 5),
-                value: QueryValue::Body {},
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 5)),
+                value: QueryValue::Body,
             }
         );
         assert_eq!(
@@ -398,10 +393,10 @@ mod tests {
             vec![(
                 Whitespace {
                     value: " ".to_string(),
-                    source_info: SourceInfo::new(1, 5, 1, 6)
+                    source_info: SourceInfo::new(Pos::new(1, 5), Pos::new(1, 6))
                 },
                 Filter {
-                    source_info: SourceInfo::new(1, 6, 1, 15),
+                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 15)),
                     value: FilterValue::UrlDecode,
                 }
             )]
