@@ -23,16 +23,16 @@ use crate::parser::reader::Reader;
 use crate::parser::{template, ParseResult};
 
 /// Steps:
-/// 1- parse String until end of stream or end of line or #
+/// 1- parse String until end of stream, end of line
 ///    the string does not contain trailing space
 /// 2- templatize
-pub fn unquoted_template(reader: &mut Reader) -> ParseResult<'static, Template> {
-    let start = reader.state.clone();
+pub fn unquoted_template(reader: &mut Reader) -> ParseResult<Template> {
+    let start = reader.state;
     let mut chars = vec![];
     let mut spaces = vec![];
-    let mut end = start.clone();
+    let mut end = start;
     loop {
-        let pos = reader.state.pos.clone();
+        let pos = reader.state.pos;
         match any_char(vec!['#'], reader) {
             Err(e) => {
                 if e.recoverable {
@@ -53,16 +53,16 @@ pub fn unquoted_template(reader: &mut Reader) -> ParseResult<'static, Template>
                         spaces = vec![];
                     }
                     chars.push((c, s, pos));
-                    end = reader.state.clone();
+                    end = reader.state;
                 }
             }
         }
     }
-    reader.state = end.clone();
+    reader.state = end;
     let encoded_string = template::EncodedString {
         source_info: SourceInfo {
-            start: start.pos.clone(),
-            end: end.pos.clone(),
+            start: start.pos,
+            end: end.pos,
         },
         chars,
     };
@@ -77,86 +77,23 @@ pub fn unquoted_template(reader: &mut Reader) -> ParseResult<'static, Template>
     })
 }
 
-pub fn unquoted_string_key(reader: &mut Reader) -> ParseResult<'static, EncodedString> {
-    let start = reader.state.pos.clone();
-    let mut value = "".to_string();
-    let mut encoded = "".to_string();
-    loop {
-        let save = reader.state.clone();
-        match escape_char(reader) {
-            Ok(c) => {
-                value.push(c);
-                encoded.push_str(reader.peek_back(save.cursor).as_str())
-            }
-            Err(e) => {
-                if e.recoverable {
-                    reader.state = save.clone();
-                    match reader.read() {
-                        None => break,
-                        Some(c) => {
-                            if c.is_alphanumeric()
-                                || c == '_'
-                                || c == '-'
-                                || c == '.'
-                                || c == '['
-                                || c == ']'
-                                || c == '@'
-                                || c == '$'
-                            {
-                                value.push(c);
-                                encoded.push_str(reader.peek_back(save.cursor).as_str())
-                            } else {
-                                reader.state = save;
-                                break;
-                            }
-                        }
-                    }
-                } else {
-                    return Err(e);
-                }
-            }
-        }
-    }
-
-    // check nonempty/ starts with [
-    if value.is_empty() || encoded.starts_with('[') {
-        return Err(Error {
-            pos: start,
-            recoverable: true,
-            inner: ParseError::Expecting {
-                value: "key string".to_string(),
-            },
-        });
-    }
-
-    let quotes = false;
-    let end = reader.state.pos.clone();
-    let source_info = SourceInfo { start, end };
-    Ok(EncodedString {
-        value,
-        encoded,
-        quotes,
-        source_info,
-    })
-}
-
 // TODO: should return an EncodedString
 // (decoding escape sequence)
-pub fn quoted_string(reader: &mut Reader) -> ParseResult<'static, String> {
+pub fn quoted_oneline_string(reader: &mut Reader) -> ParseResult<String> {
     literal("\"", reader)?;
-    let s = reader.read_while(|c| *c != '"');
+    let s = reader.read_while(|c| *c != '"' && *c != '\n');
     literal("\"", reader)?;
     Ok(s)
 }
 
-pub fn quoted_template(reader: &mut Reader) -> ParseResult<'static, Template> {
-    let start = reader.state.clone().pos;
-    let mut end = start.clone();
+pub fn quoted_template(reader: &mut Reader) -> ParseResult<Template> {
+    let start = reader.state.pos;
+    let mut end = start;
     try_literal("\"", reader)?;
     let mut chars = vec![];
     loop {
-        let pos = reader.state.pos.clone();
-        let save = reader.state.clone();
+        let pos = reader.state.pos;
+        let save = reader.state;
         match any_char(vec!['"'], reader) {
             Err(e) => {
                 if e.recoverable {
@@ -168,16 +105,13 @@ pub fn quoted_template(reader: &mut Reader) -> ParseResult<'static, Template> {
             }
             Ok((c, s)) => {
                 chars.push((c, s, pos));
-                end = reader.state.clone().pos;
+                end = reader.state.pos;
             }
         }
     }
     literal("\"", reader)?;
     let encoded_string = template::EncodedString {
-        source_info: SourceInfo {
-            start: start.clone(),
-            end,
-        },
+        source_info: SourceInfo { start, end },
         chars,
     };
     let elements = template::templatize(encoded_string)?;
@@ -186,20 +120,20 @@ pub fn quoted_template(reader: &mut Reader) -> ParseResult<'static, Template> {
         elements,
         source_info: SourceInfo {
             start,
-            end: reader.state.pos.clone(),
+            end: reader.state.pos,
         },
     })
 }
 
-pub fn backtick_template(reader: &mut Reader) -> ParseResult<'static, Template> {
+pub fn backtick_template(reader: &mut Reader) -> ParseResult<Template> {
     let delimiter = Some('`');
-    let start = reader.state.clone().pos;
-    let mut end = start.clone();
+    let start = reader.state.pos;
+    let mut end = start;
     try_literal("`", reader)?;
     let mut chars = vec![];
     loop {
-        let pos = reader.state.pos.clone();
-        let save = reader.state.clone();
+        let pos = reader.state.pos;
+        let save = reader.state;
         match any_char(vec!['`', '\n'], reader) {
             Err(e) => {
                 if e.recoverable {
@@ -211,16 +145,13 @@ pub fn backtick_template(reader: &mut Reader) -> ParseResult<'static, Template>
             }
             Ok((c, s)) => {
                 chars.push((c, s, pos));
-                end = reader.state.clone().pos;
+                end = reader.state.pos;
             }
         }
     }
     literal("`", reader)?;
     let encoded_string = template::EncodedString {
-        source_info: SourceInfo {
-            start: start.clone(),
-            end,
-        },
+        source_info: SourceInfo { start, end },
         chars,
     };
     let elements = template::templatize(encoded_string)?;
@@ -229,37 +160,33 @@ pub fn backtick_template(reader: &mut Reader) -> ParseResult<'static, Template>
         elements,
         source_info: SourceInfo {
             start,
-            end: reader.state.pos.clone(),
+            end: reader.state.pos,
         },
     })
 }
 
-fn any_char(except: Vec<char>, reader: &mut Reader) -> ParseResult<'static, (char, String)> {
-    let start = reader.state.clone();
+fn any_char(except: Vec<char>, reader: &mut Reader) -> ParseResult<(char, String)> {
+    let start = reader.state;
     match escape_char(reader) {
         Ok(c) => Ok((c, reader.peek_back(start.cursor))),
         Err(e) => {
             if e.recoverable {
-                reader.state = start.clone();
+                reader.state = start;
                 match reader.read() {
-                    None => Err(Error {
-                        pos: start.pos,
-                        recoverable: true,
-                        inner: ParseError::Expecting {
+                    None => {
+                        let inner = ParseError::Expecting {
                             value: "char".to_string(),
-                        },
-                    }),
+                        };
+                        Err(Error::new(start.pos, true, inner))
+                    }
                     Some(c) => {
                         if except.contains(&c)
-                            || vec!['\\', '\x08', '\n', '\x0c', '\r', '\t'].contains(&c)
+                            || ['\\', '\x08', '\n', '\x0c', '\r', '\t'].contains(&c)
                         {
-                            Err(Error {
-                                pos: start.pos,
-                                recoverable: true,
-                                inner: ParseError::Expecting {
-                                    value: "char".to_string(),
-                                },
-                            })
+                            let inner = ParseError::Expecting {
+                                value: "char".to_string(),
+                            };
+                            Err(Error::new(start.pos, true, inner))
                         } else {
                             Ok((c, reader.peek_back(start.cursor)))
                         }
@@ -272,9 +199,9 @@ fn any_char(except: Vec<char>, reader: &mut Reader) -> ParseResult<'static, (cha
     }
 }
 
-fn escape_char(reader: &mut Reader) -> ParseResult<'static, char> {
+fn escape_char(reader: &mut Reader) -> ParseResult<char> {
     try_literal("\\", reader)?;
-    let start = reader.state.clone();
+    let start = reader.state;
     match reader.read() {
         Some('#') => Ok('#'),
         Some('"') => Ok('"'),
@@ -287,32 +214,22 @@ fn escape_char(reader: &mut Reader) -> ParseResult<'static, char> {
         Some('r') => Ok('\r'),
         Some('t') => Ok('\t'),
         Some('u') => unicode(reader),
-        _ => Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::EscapeChar {},
-        }),
+        _ => Err(Error::new(start.pos, false, ParseError::EscapeChar)),
     }
 }
 
-fn unicode(reader: &mut Reader) -> ParseResult<'static, char> {
+pub(crate) fn unicode(reader: &mut Reader) -> ParseResult<char> {
     literal("{", reader)?;
     let v = hex_value(reader)?;
     let c = match std::char::from_u32(v) {
-        None => {
-            return Err(Error {
-                pos: reader.clone().state.pos,
-                recoverable: false,
-                inner: ParseError::Unicode {},
-            });
-        }
+        None => return Err(Error::new(reader.state.pos, false, ParseError::Unicode)),
         Some(c) => c,
     };
     literal("}", reader)?;
     Ok(c)
 }
 
-fn hex_value(reader: &mut Reader) -> ParseResult<'static, u32> {
+fn hex_value(reader: &mut Reader) -> ParseResult<u32> {
     let mut digits = one_or_more(hex_digit, reader)?;
     let mut v = 0;
     let mut weight = 1;
@@ -338,7 +255,7 @@ mod tests {
             Template {
                 delimiter: None,
                 elements: vec![],
-                source_info: SourceInfo::new(1, 1, 1, 1),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             }
         );
         assert_eq!(reader.state.cursor, 0);
@@ -355,7 +272,7 @@ mod tests {
                     value: "a".to_string(),
                     encoded: "a".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 2),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 2)),
             }
         );
         assert_eq!(reader.state.cursor, 1);
@@ -372,7 +289,7 @@ mod tests {
                     value: "a#".to_string(),
                     encoded: "a\\u{23}".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 8),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 8)),
             }
         );
         assert_eq!(reader.state.cursor, 7);
@@ -389,7 +306,7 @@ mod tests {
                     value: "\"hi\"".to_string(),
                     encoded: "\"hi\"".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 5),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 5)),
             }
         );
         assert_eq!(reader.state.cursor, 4);
@@ -409,16 +326,16 @@ mod tests {
                     },
                     TemplateElement::Expression(Expr {
                         space0: Whitespace {
-                            value: "".to_string(),
-                            source_info: SourceInfo::new(1, 14, 1, 14),
+                            value: String::new(),
+                            source_info: SourceInfo::new(Pos::new(1, 14), Pos::new(1, 14)),
                         },
                         variable: Variable {
                             name: "name".to_string(),
-                            source_info: SourceInfo::new(1, 14, 1, 18),
+                            source_info: SourceInfo::new(Pos::new(1, 14), Pos::new(1, 18)),
                         },
                         space1: Whitespace {
-                            value: "".to_string(),
-                            source_info: SourceInfo::new(1, 18, 1, 18),
+                            value: String::new(),
+                            source_info: SourceInfo::new(Pos::new(1, 18), Pos::new(1, 18)),
                         },
                     }),
                     TemplateElement::String {
@@ -426,153 +343,12 @@ mod tests {
                         encoded: "!".to_string(),
                     },
                 ],
-                source_info: SourceInfo::new(1, 1, 1, 21),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 21)),
             }
         );
         assert_eq!(reader.state.cursor, 20);
     }
 
-    #[test]
-    fn test_unquoted_template_trailing_space() {
-        let mut reader = Reader::new("hello world # comment");
-        assert_eq!(
-            unquoted_template(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
-                    value: "hello world".to_string(),
-                    encoded: "hello world".to_string(),
-                },],
-                source_info: SourceInfo::new(1, 1, 1, 12),
-            }
-        );
-        assert_eq!(reader.state.cursor, 11);
-        assert_eq!(
-            reader.state.pos,
-            Pos {
-                line: 1,
-                column: 12
-            }
-        );
-    }
-
-    #[test]
-    fn test_unquoted_key() {
-        let mut reader = Reader::new("key");
-        assert_eq!(
-            unquoted_string_key(&mut reader).unwrap(),
-            EncodedString {
-                value: "key".to_string(),
-                encoded: "key".to_string(),
-                quotes: false,
-                source_info: SourceInfo::new(1, 1, 1, 4),
-            }
-        );
-        assert_eq!(reader.state.cursor, 3);
-
-        let mut reader = Reader::new("key\\u{20}\\u{3a} :");
-        assert_eq!(
-            unquoted_string_key(&mut reader).unwrap(),
-            EncodedString {
-                value: "key :".to_string(),
-                encoded: "key\\u{20}\\u{3a}".to_string(),
-                quotes: false,
-                source_info: SourceInfo::new(1, 1, 1, 16),
-            }
-        );
-        assert_eq!(reader.state.cursor, 15);
-
-        let mut reader = Reader::new("$top:");
-        assert_eq!(
-            unquoted_string_key(&mut reader).unwrap(),
-            EncodedString {
-                value: "$top".to_string(),
-                encoded: "$top".to_string(),
-                quotes: false,
-                source_info: SourceInfo::new(1, 1, 1, 5),
-            }
-        );
-        assert_eq!(reader.state.cursor, 4);
-    }
-
-    #[test]
-    fn test_unquoted_key_with_square_bracket() {
-        let mut reader = Reader::new("values\\u{5b}0\\u{5d} :");
-        assert_eq!(
-            unquoted_string_key(&mut reader).unwrap(),
-            EncodedString {
-                value: "values[0]".to_string(),
-                encoded: "values\\u{5b}0\\u{5d}".to_string(),
-                quotes: false,
-                source_info: SourceInfo::new(1, 1, 1, 20),
-            }
-        );
-        assert_eq!(reader.state.cursor, 19);
-
-        let mut reader = Reader::new("values[0] :");
-        assert_eq!(
-            unquoted_string_key(&mut reader).unwrap(),
-            EncodedString {
-                value: "values[0]".to_string(),
-                encoded: "values[0]".to_string(),
-                quotes: false,
-                source_info: SourceInfo::new(1, 1, 1, 10),
-            }
-        );
-        assert_eq!(reader.state.cursor, 9);
-    }
-
-    #[test]
-    fn test_unquoted_keys_ignore_start_square_bracket() {
-        let mut reader = Reader::new("[0]:");
-        let error = unquoted_string_key(&mut reader).err().unwrap();
-        assert!(error.recoverable);
-        assert_eq!(reader.state.cursor, 3);
-    }
-
-    #[test]
-    fn test_unquoted_keys_accept_start_escape_square_bracket() {
-        let mut reader = Reader::new("\\u{5b}0\\u{5d}");
-        assert_eq!(
-            unquoted_string_key(&mut reader).unwrap(),
-            EncodedString {
-                value: "[0]".to_string(),
-                encoded: "\\u{5b}0\\u{5d}".to_string(),
-                quotes: false,
-                source_info: SourceInfo::new(1, 1, 1, 14),
-            }
-        );
-        assert_eq!(reader.state.cursor, 13);
-    }
-
-    #[test]
-    fn test_unquoted_key_error() {
-        let mut reader = Reader::new("");
-        let error = unquoted_string_key(&mut reader).err().unwrap();
-        assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(
-            error.inner,
-            ParseError::Expecting {
-                value: "key string".to_string()
-            }
-        );
-
-        let mut reader = Reader::new("\\l");
-        let error = unquoted_string_key(&mut reader).err().unwrap();
-        assert_eq!(error.pos, Pos { line: 1, column: 2 });
-        assert_eq!(error.inner, ParseError::EscapeChar {});
-
-        let mut reader = Reader::new(r#"{"id":1}"#);
-        let error = unquoted_string_key(&mut reader).err().unwrap();
-        assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(
-            error.inner,
-            ParseError::Expecting {
-                value: "key string".to_string()
-            }
-        );
-    }
-
     #[test]
     fn test_quoted_template() {
         let mut reader = Reader::new("\"\"");
@@ -581,7 +357,7 @@ mod tests {
             Template {
                 delimiter: Some('"'),
                 elements: vec![],
-                source_info: SourceInfo::new(1, 1, 1, 3),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 3)),
             }
         );
         assert_eq!(reader.state.cursor, 2);
@@ -595,7 +371,7 @@ mod tests {
                     value: "a#".to_string(),
                     encoded: "a#".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 5),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 5)),
             }
         );
         assert_eq!(reader.state.cursor, 4);
@@ -609,7 +385,7 @@ mod tests {
                     value: "{0}".to_string(),
                     encoded: "{0}".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 6),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6)),
             }
         );
         assert_eq!(reader.state.cursor, 5);
@@ -627,7 +403,7 @@ mod tests {
                     value: "\"hi\"".to_string(),
                     encoded: "\\\"hi\\\"".to_string()
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 9),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 9)),
             }
         );
         assert_eq!(reader.state.cursor, 8);
@@ -650,11 +426,11 @@ mod tests {
     #[test]
     fn test_quoted_string() {
         let mut reader = Reader::new("\"\"");
-        assert_eq!(quoted_string(&mut reader).unwrap(), "");
+        assert_eq!(quoted_oneline_string(&mut reader).unwrap(), "");
         assert_eq!(reader.state.cursor, 2);
 
         let mut reader = Reader::new("\"Hello\"");
-        assert_eq!(quoted_string(&mut reader).unwrap(), "Hello");
+        assert_eq!(quoted_oneline_string(&mut reader).unwrap(), "Hello");
         assert_eq!(reader.state.cursor, 7);
     }
 
@@ -666,7 +442,7 @@ mod tests {
             Template {
                 delimiter: Some('`'),
                 elements: vec![],
-                source_info: SourceInfo::new(1, 1, 1, 3),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 3)),
             }
         );
         assert_eq!(reader.state.cursor, 2);
@@ -680,7 +456,7 @@ mod tests {
                     value: "foo#".to_string(),
                     encoded: "foo#".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 7),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 7)),
             }
         );
         assert_eq!(reader.state.cursor, 6);
@@ -694,7 +470,7 @@ mod tests {
                     value: "{0}".to_string(),
                     encoded: "{0}".to_string(),
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 6),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6)),
             }
         );
         assert_eq!(reader.state.cursor, 5);
@@ -712,7 +488,7 @@ mod tests {
                     value: "`hi`".to_string(),
                     encoded: "\\`hi\\`".to_string()
                 }],
-                source_info: SourceInfo::new(1, 1, 1, 9),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 9)),
             }
         );
         assert_eq!(reader.state.cursor, 8);
