@@ -15,7 +15,8 @@
  * limitations under the License.
  *
  */
-use crate::ast::{Pos, SourceInfo, TemplateElement};
+use crate::ast::{Expr, Pos, SourceInfo, TemplateElement};
+use crate::parser::primitives::{literal, try_literal};
 use crate::parser::reader::*;
 use crate::parser::{error, expr, ParseResult};
 
@@ -24,75 +25,82 @@ pub struct EncodedString {
     pub chars: Vec<(char, String, Pos)>,
 }
 
-pub fn templatize(encoded_string: EncodedString) -> ParseResult<'static, Vec<TemplateElement>> {
+pub fn template(reader: &mut Reader) -> ParseResult<Expr> {
+    try_literal("{{", reader)?;
+    let expression = expr::parse2(reader)?;
+    literal("}}", reader)?;
+    Ok(expression)
+}
+
+pub fn templatize(encoded_string: EncodedString) -> ParseResult<Vec<TemplateElement>> {
     enum State {
-        String {},
-        Template {},
-        FirstOpenBracket {},
-        FirstCloseBracket {},
+        String,
+        Template,
+        FirstOpenBracket,
+        FirstCloseBracket,
     }
 
     let mut elements = vec![];
 
-    let mut value = "".to_string();
-    let mut encoded = "".to_string();
-    let mut state = State::String {};
+    let mut value = String::new();
+    let mut encoded = String::new();
+    let mut state = State::String;
     let mut expression_start = None;
 
     for (c, s, pos) in encoded_string.chars {
         match state {
-            State::String {} => {
+            State::String => {
                 if s.as_str() == "{" {
-                    state = State::FirstOpenBracket {};
+                    state = State::FirstOpenBracket;
                 } else {
                     value.push(c);
                     encoded.push_str(&s.clone());
                 }
             }
 
-            State::FirstOpenBracket {} => {
+            State::FirstOpenBracket => {
                 if s.as_str() == "{" {
                     if !value.is_empty() {
                         elements.push(TemplateElement::String { value, encoded });
-                        value = "".to_string();
-                        encoded = "".to_string();
+                        value = String::new();
+                        encoded = String::new();
                     }
-                    state = State::Template {};
+                    state = State::Template;
                 } else {
                     value.push('{');
                     encoded.push('{');
 
                     value.push(c);
                     encoded.push_str(&s.clone());
-                    state = State::String {};
+                    state = State::String;
                 }
             }
 
-            State::Template {} => {
+            State::Template => {
                 if expression_start.is_none() {
                     expression_start = Some(pos);
                 }
                 if s.as_str() == "}" {
-                    state = State::FirstCloseBracket {};
+                    state = State::FirstCloseBracket;
                 } else {
                     value.push(c);
                     encoded.push_str(&s.clone());
                 }
             }
 
-            State::FirstCloseBracket {} => {
+            State::FirstCloseBracket => {
                 if s.as_str() == "}" {
                     let mut reader = Reader::new(encoded.as_str());
                     reader.state = ReaderState {
                         cursor: 0,
-                        pos: expression_start.unwrap().clone(),
+                        pos: expression_start.unwrap(),
                     };
                     let expression = expr::parse2(&mut reader)?;
                     elements.push(TemplateElement::Expression(expression));
-                    value = "".to_string();
-                    encoded = "".to_string();
+                    value = String::new();
+                    encoded = String::new();
                     expression_start = None;
-                    state = State::String {};
+                    state = State::String;
                 } else {
                     value.push('}');
                     value.push(c);
@@ -104,19 +112,20 @@ pub fn templatize(encoded_string: EncodedString) -> ParseResult<'static, Vec<Tem
     }
 
     match state {
-        State::String {} => {}
-        State::FirstOpenBracket {} => {
+        State::String => {}
+        State::FirstOpenBracket => {
             value.push('{');
             encoded.push('{');
         }
-        State::Template {} | State::FirstCloseBracket {} => {
-            return Err(error::Error {
-                pos: encoded_string.source_info.end,
-                recoverable: false,
-                inner: error::ParseError::Expecting {
-                    value: "}}".to_string(),
-                },
-            });
+        State::Template | State::FirstCloseBracket => {
+            let inner = error::ParseError::Expecting {
+                value: "}}".to_string(),
+            };
+            return Err(error::Error::new(
+                encoded_string.source_info.end,
+                false,
+                inner,
+            ));
         }
     }
 
@@ -134,7 +143,7 @@ mod tests {
     #[test]
     fn test_templatize_empty_string() {
         let encoded_string = EncodedString {
-            source_info: SourceInfo::new(1, 1, 1, 1),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             chars: vec![],
         };
         assert_eq!(templatize(encoded_string).unwrap(), vec![]);
@@ -144,7 +153,7 @@ mod tests {
     fn test_templatize_hello_world() {
         // Hi\u0020{{name}}!
         let encoded_string = EncodedString {
-            source_info: SourceInfo::new(1, 1, 1, 18),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 18)),
             chars: vec![
                 ('H', "H".to_string(), Pos { line: 1, column: 1 }),
                 ('i', "i".to_string(), Pos { line: 1, column: 2 }),
@@ -225,16 +234,16 @@ mod tests {
                 },
                 TemplateElement::Expression(Expr {
                     space0: Whitespace {
-                        value: "".to_string(),
-                        source_info: SourceInfo::new(1, 11, 1, 11),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 11)),
                     },
                     variable: Variable {
                         name: "name".to_string(),
-                        source_info: SourceInfo::new(1, 11, 1, 15),
+                        source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 15)),
                     },
                     space1: Whitespace {
-                        value: "".to_string(),
-                        source_info: SourceInfo::new(1, 15, 1, 15),
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 15)),
                     },
                 }),
                 TemplateElement::String {
@@ -249,7 +258,7 @@ mod tests {
     fn test_templatize_expression_only() {
         // {{x}}!
         let encoded_string = EncodedString {
-            source_info: SourceInfo::new(1, 1, 1, 7),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 7)),
             chars: vec![
                 ('{', "{".to_string(), Pos { line: 1, column: 1 }),
                 ('{', "{".to_string(), Pos { line: 1, column: 2 }),
@@ -262,16 +271,16 @@ mod tests {
             templatize(encoded_string).unwrap(),
             vec![TemplateElement::Expression(Expr {
                 space0: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 3, 1, 3),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 3)),
                 },
                 variable: Variable {
                     name: "x".to_string(),
-                    source_info: SourceInfo::new(1, 3, 1, 4),
+                    source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
                 space1: Whitespace {
-                    value: "".to_string(),
-                    source_info: SourceInfo::new(1, 4, 1, 4),
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
                 },
             }),]
         );
@@ -282,7 +291,7 @@ mod tests {
         // missing closing
         // {{x
         let encoded_string = EncodedString {
-            source_info: SourceInfo::new(1, 1, 1, 4),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 4)),
             chars: vec![
                 ('{', "{".to_string(), Pos { line: 1, column: 1 }),
                 ('{', "{".to_string(), Pos { line: 1, column: 2 }),
@@ -305,7 +314,7 @@ mod tests {
         // \{\{
         // This is a valid string "{{"
         let encoded_string = EncodedString {
-            source_info: SourceInfo::new(1, 1, 1, 4),
+            source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 4)),
             chars: vec![
                 ('{', "\\{".to_string(), Pos { line: 1, column: 1 }),
                 ('{', "\\{".to_string(), Pos { line: 1, column: 2 }),
