@@ -21,40 +21,29 @@ use crate::parser::primitives::*;
 use crate::parser::reader::Reader;
 use crate::parser::{expr, ParseResult};
 
-pub fn url(reader: &mut Reader) -> ParseResult<'static, Template> {
-    // Can not be json-encoded, nor empty.
+pub fn url(reader: &mut Reader) -> ParseResult<Template> {
+    // Must be neither JSON-encoded nor empty.
     // But more restrictive: whitelist characters, not empty
-
-    let start = reader.state.clone();
+    let start = reader.state;
     let mut elements = vec![];
-    let mut buffer = String::from("");
+    let mut buffer = String::new();
 
-    if reader.is_eof() {
-        return Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::Url {},
-        });
-    }
-
-    // Urls must begin with http/https or { (templates).
-    let first_char = reader.peek();
-    if first_char != Some('h') && first_char != Some('{') {
-        return Err(Error {
-            pos: reader.state.pos.clone(),
-            recoverable: false,
-            inner: ParseError::UrlIllegalCharacter(first_char.unwrap()),
-        });
+    if !url_prefix_valid(reader) {
+        return Err(Error::new(
+            reader.state.pos,
+            false,
+            ParseError::UrlInvalidStart,
+        ));
     }
 
     loop {
-        let save = reader.state.clone();
+        let save = reader.state;
         match line_terminator(reader) {
             Ok(_) => {
                 reader.state = save;
                 break;
             }
-            _ => reader.state = save.clone(),
+            _ => reader.state = save,
         }
 
         match expr::parse(reader) {
@@ -64,7 +53,7 @@ pub fn url(reader: &mut Reader) -> ParseResult<'static, Template> {
                         value: buffer.clone(),
                         encoded: buffer.clone(),
                     });
-                    buffer = String::from("");
+                    buffer = String::new();
                 }
                 elements.push(TemplateElement::Expression(value));
             }
@@ -72,12 +61,12 @@ pub fn url(reader: &mut Reader) -> ParseResult<'static, Template> {
                 if !e.recoverable {
                     return Err(e);
                 } else {
-                    reader.state = save.clone();
+                    reader.state = save;
                     match reader.read() {
                         None => break,
                         Some(c) => {
                             if c.is_alphanumeric()
-                                | vec![
+                                | [
                                     ':', '/', '.', '-', '?', '=', '&', '_', '%', '*', ',', '@',
                                     '~', '+', '!', '$', '\'', '(', ')', ';', '[', ']',
                                 ]
@@ -94,6 +83,7 @@ pub fn url(reader: &mut Reader) -> ParseResult<'static, Template> {
             }
         }
     }
+
     if !buffer.is_empty() {
         elements.push(TemplateElement::String {
             value: buffer.clone(),
@@ -101,25 +91,16 @@ pub fn url(reader: &mut Reader) -> ParseResult<'static, Template> {
         });
     }
 
-    if elements.is_empty() {
-        reader.state = start.clone();
-        return Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::Url {},
-        });
-    }
-
-    // url should be followed by a line terminator
-    let save = reader.state.clone();
+    // URLs should be followed by a line terminator
+    let save = reader.state;
     if line_terminator(reader).is_err() {
         reader.state = save;
         let c = reader.peek().unwrap();
-        return Err(Error {
-            pos: reader.state.pos.clone(),
-            recoverable: false,
-            inner: ParseError::UrlIllegalCharacter(c),
-        });
+        return Err(Error::new(
+            reader.state.pos,
+            false,
+            ParseError::UrlIllegalCharacter(c),
+        ));
     }
 
     reader.state = save;
@@ -128,11 +109,23 @@ pub fn url(reader: &mut Reader) -> ParseResult<'static, Template> {
         elements,
         source_info: SourceInfo {
             start: start.pos,
-            end: reader.state.clone().pos,
+            end: reader.state.pos,
         },
     })
 }
 
+/// Returns true if url starts with http://, https:// or {{
+fn url_prefix_valid(reader: &mut Reader) -> bool {
+    let prefixes = ["https://", "http://", "{{"];
+    for expected_p in prefixes.iter() {
+        let current_p = reader.peek_n(expected_p.len());
+        if &current_p == expected_p {
+            return true;
+        }
+    }
+    false
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -148,7 +141,7 @@ mod tests {
                     encoded: String::from("http://google.fr"),
                 }],
                 delimiter: None,
-                source_info: SourceInfo::new(1, 1, 1, 17),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 17)),
             }
         );
         assert_eq!(reader.state.cursor, 16);
@@ -167,7 +160,7 @@ mod tests {
                     ),
                 }],
                 delimiter: None,
-                source_info: SourceInfo::new(1, 1, 1, 57),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 57)),
             }
         );
         assert_eq!(reader.state.cursor, 56);
@@ -186,16 +179,16 @@ mod tests {
                     },
                     TemplateElement::Expression(Expr {
                         space0: Whitespace {
-                            value: String::from(""),
-                            source_info: SourceInfo::new(1, 10, 1, 10),
+                            value: String::new(),
+                            source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 10)),
                         },
                         variable: Variable {
                             name: String::from("host"),
-                            source_info: SourceInfo::new(1, 10, 1, 14),
+                            source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 14)),
                         },
                         space1: Whitespace {
-                            value: String::from(""),
-                            source_info: SourceInfo::new(1, 14, 1, 14),
+                            value: String::new(),
+                            source_info: SourceInfo::new(Pos::new(1, 14), Pos::new(1, 14)),
                         },
                     }),
                     TemplateElement::String {
@@ -204,7 +197,7 @@ mod tests {
                     },
                 ],
                 delimiter: None,
-                source_info: SourceInfo::new(1, 1, 1, 19),
+                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 19)),
             }
         );
         assert_eq!(reader.state.cursor, 18);
@@ -256,7 +249,7 @@ mod tests {
         let mut reader = Reader::new(" # eol");
         let error = url(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(error.inner, ParseError::UrlIllegalCharacter(' '));
+        assert_eq!(error.inner, ParseError::UrlInvalidStart);
     }
 
     #[test]
@@ -272,9 +265,11 @@ mod tests {
             "http://www.google.com/?q=go+language",
             "http://www.google.com/?q=go%20language",
             "http://www.google.com/a%20b?q=c+d",
-            "http:www.google.com/?q=go+language",
-            "http:www.google.com/?q=go+language",
-            "http:%2f%2fwww.google.com/?q=go+language",
+            // The following URLs are supported in the Go test file
+            // but are not considered as valid URLs by curl
+            // "http:www.google.com/?q=go+language",
+            // "http:www.google.com/?q=go+language",
+            // "http:%2f%2fwww.google.com/?q=go+language",
             "http://user:password@google.com",
             "http://user:password@google.com",
             "http://j@ne:password@google.com",
@@ -319,4 +314,22 @@ mod tests {
             assert!(url(&mut reader).is_ok());
         }
     }
+
+    #[test]
+    fn test_invalid_urls() {
+        // from official url_test.go file
+        let invalid_urls = [
+            "foo.com",
+            "httpfoo.com",
+            "http:foo.com",
+            "https:foo.com",
+            "https:/foo.com",
+            "{https://foo.com",
+        ];
+
+        for s in invalid_urls {
+            let mut reader = Reader::new(s);
+            assert!(url(&mut reader).is_err());
+        }
+    }
 }
