@@ -15,12 +15,15 @@
  * limitations under the License.
  *
  */
-use super::OptionsError;
-use crate::cli::options::{InputFormat, OutputFormat};
-use atty::Stream;
+use std::io;
+use std::io::IsTerminal;
+use std::path::{Path, PathBuf};
+
 use clap::parser::ValueSource;
 use clap::ArgMatches;
-use std::path::{Path, PathBuf};
+
+use super::OptionsError;
+use crate::cli::options::{InputFormat, OutputFormat};
 
 pub fn check(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "check")
@@ -32,7 +35,7 @@ pub fn color(arg_matches: &ArgMatches) -> bool {
     } else if has_flag(arg_matches, "no_color") || has_flag(arg_matches, "in_place") {
         false
     } else {
-        atty::is(Stream::Stdout)
+        io::stdout().is_terminal()
     }
 }
 
@@ -98,7 +101,7 @@ pub fn input_files(arg_matches: &ArgMatches) -> Result<Vec<String>, OptionsError
             }
         }
     }
-    if files.is_empty() && !atty::is(Stream::Stdin) {
+    if files.is_empty() && !io::stdin().is_terminal() {
         files.push("-".to_string());
     }
     Ok(files)
@@ -110,7 +113,7 @@ pub fn output_file(arg_matches: &ArgMatches) -> Option<PathBuf> {
 
 pub fn standalone(arg_matches: &ArgMatches) -> Result<bool, OptionsError> {
     if has_flag(arg_matches, "standalone") {
-        if get_string(arg_matches, "format") != Some("html".to_string()) {
+        if get_string(arg_matches, "output_format") != Some("html".to_string()) {
             Err(OptionsError::Error(
                 "use --standalone option only with html output".to_string(),
             ))
