@@ -19,11 +19,11 @@
 mod commands;
 mod matches;
 
-use atty::Stream;
-use clap::ArgMatches;
 use std::env;
 use std::path::PathBuf;
 
+use clap::ArgMatches;
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Options {
     pub check: bool,
@@ -69,7 +69,7 @@ pub fn parse() -> Result<Options, OptionsError> {
     let mut command = clap::Command::new("hurlfmt")
         .version(clap::crate_version!())
         .disable_colored_help(true)
-        .about("Format hurl FILE")
+        .about("Format Hurl files")
         .arg(commands::check())
         .arg(commands::color())
         .arg(commands::format())
@@ -84,7 +84,7 @@ pub fn parse() -> Result<Options, OptionsError> {
     let arg_matches = command.try_get_matches_from_mut(env::args_os())?;
     let opts = parse_matches(&arg_matches)?;
 
-    if opts.input_files.is_empty() && atty::is(Stream::Stdin) {
+    if opts.input_files.is_empty() {
         let help = command.render_help().to_string();
         return Err(OptionsError::Error(help));
     }
