@@ -208,7 +208,7 @@ impl ToJson for File {
             ("type".to_string(), JValue::String("file".to_string())),
             (
                 "filename".to_string(),
-                JValue::String(self.filename.value.clone()),
+                JValue::String(self.filename.to_string()),
             ),
         ])
     }
@@ -219,6 +219,7 @@ fn get_json_version(version_value: &VersionValue) -> Option<String> {
         VersionValue::Version1 => Some("HTTP/1.0".to_string()),
         VersionValue::Version11 => Some("HTTP/1.1".to_string()),
         VersionValue::Version2 => Some("HTTP/2".to_string()),
+        VersionValue::Version3 => Some("HTTP/3".to_string()),
         VersionValue::VersionAny => None,
         VersionValue::VersionAnyLegacy => None,
     }
@@ -227,7 +228,7 @@ fn get_json_version(version_value: &VersionValue) -> Option<String> {
 impl ToJson for KeyValue {
     fn to_json(&self) -> JValue {
         let attributes = vec![
-            ("name".to_string(), JValue::String(self.key.value.clone())),
+            ("name".to_string(), JValue::String(self.key.to_string())),
             ("value".to_string(), JValue::String(self.value.to_string())),
         ];
         JValue::Object(attributes)
@@ -246,10 +247,10 @@ impl ToJson for MultipartParam {
 impl ToJson for FileParam {
     fn to_json(&self) -> JValue {
         let mut attributes = vec![
-            ("name".to_string(), JValue::String(self.key.value.clone())),
+            ("name".to_string(), JValue::String(self.key.to_string())),
             (
                 "filename".to_string(),
-                JValue::String(self.value.filename.value.clone()),
+                JValue::String(self.value.filename.to_string()),
             ),
         ];
         if let Some(content_type) = self.value.content_type.clone() {
@@ -262,7 +263,7 @@ impl ToJson for FileParam {
 impl ToJson for Cookie {
     fn to_json(&self) -> JValue {
         let attributes = vec![
-            ("name".to_string(), JValue::String(self.name.value.clone())),
+            ("name".to_string(), JValue::String(self.name.to_string())),
             ("value".to_string(), JValue::String(self.value.to_string())),
         ];
         JValue::Object(attributes)
@@ -271,90 +272,92 @@ impl ToJson for Cookie {
 
 impl ToJson for EntryOption {
     fn to_json(&self) -> JValue {
-        let attributes = match self {
-            EntryOption::CaCertificate(value) => vec![
-                ("name".to_string(), JValue::String("cacert".to_string())),
-                (
-                    "value".to_string(),
-                    JValue::String(value.filename.value.clone()),
-                ),
-            ],
-            EntryOption::ClientCert(value) => vec![
-                ("name".to_string(), JValue::String("cert".to_string())),
-                (
-                    "value".to_string(),
-                    JValue::String(value.filename.value.clone()),
-                ),
-            ],
-            EntryOption::ClientKey(value) => vec![
-                ("name".to_string(), JValue::String("key".to_string())),
-                (
-                    "value".to_string(),
-                    JValue::String(value.filename.value.clone()),
-                ),
-            ],
-            EntryOption::Compressed(value) => vec![
-                ("name".to_string(), JValue::String("compressed".to_string())),
-                ("value".to_string(), JValue::Boolean(value.value)),
-            ],
-            EntryOption::Insecure(value) => vec![
-                ("name".to_string(), JValue::String("insecure".to_string())),
-                ("value".to_string(), JValue::Boolean(value.value)),
-            ],
-            EntryOption::FollowLocation(value) => vec![
-                ("name".to_string(), JValue::String("location".to_string())),
-                ("value".to_string(), JValue::Boolean(value.value)),
-            ],
-            EntryOption::MaxRedirect(value) => vec![
-                ("name".to_string(), JValue::String("max-redirs".to_string())),
-                ("value".to_string(), JValue::Number(value.value.to_string())),
-            ],
-            EntryOption::PathAsIs(value) => vec![
-                ("name".to_string(), JValue::String("path-as-is".to_string())),
-                ("value".to_string(), JValue::Boolean(value.value)),
-            ],
-            EntryOption::Proxy(value) => vec![
-                ("name".to_string(), JValue::String("proxy".to_string())),
-                ("value".to_string(), JValue::String(value.value.clone())),
-            ],
-            EntryOption::Retry(value) => vec![
-                ("name".to_string(), JValue::String("retry".to_string())),
-                ("value".to_string(), JValue::Number(value.value.to_string())),
-            ],
-            EntryOption::RetryInterval(value) => vec![
-                (
-                    "name".to_string(),
-                    JValue::String("retry-interval".to_string()),
-                ),
-                ("value".to_string(), JValue::Number(value.value.to_string())),
-            ],
-            EntryOption::Variable(value) => vec![
-                ("name".to_string(), JValue::String("variable".to_string())),
-                (
-                    "value".to_string(),
-                    JValue::String(format!("{}={}", value.value.name, value.value.value)),
-                ),
-            ],
-            EntryOption::Verbose(value) => vec![
-                ("name".to_string(), JValue::String("verbose".to_string())),
-                ("value".to_string(), JValue::Boolean(value.value)),
-            ],
-            EntryOption::VeryVerbose(value) => vec![
-                (
-                    "name".to_string(),
-                    JValue::String("very-verbose".to_string()),
-                ),
-                ("value".to_string(), JValue::Boolean(value.value)),
-            ],
+        let mut attributes = vec![];
+
+        let name = "name".to_string();
+        let value = JValue::String(self.kind.name().to_string());
+        attributes.push((name, value));
+
+        let name = "value".to_string();
+        let value = match &self.kind {
+            OptionKind::AwsSigV4(value) => JValue::String(value.to_string()),
+            OptionKind::CaCertificate(filename) => JValue::String(filename.to_string()),
+            OptionKind::ClientCert(filename) => JValue::String(filename.to_string()),
+            OptionKind::ClientKey(filename) => JValue::String(filename.to_string()),
+            OptionKind::Compressed(value) => value.to_json(),
+            OptionKind::ConnectTo(value) => JValue::String(value.to_string()),
+            OptionKind::Delay(value) => value.to_json(),
+            OptionKind::FollowLocation(value) => value.to_json(),
+            OptionKind::FollowLocationTrusted(value) => value.to_json(),
+            OptionKind::Http10(value) => value.to_json(),
+            OptionKind::Http11(value) => value.to_json(),
+            OptionKind::Http2(value) => value.to_json(),
+            OptionKind::Http3(value) => value.to_json(),
+            OptionKind::Insecure(value) => value.to_json(),
+            OptionKind::IpV4(value) => value.to_json(),
+            OptionKind::IpV6(value) => value.to_json(),
+            OptionKind::MaxRedirect(value) => value.to_json(),
+            OptionKind::Output(filename) => JValue::String(filename.to_string()),
+            OptionKind::PathAsIs(value) => value.to_json(),
+            OptionKind::Proxy(value) => JValue::String(value.to_string()),
+            OptionKind::Resolve(value) => JValue::String(value.to_string()),
+            OptionKind::Retry(value) => value.to_json(),
+            OptionKind::RetryInterval(value) => value.to_json(),
+            OptionKind::Skip(value) => value.to_json(),
+            OptionKind::UnixSocket(value) => JValue::String(value.to_string()),
+            OptionKind::Variable(value) => {
+                JValue::String(format!("{}={}", value.name, value.value))
+            }
+            OptionKind::Verbose(value) => value.to_json(),
+            OptionKind::VeryVerbose(value) => value.to_json(),
         };
+        attributes.push((name, value));
+
         JValue::Object(attributes)
     }
 }
 
+impl ToJson for BooleanOption {
+    fn to_json(&self) -> JValue {
+        match self {
+            BooleanOption::Literal(value) => JValue::Boolean(*value),
+            BooleanOption::Expression(expr) => expr.to_json(),
+        }
+    }
+}
+
+impl ToJson for NaturalOption {
+    fn to_json(&self) -> JValue {
+        match self {
+            NaturalOption::Literal(value) => JValue::Number(value.to_string()),
+            NaturalOption::Expression(expr) => expr.to_json(),
+        }
+    }
+}
+
+impl ToJson for RetryOption {
+    fn to_json(&self) -> JValue {
+        match self {
+            RetryOption::Literal(value) => value.to_json(),
+            RetryOption::Expression(expr) => expr.to_json(),
+        }
+    }
+}
+
+impl ToJson for Retry {
+    fn to_json(&self) -> JValue {
+        match self {
+            Retry::None => JValue::Number("0".to_string()),
+            Retry::Finite(value) => JValue::Number(value.to_string()),
+            Retry::Infinite => JValue::Number("-1".to_string()),
+        }
+    }
+}
+
 impl ToJson for Capture {
     fn to_json(&self) -> JValue {
         let mut attributes = vec![
-            ("name".to_string(), JValue::String(self.name.value.clone())),
+            ("name".to_string(), JValue::String(self.name.to_string())),
             ("query".to_string(), self.query.to_json()),
         ];
         if !self.filters.is_empty() {
@@ -387,13 +390,13 @@ impl ToJson for Query {
 fn query_value_attributes(query_value: &QueryValue) -> Vec<(String, JValue)> {
     let mut attributes = vec![];
     match query_value {
-        QueryValue::Status {} => {
+        QueryValue::Status => {
             attributes.push(("type".to_string(), JValue::String("status".to_string())));
         }
-        QueryValue::Url {} => {
+        QueryValue::Url => {
             attributes.push(("type".to_string(), JValue::String("url".to_string())));
         }
-        QueryValue::Body {} => {
+        QueryValue::Body => {
             attributes.push(("type".to_string(), JValue::String("body".to_string())));
         }
         QueryValue::Jsonpath { expr, .. } => {
@@ -420,16 +423,16 @@ fn query_value_attributes(query_value: &QueryValue) -> Vec<(String, JValue)> {
             attributes.push(("type".to_string(), JValue::String("variable".to_string())));
             attributes.push(("name".to_string(), JValue::String(name.to_string())));
         }
-        QueryValue::Duration {} => {
+        QueryValue::Duration => {
             attributes.push(("type".to_string(), JValue::String("duration".to_string())));
         }
-        QueryValue::Bytes {} => {
+        QueryValue::Bytes => {
             attributes.push(("type".to_string(), JValue::String("bytes".to_string())));
         }
-        QueryValue::Sha256 {} => {
+        QueryValue::Sha256 => {
             attributes.push(("type".to_string(), JValue::String("sha256".to_string())));
         }
-        QueryValue::Md5 {} => {
+        QueryValue::Md5 => {
             attributes.push(("type".to_string(), JValue::String("md5".to_string())));
         }
         QueryValue::Certificate {
@@ -515,10 +518,6 @@ impl ToJson for Predicate {
                 ));
                 add_predicate_value(&mut attributes, value);
             }
-            PredicateFuncValue::CountEqual { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("count".to_string())));
-                add_predicate_value(&mut attributes, value);
-            }
             PredicateFuncValue::StartWith { value, .. } => {
                 attributes.push(("type".to_string(), JValue::String("start-with".to_string())));
                 add_predicate_value(&mut attributes, value);
@@ -539,28 +538,31 @@ impl ToJson for Predicate {
                 attributes.push(("type".to_string(), JValue::String("match".to_string())));
                 add_predicate_value(&mut attributes, value);
             }
-            PredicateFuncValue::IsInteger {} => {
+            PredicateFuncValue::IsInteger => {
                 attributes.push(("type".to_string(), JValue::String("isInteger".to_string())));
             }
-            PredicateFuncValue::IsFloat {} => {
+            PredicateFuncValue::IsFloat => {
                 attributes.push(("type".to_string(), JValue::String("isFloat".to_string())));
             }
-            PredicateFuncValue::IsBoolean {} => {
+            PredicateFuncValue::IsBoolean => {
                 attributes.push(("type".to_string(), JValue::String("isBoolean".to_string())));
             }
-            PredicateFuncValue::IsString {} => {
+            PredicateFuncValue::IsString => {
                 attributes.push(("type".to_string(), JValue::String("isString".to_string())));
             }
-            PredicateFuncValue::IsCollection {} => {
+            PredicateFuncValue::IsCollection => {
                 attributes.push((
                     "type".to_string(),
                     JValue::String("isCollection".to_string()),
                 ));
             }
-            PredicateFuncValue::Exist {} => {
+            PredicateFuncValue::IsDate => {
+                attributes.push(("type".to_string(), JValue::String("isDate".to_string())));
+            }
+            PredicateFuncValue::Exist => {
                 attributes.push(("type".to_string(), JValue::String("exist".to_string())));
             }
-            PredicateFuncValue::IsEmpty {} => {
+            PredicateFuncValue::IsEmpty => {
                 attributes.push(("type".to_string(), JValue::String("isEmpty".to_string())));
             }
         }
@@ -580,10 +582,10 @@ fn json_predicate_value(predicate_value: PredicateValue) -> (JValue, Option<Stri
     match predicate_value {
         PredicateValue::String(value) => (JValue::String(value.to_string()), None),
         PredicateValue::MultilineString(value) => (JValue::String(value.value().to_string()), None),
-        PredicateValue::Integer(value) => (JValue::Number(value.to_string()), None),
-        PredicateValue::Float(value) => (JValue::Number(value.to_string()), None),
         PredicateValue::Bool(value) => (JValue::Boolean(value), None),
-        PredicateValue::Null {} => (JValue::Null, None),
+        PredicateValue::Null => (JValue::Null, None),
+        PredicateValue::Number(value) => (JValue::Number(value.to_string()), None),
+        PredicateValue::File(value) => (value.to_json(), None),
         PredicateValue::Hex(value) => {
             let base64_string = general_purpose::STANDARD.encode(value.value);
             (JValue::String(base64_string), Some("base64".to_string()))
@@ -602,7 +604,7 @@ fn json_predicate_value(predicate_value: PredicateValue) -> (JValue, Option<Stri
 impl ToJson for JsonValue {
     fn to_json(&self) -> JValue {
         match self {
-            JsonValue::Null {} => JValue::Null {},
+            JsonValue::Null => JValue::Null,
             JsonValue::Number(s) => JValue::Number(s.to_string()),
             JsonValue::String(s) => JValue::String(s.to_string()),
             JsonValue::Boolean(v) => JValue::Boolean(*v),
@@ -659,6 +661,10 @@ impl ToJson for FilterValue {
                 attributes.push(("type".to_string(), JValue::String("format".to_string())));
                 attributes.push(("fmt".to_string(), JValue::String(fmt.to_string())));
             }
+            FilterValue::JsonPath { expr, .. } => {
+                attributes.push(("type".to_string(), JValue::String("jsonpath".to_string())));
+                attributes.push(("expr".to_string(), JValue::String(expr.to_string())));
+            }
             FilterValue::Nth { n, .. } => {
                 attributes.push(("type".to_string(), JValue::String("nth".to_string())));
                 attributes.push(("n".to_string(), JValue::Number(n.to_string())));
@@ -706,7 +712,7 @@ impl ToJson for FilterValue {
                 attributes.push(("type".to_string(), JValue::String("toInt".to_string())));
             }
             FilterValue::XPath { expr, .. } => {
-                attributes.push(("type".to_string(), JValue::String("toDate".to_string())));
+                attributes.push(("type".to_string(), JValue::String("xpath".to_string())));
                 attributes.push(("expr".to_string(), JValue::String(expr.to_string())));
             }
         }
@@ -714,14 +720,20 @@ impl ToJson for FilterValue {
     }
 }
 
+impl ToJson for Expr {
+    fn to_json(&self) -> JValue {
+        JValue::String(format!("{{{{{}}}}}", self))
+    }
+}
+
 #[cfg(test)]
 pub mod tests {
     use super::*;
 
     fn whitespace() -> Whitespace {
         Whitespace {
-            value: "".to_string(),
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            value: String::new(),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         }
     }
 
@@ -747,17 +759,19 @@ pub mod tests {
                         value: "http://example.com".to_string(),
                         encoded: "not_used".to_string(),
                     }],
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
                 line_terminator0: line_terminator(),
                 headers: vec![KeyValue {
                     line_terminators: vec![],
                     space0: whitespace(),
-                    key: EncodedString {
-                        value: "Foo".to_string(),
-                        encoded: "unused".to_string(),
-                        quotes: false,
-                        source_info: SourceInfo::new(0, 0, 0, 0),
+                    key: Template {
+                        delimiter: None,
+                        elements: vec![TemplateElement::String {
+                            value: "Foo".to_string(),
+                            encoded: "unused".to_string(),
+                        }],
+                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                     },
                     space1: whitespace(),
                     space2: whitespace(),
@@ -767,13 +781,13 @@ pub mod tests {
                             value: "Bar".to_string(),
                             encoded: "unused".to_string(),
                         }],
-                        source_info: SourceInfo::new(0, 0, 0, 0),
+                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                     },
                     line_terminator0: line_terminator(),
                 }],
                 sections: vec![],
                 body: None,
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             }
             .to_json(),
             JValue::Object(vec![
@@ -800,19 +814,19 @@ pub mod tests {
                 line_terminators: vec![],
                 version: Version {
                     value: VersionValue::Version11,
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
                 space0: whitespace(),
                 status: Status {
                     value: StatusValue::Specific(200),
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
                 space1: whitespace(),
                 line_terminator0: line_terminator(),
                 headers: vec![],
                 sections: vec![],
                 body: None,
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             }
             .to_json(),
             JValue::Object(vec![
@@ -828,19 +842,19 @@ pub mod tests {
                 line_terminators: vec![],
                 version: Version {
                     value: VersionValue::VersionAny,
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
                 space0: whitespace(),
                 status: Status {
                     value: StatusValue::Any,
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
                 space1: whitespace(),
                 line_terminator0: line_terminator(),
                 headers: vec![],
                 sections: vec![],
                 body: None,
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             }
             .to_json(),
             JValue::Object(vec![])
@@ -849,7 +863,7 @@ pub mod tests {
 
     fn header_query() -> Query {
         Query {
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Header {
                 space0: whitespace(),
                 name: Template {
@@ -858,7 +872,7 @@ pub mod tests {
                         value: "Content-Length".to_string(),
                         encoded: "10".to_string(),
                     }],
-                    source_info: SourceInfo::new(0, 0, 0, 0),
+                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 },
             },
         }
@@ -868,11 +882,13 @@ pub mod tests {
         Capture {
             line_terminators: vec![],
             space0: whitespace(),
-            name: EncodedString {
-                value: "size".to_string(),
-                encoded: "unused".to_string(),
-                quotes: false,
-                source_info: SourceInfo::new(0, 0, 0, 0),
+            name: Template {
+                delimiter: None,
+                elements: vec![TemplateElement::String {
+                    value: "size".to_string(),
+                    encoded: "unused".to_string(),
+                }],
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             space1: whitespace(),
             space2: whitespace(),
@@ -899,10 +915,10 @@ pub mod tests {
             not: false,
             space0: whitespace(),
             predicate_func: PredicateFunc {
-                source_info: SourceInfo::new(0, 0, 0, 0),
+                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 value: PredicateFuncValue::Equal {
                     space0: whitespace(),
-                    value: PredicateValue::Integer(value),
+                    value: PredicateValue::Number(Number::Integer(value)),
                     operator: false,
                 },
             },
