@@ -33,16 +33,16 @@ pub enum JValue {
 }
 
 impl JValue {
-    pub fn format(self) -> String {
+    pub fn format(&self) -> String {
         match self {
             JValue::Null => "null".to_string(),
-            JValue::Number(n) => n,
+            JValue::Number(n) => n.to_string(),
             JValue::String(s) => format!("\"{}\"", s.chars().map(format_char).collect::<String>()),
             JValue::Boolean(b) => b.to_string(),
             JValue::List(elem) => {
                 let s = elem
                     .iter()
-                    .map(|e| e.clone().format())
+                    .map(|e| e.format())
                     .collect::<Vec<String>>()
                     .join(",");
                 format!("[{s}]")
@@ -50,7 +50,7 @@ impl JValue {
             JValue::Object(key_values) => {
                 let s = key_values
                     .iter()
-                    .map(|(k, v)| format!("\"{}\":{}", k, v.clone().format()))
+                    .map(|(k, v)| format!("\"{}\":{}", k, v.format()))
                     .collect::<Vec<String>>()
                     .join(",");
                 format!("{{{s}}}")
@@ -95,7 +95,7 @@ pub mod tests {
 
     #[test]
     pub fn format_scalars() {
-        assert_eq!(JValue::Null {}.format(), "null");
+        assert_eq!(JValue::Null.format(), "null");
         assert_eq!(JValue::Number("1.0".to_string()).format(), "1.0");
         assert_eq!(JValue::String("hello".to_string()).format(), "\"hello\"");
         assert_eq!(JValue::Boolean(true).format(), "true");
