@@ -66,7 +66,7 @@ impl Tokenizable for Entry {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![];
         tokens.append(&mut self.request.tokenize());
-        if let Some(response) = self.clone().response {
+        if let Some(response) = &self.response {
             tokens.append(&mut response.tokenize());
         }
         tokens
@@ -90,7 +90,7 @@ impl Tokenizable for Request {
         tokens.append(&mut self.line_terminator0.tokenize());
         tokens.append(&mut self.headers.iter().flat_map(|e| e.tokenize()).collect());
         tokens.append(&mut self.sections.iter().flat_map(|e| e.tokenize()).collect());
-        if let Some(body) = self.clone().body {
+        if let Some(body) = &self.body {
             tokens.append(&mut body.tokenize());
         }
         tokens
@@ -206,7 +206,9 @@ impl Tokenizable for SectionValue {
                 tokens.append(&mut items.iter().flat_map(|e| e.tokenize()).collect());
             }
             SectionValue::BasicAuth(item) => {
-                tokens.append(&mut item.tokenize());
+                if let Some(kv) = item {
+                    tokens.append(&mut kv.tokenize())
+                }
             }
             SectionValue::FormParams(items) => {
                 tokens.append(&mut items.iter().flat_map(|e| e.tokenize()).collect());
@@ -358,7 +360,7 @@ impl Tokenizable for Capture {
         tokens.append(&mut self.space2.tokenize());
         tokens.append(&mut self.query.tokenize());
         for (space, filter) in &self.filters {
-            tokens.append(&mut space.clone().tokenize());
+            tokens.append(&mut space.tokenize());
             tokens.append(&mut filter.tokenize());
         }
         tokens.append(&mut self.line_terminator0.tokenize());
@@ -379,7 +381,7 @@ impl Tokenizable for Assert {
         tokens.append(&mut self.space0.tokenize());
         tokens.append(&mut self.query.tokenize());
         for (space, filter) in &self.filters {
-            tokens.append(&mut space.clone().tokenize());
+            tokens.append(&mut space.tokenize());
             tokens.append(&mut filter.tokenize());
         }
         tokens.append(&mut self.space1.tokenize());
@@ -393,7 +395,7 @@ impl Tokenizable for Assert {
 
 impl Tokenizable for Query {
     fn tokenize(&self) -> Vec<Token> {
-        self.value.clone().tokenize()
+        self.value.tokenize()
     }
 }
 
@@ -401,8 +403,8 @@ impl Tokenizable for QueryValue {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![];
         match self.clone() {
-            QueryValue::Status {} => tokens.push(Token::QueryType(String::from("status"))),
-            QueryValue::Url {} => tokens.push(Token::QueryType(String::from("url"))),
+            QueryValue::Status => tokens.push(Token::QueryType(String::from("status"))),
+            QueryValue::Url => tokens.push(Token::QueryType(String::from("url"))),
             QueryValue::Header { space0, name } => {
                 tokens.push(Token::QueryType(String::from("header")));
                 tokens.append(&mut space0.tokenize());
@@ -415,7 +417,7 @@ impl Tokenizable for QueryValue {
                 tokens.append(&mut expr.tokenize());
                 tokens.push(Token::CodeDelimiter("\"".to_string()));
             }
-            QueryValue::Body {} => tokens.push(Token::QueryType(String::from("body"))),
+            QueryValue::Body => tokens.push(Token::QueryType(String::from("body"))),
             QueryValue::Xpath { space0, expr } => {
                 tokens.push(Token::QueryType(String::from("xpath")));
                 tokens.append(&mut space0.tokenize());
@@ -436,10 +438,10 @@ impl Tokenizable for QueryValue {
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut name.tokenize());
             }
-            QueryValue::Duration {} => tokens.push(Token::QueryType(String::from("duration"))),
-            QueryValue::Bytes {} => tokens.push(Token::QueryType(String::from("bytes"))),
-            QueryValue::Sha256 {} => tokens.push(Token::QueryType(String::from("sha256"))),
-            QueryValue::Md5 {} => tokens.push(Token::QueryType(String::from("md5"))),
+            QueryValue::Duration => tokens.push(Token::QueryType(String::from("duration"))),
+            QueryValue::Bytes => tokens.push(Token::QueryType(String::from("bytes"))),
+            QueryValue::Sha256 => tokens.push(Token::QueryType(String::from("sha256"))),
+            QueryValue::Md5 => tokens.push(Token::QueryType(String::from("md5"))),
             QueryValue::Certificate {
                 space0,
                 attribute_name: field,
@@ -549,11 +551,6 @@ impl Tokenizable for PredicateFuncValue {
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
-            PredicateFuncValue::CountEqual { space0, value } => {
-                tokens.push(Token::PredicateType(self.name()));
-                tokens.append(&mut space0.tokenize());
-                tokens.append(&mut value.tokenize());
-            }
             PredicateFuncValue::StartWith { space0, value } => {
                 tokens.push(Token::PredicateType(self.name()));
                 tokens.append(&mut space0.tokenize());
@@ -580,25 +577,28 @@ impl Tokenizable for PredicateFuncValue {
                 tokens.append(&mut value.tokenize());
             }
 
-            PredicateFuncValue::IsInteger {} => {
+            PredicateFuncValue::IsInteger => {
                 tokens.push(Token::PredicateType(self.name()));
             }
-            PredicateFuncValue::IsFloat {} => {
+            PredicateFuncValue::IsFloat => {
                 tokens.push(Token::PredicateType(self.name()));
             }
-            PredicateFuncValue::IsBoolean {} => {
+            PredicateFuncValue::IsBoolean => {
                 tokens.push(Token::PredicateType(self.name()));
             }
-            PredicateFuncValue::IsString {} => {
+            PredicateFuncValue::IsString => {
                 tokens.push(Token::PredicateType(self.name()));
             }
-            PredicateFuncValue::IsCollection {} => {
+            PredicateFuncValue::IsCollection => {
                 tokens.push(Token::PredicateType(self.name()));
             }
-            PredicateFuncValue::Exist {} => {
+            PredicateFuncValue::IsDate => {
                 tokens.push(Token::PredicateType(self.name()));
             }
-            PredicateFuncValue::IsEmpty {} => {
+            PredicateFuncValue::Exist => {
+                tokens.push(Token::PredicateType(self.name()));
+            }
+            PredicateFuncValue::IsEmpty => {
                 tokens.push(Token::PredicateType(self.name()));
             }
         }
@@ -611,10 +611,10 @@ impl Tokenizable for PredicateValue {
         match self {
             PredicateValue::String(value) => value.tokenize(),
             PredicateValue::MultilineString(value) => value.tokenize(),
-            PredicateValue::Integer(value) => vec![Token::Number(value.to_string())],
-            PredicateValue::Float(value) => vec![Token::Number(value.to_string())],
             PredicateValue::Bool(value) => vec![Token::Boolean(value.to_string())],
-            PredicateValue::Null {} => vec![Token::Keyword("null".to_string())],
+            PredicateValue::Null => vec![Token::Keyword("null".to_string())],
+            PredicateValue::Number(value) => vec![Token::Number(value.to_string())],
+            PredicateValue::File(value) => value.tokenize(),
             PredicateValue::Hex(value) => vec![Token::String(value.to_string())],
             PredicateValue::Base64(value) => value.tokenize(),
             PredicateValue::Expression(value) => value.tokenize(),
@@ -683,14 +683,14 @@ impl Tokenizable for EncodedString {
         let mut tokens: Vec<Token> = vec![];
         if self.quotes {
             tokens.push(Token::StringDelimiter(
-                if self.clone().quotes { "\"" } else { "" }.to_string(),
+                if self.quotes { "\"" } else { "" }.to_string(),
             ));
         }
         tokens.push(Token::String(self.encoded.clone()));
 
         if self.quotes {
             tokens.push(Token::StringDelimiter(
-                if self.clone().quotes { "\"" } else { "" }.to_string(),
+                if self.quotes { "\"" } else { "" }.to_string(),
             ));
         }
         tokens
@@ -750,7 +750,7 @@ impl Tokenizable for LineTerminator {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![];
         tokens.append(&mut self.space0.tokenize());
-        if let Some(comment) = self.clone().comment {
+        if let Some(comment) = &self.comment {
             tokens.append(&mut comment.tokenize());
         }
         tokens.append(&mut self.newline.tokenize());
@@ -762,7 +762,7 @@ impl Tokenizable for Whitespace {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![];
         if !self.value.is_empty() {
-            tokens.push(Token::Whitespace(self.clone().value));
+            tokens.push(Token::Whitespace(self.value.clone()));
         }
         tokens
     }
@@ -770,14 +770,7 @@ impl Tokenizable for Whitespace {
 
 impl Tokenizable for Comment {
     fn tokenize(&self) -> Vec<Token> {
-        vec![Token::Comment(format!("#{}", self.clone().value))]
-    }
-}
-
-impl Tokenizable for Filename {
-    fn tokenize(&self) -> Vec<Token> {
-        let s = self.clone().value.replace(' ', "\\ ");
-        vec![Token::String(s)]
+        vec![Token::Comment(format!("#{}", self.value.clone()))]
     }
 }
 
@@ -818,7 +811,7 @@ impl Tokenizable for JsonValue {
                 }
                 tokens.push(Token::CodeDelimiter("}".to_string()));
             }
-            JsonValue::Null {} => {
+            JsonValue::Null => {
                 tokens.push(Token::Keyword("null".to_string()));
             }
             JsonValue::Expression(exp) => {
@@ -852,237 +845,86 @@ impl Tokenizable for JsonObjectElement {
         tokens
     }
 }
-
 impl Tokenizable for EntryOption {
+    fn tokenize(&self) -> Vec<Token> {
+        let mut tokens: Vec<Token> = vec![];
+        tokens.append(
+            &mut self
+                .line_terminators
+                .iter()
+                .flat_map(|e| e.tokenize())
+                .collect(),
+        );
+        tokens.append(&mut self.space0.tokenize());
+        tokens.push(Token::String(self.kind.name().to_string()));
+        tokens.append(&mut self.space1.tokenize());
+        tokens.push(Token::Colon(String::from(":")));
+        tokens.append(&mut self.space2.tokenize());
+        tokens.append(&mut self.kind.tokenize());
+        tokens.append(&mut self.line_terminator0.tokenize());
+        tokens
+    }
+}
+
+impl Tokenizable for OptionKind {
     fn tokenize(&self) -> Vec<Token> {
         match self {
-            EntryOption::CaCertificate(option) => option.tokenize(),
-            EntryOption::ClientCert(option) => option.tokenize(),
-            EntryOption::ClientKey(option) => option.tokenize(),
-            EntryOption::Compressed(option) => option.tokenize(),
-            EntryOption::Insecure(option) => option.tokenize(),
-            EntryOption::FollowLocation(option) => option.tokenize(),
-            EntryOption::MaxRedirect(option) => option.tokenize(),
-            EntryOption::PathAsIs(option) => option.tokenize(),
-            EntryOption::Proxy(option) => option.tokenize(),
-            EntryOption::Retry(option) => option.tokenize(),
-            EntryOption::RetryInterval(option) => option.tokenize(),
-            EntryOption::Variable(option) => option.tokenize(),
-            EntryOption::Verbose(option) => option.tokenize(),
-            EntryOption::VeryVerbose(option) => option.tokenize(),
+            OptionKind::AwsSigV4(value) => value.tokenize(),
+            OptionKind::CaCertificate(filename) => filename.tokenize(),
+            OptionKind::ClientCert(filename) => filename.tokenize(),
+            OptionKind::ClientKey(filename) => filename.tokenize(),
+            OptionKind::Compressed(value) => value.tokenize(),
+            OptionKind::ConnectTo(value) => value.tokenize(),
+            OptionKind::Delay(value) => value.tokenize(),
+            OptionKind::FollowLocation(value) => value.tokenize(),
+            OptionKind::FollowLocationTrusted(value) => value.tokenize(),
+            OptionKind::Http10(value) => value.tokenize(),
+            OptionKind::Http11(value) => value.tokenize(),
+            OptionKind::Http2(value) => value.tokenize(),
+            OptionKind::Http3(value) => value.tokenize(),
+            OptionKind::Insecure(value) => value.tokenize(),
+            OptionKind::IpV4(value) => value.tokenize(),
+            OptionKind::IpV6(value) => value.tokenize(),
+            OptionKind::MaxRedirect(value) => value.tokenize(),
+            OptionKind::Output(filename) => filename.tokenize(),
+            OptionKind::PathAsIs(value) => value.tokenize(),
+            OptionKind::Proxy(value) => value.tokenize(),
+            OptionKind::Resolve(value) => value.tokenize(),
+            OptionKind::Retry(value) => value.tokenize(),
+            OptionKind::RetryInterval(value) => value.tokenize(),
+            OptionKind::Skip(value) => value.tokenize(),
+            OptionKind::UnixSocket(value) => value.tokenize(),
+            OptionKind::Variable(value) => value.tokenize(),
+            OptionKind::Verbose(value) => value.tokenize(),
+            OptionKind::VeryVerbose(value) => value.tokenize(),
         }
     }
 }
 
-impl Tokenizable for CaCertificateOption {
+impl Tokenizable for BooleanOption {
     fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("cacert".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.append(&mut self.filename.tokenize());
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
+        match self {
+            BooleanOption::Literal(value) => vec![Token::Boolean(value.to_string())],
+            BooleanOption::Expression(expr) => expr.tokenize(),
+        }
     }
 }
 
-impl Tokenizable for ClientCertOption {
+impl Tokenizable for NaturalOption {
     fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("cert".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.append(&mut self.filename.tokenize());
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
-impl Tokenizable for ClientKeyOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("key".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.append(&mut self.filename.tokenize());
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
-impl Tokenizable for CompressedOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("compressed".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.push(Token::Boolean(self.value.to_string()));
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
-impl Tokenizable for InsecureOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("insecure".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.push(Token::Boolean(self.value.to_string()));
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
-impl Tokenizable for FollowLocationOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("location".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.push(Token::Boolean(self.value.to_string()));
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
-impl Tokenizable for MaxRedirectOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("max-redirs".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.push(Token::Number(self.value.to_string()));
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
-impl Tokenizable for PathAsIsOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("path-as-is".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.push(Token::Boolean(self.value.to_string()));
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
-impl Tokenizable for ProxyOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("proxy".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.push(Token::String(self.value.clone()));
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
+        match self {
+            NaturalOption::Literal(value) => vec![Token::Number(value.to_string())],
+            NaturalOption::Expression(expr) => expr.tokenize(),
+        }
     }
 }
 
 impl Tokenizable for RetryOption {
     fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        if !matches!(self.value, Retry::None) {
-            tokens.push(Token::String("retry".to_string()));
-            tokens.append(&mut self.space1.tokenize());
-            tokens.push(Token::Colon(String::from(":")));
-            tokens.append(&mut self.space2.tokenize());
-            tokens.append(&mut self.value.tokenize());
+        match self {
+            RetryOption::Literal(retry) => retry.tokenize(),
+            RetryOption::Expression(expr) => expr.tokenize(),
         }
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
     }
 }
 
@@ -1096,48 +938,6 @@ impl Tokenizable for Retry {
     }
 }
 
-impl Tokenizable for RetryIntervalOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("retry-interval".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.push(Token::Number(self.value.to_string()));
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
-impl Tokenizable for VariableOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("variable".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.append(&mut self.value.tokenize());
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
 impl Tokenizable for VariableDefinition {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![Token::String(self.name.clone())];
@@ -1152,57 +952,14 @@ impl Tokenizable for VariableDefinition {
 impl Tokenizable for VariableValue {
     fn tokenize(&self) -> Vec<Token> {
         match self {
-            VariableValue::Null { .. } => vec![Token::Keyword("null".to_string())],
+            VariableValue::Null => vec![Token::Keyword("null".to_string())],
             VariableValue::Bool(v) => vec![Token::Boolean(v.to_string())],
-            VariableValue::Integer(v) => vec![Token::Number(v.to_string())],
-            VariableValue::Float(v) => vec![Token::Number(v.to_string())],
+            VariableValue::Number(v) => vec![Token::Number(v.to_string())],
             VariableValue::String(v) => v.tokenize(),
         }
     }
 }
 
-impl Tokenizable for VerboseOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("verbose".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.push(Token::Boolean(self.value.to_string()));
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
-impl Tokenizable for VeryVerboseOption {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(
-            &mut self
-                .line_terminators
-                .iter()
-                .flat_map(|e| e.tokenize())
-                .collect(),
-        );
-        tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String("very-verbose".to_string()));
-        tokens.append(&mut self.space1.tokenize());
-        tokens.push(Token::Colon(String::from(":")));
-        tokens.append(&mut self.space2.tokenize());
-        tokens.push(Token::Boolean(self.value.to_string()));
-        tokens.append(&mut self.line_terminator0.tokenize());
-        tokens
-    }
-}
-
 impl Tokenizable for Filter {
     fn tokenize(&self) -> Vec<Token> {
         match self.value.clone() {
@@ -1225,6 +982,12 @@ impl Tokenizable for Filter {
             FilterValue::HtmlUnescape => {
                 vec![Token::FilterType(String::from("htmlUnescape"))]
             }
+            FilterValue::JsonPath { space0, expr } => {
+                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("jsonpath"))];
+                tokens.append(&mut space0.tokenize());
+                tokens.append(&mut expr.tokenize());
+                tokens
+            }
             FilterValue::Nth { space0, n } => {
                 let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("nth"))];
                 tokens.append(&mut space0.tokenize());
