@@ -52,14 +52,14 @@ fn check_request(request: &Request) -> Vec<Error> {
     let mut errors = vec![];
     if !request.space0.value.is_empty() {
         errors.push(Error {
-            source_info: request.space0.source_info.clone(),
-            inner: LinterError::UnnecessarySpace {},
+            source_info: request.space0.source_info,
+            inner: LinterError::UnnecessarySpace,
         });
     }
     if request.space1.value != " " {
         errors.push(Error {
-            source_info: request.space1.source_info.clone(),
-            inner: LinterError::OneSpace {},
+            source_info: request.space1.source_info,
+            inner: LinterError::OneSpace,
         });
     }
     for error in check_line_terminator(&request.line_terminator0) {
@@ -81,7 +81,7 @@ fn lint_request(request: &Request) -> Request {
     let mut sections: Vec<Section> = request.sections.iter().map(lint_section).collect();
     sections.sort_by_key(|k| section_value_index(k.value.clone()));
 
-    let source_info = SourceInfo::new(0, 0, 0, 0);
+    let source_info = SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0));
     Request {
         line_terminators,
         space0,
@@ -100,8 +100,8 @@ fn check_response(response: &Response) -> Vec<Error> {
     let mut errors = vec![];
     if !response.space0.value.is_empty() {
         errors.push(Error {
-            source_info: response.space0.source_info.clone(),
-            inner: LinterError::UnnecessarySpace {},
+            source_info: response.space0.source_info,
+            inner: LinterError::UnnecessarySpace,
         });
     }
     errors
@@ -129,7 +129,7 @@ fn lint_response(response: &Response) -> Response {
         headers,
         sections,
         body,
-        source_info: SourceInfo::new(0, 0, 0, 0),
+        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
     }
 }
 
@@ -142,7 +142,7 @@ fn lint_section(section: &Section) -> Section {
         space0: empty_whitespace(),
         value,
         line_terminator0,
-        source_info: SourceInfo::new(0, 0, 0, 0),
+        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
     }
 }
 
@@ -151,7 +151,9 @@ fn lint_section_value(section_value: &SectionValue) -> SectionValue {
         SectionValue::QueryParams(params) => {
             SectionValue::QueryParams(params.iter().map(lint_key_value).collect())
         }
-        SectionValue::BasicAuth(param) => SectionValue::BasicAuth(lint_key_value(param)),
+        SectionValue::BasicAuth(param) => {
+            SectionValue::BasicAuth(param.as_ref().map(lint_key_value))
+        }
         SectionValue::Captures(captures) => {
             SectionValue::Captures(captures.iter().map(lint_capture).collect())
         }
@@ -225,15 +227,15 @@ fn lint_capture(capture: &Capture) -> Capture {
 
 fn lint_query(query: &Query) -> Query {
     Query {
-        source_info: SourceInfo::new(0, 0, 0, 0),
+        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         value: lint_query_value(&query.value),
     }
 }
 
 fn lint_query_value(query_value: &QueryValue) -> QueryValue {
     match query_value {
-        QueryValue::Status {} => QueryValue::Status {},
-        QueryValue::Url {} => QueryValue::Url {},
+        QueryValue::Status => QueryValue::Status,
+        QueryValue::Url => QueryValue::Url,
         QueryValue::Header { name, .. } => QueryValue::Header {
             name: name.clone(),
             space0: one_whitespace(),
@@ -251,7 +253,7 @@ fn lint_query_value(query_value: &QueryValue) -> QueryValue {
                 },
             }
         }
-        QueryValue::Body {} => QueryValue::Body {},
+        QueryValue::Body => QueryValue::Body,
         QueryValue::Xpath { expr, .. } => QueryValue::Xpath {
             expr: expr.clone(),
             space0: one_whitespace(),
@@ -268,10 +270,10 @@ fn lint_query_value(query_value: &QueryValue) -> QueryValue {
             name: name.clone(),
             space0: one_whitespace(),
         },
-        QueryValue::Duration {} => QueryValue::Duration {},
-        QueryValue::Bytes {} => QueryValue::Bytes {},
-        QueryValue::Sha256 {} => QueryValue::Sha256 {},
-        QueryValue::Md5 {} => QueryValue::Md5 {},
+        QueryValue::Duration => QueryValue::Duration,
+        QueryValue::Bytes => QueryValue::Bytes,
+        QueryValue::Sha256 => QueryValue::Sha256,
+        QueryValue::Md5 => QueryValue::Md5,
         QueryValue::Certificate {
             attribute_name: field,
             ..
@@ -327,7 +329,7 @@ fn lint_predicate(predicate: &Predicate) -> Predicate {
 
 fn lint_predicate_func(predicate_func: &PredicateFunc) -> PredicateFunc {
     PredicateFunc {
-        source_info: SourceInfo::new(0, 0, 0, 0),
+        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         value: lint_predicate_func_value(&predicate_func.value),
     }
 }
@@ -388,17 +390,14 @@ fn lint_predicate_func_value(predicate_func_value: &PredicateFuncValue) -> Predi
             space0: one_whitespace(),
             value: lint_predicate_value(value),
         },
-        PredicateFuncValue::CountEqual { value, .. } => PredicateFuncValue::CountEqual {
-            space0: one_whitespace(),
-            value: value.clone(),
-        },
-        PredicateFuncValue::IsInteger {} => PredicateFuncValue::IsInteger {},
-        PredicateFuncValue::IsFloat {} => PredicateFuncValue::IsFloat {},
-        PredicateFuncValue::IsBoolean {} => PredicateFuncValue::IsBoolean {},
-        PredicateFuncValue::IsString {} => PredicateFuncValue::IsString {},
-        PredicateFuncValue::IsCollection {} => PredicateFuncValue::IsCollection {},
-        PredicateFuncValue::Exist {} => PredicateFuncValue::Exist {},
-        PredicateFuncValue::IsEmpty {} => PredicateFuncValue::IsEmpty {},
+        PredicateFuncValue::IsInteger => PredicateFuncValue::IsInteger,
+        PredicateFuncValue::IsFloat => PredicateFuncValue::IsFloat,
+        PredicateFuncValue::IsBoolean => PredicateFuncValue::IsBoolean,
+        PredicateFuncValue::IsString => PredicateFuncValue::IsString,
+        PredicateFuncValue::IsCollection => PredicateFuncValue::IsCollection,
+        PredicateFuncValue::IsDate => PredicateFuncValue::IsDate,
+        PredicateFuncValue::Exist => PredicateFuncValue::Exist,
+        PredicateFuncValue::IsEmpty => PredicateFuncValue::IsEmpty,
     }
 }
 
@@ -408,10 +407,10 @@ fn lint_predicate_value(predicate_value: &PredicateValue) -> PredicateValue {
         PredicateValue::MultilineString(value) => {
             PredicateValue::MultilineString(lint_multiline_string(value))
         }
-        PredicateValue::Integer(value) => PredicateValue::Integer(*value),
-        PredicateValue::Float(value) => PredicateValue::Float(value.clone()),
         PredicateValue::Bool(value) => PredicateValue::Bool(*value),
-        PredicateValue::Null {} => PredicateValue::Null {},
+        PredicateValue::Null => PredicateValue::Null,
+        PredicateValue::Number(value) => PredicateValue::Number(value.clone()),
+        PredicateValue::File(value) => PredicateValue::File(lint_file(value)),
         PredicateValue::Hex(value) => PredicateValue::Hex(lint_hex(value)),
         PredicateValue::Base64(value) => PredicateValue::Base64(lint_base64(value)),
         PredicateValue::Expression(value) => PredicateValue::Expression(value.clone()),
@@ -503,10 +502,7 @@ fn lint_hex(hex: &Hex) -> Hex {
 fn lint_file(file: &File) -> File {
     File {
         space0: empty_whitespace(),
-        filename: Filename {
-            value: file.filename.value.clone(),
-            source_info: SourceInfo::new(0, 0, 0, 0),
-        },
+        filename: lint_template(&file.filename),
         space1: empty_whitespace(),
     }
 }
@@ -557,15 +553,15 @@ fn lint_file_param(file_param: &FileParam) -> FileParam {
 
 fn empty_whitespace() -> Whitespace {
     Whitespace {
-        value: "".to_string(),
-        source_info: SourceInfo::new(0, 0, 0, 0),
+        value: String::new(),
+        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
     }
 }
 
 fn one_whitespace() -> Whitespace {
     Whitespace {
         value: " ".to_string(),
-        source_info: SourceInfo::new(0, 0, 0, 0),
+        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
     }
 }
 
@@ -576,8 +572,8 @@ fn check_line_terminator(line_terminator: &LineTerminator) -> Vec<Error> {
         None => {
             if !line_terminator.space0.value.is_empty() {
                 errors.push(Error {
-                    source_info: line_terminator.space0.source_info.clone(),
-                    inner: LinterError::UnnecessarySpace {},
+                    source_info: line_terminator.space0.source_info,
+                    inner: LinterError::UnnecessarySpace,
                 });
             }
         }
@@ -589,18 +585,18 @@ fn lint_line_terminator(line_terminator: &LineTerminator) -> LineTerminator {
     let space0 = match line_terminator.comment {
         None => empty_whitespace(),
         Some(_) => Whitespace {
-            value: line_terminator.clone().space0.value,
-            source_info: SourceInfo::new(0, 0, 0, 0),
+            value: line_terminator.space0.value.clone(),
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         },
     };
     let comment = line_terminator.comment.as_ref().map(lint_comment);
     let newline = Whitespace {
         value: if line_terminator.newline.value.is_empty() {
-            "".to_string()
+            String::new()
         } else {
             "\n".to_string()
         },
-        source_info: SourceInfo::new(0, 0, 0, 0),
+        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
     };
     LineTerminator {
         space0,
@@ -616,6 +612,7 @@ fn lint_comment(comment: &Comment) -> Comment {
         } else {
             format!(" {}", comment.value)
         },
+        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
     }
 }
 
@@ -629,7 +626,7 @@ fn lint_entry_option(entry_option: &EntryOption) -> EntryOption {
 
 fn lint_filter(filter: &Filter) -> Filter {
     Filter {
-        source_info: SourceInfo::new(0, 0, 0, 0),
+        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
         value: lint_filter_value(&filter.value),
     }
 }
