@@ -26,7 +26,7 @@ use proptest::prelude::*;
 
 fn whitespace() -> BoxedStrategy<String> {
     prop_oneof![
-        Just("".to_string()),
+        Just(String::new()),
         Just(" ".to_string()),
         Just("  ".to_string()),
     ]
@@ -54,16 +54,16 @@ fn value_boolean() -> BoxedStrategy<JsonValue> {
 }
 
 fn value_string() -> BoxedStrategy<JsonValue> {
-    let source_info = SourceInfo::new(0, 0, 0, 0);
+    let source_info = SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0));
     let variable = Variable {
         name: "name".to_string(),
-        source_info: source_info.clone(),
+        source_info,
     };
     prop_oneof![
         Just(JsonValue::String(Template {
             elements: vec![],
             delimiter: Some('"'),
-            source_info: source_info.clone()
+            source_info
         })),
         Just(JsonValue::String(Template {
             elements: vec![TemplateElement::String {
@@ -71,7 +71,7 @@ fn value_string() -> BoxedStrategy<JsonValue> {
                 value: "Hello".to_string(),
             }],
             delimiter: Some('"'),
-            source_info: source_info.clone()
+            source_info
         })),
         Just(JsonValue::String(Template {
             elements: vec![
@@ -81,13 +81,13 @@ fn value_string() -> BoxedStrategy<JsonValue> {
                 },
                 TemplateElement::Expression(Expr {
                     space0: Whitespace {
-                        value: "".to_string(),
-                        source_info: source_info.clone()
+                        value: String::new(),
+                        source_info
                     },
                     variable,
                     space1: Whitespace {
-                        value: "".to_string(),
-                        source_info: source_info.clone()
+                        value: String::new(),
+                        source_info
                     },
                 })
             ],
@@ -119,7 +119,7 @@ fn value() -> BoxedStrategy<JsonValue> {
                     JsonValue::List {
                         space0,
                         elements: vec![JsonListElement {
-                            space0: "".to_string(),
+                            space0: String::new(),
                             value,
                             space1,
                         }],
@@ -138,7 +138,7 @@ fn value() -> BoxedStrategy<JsonValue> {
                             space0: space00,
                             elements: vec![
                                 JsonListElement {
-                                    space0: "".to_string(),
+                                    space0: String::new(),
                                     value: value0,
                                     space1: space01
                                 },
@@ -163,7 +163,7 @@ fn value() -> BoxedStrategy<JsonValue> {
                             space0: space00,
                             elements: vec![
                                 JsonListElement {
-                                    space0: "".to_string(),
+                                    space0: String::new(),
                                     value: value0,
                                     space1: space01
                                 },
@@ -188,7 +188,7 @@ fn value() -> BoxedStrategy<JsonValue> {
                             space0: space00,
                             elements: vec![
                                 JsonListElement {
-                                    space0: "".to_string(),
+                                    space0: String::new(),
                                     value: value0,
                                     space1: space01
                                 },
@@ -216,14 +216,14 @@ fn value() -> BoxedStrategy<JsonValue> {
                         JsonValue::Object {
                             space0,
                             elements: vec![JsonObjectElement {
-                                space0: "".to_string(),
+                                space0: String::new(),
                                 name: Template {
                                     delimiter: None,
                                     elements: vec![TemplateElement::String {
                                         value: "key1".to_string(),
                                         encoded: "key1".to_string(),
                                     }],
-                                    source_info: SourceInfo::new(1, 1, 1, 1),
+                                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                                 },
                                 space1,
                                 space2,
